package com.persagy.dmp.alarm.config;

import com.persagy.common.web.BaseResponse;
import com.persagy.dmp.alarm.jms.MessageProcesser;
import com.persagy.dmp.rwd.model.DmpMessage;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @author: yaoll
 * @date: 2020-10-22
 * @verison: 1.0
 */
@Aspect
@Component
public class DmpMessageAspect {
	@Autowired
	private MessageProcesser messageProcesser;

	@Pointcut("execution(public * com.persagy.dmp.*.controller.*.*(..))")
	public void controllerPoint() {
	}

	@AfterReturning(returning = "ret", pointcut = "controllerPoint()")
	public void doAfterReturning(Object ret) {
		if (ret != null) {
			if (ret instanceof BaseResponse) {
				BaseResponse resp = (BaseResponse) ret;
				List<DmpMessage> messageList = resp.getMessageList();
				if (messageList != null && messageList.size() > 0) {
					for (DmpMessage message : messageList) {
						messageProcesser.convertAndSend(message);
					}
				}
			}
		}
	}
}
