package com.persagy.dmp.alarm.config;

import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.json.JacksonMapper;
import com.persagy.common.web.BaseResponse;
import com.persagy.dmp.config.DmpParameterStorage;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.*;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;

import static com.persagy.dmp.alarm.config.RequestLogAspect.prepare;

/**
 * @description: 查询切面
 * @author: lixing
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/12/3 9:42 上午
 * @version: V1.0
 */
@Slf4j
@Aspect
@Component
public class QueryAspect {
	@Pointcut("execution(public * com.persagy.dmp.alarm.service.*.query(..))")
	public void queryPoint() {
	}

	/**
	 * @description: 在查询条件中增加根据项目id和集团编码
	 * @param: joinPoint
	 * @return: void
	 * @exception:
	 * @author: lixing
	 * @company: Persagy Technology Co.,Ltd
	 * @since: 2020/12/3 9:56 上午
	 * @version: V1.0
	 */
	@Before("queryPoint()")
	public void doBefore(JoinPoint joinPoint) {
		ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
		HttpServletRequest request = attributes.getRequest();

		Object[] args = joinPoint.getArgs();
		for (Object arg : args) {
			if (arg instanceof JacksonCriteria) {
				JacksonCriteria criteria = (JacksonCriteria) arg;
				criteria.add("groupCode", request.getParameter("groupCode"));
				/* 查询接口中，参数为0代表查询所有；报警条目查询例外，0代表查询通用*/
				// 查询参数中不包含projectId，且param中projectId不等于0，将projectId放入查询条件
				if (StringUtils.isEmpty(criteria.getString("projectId")) &&
						!"0".equals(request.getParameter("projectId"))) {
					criteria.add("projectId", request.getParameter("projectId"));
				}
				// 查询参数中不包含appId，且param中appId不等于0，将appId放入查询条件
				if (StringUtils.isEmpty(criteria.getString("appId")) &&
						!"0".equals(request.getParameter("appId"))) {
					criteria.add("appId", request.getParameter("appId"));
				}
			}
		}
	}
}
