package com.persagy.dmp.alarm.config;

import com.persagy.common.criteria.JacksonCriteria;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;

/**
 * @description: 查询切面
 * @author: lixing
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/12/3 9:42 上午
 * @version: V1.0
 */
@Slf4j
@Aspect
@Component
public class RequestParamCheckAspect {

	private static final String errorInfo = "request url中参数[%s]不能为空";
	@Pointcut("execution(public * com.persagy.dmp.alarm.controller.*.*(..))")
	public void point() {
	}

	/**
	 * @description: 校验参数中的必填项
	 * @param: joinPoint
	 * @return: void
	 * @exception:
	 * @author: lixing
	 * @company: Persagy Technology Co.,Ltd
	 * @since: 2020/12/3 9:56 上午
	 * @version: V1.0
	 */
	@Before("point()")
	public void doBefore(JoinPoint joinPoint) throws Exception{
		ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
		HttpServletRequest request = attributes.getRequest();
		String projectId = request.getParameter("projectId");
		String groupCode = request.getParameter("groupCode");
		String appId = request.getParameter("appId");
		String userId = request.getParameter("userId");

		if (StringUtils.isEmpty(projectId)) {
			throw new RuntimeException(String.format(errorInfo, "projectId"));
		}
		if (StringUtils.isEmpty(groupCode)) {
			throw new RuntimeException(String.format(errorInfo, "groupCode"));
		}
		if (StringUtils.isEmpty(userId)) {
			throw new RuntimeException(String.format(errorInfo, "userId"));
		}
	}

}
