package com.persagy.dmp.alarm.controller;

import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.web.MapResponse;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.alarm.model.AlarmConfigModel;
import com.persagy.dmp.alarm.model.AlarmConfigModels;
import com.persagy.dmp.alarm.service.AlarmConfigService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping("/alarm/config")
public class AlarmConfigController {
    @Autowired
    private AlarmConfigService service;

    @PostMapping("/query")
    public PagedResponse<AlarmConfigModel> query(@RequestBody JacksonCriteria criteria) {
        return service.query(criteria);
    }

    @PostMapping("/create")
    public MapResponse create(@RequestBody AlarmConfigModel param) {
        return service.create(param);
    }

    /**
     * @description: 批量创建报警定义
     * @param: param
     * @return: com.persagy.common.web.MapResponse
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/22 2:37 下午
     * @version: V1.0
     */
    @PostMapping("/batch_create")
    public MapResponse batchCreate(@RequestBody AlarmConfigModels configModelsObj) {
        List<AlarmConfigModel> configModels = configModelsObj.getAlarmConfigs();
        if (configModels == null) {
            MapResponse response = new MapResponse();
            response.setFail("请传入要创建的报警定义！");
            return response;
        }
        return service.batchCreate(configModels);
    }

    @PostMapping("/update")
    public MapResponse update(@RequestBody AlarmConfigModel param) {
        return service.update(param);
    }

    /**
     * @description: 批量更新报警定义
     * @param: param
     * @return: com.persagy.common.web.MapResponse
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/22 2:37 下午
     * @version: V1.0
     */
    @PostMapping("/batch_update")
    public MapResponse batchUpdate(@RequestBody AlarmConfigModels configModelsObj) {
        List<AlarmConfigModel> configModels = configModelsObj.getAlarmConfigs();
        if (configModels == null) {
            MapResponse response = new MapResponse();
            response.setFail("请传入要修改的报警定义！");
            return response;
        }
        return service.batchUpdate(configModels);
    }

    //	@PostMapping("/delete")
    //	public MapResponse delete(@RequestBody AlarmConfigModel param){
    //		return service.delete(param);
    //	}


    @PostMapping("/init")
    public MapResponse init(@RequestBody AlarmConfigModel param) {
        return service.init(param);
    }
}
