package com.persagy.dmp.alarm.entity;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.vladmihalcea.hibernate.type.json.JsonStringType;
import lombok.Data;
import lombok.Getter;
import lombok.Setter;
import com.persagy.dmp.alarm.model.AlarmConfigModel;
import org.hibernate.annotations.GenericGenerator;
import org.hibernate.annotations.Type;
import org.hibernate.annotations.TypeDef;

import javax.persistence.GeneratedValue;
import javax.persistence.*;
import java.util.Date;

@Setter
@Getter
@Entity
@TypeDef(name = "json", typeClass = JsonStringType.class)
@Table(name = "alarm_config")
public class AlarmConfig {

	@Id
	@GeneratedValue(generator = "assignedId")
	@GenericGenerator(name = "assignedId", strategy = "assigned")
	private String id;
	private String objId;
	private String classCode;
	private String projectId;
	private String itemCode;
	private String targetId;
	private String category;
	@Column(name="`level`")
	private String level;
	private String remark;
	private String groupCode;

	@Type(type = "json")
	@Column(name="`condition`")
	private ObjectNode condition;
	@Column(name="`open`")
	private Integer open;
	private Integer concern;
	private Integer userDefined;
	private String createUser;
	private Date createTime;
	private String updateUser;
	private Date updateTime;
	private Integer valid;

	public static AlarmConfig fromModel(AlarmConfigModel model) {
		AlarmConfig entity = new AlarmConfig();
		entity.setId(model.getId());
		entity.setObjId(model.getObjId());
		entity.setClassCode(model.getClassCode());
		entity.setProjectId(model.getProjectId());
		entity.setItemCode(model.getItemCode());
		entity.setTargetId(model.getTargetId());
		entity.setCategory(model.getCategory());
		entity.setLevel(model.getLevel());
		entity.setRemark(model.getRemark());
		entity.setCondition(model.getCondition());
		entity.setOpen(model.getOpen());
		entity.setConcern(model.getConcern());
		entity.setUserDefined(model.getUserDefined());
		entity.setCreateUser(model.getCreateUser());
		entity.setCreateTime(model.getCreateTime());
		entity.setUpdateUser(model.getUpdateUser());
		entity.setUpdateTime(model.getUpdateTime());
		entity.setValid(model.getValid());
		entity.setGroupCode(model.getGroupCode());
		return entity;
	}

	public AlarmConfigModel toModel() {
		AlarmConfigModel model = new AlarmConfigModel();
		model.setId(this.id);
		model.setObjId(this.getObjId());
		model.setClassCode(this.getClassCode());
		model.setProjectId(this.getProjectId());
		model.setItemCode(this.getItemCode());
		model.setTargetId(this.getTargetId());
		model.setCategory(this.getCategory());
		model.setLevel(this.getLevel());
		model.setRemark(this.getRemark());
		model.setCondition(this.getCondition());
		model.setOpen(this.getOpen());
		model.setConcern(this.getConcern());
		model.setUserDefined(this.getUserDefined());
		model.setCreateUser(this.getCreateUser());
		model.setCreateTime(this.getCreateTime());
		model.setUpdateUser(this.getUpdateUser());
		model.setUpdateTime(this.getUpdateTime());
		model.setValid(this.getValid());
		model.setGroupCode(this.getGroupCode());
		return model;
	}

	/**
	 * @description: <p>报警定义唯一标识; 当报警条目和报警匹配条件变化时会重新生成报警定义，
	 * 因此在与其他系统进行通讯时，一般不使用报警定义的id作为其唯一标识，而是使用itemCode + objID作为其唯一标识</p>
	 * @author: lixing
	 * @company: Persagy Technology Co.,Ltd
	 * @since: 2020/10/26 11:25 上午
	 * @version: V1.0
	 */
	@Data
	public class AlarmConfigUnique {
		private String itemCode;
		private String objId;

		AlarmConfigUnique(String itemCode, String objId) {
			this.itemCode = itemCode;
			this.objId = objId;
		}
	}

	public AlarmConfigUnique getAlarmConfigUnique() {
		return new AlarmConfigUnique(this.itemCode, this.objId);
	}

}
