package com.persagy.dmp.alarm.entity;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.vladmihalcea.hibernate.type.json.JsonStringType;
import lombok.Getter;
import lombok.Setter;
import com.persagy.dmp.alarm.model.AlarmItemModel;
import org.hibernate.annotations.GenericGenerator;
import org.hibernate.annotations.Type;
import org.hibernate.annotations.TypeDef;

import javax.persistence.GeneratedValue;
import javax.persistence.*;
import java.util.Date;

@Setter
@Getter
@Entity
@TypeDef(name = "json", typeClass = JsonStringType.class)
@Table(name = "alarm_item")
public class AlarmItem {

	@Id
	@GeneratedValue(generator = "assignedId")
	@GenericGenerator(name = "assignedId", strategy = "assigned")
	private String id;
	private String code;
	private String name;
	private String category;
	private String level;
	private String remark;

	@Type(type = "json")
	private ObjectNode condition;
	private String dealFun;
	private String createUser;
	private Date createTime;
	private String updateUser;
	private Date updateTime;
	private Integer valid;
	private String projectId;
	private String groupCode;
	@Column(name="`app_id`")
	private String appId;

	public static AlarmItem fromModel(AlarmItemModel model) {
		AlarmItem entity = new AlarmItem();
		entity.setId(model.getId());
		entity.setCode(model.getCode());
		entity.setName(model.getName());
		entity.setCategory(model.getCategory());
		entity.setLevel(model.getLevel());
		entity.setRemark(model.getRemark());
		entity.setCondition(model.getCondition());
		entity.setDealFun(model.getDealFun());
		entity.setCreateUser(model.getCreateUser());
		entity.setCreateTime(model.getCreateTime());
		entity.setUpdateUser(model.getUpdateUser());
		entity.setUpdateTime(model.getUpdateTime());
		entity.setValid(model.getValid());
		entity.setProjectId(model.getProjectId());
		entity.setGroupCode(model.getGroupCode());
		entity.setAppId(model.getAppId());
		return entity;
	}

	public AlarmItemModel toModel() {
		AlarmItemModel model = new AlarmItemModel();
		model.setId(this.id);
		model.setCode(this.getCode());
		model.setName(this.getName());
		model.setCategory(this.getCategory());
		model.setLevel(this.getLevel());
		model.setRemark(this.getRemark());
		model.setCondition(this.getCondition());
		model.setDealFun(this.getDealFun());
		model.setCreateUser(this.getCreateUser());
		model.setCreateTime(this.getCreateTime());
		model.setUpdateUser(this.getUpdateUser());
		model.setUpdateTime(this.getUpdateTime());
		model.setValid(this.getValid());
		model.setProjectId(this.getProjectId());
		model.setGroupCode(this.getGroupCode());
		model.setAppId(this.getAppId());
		return model;
	}

}
