package com.persagy.dmp.alarm.entity;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.dmp.alarm.enumeration.EnumAlarmLogType;
import com.vladmihalcea.hibernate.type.json.JsonStringType;
import lombok.Getter;
import lombok.Setter;
import com.persagy.dmp.alarm.model.AlarmLogModel;
import org.hibernate.annotations.GenericGenerator;
import org.hibernate.annotations.Type;
import org.hibernate.annotations.TypeDef;

import javax.persistence.GeneratedValue;
import javax.persistence.*;
import java.util.Date;

@Setter
@Getter
@Entity
@Table(name = "alarm_log")
@TypeDef(name = "json", typeClass = JsonStringType.class)
public class AlarmLog {

	@Id
	@GeneratedValue(generator = "assignedId")
	@GenericGenerator(name = "assignedId", strategy = "assigned")
	private String id;
	private String recordId;
	@Column(name = "`type`")
	private EnumAlarmLogType type;
	private Integer nature;
	@Column(name = "`before`")
	private Integer before;
	@Column(name = "`after`")
	private Integer after;
	@Type(type = "json")
	private ObjectNode remark;
	private Date createTime;
	private String createUser;
	private Date updateTime;
	private Integer valid;
	private String projectId;
	private String groupCode;

	public static AlarmLog fromModel(AlarmLogModel model) {
		AlarmLog entity = new AlarmLog();
		entity.setId(model.getId());
		entity.setRecordId(model.getRecordId());
		entity.setType(model.getType());
		entity.setNature(model.getNature());
		entity.setBefore(model.getBefore());
		entity.setAfter(model.getAfter());
		entity.setRemark(model.getRemark());
		entity.setCreateTime(model.getCreateTime());
		entity.setProjectId(model.getProjectId());
		entity.setGroupCode(model.getGroupCode());
		entity.setCreateUser(model.getCreateUser());
//		entity.setUpdateTime(model.getUpdateTime());
//		entity.setValid(model.getValid());
		return entity;
	}

	public AlarmLogModel toModel() {
		AlarmLogModel model = new AlarmLogModel();
		model.setId(this.id);
		model.setRecordId(this.getRecordId());
		model.setType(this.getType());
		model.setNature(this.getNature());
		model.setBefore(this.getBefore());
		model.setAfter(this.getAfter());
		model.setRemark(this.getRemark());
		model.setCreateTime(this.getCreateTime());
		model.setProjectId(this.getProjectId());
		model.setGroupCode(this.getGroupCode());
		model.setCreateUser(this.getCreateUser());
//		model.setUpdateTime(this.getUpdateTime());
//		model.setValid(this.getValid());
		return model;
	}

}
