package com.persagy.dmp.alarm.entity;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.vladmihalcea.hibernate.type.json.JsonStringType;
import lombok.Getter;
import lombok.Setter;
import com.persagy.dmp.alarm.model.AlarmRecordModel;
import org.hibernate.annotations.GenericGenerator;
import org.hibernate.annotations.Type;
import org.hibernate.annotations.TypeDef;

import javax.persistence.GeneratedValue;
import javax.persistence.*;
import java.util.Date;

@Setter
@Getter
@Entity
@Table(name = "alarm_record")
@TypeDef(name = "json", typeClass = JsonStringType.class)
public class AlarmRecord {

	@Id
	@GeneratedValue(generator = "assignedId")
	@GenericGenerator(name = "assignedId", strategy = "assigned")
	private String id;
	private String itemCode;
	private String projectId;
	private String objId;
	private String classCode;
	private String level;
	private String remark;
	private String name;
	private String category;
	private Integer concern;
	private Integer state;
	private Date effectStartTime;
	private Date effectEndTime;
	/** 报警性质 */
	private Integer nature;
	/** 处理方式 */
	private Integer treatMode;
	/** 处理状态 */
	private Integer treatState;
	private String createUser;
	@Type(type = "json")
	@Column(name="`condition`")
	private ObjectNode condition;
	@Type(type = "json")
	private ObjectNode triggerInfo;
	private Date triggerTime;
	@Type(type = "json")
	private ObjectNode endInfo;
	private Date endTime;
	private Date createTime;
	private String updateUser;
	private Date updateTime;
	private Integer valid;
	private String groupCode;
	private String orderId;
	private String orderState;
	@Type(type = "json")
	private ObjectNode supplement;

	public static AlarmRecord fromModel(AlarmRecordModel model) {
		AlarmRecord entity = new AlarmRecord();
		entity.setId(model.getId());
		entity.setItemCode(model.getItemCode());
		entity.setProjectId(model.getProjectId());
		entity.setObjId(model.getObjId());
		entity.setClassCode(model.getClassCode());
		entity.setLevel(model.getLevel());
		entity.setRemark(model.getRemark());
		entity.setName(model.getName());
		entity.setCategory(model.getCategory());
		entity.setConcern(model.getConcern());
		entity.setState(model.getState());
		entity.setEffectStartTime(model.getEffectStartTime());
		entity.setEffectEndTime(model.getEffectEndTime());
		entity.setNature(model.getNature());
		entity.setTreatMode(model.getTreatMode());
		entity.setTreatState(model.getTreatState());
		entity.setCreateUser(model.getCreateUser());
		entity.setCondition(model.getCondition());
		entity.setTriggerInfo(model.getTriggerInfo());
		entity.setTriggerTime(model.getTriggerTime());
		entity.setEndInfo(model.getEndInfo());
		entity.setEndTime(model.getEndTime());
		entity.setCreateTime(model.getCreateTime());
		entity.setUpdateUser(model.getUpdateUser());
		entity.setUpdateTime(model.getUpdateTime());
		entity.setValid(model.getValid());
		entity.setGroupCode(model.getGroupCode());
		entity.setOrderId(model.getOrderId());
		entity.setOrderState(model.getOrderState());
		entity.setSupplement(model.getSupplement());
		return entity;
	}

	public AlarmRecordModel toModel() {
		AlarmRecordModel model = new AlarmRecordModel();
		model.setId(this.id);
		model.setItemCode(this.getItemCode());
		model.setProjectId(this.getProjectId());
		model.setObjId(this.getObjId());
		model.setClassCode(this.getClassCode());
		model.setLevel(this.getLevel());
		model.setRemark(this.getRemark());
		model.setName(this.getName());
		model.setCategory(this.getCategory());
		model.setConcern(this.getConcern());
		model.setState(this.getState());
		model.setEffectStartTime(this.getEffectStartTime());
		model.setEffectEndTime(this.getEffectEndTime());
		model.setNature(this.getNature());
		model.setTreatMode(this.getTreatMode());
		model.setTreatState(this.getTreatState());
		model.setCreateUser(this.getCreateUser());
		model.setCondition(this.getCondition());
		model.setTriggerInfo(this.getTriggerInfo());
		model.setTriggerTime(this.getTriggerTime());
		model.setEndInfo(this.getEndInfo());
		model.setEndTime(this.getEndTime());
		model.setCreateTime(this.getCreateTime());
		model.setUpdateUser(this.getUpdateUser());
		model.setUpdateTime(this.getUpdateTime());
		model.setValid(this.getValid());
		model.setGroupCode(this.getGroupCode());
		model.setOrderId(this.getOrderId());
		model.setOrderState(this.getOrderState());
		model.setSupplement(this.getSupplement());
		return model;
	}

}
