package com.persagy.dmp.alarm.entity;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.vladmihalcea.hibernate.type.json.JsonStringType;
import lombok.Getter;
import lombok.Setter;
import com.persagy.dmp.alarm.model.AlarmTargetModel;
import org.hibernate.annotations.GenericGenerator;
import org.hibernate.annotations.Type;
import org.hibernate.annotations.TypeDef;

import javax.persistence.GeneratedValue;
import javax.persistence.*;
import java.util.Date;

@Setter
@Getter
@Entity
@TypeDef(name = "json", typeClass = JsonStringType.class)
@Table(name = "alarm_target")
public class AlarmTarget {

	@Id
	@GeneratedValue(generator = "assignedId")
	@GenericGenerator(name = "assignedId", strategy = "assigned")
	private String id;
	private String itemCode;
	private String classCode;

	@Type(type = "json")
	@Column(name = "`condition`")
	private ObjectNode condition;
	private Integer concern;
	private String level;
	private Integer open;
	private String createUser;
	private Date createTime;
	private String updateUser;
	private Date updateTime;
	private Integer valid = 1;
	private String projectId;
	private String groupCode;

	public static AlarmTarget fromModel(AlarmTargetModel model) {
		AlarmTarget entity = new AlarmTarget();
		entity.setId(model.getId());
		entity.setItemCode(model.getItemCode());
		entity.setClassCode(model.getClassCode());
		entity.setCondition(model.getCondition());
		entity.setLevel(model.getLevel());
		entity.setConcern(model.getConcern());
		entity.setOpen(model.getOpen());
//		entity.setCreateUser(model.getCreateUser());
//		entity.setCreateTime(model.getCreateTime());
//		entity.setUpdateUser(model.getUpdateUser());
//		entity.setUpdateTime(model.getUpdateTime());
//		entity.setValid(model.getValid());
//		entity.setProjectId(model.getProjectId());
//		entity.setGroupCode(model.getGroupCode());
		return entity;
	}

	public AlarmTargetModel toModel() {
		AlarmTargetModel model = new AlarmTargetModel();
		model.setId(this.id);
		model.setItemCode(this.getItemCode());
		model.setClassCode(this.getClassCode());
		model.setCondition(this.getCondition());
		model.setLevel(this.getLevel());
		model.setConcern(this.getConcern());
		model.setOpen(this.getOpen());
		model.setCreateUser(this.getCreateUser());
		model.setCreateTime(this.getCreateTime());
		model.setUpdateUser(this.getUpdateUser());
		model.setUpdateTime(this.getUpdateTime());
		model.setValid(this.getValid());
		model.setProjectId(this.getProjectId());
		model.setGroupCode(this.getGroupCode());
		return model;
	}

}
