package com.persagy.dmp.alarm.jms;

import com.persagy.common.json.JacksonMapper;
import com.persagy.dmp.alarm.enumeration.EnumAlarmMessageType;
import com.persagy.dmp.alarm.jms.executor.AlarmTargetUpdateExecutor;
import com.persagy.dmp.common.SpringUtils;
import com.persagy.dmp.config.DmpParameterStorage;
import com.persagy.dmp.rwd.model.DmpMessage;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.HashSet;
import java.util.Set;

/**
 * @author: yaoll
 * @date: 2020-10-22
 * @verison: 1.0
 */
@Slf4j
@Component
public class MessageProcesser {

	@Value("${persagy.dmp.exchange}")
	private String exchange;

	@Value("${persagy.dmp.alarm.routingKey}")
	private String rwdRoutingKey;

	@Autowired
	private RabbitTemplate rabbitTemplate;

	public void convertAndSend(DmpMessage message) {
		log.info("向rabbitMQ发送消息，message:" + JacksonMapper.toSimpleJson(message));
		rabbitTemplate.convertAndSend(exchange, rwdRoutingKey, message);
	}

	private static final Set<String> beanNames = new HashSet<>();

	static {
		beanNames.add(EnumAlarmMessageType.ALARM_TARGET_CREATE.getValue());
		beanNames.add(EnumAlarmMessageType.ALARM_TARGET_UPDATE.getValue());
		beanNames.add(EnumAlarmMessageType.ALARM_ITEM_UPDATE.getValue());
//		由其他服务消费
//		beanNames.add(RabbitMQConstants.ALARM_CONFIG_CREATE);
//		beanNames.add(RabbitMQConstants.ALARM_CONFIG_DELETE);
	}

	public void listen(DmpMessage message) {
		String type = message.getType();
		if (!beanNames.contains(type)) {
			return;
		}

		String executorName = type + "Executor";

		MessageExecutor executor = null;
		try {
			executor = SpringUtils.getBean(executorName, MessageExecutor.class);
		} catch (Exception e) {
			log.error(e.getMessage());
		}
		if (executor == null) {
			log.error("bean for [" + executorName + "] not exists");
		}

		try {
			DmpParameterStorage.ParameterData data = new DmpParameterStorage.ParameterData();
			data.setGroupCode(message.getGroupCode());
			data.setProjectId(message.getProjectId());
			data.setAppId(message.getAppId());

			DmpParameterStorage.set(data);

			executor.execute(message);
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			DmpParameterStorage.clear();
		}
	}

}
