package com.persagy.dmp.alarm.jms.executor;

import com.persagy.common.json.JacksonMapper;
import com.persagy.common.web.BaseResponse;
import com.persagy.common.web.MapResponse;
import com.persagy.dmp.alarm.jms.MessageExecutor;
import com.persagy.dmp.alarm.service.AlarmConfigService;
import com.persagy.dmp.rwd.model.DmpMessage;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

/**
 * @author: lixing
 * @date: 2020-10-23
 * @verison: 1.0
 */
@Slf4j
@Component
public class AlarmItemUpdateExecutor implements MessageExecutor {
	@Autowired
	private AlarmConfigService alarmConfigService;

	@Override
	public void execute(DmpMessage message) {
		log.info("AlarmItemUpdate : {}", JacksonMapper.toSimpleJson(message));

		/* 验证消息是否符合要求 */
		String itemCode = message.getStr1();

		if (StringUtils.isEmpty(itemCode)) {
			log.error("AlarmItemUpdate消息中缺少itemCode信息");
		}

		MapResponse response = alarmConfigService.batchUpdateWhenItemUpdate(itemCode, message.getProjectId(), message.getGroupCode());
		if (BaseResponse.FAIL.equals(response.getResult())) {
			log.error("消费AlarmItemUpdate消息异常，{}", response.getMessage());
		}
	}
}
