package com.persagy.dmp.alarm.jms.executor;

import com.persagy.common.json.JacksonMapper;
import com.persagy.common.web.BaseResponse;
import com.persagy.common.web.MapResponse;
import com.persagy.dmp.alarm.jms.MessageExecutor;
import com.persagy.dmp.alarm.service.AlarmConfigService;
import com.persagy.dmp.alarm.util.CheckRequiredParam;
import com.persagy.dmp.rwd.model.DmpMessage;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author: lixing
 * @date: 2020-10-23
 * @verison: 1.0
 */
@Slf4j
@Component
public class AlarmTargetDeleteExecutor implements MessageExecutor {
	@Autowired
	private AlarmConfigService alarmConfigService;

	@Override
	public void execute(DmpMessage message) {
		log.info("AlarmTargetDelete : {}", JacksonMapper.toSimpleJson(message));

		/* 验证消息是否符合要求 */
		CheckRequiredParam.check(message, "targetId", "projectId", "groupCode");

		MapResponse response = alarmConfigService.batchDeleteWhenTargetDelete(message.getTargetId(), message.getProjectId(), message.getGroupCode());
		if (BaseResponse.FAIL.equals(response.getResult())) {
			log.error("消费AlarmTargetDelete消息异常，{}", response.getMessage());
		}
	}
}
