package com.persagy.dmp.alarm.jms.executor;

import com.persagy.common.json.JacksonMapper;
import com.persagy.common.web.BaseResponse;
import com.persagy.common.web.MapResponse;
import com.persagy.dmp.alarm.jms.MessageExecutor;
import com.persagy.dmp.alarm.service.AlarmConfigService;
import com.persagy.dmp.alarm.util.CheckRequiredParam;
import com.persagy.dmp.rwd.model.DmpMessage;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

/**
 * @description: 对象类删除
 * @author: lixing
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/12/10 10:53 上午
 * @version: V1.0
 */
@Slf4j
@Component
public class InstanceObjDeleteExecutor implements MessageExecutor {
	@Autowired
	private AlarmConfigService alarmConfigService;

	@Override
	public void execute(DmpMessage message) {
		log.info("InstanceObjDelete : {}", JacksonMapper.toSimpleJson(message));

		/* 验证消息是否符合要求 */
		CheckRequiredParam.check(message, "targetId", "projectId");

		MapResponse response = alarmConfigService.deleteOnRemoveObj(message.getTargetId(), message.getProjectId());
		if (BaseResponse.FAIL.equals(response.getResult())) {
			log.error("消费InstanceObjDelete消息异常，{}", response.getMessage());
		}
	}
}
