package com.persagy.dmp.alarm.service;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.dmp.alarm.util.CheckRequiredParam;
import com.querydsl.core.types.dsl.BooleanExpression;
import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.web.PagedResponse;
import com.persagy.common.web.MapResponse;
import javax.transaction.Transactional;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;
import com.persagy.dmp.alarm.model.AlarmCommentModel;
import org.springframework.beans.factory.annotation.Autowired;
import com.persagy.dmp.alarm.entity.*;
import com.persagy.dmp.alarm.repository.AlarmCommentRepository;
import org.springframework.util.StringUtils;

@Service
public class AlarmCommentService extends AlarmBaseService{

	@Autowired
	private AlarmCommentRepository alarmCommentRepository;

	@Autowired
	private CriteriaUtils criteriaUtils;

	private List<BooleanExpression> parse(ObjectNode criteria) {
		// TODO
		return null;
	}

	public PagedResponse<AlarmCommentModel> query(JacksonCriteria criteria) {
		PagedResponse<AlarmComment> resp = criteriaUtils.query(QAlarmComment.alarmComment, this::parse, criteria);
		PagedResponse<AlarmCommentModel> result = new PagedResponse<>();
		result.setCount(resp.getCount());
		List<AlarmComment> dataList = resp.getData();
		if (dataList != null && dataList.size() > 0) {
			List<AlarmCommentModel> collect = dataList.stream().map(entity -> {
				AlarmCommentModel model = entity.toModel();
				return model;
			}).collect(Collectors.toList());
			result.setData(collect);
		}
		return result;
	}

	@Transactional
	public MapResponse create(AlarmCommentModel param) {
		MapResponse response = new MapResponse();
		AlarmParam alarmParam = prepareParam(response);
		if (alarmParam == null) {
			return response;
		}
		// 必填项校验
		String checkResult = CheckRequiredParam.check(param, "recordId", "content");
		if (!StringUtils.isEmpty(checkResult)) {
			response.setFail(checkResult);
			return response;
		}
		AlarmComment entity = AlarmComment.fromModel(param);
		entity.setId(UUID.randomUUID().toString());
		entity.setCreateUser(alarmParam.userId);
		entity.setProjectId(alarmParam.projectId);
		entity.setGroupCode(alarmParam.groupCode);
		entity.setCreateTime(new Date());
		alarmCommentRepository.save(entity);
		response.add("id", entity.getId());
		return response;
	}
}

