package com.persagy.dmp.alarm.service;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.date.DateUtils;
import com.persagy.common.web.MapResponse;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.alarm.entity.AlarmItem;
import com.persagy.dmp.alarm.entity.QAlarmItem;
import com.persagy.dmp.alarm.enumeration.EnumAlarmMessageType;
import com.persagy.dmp.alarm.model.AlarmItemModel;
import com.persagy.dmp.alarm.repository.AlarmItemRepository;
import com.persagy.dmp.rwd.model.DmpMessage;
import com.querydsl.core.types.dsl.BooleanExpression;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.transaction.Transactional;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class AlarmItemService extends AlarmBaseService {

    @Autowired
    private AlarmItemRepository alarmItemRepository;

    @Autowired
    private CriteriaUtils criteriaUtils;

    private List<BooleanExpression> parse(ObjectNode criteria) {
        //        List<BooleanExpression> expressions = new LinkedList<>();
        //        ArrayNode itemCodes = (ArrayNode) criteria.findValue("itemCodes");
        //        if (itemCodes != null) {
        //            QAlarmItem qAlarmItem = QAlarmItem.alarmItem;
        //            expressions.addAll(CriteriaUtils.parse(qAlarmItem.code, itemCodes));
        //        }
        //        return expressions;
        return null;
    }

    public PagedResponse<AlarmItemModel> query(JacksonCriteria criteria) {
        // 报警条目查询中的0代表的是通用
        addDefaultQueryCondition(criteria, "projectId", "0");
        addDefaultQueryCondition(criteria, "groupCode", "0");
        addDefaultQueryCondition(criteria, "appId", "0");

        PagedResponse<AlarmItem> resp = criteriaUtils.query(QAlarmItem.alarmItem, this::parse, criteria);
        PagedResponse<AlarmItemModel> result = new PagedResponse<>();
        result.setCount(resp.getCount());
        List<AlarmItem> dataList = resp.getData();
        if (dataList != null && dataList.size() > 0) {
            List<AlarmItemModel> collect = dataList.stream().map(entity -> {
                AlarmItemModel model = entity.toModel();
                return model;
            }).collect(Collectors.toList());
            result.setData(collect);
        }
        return result;
    }

    /**
     * @description: 添加默认查询条件
     * 例如：当传入的查询条件中projectId = 1时，实际需要查询projectId = [0, 1]的数据，使用此方法将0添加到查询条件中
     * @param: criteria
     * @param: paramName
     * @param: defaultCondition
     * @return: void
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/12/5 5:29 下午
     * @version: V1.0
     */
    private void addDefaultQueryCondition(JacksonCriteria criteria, String paramName, String defaultCondition) {
        // 这里不校验已有查询条件和默认查询条件是否重复
        ObjectNode queryBody = criteria.getCriteria();
        JsonNode jsonNode = queryBody.get(paramName);
        if (jsonNode == null) {
            criteria.add(paramName, defaultCondition);
        } else if (jsonNode.isArray()) {
            ((ArrayNode) jsonNode).add(defaultCondition);
        } else {
            List<String> conditions = new ArrayList<>();
            conditions.add(defaultCondition);
            conditions.add(jsonNode.asText());
            criteria.add(paramName, conditions);
        }
    }

    @Transactional
    public MapResponse create(AlarmItemModel param) {
        MapResponse response = new MapResponse();
        AlarmParam alarmParam = prepareParam(response);
        if (alarmParam == null) {
            return response;
        }
        AlarmItem entity = AlarmItem.fromModel(param);
        entity.setCreateUser(alarmParam.userId);
        entity.setCreateTime(new Date());
        alarmItemRepository.save(entity);
        response.add("id", entity.getId());
        /* 拼装需要推送至消息队列的消息 */
        DmpMessage msg = new DmpMessage();
        msg.setMid(UUID.randomUUID().toString());
        msg.setType(EnumAlarmMessageType.ALARM_ITEM_CREATE.getValue());
        msg.setGroupCode(alarmParam.groupCode);
        msg.setProjectId(alarmParam.projectId);
        msg.setTargetId(entity.getId());
        msg.setStr1(entity.getCode());
        msg.setAppId(alarmParam.appId);
        msg.setSendTime(DateUtils.format(new Date()));
        response.add(msg);
        return response;
    }

    @Transactional
    public MapResponse update(AlarmItemModel param) {
        MapResponse response = new MapResponse();
        AlarmParam alarmParam = prepareParam(response);
        if (alarmParam == null) {
            return response;
        }
        AlarmItem entity = AlarmItem.fromModel(param);
        entity.setUpdateUser(alarmParam.userId);
        entity.setUpdateTime(new Date());
        alarmItemRepository.save(entity);
        response.add("id", entity.getId());
        /* 拼装需要推送至消息队列的消息 */
        DmpMessage msg = new DmpMessage();
        msg.setMid(UUID.randomUUID().toString());
        msg.setType(EnumAlarmMessageType.ALARM_ITEM_UPDATE.getValue());
        msg.setGroupCode(alarmParam.groupCode);
        msg.setProjectId(alarmParam.projectId);
        msg.setTargetId(entity.getId());
        msg.setStr1(entity.getCode());
        msg.setAppId(alarmParam.appId);
        msg.setSendTime(DateUtils.format(new Date()));
        response.add(msg);
        return response;
    }
    //
    //	@Transactional
    //	public MapResponse delete(AlarmItemModel param) {
    //		MapResponse response = new MapResponse();
    //		// TODO
    //		alarmItemRepository.deleteById(param.getId());
    //		return response;
    //	}

    /**
     * @description: 根据itemCodes获取itemCode, alarmItem的映射map
     * @param: itemCodes 报警条目编码列表
     * @return: java.util.HashMap<java.lang.String, com.persagy.dmp.alarm.model.AlarmItemModel>
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/20 2:25 下午
     * @version: V1.0
     */
    public Map<String, AlarmItemModel> getItemCodeModelMap(List<String> itemCodes) {
        HashMap<String, AlarmItemModel> itemCodeModelMap = new HashMap<>();
        if (CollectionUtils.isEmpty(itemCodes)) {
            return itemCodeModelMap;
        }
        JacksonCriteria criteria = JacksonCriteria.newInstance();
        criteria.add("itemCode", itemCodes);
        PagedResponse<AlarmItemModel> response = query(criteria);
        if (response == null) {
            return itemCodeModelMap;
        }
        List<AlarmItemModel> data = response.getData();
        if (CollectionUtils.isEmpty(data)) {
            return itemCodeModelMap;
        }

        return data.stream().collect(
                Collectors.toMap(AlarmItemModel::getCode, itemModel -> itemModel, (k1, k2) -> k1)
        );
    }

    public AlarmItemModel getAlarmItemByItemCode(String itemCode) {
        JacksonCriteria criteria = JacksonCriteria.newInstance();
        criteria.add("itemCode", itemCode);
        List<AlarmItemModel> alarmItemModels = queryList(criteria);
        if (CollectionUtils.isEmpty(alarmItemModels)) {
            return null;
        }
        return alarmItemModels.get(0);
    }

    public List<AlarmItemModel> queryList(JacksonCriteria criteria) {
        PagedResponse<AlarmItemModel> pagedResponse = query(criteria);
        if (pagedResponse == null) {
            return null;
        }
        List<AlarmItemModel> data = pagedResponse.getData();
        if (CollectionUtils.isEmpty(data)) {
            return null;
        }
        return data;
    }
}

