package com.persagy.dmp.alarm.service;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.web.MapResponse;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.alarm.entity.AlarmLog;
import com.persagy.dmp.alarm.entity.QAlarmLog;
import com.persagy.dmp.alarm.enumeration.EnumAlarmLogType;
import com.persagy.dmp.alarm.model.AlarmLogModel;
import com.persagy.dmp.alarm.repository.AlarmLogRepository;
import com.querydsl.core.types.dsl.BooleanExpression;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.transaction.Transactional;
import java.util.Date;
import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

@Service
public class AlarmLogService extends AlarmBaseService {

    @Autowired
    private AlarmLogRepository alarmLogRepository;

    @Autowired
    private CriteriaUtils criteriaUtils;

    private List<BooleanExpression> parse(ObjectNode criteria) {
        // TODO
        return null;
    }

    public PagedResponse<AlarmLogModel> query(JacksonCriteria criteria) {
        PagedResponse<AlarmLog> resp = criteriaUtils.query(QAlarmLog.alarmLog, this::parse, criteria);
        PagedResponse<AlarmLogModel> result = new PagedResponse<>();
        result.setCount(resp.getCount());
        List<AlarmLog> dataList = resp.getData();
        if (dataList != null && dataList.size() > 0) {
            List<AlarmLogModel> collect = dataList.stream().map(entity -> {
                AlarmLogModel model = entity.toModel();
                return model;
            }).collect(Collectors.toList());
            result.setData(collect);
        }
        return result;
    }

    /**
     * @description: 查询日志创建时间
     * @param: recordId 报警记录id
     * @param: type 日志类型
     * @param: after
     * @return: java.util.Date
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/12/8 4:40 下午
     * @version: V1.0
     */
    private Date getCreateTime(String recordId, EnumAlarmLogType type, int after) {
		JacksonCriteria criteria = JacksonCriteria.newInstance();
		criteria.add("recordId", recordId);
		criteria.add("type", type);
		criteria.add("after", after);
		PagedResponse<AlarmLog> resp = criteriaUtils.query(QAlarmLog.alarmLog, this::parse, criteria);
		List<AlarmLog> logs = resp.getData();
		if (CollectionUtils.isEmpty(logs)) {
			return null;
		} else {
			return logs.get(0).getCreateTime();
		}
	}

	/**
	 * @description: 获取忽略报警的时间
	 * @param: recordId
	 * @return: java.util.Date
	 * @exception:
	 * @author: lixing
	 * @company: Persagy Technology Co.,Ltd
	 * @since: 2020/12/8 4:40 下午
	 * @version: V1.0
	 */
    public Date getIgnoreAlarmTime(String recordId) {
		// 已忽略
    	int after = 1;
        return getCreateTime(recordId, EnumAlarmLogType.TREATMODE, after);
    }

    /**
     * @description: 获取报警记录处理完成时间
     * @param: recordId
     * @return: java.util.Date
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/12/8 4:41 下午
     * @version: V1.0
     */
    public Date getTreatEndTime(String recordId) {
    	// 处理完成
    	int after = 3;
		return getCreateTime(recordId, EnumAlarmLogType.TREATSTATE, after);
    }

    @Transactional
    public MapResponse create(AlarmLogModel param) {
        MapResponse response = new MapResponse();
        AlarmParam alarmParam = prepareParam(response);
        if (alarmParam == null) {
            return response;
        }
        AlarmLog entity = AlarmLog.fromModel(param);
        entity.setId(UUID.randomUUID().toString());
        entity.setCreateTime(new Date());
        entity.setProjectId(alarmParam.projectId);
        entity.setGroupCode(alarmParam.groupCode);
        entity.setCreateUser(alarmParam.userId);
        //		param.setUpdateTime(new Date());
        alarmLogRepository.save(entity);
        response.add("id", entity.getId());
        return response;
    }


    //	@Transactional
    //	public MapResponse update(AlarmLogModel param) {
    //		MapResponse response = new MapResponse();
    //		// TODO
    //		AlarmLog entity = AlarmLog.fromModel(param);
    //		alarmLogRepository.save(entity);
    //		response.add("id", entity.getId());
    //		return response;
    //	}
    //
    //	@Transactional
    //	public MapResponse delete(AlarmLogModel param) {
    //		MapResponse response = new MapResponse();
    //		// TODO
    //		alarmLogRepository.deleteById(param.getId());
    //		return response;
    //	}
}

