package com.persagy.dmp.alarm.service;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.google.common.collect.Sets;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.alarm.client.PhysicalWorldClient;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.validation.constraints.NotNull;
import java.util.*;

/**
 * @description: 对接物理世界接口
 * @author: lixing
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/10/20 5:04 下午
 * @version: V1.0
 **/
@Service
@Slf4j
public class PhysicalWorldService {
    @Autowired
    private PhysicalWorldClient physicalWorldClient;
//    private PhysicalWorldClient physicalWorldClient = null;

    public Set<String> queryObjectIdListByClassCode(
            @NotNull String projectId,
            @NotNull String groupCode,
            @NotNull String classCode) throws Exception {
        JacksonCriteria criteria = JacksonCriteria.newInstance();
        criteria.add("classCode", classCode);
        PagedResponse<ObjectNode> objectNodePagedResponse = physicalWorldClient.queryObj(projectId, groupCode, criteria);
        log.info("物理世界查询对象返回值：", objectNodePagedResponse == null ? null : objectNodePagedResponse.toString());
        List<ObjectNode> data = objectNodePagedResponse.getData();
        Set<String> objIdList = new HashSet<>();
        for (ObjectNode obj : data) {
            String id = obj.get("id").asText();
            objIdList.add(id);
        }
        return objIdList;
    }

    /**
     * @description: 查询设备类型和设备id列表的映射关系
     * @param: projectId
     * @param: classCodes
     * @return: java.util.Map
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/21 2:27 下午
     * @version: V1.0
     */
    public Map<String, Set<String>> queryCodeIdListMapByClassCodes(
            @NotNull String projectId,
            @NotNull String groupCode,
            @NotNull List<String> classCodes) throws Exception {
        JacksonCriteria criteria = JacksonCriteria.newInstance();
        criteria.add("classCode", classCodes);
        PagedResponse<ObjectNode> objectNodePagedResponse = physicalWorldClient.queryObj(projectId, groupCode, criteria);
        log.info("物理世界查询对象返回值：", objectNodePagedResponse == null ? null : objectNodePagedResponse.toString());
        List<ObjectNode> data = objectNodePagedResponse.getData();
        Map<String, Set<String>> resultMap = new HashMap<>();
        for (ObjectNode obj : data) {
            String id = obj.get("id").asText();
            String classCode = obj.get("classCode").asText();
            if (resultMap.get(classCode) == null) {
                resultMap.put(classCode, Sets.newHashSet(id));
            } else {
                resultMap.get(classCode).add(id);
            }
        }
        return resultMap;
    }
}
