package com.persagy.dmp.alarm.util;

import org.springframework.util.StringUtils;

import java.lang.reflect.Field;
import java.lang.reflect.Method;

/**
 * @description: 必填项校验
 * @author: lixing
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/11/25 9:46 上午
 * @version: V1.0
 **/
public class CheckRequiredParam {
    /**
     * @description: 反射校验必填项
     * @param: t
     * @param: params
     * @return: java.lang.String
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/11/25 9:49 上午
     * @version: V1.0
     */
    public static <T> String check(T t, String... params) {
        // message用来存放校验结果
        StringBuilder message = new StringBuilder();
        Class<?> clazz = t.getClass();
        // 校验参数
        for (String param : params) {
            if (StringUtils.isEmpty(param)) {
                message.append("校验必填项参数错误，参数不能为空");
                return message.toString();
            }
            try {
                // 属性, 这里需要注意getField方法无法获取私有属性，应使用getDeclaredField方法
                Field field = clazz.getDeclaredField(param);
                // 属性类型
                Class<?> fieldType = field.getType();

                // get方法
                Method method = clazz.getMethod("get" + firstUpperCase(param));

                // 执行get方法，验证是否为空
                try {
                    if (fieldType == String.class) {
                        String result = (String) method.invoke(t);
                        if (StringUtils.isEmpty(result)) {
                            message.append(param).append("必填!");
                        }
                    } else {
                        Object result = method.invoke(t);
                        if (result == null) {
                            message.append(param).append("必填!");
                        }
                    }

                } catch (Exception e) {
                    message.append(e.getMessage());
                    return message.toString();
                }

            } catch (NoSuchFieldException e) {
                message.append("校验必填项参数错误，").
                        append(clazz.getSimpleName()).
                        append("不包含属性：").
                        append(param);
                return message.toString();
            } catch (NoSuchMethodException e) {
                message.append("校验必填项参数错误，").
                        append(clazz.getSimpleName()).
                        append("不包含方法：get").
                        append(firstUpperCase(param)).
                        append("()");
                return message.toString();
            }
        }
        return message.toString();
    }

    /**
     * @description: 字符串首字母大写
     * @param: s
     * @return: java.lang.String
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/11/25 10:08 上午
     * @version: V1.0
     */
    private static String firstUpperCase(String s) {
        if (StringUtils.isEmpty(s)) {
            return null;
        }
        char[] chars = s.toCharArray();
        chars[0] -= 32;
        return String.valueOf(chars);
    }
}
