package com.persagy.common.criteria;

import org.apache.commons.collections4.MapUtils;
import org.hibernate.transform.Transformers;

import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.Parameter;
import javax.persistence.Query;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

public class JpaUtils {

	private EntityManager entityManager;

	public JpaUtils(EntityManager entityManager) {
		this.entityManager = entityManager;
	}

	private Set<String> getParameterNames(Query q) {
		Set<Parameter<?>> parameters = q.getParameters();
		if (parameters == null || parameters.size() == 0) {
			return Collections.EMPTY_SET;
		} else {
			return parameters.stream().map(Parameter::getName).collect(Collectors.toSet());
		}
	}

	private Query createQuery(String sql, Object... args) {
		Query q = entityManager.createNativeQuery(sql);
		if (args != null && args.length > 0) {
			for (int i = 0; i < args.length; i++) {
				q.setParameter(i + 1, args[i]);
			}
		}
		return q;
	}

	private Query createQuery(String sql, Map<String, Object> args) {
		Query q = entityManager.createNativeQuery(sql);
		if (args != null && !args.isEmpty()) {
			Set<String> parameterNames = getParameterNames(q);
			for (String key : parameterNames) {
				q.setParameter(key, args.get(key));
			}
		}
		return q;
	}


	public int update(String sql, Object... args) {
		return createQuery(sql, args).executeUpdate();
	}

	public int update(String sql, Map<String, Object> args) {
		return createQuery(sql, args).executeUpdate();
	}

	public List<Map<String, Object>> queryList(String sql, Object... args) {
		Query q = createQuery(sql, args);
		q.unwrap(org.hibernate.SQLQuery.class).setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP);
		return q.getResultList();
	}

	public List<Map<String, Object>> queryList(String sql, Map<String, Object> args) {
		Query q = createQuery(sql, args);
		q.unwrap(org.hibernate.SQLQuery.class).setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP);
		return q.getResultList();
	}

	public Map<String, Object> queryOne(String sql, Object... args) {
		try {
			Query q = createQuery(sql, args);
			q.unwrap(org.hibernate.SQLQuery.class).setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP);
			return (Map<String, Object>) q.getSingleResult();
		} catch (Exception e) {
			return null;
		}
	}

	public <T> List<T> querySingleColumn(String sql, Object... args) {
		Query q = createQuery(sql, args);
		return q.getResultList();
	}

	public <T> List<T> querySingleColumn(String sql, Map<String, Object> args) {
		Query q = createQuery(sql, args);
		return q.getResultList();
	}

	public Map<String, Object> queryOne(String sql, Map<String, Object> args) {
		try {
			Query q = createQuery(sql, args);
			q.unwrap(org.hibernate.SQLQuery.class).setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP);
			return (Map<String, Object>) q.getSingleResult();
		} catch (NoResultException e) {
			return null;
		}
	}

	public Long count(String sql, String countKey, Object... args) {
		Map<String, Object> stringObjectMap = queryOne(sql, args);
		return MapUtils.getLong(stringObjectMap, countKey);
	}

	public Long count(String sql, String countKey, Map<String, Object> args) {
		Map<String, Object> stringObjectMap = queryOne(sql, args);
		return MapUtils.getLong(stringObjectMap, countKey);
	}

}
