package com.persagy.dmp.common.http;

/**
 * @author: yaoll
 * @date: 2020-10-14
 * @verison: 1.0
 */

import com.persagy.common.json.JacksonMapper;
import lombok.Data;
import lombok.Getter;
import lombok.Setter;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpEntity;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpRequestBase;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;

@Getter
@Setter
@Slf4j
public class HttpUtils {

	public static class HttpHandler {
		/**
		 * @param requestId
		 * @param method
		 * @param duration  耗时, 毫秒数
		 * @param url
		 * @param body
		 * @param status
		 * @param response
		 */
		public void handle(String requestId, String method, long duration, String url, String body, int status, String response) {

		}

	}

	private static final Logger logger = LoggerFactory.getLogger(HttpUtils.class);

	public static final String UTF8 = "UTF-8";

	private static final AtomicInteger flag = new AtomicInteger(0);

	private static String getFlag() {
		int result = flag.addAndGet(1);
		if (result > 99999999) {
			flag.set(0);
		}
		return 100000000 + result + "";
	}

	@Data
	public static class HttpResult {
		private Integer status;
		private String content;

		public boolean success() {
			return this.status == 200;
		}
	}

	@Setter
	private HttpHandler httpHandler = new HttpHandler();

	/**
	 * 这个方法需要重构
	 *
	 * @param url
	 * @return
	 */
	@Deprecated
	public String getJson(String url) {
		HttpResult result = get(url, 0);
		return result.getContent();
	}

	public String get(String url, Map<String, Object> param) {
		return get(url, param, false);
	}

	public String get(String url, Map<String, Object> param, boolean urlEncode) {
		String path = parseUrl(url, param, urlEncode);
		HttpResult httpResult = get(path, 0);
		return httpResult.getContent();
	}

	public String postJson(String url, Map<String, Object> param, Object requestBody) {
		return postJson(url, param, requestBody, 0);
	}

	public String postJson(String url, Map<String, Object> param, Object requestBody, int timeout) {
		String server = parseUrl(url, param, false);
		String requestBodyStr;
		if (requestBody instanceof String) {
			requestBodyStr = (String) requestBody;
		} else {
			requestBodyStr = JacksonMapper.toSimpleJson(requestBody);
		}
		HttpResult result = post(server, requestBodyStr, timeout);
		return result.getContent();
	}

	public HttpResult post(String url, String requestBody, int timeout) {
		String flag = getFlag();
		long begin = System.currentTimeMillis();
		CloseableHttpClient client = HttpClients.createDefault();
		CloseableHttpResponse resp = null;
		HttpResult result = new HttpResult();
		long duration = 0;
		try {
			logger.info("{} httputils request: {}", flag, url);
			logger.info("{} httputils requestbody: {}", flag, requestBody);
			HttpPost post = new HttpPost(url);
			post.addHeader("contentType", "application/json;charset=UTF-8");
			post.addHeader("Accept", "application/json");
			if (requestBody != null) {
				StringEntity entity = new StringEntity(requestBody, UTF8);
				entity.setContentType("application/json");
				post.setEntity(entity);
			}
			setTimeOut(post, timeout);
			resp = client.execute(post);
			int status = resp.getStatusLine().getStatusCode();
			result.setStatus(status);
			HttpEntity r = resp.getEntity();
			String respContent = EntityUtils.toString(r, UTF8);
			result.setContent(respContent);
			logger.info("{} httputils resp : {}", flag, getLogMsg(respContent));
			duration = System.currentTimeMillis() - begin;
			httpHandler.handle(flag, "POST", duration, url, requestBody, status, respContent);
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("{} httputils exception : ", flag, e);
			duration = System.currentTimeMillis() - begin;
			httpHandler.handle(flag, "POST", duration, url, requestBody, -1, e.getMessage());
			throw new RuntimeException(e);
		} finally {
			close(client, resp);
			logger.info("{} httputils timeconsuming : {} {}", flag, duration, url);
		}
		return result;
	}

	public HttpResult get(String url, int timeout) {
		String flag = getFlag();
		long begin = System.currentTimeMillis();
		CloseableHttpClient client = HttpClients.createDefault();
		CloseableHttpResponse resp = null;
		HttpResult result = new HttpResult();
		long duration = 0;
		try {
			logger.info("{} httputils request: {}", flag, url);
			HttpGet get = new HttpGet(url);
			setTimeOut(get, timeout);

			resp = client.execute(get);

			int status = resp.getStatusLine().getStatusCode();
			result.setStatus(status);
			HttpEntity r = resp.getEntity();
			String respContent = EntityUtils.toString(r, UTF8);
			result.setContent(respContent);
			logger.info("{} httputils resp: {}", flag, getLogMsg(respContent));
			duration = System.currentTimeMillis() - begin;
			httpHandler.handle(flag, "GET", duration, url, null, status, respContent);
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("{} httputils exception: ", flag, e);
			duration = System.currentTimeMillis() - begin;
			httpHandler.handle(flag, "GET", duration, url, null, -1, e.getMessage());
			throw new RuntimeException(e);
		} finally {
			close(client, resp);
			logger.info("{} httputils timeconsuming: {} {}", flag, duration, url);
		}
		return result;
	}

//    /**
//     * 发送get请求
//     *
//     * @param url       地址
//     * @param param     参数
//     * @param urlEncode 是否对uri进行decode
//     * @param timeout   超时时间
//     * @return
//     */
//    public HttpResult sendGet(String url, Map<?, Object> param, boolean urlEncode, int timeout) {
//        String path = parseUrl(url, param, urlEncode);
//        return get(path, timeout);
//    }

	private String parseUri(Map<?, Object> param) {
		StringBuilder buff = new StringBuilder();
		if (param != null && param.size() > 0) {
			for (Map.Entry<?, Object> entry : param.entrySet()) {
				Object key = entry.getKey();
				String value = entry.getValue().toString();
				buff.append("&").append(key.toString()).append("=").append(value);
			}
			return buff.substring(1);
		} else {
			return "";
		}
	}

	public String parseUrl(String url, String uri, boolean needEncode) {
		if (uri == null) {
			return url;
		}
		String result;
		if (url.indexOf("?") > -1) {
			result = url + "&" + uri;
		} else {
			result = url + "?" + uri;
		}

		if (needEncode) {
			int idx = result.indexOf("?");
			if (idx == -1) {
				return result;
			}
			try {
				String queryStr = URLEncoder.encode(result.substring(idx), UTF8);
				return result.substring(0, idx) + queryStr;
			} catch (UnsupportedEncodingException e) {
				throw new RuntimeException(e);
			}
		} else {
			return result;
		}
	}

	public String parseUrl(String url, Map<?, Object> param, boolean needEncode) {
		return parseUrl(url, parseUri(param), needEncode);
	}

	private void setTimeOut(HttpRequestBase req, int timeout) {
		if (timeout > 0) {
			RequestConfig requestConfig = RequestConfig.custom().setConnectTimeout(3000).setSocketTimeout(timeout).build();
			req.setConfig(requestConfig);
		}
	}

	private String getLogMsg(String respContent) {
		if (respContent == null) {
			return "null";
		}
		if (respContent.length() > 200) {
			return respContent.substring(0, 200);
		}
		return respContent;
	}

	private void close(CloseableHttpClient client, CloseableHttpResponse response) {
		if (response != null) {
			try {
				response.close();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		if (client != null) {
			try {
				client.close();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

}
