package com.persagy.dmp.config;

import lombok.Getter;
import lombok.Setter;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * @author: yaoll
 * @date: 2020-09-22
 * @verison: 1.0
 */
public class DmpParameterStorage extends HandlerInterceptorAdapter {

	private static final ThreadLocal<ParameterData> storage = new ThreadLocal();

	@Override
	public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
		ParameterData data = new ParameterData();
		data.groupCode = request.getParameter("groupCode");
		data.schemaId = request.getParameter("schemaId");
		data.projectId = request.getParameter("projectId");
		data.appId = request.getParameter("appId");
		data.userId = request.getParameter("userId");

		storage.set(data);
		return true;
	}

	@Override
	public void postHandle(HttpServletRequest request, HttpServletResponse response, Object handler, ModelAndView modelAndView) throws Exception {
		storage.remove();
	}

	@Override
	public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object handler, Exception ex) throws Exception {
		storage.remove();
	}

	@Setter
	@Getter
	public static class ParameterData {
		private String groupCode;
		private String schemaId;
		private String projectId;
		private String appId;
		private String userId;
	}

	public static void set(ParameterData data) {
		storage.set(data);
	}

	public static void clear(){
		storage.remove();
	}

	public static String getGroupCode() {
		return storage.get().groupCode;
	}

	public static String getSchemaId() {
		return storage.get().schemaId;
	}

	public static String getProjectId() {
		return storage.get().projectId;
	}

	public static String getAppId() {
		return storage.get().appId;
	}

	public static String getUserId() {
		return storage.get().userId;
	}

}
