package com.persagy.dmp.dic.service;

import com.persagy.common.web.BaseResponse;
import com.persagy.dmp.config.DmpParameterStorage;
import com.persagy.dmp.dic.repository.*;
import lombok.Getter;
import lombok.Setter;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.UUID;

public class DicBaseService {

    @Autowired
    protected DtTypeRepository dtTypeRepository;

    @Autowired
    protected DtDataRepository dtDataRepository;

    @Autowired
    protected DtDataMapRepository dtDataMapRepository;

    @Autowired
    protected KlTypeRepository klTypeRepository;

    @Autowired
    protected KlDataRepository klDataRepository;

    public static final String USER_ID = "userId";
    public static final String GROUP_CODE = "groupCode";
    public static final String PROJECT_ID = "projectId";

    @Getter
    @Setter
    protected static class DicParam {
        String groupCode;
        String userId;
    }

    protected <T> DicParam prepareParam(BaseResponse<T> resp, String... requiredColumns) {
        DicParam dicParam = new DicParam();
        dicParam.groupCode = DmpParameterStorage.getGroupCode();
        dicParam.userId = DmpParameterStorage.getUserId();
        if(requiredColumns != null){
            for (int i = 0; i < requiredColumns.length; i++) {
                String column = requiredColumns[i];
                if (USER_ID.equals(column) && (dicParam.userId == null || dicParam.userId.isEmpty())) {
                    resp.setFail("请求地址中的 userId 必须有值");
                    return null;
                }
                if (GROUP_CODE.equals(column) && (dicParam.groupCode == null || dicParam.groupCode.isEmpty())){
                    resp.setFail("请求地址中的 groupCode 必须有值");
                    return null;
                }
            }
        }
        return dicParam;
    }

    public String getUUID(){
        return UUID.randomUUID().toString().replace("-", "");
    }

}
