package com.persagy.dmp.dic.service;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.dmp.dic.model.DtDataModel;
import com.querydsl.core.types.dsl.BooleanExpression;
import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.web.PagedResponse;
import com.persagy.common.web.MapResponse;
import javax.transaction.Transactional;

import org.springframework.data.domain.Example;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;
import com.persagy.dmp.dic.model.DtDataMapModel;
import org.springframework.beans.factory.annotation.Autowired;
import com.persagy.dmp.dic.entity.*;
import com.persagy.dmp.dic.repository.DtDataMapRepository;

@Service
public class DtDataMapService extends DicBaseService {

	@Autowired
	private DtDataMapRepository dtDataMapRepository;

	@Autowired
	private CriteriaUtils criteriaUtils;

	private List<BooleanExpression> parse(ObjectNode criteria) {
		// TODO
		return null;
	}

	public PagedResponse<DtDataMapModel> query(JacksonCriteria criteria) {
		MapResponse response = new MapResponse();
		PagedResponse<DtDataMapModel> result = new PagedResponse<>();

		DicParam dicParam = prepareParam(response, GROUP_CODE, USER_ID);
		if (!response.success()) {
			result.setFail(response.getMessage());
			return result;
		}
		criteria.add("groupCode", dicParam.groupCode);
		PagedResponse<DtDataMap> resp = criteriaUtils.query(QDtDataMap.dtDataMap, this::parse, criteria);

		result.setCount(resp.getCount());
		List<DtDataMap> dataList = resp.getData();
		if (dataList != null && dataList.size() > 0) {
			List<DtDataMapModel> collect = dataList.stream().map(entity -> {
				DtDataMapModel model = entity.toModel();
				return model;
			}).collect(Collectors.toList());
			result.setData(collect);
		}
		return result;
	}

	@Transactional
	public MapResponse create(DtDataMapModel param) {
		MapResponse response = new MapResponse();
		// TODO
		DicParam dicParam = prepareParam(response, GROUP_CODE, USER_ID);
		if (!response.success()) {
			return response;
		}
		if (dicParam == null) {
			response.setFail("url 参数不能为空！");
			return response;
		}
		if (param == null){
			response.setFail("无数据！");
			return response;
		}
		DtDataMap entity = DtDataMap.fromModel(param);
		entity.setId(getUUID());
		if (entity.getCode() == null){
			response.setFail("code 不能为空！");
			return response;
		}
		if (entity.getLabel() == null){
			response.setFail("label 不能为空！");
			return response;
		}
		if (entity.getTargetCode() == null){
			response.setFail("targetCode 不能为空！");
			return response;
		}
		if (entity.getDictDataId() == null){
			response.setFail("dictDataId 不能为空！");
			return response;
		}
		if (entity.getType() == null){
			response.setFail("type 不能为空！");
			return response;
		}
		entity.setStatus(1);
		entity.setCreateUser(dicParam.userId);
		entity.setCreateTime(new Date());
		dtDataMapRepository.save(entity);
		response.add("id", entity.getId());
		return response;
	}

	@Transactional
	public MapResponse update(DtDataMapModel param) {
		MapResponse response = new MapResponse();
		// TODO
		DicParam dicParam = prepareParam(response, GROUP_CODE, USER_ID);
		if (!response.success()) {
			return response;
		}
		if (dicParam == null) {
			response.setFail("url 参数不能为空！");
			return response;
		}
		if (param == null){
			response.setFail("无数据！");
			return response;
		}
		if (param.getId() == null){
			response.setFail("更新操作id不能为空！");
			return response;
		}
		Optional<DtDataMap> optional = dtDataMapRepository.findById(param.getId());
		if (!optional.isPresent()){
			response.setFail("更新的数据不存在！");
			return response;
		}
		DtDataMap one = optional.get();
		if (param.getDictDataId() != null){
			one.setDictDataId(param.getDictDataId());
		}
		if (param.getTargetCode() != null){
			one.setTargetCode(param.getTargetCode());
		}
		if (param.getLabel() != null){
			one.setLabel(param.getLabel());
		}
		if (param.getOrderNum() != null){
			one.setOrderNum(param.getOrderNum());
		}
		if (param.getStatus() != null){
			one.setStatus(param.getStatus());
		}
		one.setUpdateUser(dicParam.userId);
		one.setUpdateTime(new Date());

		dtDataMapRepository.save(one);
		response.add("id", one.getId());

		return response;
	}

	@Transactional
	public MapResponse delete(DtDataMapModel param) {
		MapResponse response = new MapResponse();
		// TODO
		DicParam dicParam = prepareParam(response, GROUP_CODE, USER_ID);
		if (!response.success()) {
			return response;
		}
		if (dicParam == null) {
			response.setFail("url 参数不能为空！");
			return response;
		}
		if (param == null){
			response.setFail("无数据！");
			return response;
		}
		if (param.getId() == null){
			response.setFail("删除操作id不能为空！");
			return response;
		}
		Optional<DtDataMap> optional = dtDataMapRepository.findById(param.getId());
		if (!optional.isPresent()){
			response.setFail("删除的数据不存在！");
			return response;
		}
		DtDataMap one = optional.get();

		one.setStatus(0);
		one.setUpdateTime(new Date());
		one.setUpdateUser(dicParam.userId);

		dtDataMapRepository.save(one);
		response.add("id", one.getId());
		return response;
	}
}

