package com.persagy.dmp.dic.service;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.google.common.collect.Lists;
import com.persagy.dmp.dic.model.DtTypeModel;
import com.persagy.dmp.dic.repository.DtDataMapRepository;
import com.querydsl.core.types.dsl.BooleanExpression;
import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.web.PagedResponse;
import com.persagy.common.web.MapResponse;
import javax.transaction.Transactional;

import com.querydsl.sql.types.Null;
import org.springframework.data.domain.Example;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;
import com.persagy.dmp.dic.model.DtDataModel;
import org.springframework.beans.factory.annotation.Autowired;
import com.persagy.dmp.dic.entity.*;
import com.persagy.dmp.dic.repository.DtDataRepository;

@Service
public class DtDataService extends DicBaseService {

	@Autowired
	private DtDataRepository dtDataRepository;

	@Autowired
	private DtDataMapRepository dtDataMapRepository;

	@Autowired
	private CriteriaUtils criteriaUtils;

	private List<BooleanExpression> parse(ObjectNode criteria) {
		// TODO
		return null;
	}

	public PagedResponse<DtDataModel> query(JacksonCriteria criteria) {
		MapResponse response = new MapResponse();
		PagedResponse<DtDataModel> result = new PagedResponse<>();

		DicParam dicParam = prepareParam(response, GROUP_CODE, USER_ID);
		if (!response.success()) {
			result.setFail(response.getMessage());
			return result;
		}
		criteria.add("groupCode", dicParam.groupCode);

		PagedResponse<DtData> resp = criteriaUtils.query(QDtData.dtData, this::parse, criteria);
		result.setCount(resp.getCount());
		List<DtData> dataList = resp.getData();
		if (dataList != null && dataList.size() > 0) {
			List<DtDataModel> collect = dataList.stream().map(entity -> {
				DtDataModel model = entity.toModel();
				return model;
			}).collect(Collectors.toList());
			result.setData(collect);
		}
		return result;
	}

	@Transactional
	public MapResponse create(DtDataModel param) {
		MapResponse response = new MapResponse();
		// TODO
		DicParam dicParam = prepareParam(response, GROUP_CODE, USER_ID);
		if (!response.success()) {
			return response;
		}
		if (dicParam == null) {
			response.setFail("url 参数不能为空！");
			return response;
		}
		if (param == null){
			response.setFail("无数据！");
			return response;
		}
		DtData entity = DtData.fromModel(param);
		entity.setId(getUUID());
		if (entity.getCode() == null){
			response.setFail("code 不能为空！");
			return response;
		}
		if (entity.getDictTypeId() == null){
			response.setFail("dictType 不能为空！");
			return response;
		}
		if (entity.getLabel() == null){
			response.setFail("label 不能为空！");
			return response;
		}
		if (entity.getType() == null){
			response.setFail("type 不能为空！");
			return response;
		}
		entity.setStatus(1);
		entity.setCreateUser(dicParam.userId);
		entity.setCreateTime(new Date());
		dtDataRepository.save(entity);
		response.add("id", entity.getId());

		return response;
	}

	@Transactional
	public MapResponse update(DtDataModel param) {
		MapResponse response = new MapResponse();
		// TODO
		DicParam dicParam = prepareParam(response, GROUP_CODE, USER_ID);
		if (!response.success()) {
			return response;
		}
		if (dicParam == null) {
			response.setFail("url 参数不能为空！");
			return response;
		}
		if (param == null){
			response.setFail("无数据！");
			return response;
		}
		if (param.getId() == null){
			response.setFail("更新操作id不能为空！");
			return response;
		}
		Optional<DtData> optional = dtDataRepository.findById(param.getId());
		if (!optional.isPresent()){
			response.setFail("");
			return response;
		}
		DtData one = optional.get();
		if (param.getCode() != null){
			one.setCode(param.getCode());
		}
		if (param.getLabel() != null){
			one.setLabel(param.getLabel());
		}
		if (param.getParentId() != null){
			one.setParentId(param.getParentId());
		}
		if (param.getOrderNum() != null){
			one.setOrderNum(param.getOrderNum());
		}
		if (param.getStatus() != null){
			one.setStatus(param.getStatus());
		}
		one.setUpdateUser(dicParam.userId);
		one.setUpdateTime(new Date());

		dtDataRepository.save(one);
		response.add("id", one.getId());
		return response;
	}

	@Transactional
	public MapResponse delete(DtDataModel param) {
		MapResponse response = new MapResponse();
		// TODO
		DicParam dicParam = prepareParam(response, GROUP_CODE, USER_ID);
		if (!response.success()) {
			return response;
		}
		if (dicParam == null) {
			response.setFail("url 参数不能为空！");
			return response;
		}
		if (param == null){
			response.setFail("无数据！");
			return response;
		}
		if (param.getId() == null){
			response.setFail("删除操作id不能为空！");
			return response;
		}
		Optional<DtData> optional = dtDataRepository.findById(param.getId());
		if (!optional.isPresent()){
			response.setFail("");
			return response;
		}
		DtData one = optional.get();
		one.setStatus(0);
		one.setUpdateTime(new Date());
		one.setUpdateUser(dicParam.userId);
		dtDataRepository.save(one);
		response.add("id", one.getId());
		return response;
	}

	/**
	 * 返回映射表替代字典数据的数据集
	 * @param dtTypeModel
	 * @return
	 */
	@Transactional
	public PagedResponse<DtDataModel> queryData(DtTypeModel dtTypeModel) {

		PagedResponse<DtDataModel> result = new PagedResponse<>();

		String type = dtTypeModel.getType();
		String groupCode = dtTypeModel.getGroupCode();
		Integer status = dtTypeModel.getStatus();

		QDtData dtData = QDtData.dtData;
		BooleanExpression dtDataEq = dtData.type.eq(type);
		if (status != null){
			dtDataEq = dtDataEq.and(dtData.status.eq(status));
		}

		Iterable<DtData> dtDataIterable = dtDataRepository.findAll(dtDataEq);
		List<DtData> dtDatas = Lists.newArrayList(dtDataIterable);

		QDtDataMap dtDataMap = QDtDataMap.dtDataMap;
		BooleanExpression dtDataMapEq = dtDataMap.type.eq(type).and(dtDataMap.groupCode.eq(groupCode));
		if (status != null){
			dtDataMapEq = dtDataMapEq.and(dtDataMap.status.eq(status));
		}
		Iterable<DtDataMap> dtDataMapIterable = dtDataMapRepository.findAll(dtDataMapEq);
		if (dtDataMapIterable != null) {
			List<DtDataMap> dtDataMaps = Lists.newArrayList(dtDataMapIterable);
			Map<String, DtDataMap> dtDataMapMap = new HashMap<>(dtDataMaps.size());
			for (DtDataMap dataMap : dtDataMaps) {
				dtDataMapMap.put(dataMap.getTargetCode(), dataMap);
			}
			for (DtData data : dtDatas) {
				String code = data.getCode();
				DtDataMap ddm = dtDataMapMap.get(code);
				if (ddm == null){
					continue;
				}
				data.setCode(ddm.getCode());
				data.setLabel(ddm.getLabel());
			}
		}

		result.setCount(Long.valueOf(dtDatas.size()));
		List<DtDataModel> collect = dtDatas.stream().map(DtData::toModel).collect(Collectors.toList());
		result.setData(collect);

		return result;
	}
}

