package com.persagy.dmp.dic.service;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.web.MapResponse;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.dic.entity.DtType;
import com.persagy.dmp.dic.entity.QDtType;
import com.persagy.dmp.dic.model.DtTypeModel;
import com.persagy.dmp.dic.repository.DtTypeRepository;
import com.querydsl.core.types.dsl.BooleanExpression;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class DtTypeService extends DicBaseService {

	@Autowired
	private DtTypeRepository dtTypeRepository;

	@Autowired
	private CriteriaUtils criteriaUtils;

	private List<BooleanExpression> parse(ObjectNode criteria) {
		// TODO
		return null;
	}

	public PagedResponse<DtTypeModel> query(JacksonCriteria criteria) {
		MapResponse response = new MapResponse();
		PagedResponse<DtTypeModel> result = new PagedResponse<>();
		DicParam dicParam = prepareParam(response, GROUP_CODE, USER_ID);
		if (!response.success()) {
			result.setFail(response.getMessage());
			return result;
		}
		/** 系统默认的字典类型的groupCode为0,查询时候传入0与groupCode*/
		criteria.add("groupCode").in(Arrays.asList("0", dicParam.groupCode));

		PagedResponse<DtType> resp = criteriaUtils.query(QDtType.dtType, this::parse, criteria);
		result.setCount(resp.getCount());
		List<DtType> dataList = resp.getData();
		if (dataList != null && dataList.size() > 0) {
			List<DtTypeModel> collect = dataList.stream().map(entity -> {
				DtTypeModel model = entity.toModel();
				return model;
			}).collect(Collectors.toList());
			result.setData(collect);
		}
		return result;
	}

	@Transactional
	public MapResponse create(DtTypeModel param) {
		MapResponse response = new MapResponse();
		// TODO
		DicParam dicParam = prepareParam(response, GROUP_CODE, USER_ID);
		if (!response.success()) {
			return response;
		}
		if (dicParam == null) {
			response.setFail("url 参数不能为空！");
			return response;
		}
		if (param == null){
			response.setFail("无数据！");
			return response;
		}

		DtType entity = DtType.fromModel(param);
		entity.setId(getUUID());
		if (entity.getType() == null){
			response.setFail("type 不能为空！");
			return response;
		}
		QDtType qDtType = QDtType.dtType;
		long count = dtTypeRepository.count(qDtType.type.eq(entity.getType())
				.and(qDtType.groupCode.in(Arrays.asList("0", dicParam.groupCode))));
		if (count > 0) {
			response.setFail("type 不能重复！");
			return response;
		}
		if (entity.getName() == null){
			response.setFail("name 不能为空！");
			return response;
		}
		entity.setStatus(1);
		entity.setGroupCode(dicParam.groupCode);
		entity.setCreateUser(dicParam.userId);
		entity.setCreateTime(new Date());
		dtTypeRepository.save(entity);
		response.add("id", entity.getId());
		return response;
	}

	@Transactional
	public MapResponse update(DtTypeModel param) {
		MapResponse response = new MapResponse();
		// TODO
		DicParam dicParam = prepareParam(response, GROUP_CODE, USER_ID);
		if (!response.success()) {
			return response;
		}
		if (dicParam == null) {
			response.setFail("url 参数不能为空！");
			return response;
		}
		if (param == null){
			response.setFail("无数据！");
			return response;
		}
		if (param.getId() == null){
			response.setFail("更新操作id不能为空！");
			return response;
		}
		Optional<DtType> optional = dtTypeRepository.findById(param.getId());
		if (!optional.isPresent()){
			response.setFail("数据不存在！");
			return response;
		}
		DtType one = optional.get();
		if (param.getType() != null){
			if (!one.getType().equals(param.getType())){
				QDtType qDtType = QDtType.dtType;
				long count = dtTypeRepository.count(qDtType.type.eq(param.getType())
						.and(qDtType.groupCode.in(Arrays.asList("0", dicParam.groupCode))));
				if (count > 0) {
					response.setFail("type 不能重复！");
					return response;
				}
			}
			one.setType(param.getType());
		}
		if (param.getName() != null){
			one.setName(param.getName());
		}
		if (param.getDescription() != null){
			one.setDescription(param.getDescription());
		}
		if (param.getStatus() != null){
			one.setStatus(one.getStatus());
		}
		if (param.getRemark() != null){
			one.setRemark(param.getRemark());
		}

		one.setUpdateUser(dicParam.userId);
		one.setUpdateTime(new Date());
		dtTypeRepository.save(one);

		response.add("id", one.getId());

		return response;
	}

	@Transactional
	public MapResponse delete(DtTypeModel param) {
		MapResponse response = new MapResponse();
		// TODO
		DicParam dicParam = prepareParam(response, GROUP_CODE, USER_ID);
		if (!response.success()) {
			return response;
		}
		if (dicParam == null) {
			response.setFail("url 参数不能为空！");
			return response;
		}
		if (param == null){
			response.setFail("无数据！");
			return response;
		}
		if (param.getId() == null){
			response.setFail("删除操作id不能为空！");
			return response;
		}
		QDtType qDtType = QDtType.dtType;
		Optional<DtType> optional = dtTypeRepository.findOne(qDtType.id.eq(param.getId())
				.and(qDtType.groupCode.in(Arrays.asList("0", dicParam.groupCode))));
		if (!optional.isPresent()){
			response.setFail("数据不存在！");
			return response;
		}
		DtType one = optional.get();

		one.setStatus(0);
		one.setUpdateTime(new Date());
		one.setUpdateUser(dicParam.userId);
		dtTypeRepository.save(one);

		response.add("id", one.getId());

		return response;
	}
}

