package com.persagy.dmp.org.config;

import com.persagy.common.web.BaseResponse;
import com.persagy.dmp.org.exception.OrgCURDRunTimeException;
import lombok.extern.slf4j.Slf4j;
import org.apache.shardingsphere.sharding.route.engine.validator.impl.ShardingUpdateStatementValidator;
import org.springframework.dao.DataAccessException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import java.sql.SQLException;

/**
 * @author: yaoll
 * @date: 2020-11-01
 * @verison: 1.0
 */
@Slf4j
@RestControllerAdvice
public class GlobalExceptionHandler {

	@ExceptionHandler(Exception.class)
	public BaseResponse<String> handleException(Exception ex) {
		BaseResponse<String> resp = new BaseResponse<>();
		resp.setFail("服务异常: " + ex.getMessage());
		log.error("服务异常", ex);
		ShardingUpdateStatementValidator va;
		return resp;
	}

	@ExceptionHandler(DataAccessException.class)
	public BaseResponse<String> handleDataAccessException(DataAccessException ex) {
		BaseResponse<String> resp = new BaseResponse<>();
		resp.setFail("数据访问异常");
		return resp;
	}

	@ExceptionHandler(SQLException.class)
	public BaseResponse<String> handleSQLException(SQLException ex) {
		BaseResponse<String> resp = new BaseResponse<>();
		resp.setFail("数据操作异常");
		return resp;
	}

	/**
	 * 用于拦截增删改中手动抛出的异常
	 * @param ex
	 * @return
	 */
	@ExceptionHandler(OrgCURDRunTimeException.class)
	public BaseResponse<String> handleSQLRunTimeException(OrgCURDRunTimeException ex) {
		return ex.getBaseResponse();
	}
}
