package com.persagy.dmp.org.jms;

import com.persagy.dmp.common.SpringUtils;
import com.persagy.dmp.config.DmpParameterStorage;
import com.persagy.dmp.rwd.model.DmpMessage;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.HashSet;
import java.util.Set;

/**
 * @author: yaoll
 * @date: 2020-10-22
 * @verison: 1.0
 */
@Slf4j
@Component
public class MessageProcesser {

	@Value("${persagy.dmp.exchange}")
	private String exchange;

	@Value("${persagy.dmp.org.routingKey}")
	private String orgRoutingKey;

	@Autowired
	private RabbitTemplate rabbitTemplate;

	public void convertAndSend(DmpMessage message) {
		rabbitTemplate.convertAndSend(exchange, orgRoutingKey, message);
	}

	private static final Set<String> beanNames = new HashSet<>();

	static {
//		beanNames.add(RabbitMQConstants.ALARM_TARGET_CREATE);
	}

	public void listen(DmpMessage message) {
		String type = message.getType();
		if (!beanNames.contains(type)) {
			return;
		}

		String executorName = type + "Executor";
		MessageExecutor executor = SpringUtils.getBean(executorName, MessageExecutor.class);
		if (executor == null) {
			log.error("bean for [" + executorName + "] not exists");
		}

		try {
			DmpParameterStorage.ParameterData data = new DmpParameterStorage.ParameterData();
			data.setGroupCode(message.getGroupCode());
			data.setProjectId(message.getProjectId());
			data.setAppId(message.getAppId());

			DmpParameterStorage.set(data);

			executor.execute(message);
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			DmpParameterStorage.clear();
		}
	}

}
