package com.persagy.dmp.org.service;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.web.MapResponse;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.org.common.OrgUtils;
import com.persagy.dmp.org.entity.Application;
import com.persagy.dmp.org.entity.QApplication;
import com.persagy.dmp.org.model.ApplicationModel;
import com.persagy.dmp.org.repository.ApplicationRepository;
import com.querydsl.core.types.dsl.BooleanExpression;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author king
 */
@Service
public class ApplicationService extends OrgBaseService {

	@Autowired
	private ApplicationRepository applicationRepository;

	@Autowired
	private CriteriaUtils criteriaUtils;

	private List<BooleanExpression> parse(ObjectNode criteria) {
		// TODO
		return null;
	}

	public PagedResponse<ApplicationModel> query(JacksonCriteria criteria) {
		PagedResponse<Application> resp = criteriaUtils.query(QApplication.application, this::parse, criteria);
		PagedResponse<ApplicationModel> result = new PagedResponse<>();
		result.setCount(resp.getCount());
		List<Application> dataList = resp.getData();
		if (dataList != null && dataList.size() > 0) {
			List<ApplicationModel> collect = dataList.stream().map(entity -> {
				ApplicationModel model = entity.toModel();
				return model;
			}).collect(Collectors.toList());
			result.setData(collect);
		}
		return result;
	}

	@Transactional
	public MapResponse create(List<ApplicationModel> params) {
		MapResponse response = new MapResponse();
		// TODO
		OrgParam orgParam = prepareParam(response);
		if (orgParam != null){
			if (params != null && !params.isEmpty()){
				List<Application> applications = new ArrayList<>(params.size());
				for (ApplicationModel param : params) {
					Application entity = Application.fromModel(param);

					if (entity.getId() == null){
						entity.setId(OrgUtils.getUUID());
					}
					entity.setStatus(1);

					entity.setCreateUser(orgParam.userId);
					entity.setCreateTime(new Date());

					applications.add(entity);
				}

				if (applications == null || applications.isEmpty()){
					response.setFail("无可创建的数据！");
					return response;
				}
				applicationRepository.saveAll(applications);
				List<String> ids = applications.stream().map(Application::getId).collect(Collectors.toList());
				response.add("ids", ids);
			}
		}

		return response;
	}

	@Transactional
	public MapResponse update(List<ApplicationModel> params) {
		MapResponse response = new MapResponse();
		// TODO
		OrgParam orgParam = prepareParam(response);
		if (orgParam != null) {
			if (params != null && !params.isEmpty()) {
				List<Application> applications = new ArrayList<>(params.size());
				for (ApplicationModel param : params) {
					if (param.getId() == null){
						response.setFail("更新操作id不能为空！");
						return response;
					}
					Application one = applicationRepository.getOne(param.getId());

					if (param.getCode() != null){
						one.setCode(param.getCode());
					}
					if (param.getName() != null){
						one.setName(param.getName());
					}
					if (param.getCategory() != null){
						one.setCategory(param.getCategory());
					}
					if (param.getAppOrder() != null){
						one.setAppOrder(param.getAppOrder());
					}
					if (param.getMenuIconKey() != null){
						one.setMenuIconKey(param.getMenuIconKey());
					}
					if (param.getIconKey() != null){
						one.setIconKey(param.getIconKey());
					}
					if (param.getDescription() != null){
						one.setDescription(param.getDescription());
					}
					if (param.getObjType() != null){
						one.setObjType(param.getObjType());
					}
					if (param.getRemark() != null){
						one.setRemark(param.getRemark());
					}
					if (param.getStatus() != null){
						one.setStatus(param.getStatus());
					}
					one.setUpdateUser(orgParam.userId);
					one.setUpdateTime(new Date());

					applications.add(one);
				}
				if (applications == null || applications.isEmpty()){
					response.setFail("无更新数据！");
					return response;
				}
				applicationRepository.saveAll(applications);
				List<String> ids = applications.stream().map(Application::getId).collect(Collectors.toList());
				response.add("ids", ids);
			}
		}
		return response;
	}

	@Transactional
	public MapResponse delete(List<ApplicationModel> params) {
		MapResponse response = new MapResponse();
		// TODO 逻辑删除
		OrgParam orgParam = prepareParam(response);
		if (orgParam != null){
			if (params != null && !params.isEmpty()) {
				List<Application> applications = new ArrayList<>(params.size());
				for (ApplicationModel param : params) {
					if (param.getId() == null){
						response.setFail("删除操作id不能为空！");
						return response;
					}
					Application one = applicationRepository.getOne(param.getId());
					one.setStatus(0);
					one.setUpdateUser(orgParam.userId);
					one.setUpdateTime(new Date());

					applications.add(one);
				}
				if (applications == null || applications.isEmpty()){
					response.setFail("无删除数据！");
					return response;
				}
				applicationRepository.saveAll(applications);
				List<String> ids = applications.stream().map(Application::getId).collect(Collectors.toList());
				response.add("ids", ids);
			}
		}
		return response;
	}
}

