package com.persagy.dmp.org.service;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.google.common.collect.Lists;
import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.web.BaseResponse;
import com.persagy.common.web.MapResponse;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.config.DmpParameterStorage;
import com.persagy.dmp.org.common.OrgUtils;
import com.persagy.dmp.org.constant.OrgWithColumnConstant;
import com.persagy.dmp.org.entity.*;
import com.persagy.dmp.org.exception.OrgCURDRunTimeException;
import com.persagy.dmp.org.model.CompanyModel;
import com.persagy.dmp.org.repository.CompanyProjectRelRepository;
import com.persagy.dmp.org.repository.CompanyRepository;
import com.persagy.dmp.org.repository.ProjectRepository;
import com.querydsl.core.types.dsl.BooleanExpression;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import javax.validation.Valid;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author king
 */
@Service
public class CompanyService extends OrgBaseService {

	@Autowired
	private CompanyRepository companyRepository;

	@Autowired
	private CriteriaUtils criteriaUtils;

	@Autowired
	private CompanyProjectRelRepository companyProjectRelRepository;

	@Autowired
	private ProjectRepository projectRepository;


	private List<BooleanExpression> parse(ObjectNode criteria) {
		// TODO
		return null;
	}

	public PagedResponse<CompanyModel> query(JacksonCriteria criteria) {
		MapResponse response = new MapResponse();
		PagedResponse<CompanyModel> result = new PagedResponse<>();
		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE);
		if (!response.success()) {
			result.setFail(response.getMessage());
			return result;
		}
		criteria.add("groupCode", orgParam.groupCode);

		PagedResponse<Company> resp = criteriaUtils.query(QCompany.company, this::parse, criteria);

		result.setCount(resp.getCount());
		List<Company> dataList = resp.getData();


		Set<String> withColumns = criteria.getWithColumns();
		Boolean returnProjects = withColumns != null && withColumns.contains(OrgWithColumnConstant.COMPANY_PROJECTS);

		if (dataList != null && dataList.size() > 0) {
			List<CompanyModel> collect = dataList.stream().map(entity -> {
				CompanyModel model = entity.toModel();

				// TODO 查询公司下的项目
				if (returnProjects) {
					String companyId = model.getId();
					QCompanyProjectRel qt = QCompanyProjectRel.companyProjectRel;
					Iterable<CompanyProjectRel> all = companyProjectRelRepository.findAll(qt.companyId.eq(companyId)
							.and(qt.groupCode.eq(orgParam.groupCode)));
					if (all != null) {
						List<CompanyProjectRel> companyProjectRels = Lists.newArrayList(all);
						List<String> projectIds = companyProjectRels.stream().map(
								CompanyProjectRel::getProjectId
						).collect(Collectors.toList());
						QProject qProject = QProject.project;
						List<Project> projectList = Lists.newArrayList(projectRepository.findAll(qProject.id.in(projectIds)
								.and(qProject.groupCode.eq(orgParam.groupCode))));
						model.setProjects(projectList);
					}
				}
				return model;
			}).collect(Collectors.toList());
			result.setData(collect);
		}
		return result;
	}

	@Transactional
	public MapResponse create(List<CompanyModel> params) {
		MapResponse response = new MapResponse();
		// TODO 加集团锁,每个集团只有一个人可以操作,暂缓
		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE);
		if (!response.success()) {
			return response;
		}
		if (orgParam != null) {
			if (params == null || params.isEmpty()){
				response.setFail("传入数据为空！");
				return response;
			}
			// TODO groupCode uerId 不可为空
			if (params != null && !params.isEmpty()) {
				List<Company> companies = new ArrayList<>(params.size());
				List<CompanyProjectRel> companyProjectRels = new ArrayList<>();
				for (CompanyModel param : params) {
					Company entity = Company.fromModel(param);
					if (entity.getCode() == null || entity.getCode().isEmpty()){
						response.setFail("公司 code 不可为空！");
						return response;
					}
					// TODO code 不可重复
					QCompany qCompany = QCompany.company;
					long codeFlag = companyRepository.count(qCompany.code.eq(entity.getCode())
							.and(qCompany.groupCode.eq(orgParam.groupCode)));
					if (codeFlag > 0) {
						response.setFail("公司 code 不能重复！");
						throw new OrgCURDRunTimeException(response);
					}
					if (entity.getName() == null || entity.getName().isEmpty()){
						response.setFail("公司 name 不可为空！");
						throw new OrgCURDRunTimeException(response);
					}
					// TODO name 不可重复
					long nameFlag = companyRepository.count(qCompany.name.eq(entity.getName())
							.and(qCompany.status.eq(1))
							.and(qCompany.groupCode.eq(orgParam.groupCode)));
					if (nameFlag > 0) {
						response.setFail("公司 name 不能重复！");
						throw new OrgCURDRunTimeException(response);
					}
					entity.setId(OrgUtils.getUUID());
					entity.setStatus(1);
					entity.setGroupCode(orgParam.groupCode);
					entity.setCreateUser(orgParam.userId);
					Date date = new Date();
					entity.setCreateTime(date);
					entity.setUpdateTime(date);
					entity.setUpdateUser(orgParam.userId);
					companies.add(entity);

					// TODO 新建公司，配置项目
					String companyId = entity.getId();
					List<Project> projects = param.getProjects();
					if (projects != null && !projects.isEmpty()) {
						for (Project project : projects) {
							String projectId = project.getId();
							CompanyProjectRel cpr = new CompanyProjectRel();
							cpr.setId(OrgUtils.getUUID());
							cpr.setCompanyId(companyId);
							cpr.setGroupCode(orgParam.groupCode);
							// TODO 校验projectId
							QProject qProject = QProject.project;
							long projectFlag = projectRepository.count(qProject.id.eq(projectId)
									.and(qProject.groupCode.eq(orgParam.groupCode)));
							if (projectFlag == 0) {
								response.setFail("项目 " + projectId + " 不存在！");
								throw new OrgCURDRunTimeException(response);
							}
							cpr.setProjectId(projectId);
							companyProjectRels.add(cpr);

						}
					}
				}
				if (companies == null || companies.isEmpty()){
					response.setFail("无新建数据！");
					throw new OrgCURDRunTimeException(response);
				}
				companyRepository.saveAll(companies);
				if (companyProjectRels != null && !companyProjectRels.isEmpty()){
					companyProjectRelRepository.saveAll(companyProjectRels);
				}
				List<String> returnValue = companies.stream().map(Company::getId).collect(Collectors.toList());
				response.add("ids", returnValue);
			}
		}
		return response;
	}

	@Transactional
	public MapResponse update(List<CompanyModel> params) {
		MapResponse response = new MapResponse();
		// TODO
		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE);
		if (!response.success()) {
			return response;
		}
		if (orgParam != null) {
			if (params == null || params.isEmpty()){
				response.setFail("传入数据为空！");
				throw new OrgCURDRunTimeException(response);
			}
			if (params != null && !params.isEmpty()) {
				List<Company> companies = new ArrayList<>(params.size());
				for (CompanyModel param : params) {
					if (param.getId() == null) {
						response.setFail("更新操作id不能为空！");
						throw new OrgCURDRunTimeException(response);
					}
					QCompany qCompany = QCompany.company;
					Optional<Company> optional = companyRepository.findOne(qCompany.id.eq(param.getId())
							.and(qCompany.groupCode.eq(orgParam.groupCode)));
					if (!optional.isPresent()) {
						// TODO 数据不存在,直接报错返回
						response.setFail("数据不存在！");
						throw new OrgCURDRunTimeException(response);
					}
					Company one = optional.get();

					if (param.getCode() != null) {
						if (param.getCode() != one.getCode()){
							// TODO code 不可重复
							long codeFlag = companyRepository.count(qCompany.code.eq(param.getCode())
									.and(qCompany.groupCode.eq(orgParam.groupCode)));
							if (codeFlag > 1) {
								response.setFail("公司 code 不能重复！");
								throw new OrgCURDRunTimeException(response);
							}
						}
						one.setCode(param.getCode());
					}
					if (param.getName() != null) {
						if (param.getName() != one.getName()){
							// TODO name 不可重复
							long nameFlag = companyRepository.count(qCompany.name.eq(param.getName())
									.and(qCompany.groupCode.eq(orgParam.groupCode)));
							if (nameFlag > 1) {
								response.setFail("公司 name 不能重复！");
								throw new OrgCURDRunTimeException(response);
							}
						}
						one.setName(param.getName());
					}
					if (param.getParentId() != null) {
						one.setParentId(param.getParentId());
					}
					if (param.getRemark() != null) {
						one.setRemark(param.getRemark());
					}
					if (param.getStatus() != null) {
						one.setStatus(one.getStatus());
					}
					one.setUpdateUser(orgParam.userId);
					one.setUpdateTime(new Date());
					companies.add(one);
				}

				List<CompanyProjectRel> deletes = new ArrayList<>();
				List<CompanyProjectRel> saves = new ArrayList<>();

				// TODO 更新公司，更新公司下的项目
				for (CompanyModel param : params) {
					String companyId = param.getId();
					List<Project> projects = param.getProjects();
					if (projects != null && !projects.isEmpty()) {
						QCompanyProjectRel qt = QCompanyProjectRel.companyProjectRel;
						Iterable<CompanyProjectRel> all = companyProjectRelRepository.findAll(qt.companyId.eq(companyId)
								.and(qt.groupCode.eq(orgParam.groupCode)));
						ArrayList<CompanyProjectRel> companyProjectRels = Lists.newArrayList(all);

						// TODO 删除原有项目
						deletes.addAll(companyProjectRels);

						// TODO 保存新项目
						List<CompanyProjectRel> cprs = new ArrayList<>(projects.size());

						for (Project project : projects) {
							String projectId = project.getId();
							CompanyProjectRel cpr = new CompanyProjectRel();
							cpr.setId(OrgUtils.getUUID());
							cpr.setCompanyId(companyId);
							// TODO 校验projectId
							QProject qProject = QProject.project;
							long projectFlag = projectRepository.count(qProject.id.eq(projectId)
									.and(qProject.groupCode.eq(orgParam.groupCode)));
							if (projectFlag == 0) {
								response.setFail("项目 " + projectId + " 不存在！");
								throw new OrgCURDRunTimeException(response);
							}
							cpr.setProjectId(projectId);
							cpr.setGroupCode(orgParam.groupCode);
							cprs.add(cpr);
						}
						saves.addAll(cprs);
					}
					if (companies == null || companies.isEmpty()){
						response.setFail("无可更新的数据！");
						throw new OrgCURDRunTimeException(response);
					}
					/** 保存公司 */
					companyRepository.saveAll(companies);
					/** 删除公司关联的项目关系 */
					if (deletes != null && !deletes.isEmpty()){
						companyProjectRelRepository.deleteAll(deletes);
					}
					/** 保存公司关联的项目关系 */
					if (saves != null && !saves.isEmpty()){
						companyProjectRelRepository.saveAll(saves);
					}
					List<String> returnValue = params.stream().map(CompanyModel::getId).collect(Collectors.toList());
					response.add("ids", returnValue);
				}
			}
		}
		return response;
	}

	/**
	 * TODO 确认删除公司后,部门\岗位\人员如何处理?
	 * @param params
	 * @return
	 */
	@Transactional
	public MapResponse delete(List<CompanyModel> params) {
		MapResponse response = new MapResponse();
		// TODO 逻辑删除
		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE);
		if (!response.success()) {
			return response;
		}
		if (orgParam != null) {
			if (params == null || params.isEmpty()){
				response.setFail("传入数据为空！");
				throw new OrgCURDRunTimeException(response);
			}
			if (params != null && !params.isEmpty()) {
				List<Company> companies = new ArrayList<>(params.size());
				List<CompanyProjectRel> deletes = new ArrayList<>();
				for (CompanyModel param : params) {
					if (param.getId() == null) {
						response.setFail("删除操作id不能为空！");
						throw new OrgCURDRunTimeException(response);
					}
					QCompany qCompany = QCompany.company;
					Optional<Company> optional = companyRepository.findOne(qCompany.id.eq(param.getId())
							.and(qCompany.groupCode.eq(orgParam.groupCode)));
					if (!optional.isPresent()) {
						// TODO 数据不存在,直接报错返回
						response.setFail("要删除的数据不存在！");
						throw new OrgCURDRunTimeException(response);
					}
					Company one = optional.get();
					one.setStatus(0);
					one.setUpdateTime(new Date());
					one.setUpdateUser(orgParam.userId);

					companies.add(one);

					QCompanyProjectRel qt = QCompanyProjectRel.companyProjectRel;
					Iterable<CompanyProjectRel> all = companyProjectRelRepository.findAll(qt.companyId.eq(one.getId())
							.and(qt.groupCode.eq(orgParam.groupCode)));
					ArrayList<CompanyProjectRel> companyProjectRels = Lists.newArrayList(all);
					deletes.addAll(companyProjectRels);
				}


				if (companies == null || companies.isEmpty()){
					response.setFail("无可删除的数据！");
					throw new OrgCURDRunTimeException(response);
				}
				companyRepository.saveAll(companies);
				if (deletes != null && !deletes.isEmpty()){
					companyProjectRelRepository.deleteAll(deletes);
				}
				List<String> returnValue = companies.stream().map(Company::getId).collect(Collectors.toList());
				response.add("ids", returnValue);
			}
		}
		return response;
	}
}