package com.persagy.dmp.org.service;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.web.MapResponse;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.org.common.OrgUtils;
import com.persagy.dmp.org.entity.DeptClassify;
import com.persagy.dmp.org.entity.QDeptClassify;
import com.persagy.dmp.org.exception.OrgCURDRunTimeException;
import com.persagy.dmp.org.model.DeptClassifyModel;
import com.persagy.dmp.org.repository.DeptClassifyRepository;
import com.querydsl.core.types.dsl.BooleanExpression;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @author king
 */
@Service
public class DeptClassifyService extends OrgBaseService {

	@Autowired
	private DeptClassifyRepository deptClassifyRepository;

	@Autowired
	private CriteriaUtils criteriaUtils;

	private List<BooleanExpression> parse(ObjectNode criteria) {
		// TODO
		return null;
	}

	public PagedResponse<DeptClassifyModel> query(JacksonCriteria criteria) {

		MapResponse response = new MapResponse();
		PagedResponse<DeptClassifyModel> result = new PagedResponse<>();
		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE);
		if (!response.success()) {
			result.setFail(response.getMessage());
			return result;
		}
		criteria.add("groupCode", orgParam.groupCode);

		PagedResponse<DeptClassify> resp = criteriaUtils.query(QDeptClassify.deptClassify, this::parse, criteria);
		result.setCount(resp.getCount());
		List<DeptClassify> dataList = resp.getData();
		if (dataList != null && dataList.size() > 0) {
			List<DeptClassifyModel> collect = dataList.stream().map(entity -> {
				DeptClassifyModel model = entity.toModel();
				return model;
			}).collect(Collectors.toList());
			result.setData(collect);
		}
		return result;
	}

	@Transactional
	public MapResponse create(List<DeptClassifyModel> params) {
		MapResponse response = new MapResponse();
		// TODO
		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE);
		if (!response.success()) {
			return response;
		}
		if (orgParam != null){
			if (params == null || params.isEmpty()){
				response.setFail("传入数据为空！");
				return response;
			}
			if (params != null && !params.isEmpty()) {

				List<DeptClassify> deptClassifies = new ArrayList<>(params.size());
				for (DeptClassifyModel param : params) {
					DeptClassify entity = DeptClassify.fromModel(param);
					if (entity.getId() == null){
						entity.setId(OrgUtils.getUUID());
					}

					if (entity.getCode() == null){
						response.setFail("部门分类 code 不能为空！");
						throw new OrgCURDRunTimeException(response);
					}
					if (entity.getName() == null){
						response.setFail("部门分类 name 不能为空！");
						throw new OrgCURDRunTimeException(response);
					}
					QDeptClassify qDeptClassify = QDeptClassify.deptClassify;
					// TODO code 不可重复
					long codeFlag = deptClassifyRepository.count(qDeptClassify.code.eq(entity.getCode())
							.and(qDeptClassify.groupCode.eq(orgParam.groupCode)));
					if (codeFlag > 0) {
						response.setFail("部门分类 code 不能重复！");
						throw new OrgCURDRunTimeException(response);
					}

					// TODO name 不可重复
					long nameFlag = deptClassifyRepository.count(qDeptClassify.name.eq(entity.getName())
							.and(qDeptClassify.groupCode.eq(orgParam.groupCode)));
					if (nameFlag > 0) {
						response.setFail("部门分类 name 不能重复！");
						throw new OrgCURDRunTimeException(response);
					}

					entity.setStatus(1);
					entity.setGroupCode(orgParam.groupCode);
					entity.setCreateUser(orgParam.userId);
					Date date = new Date();
					entity.setCreateTime(date);
					entity.setUpdateTime(date);
					entity.setUpdateUser(orgParam.userId);

					deptClassifies.add(entity);
				}
				if (deptClassifies == null || deptClassifies.isEmpty()){
					response.setFail("无新建数据！");
					throw new OrgCURDRunTimeException(response);
				}
				deptClassifyRepository.saveAll(deptClassifies);
				List<String> returnValue = deptClassifies.stream().map(DeptClassify::getId).collect(Collectors.toList());
				response.add("ids", returnValue);
			}
		}
		return response;
	}

	@Transactional
	public MapResponse update(List<DeptClassifyModel> params) {
		MapResponse response = new MapResponse();
		// TODO
		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE);
		if (!response.success()) {
			return response;
		}
		if (orgParam != null){
			if (params == null || params.isEmpty()){
				response.setFail("传入数据为空！");
				return response;
			}
			if (params != null && !params.isEmpty()) {
				List<DeptClassify> deptClassifies = new ArrayList<>(params.size());
				for (DeptClassifyModel param : params) {
					if (param.getId() == null) {
						response.setFail("更新操作id不能为空！");
						throw new OrgCURDRunTimeException(response);
					}
					QDeptClassify qDeptClassify = QDeptClassify.deptClassify;
					Optional<DeptClassify> optional = deptClassifyRepository.findOne(qDeptClassify.id.eq(param.getId())
							.and(qDeptClassify.groupCode.eq(orgParam.groupCode)));
					if (!optional.isPresent()) {
						// TODO 数据不存在,直接报错返回
						response.setFail("数据不存在！");
						throw new OrgCURDRunTimeException(response);
					}
					DeptClassify one = optional.get();
					if (param.getCode() != null){
						if (param.getCode() != one.getCode()){
							// TODO code 不可重复
							long codeFlag = deptClassifyRepository.count(qDeptClassify.code.eq(param.getCode())
									.and(qDeptClassify.groupCode.eq(orgParam.groupCode)));
							if (codeFlag > 1) {
								response.setFail("部门分类 code 不能重复！");
								throw new OrgCURDRunTimeException(response);
							}
						}
						one.setCode(param.getCode());
					}
					if(param.getName() != null){
						if (param.getName() != one.getName()){
							// TODO name 不可重复
							long nameFlag = deptClassifyRepository.count(qDeptClassify.name.eq(param.getName())
									.and(qDeptClassify.groupCode.eq(orgParam.groupCode)));
							if (nameFlag > 1) {
								response.setFail("部门分类 name 不能重复！");
								throw new OrgCURDRunTimeException(response);
							}
						}
						one.setName(param.getName());
					}
					if(param.getParentId() != null){
						one.setParentId(param.getParentId());
					}
					if (param.getStatus() != null){
						one.setStatus(param.getStatus());
					}
					if(param.getOrderNum() != null ){
						one.setOrderNum(param.getOrderNum());
					}
					if(param.getRemark() != null){
						one.setRemark(param.getRemark());
					}
					one.setUpdateUser(orgParam.userId);
					one.setUpdateTime(new Date());

					deptClassifies.add(one);
				}
				if (deptClassifies == null || deptClassifies.isEmpty()){
					response.setFail("无可更新的数据！");
					return response;
				}
				deptClassifyRepository.saveAll(deptClassifies);
				List<String> returnValue = deptClassifies.stream().map(DeptClassify::getId).collect(Collectors.toList());
				response.add("ids", returnValue);
			}
		}
		return response;
	}

	@Transactional
	public MapResponse delete(List<DeptClassifyModel> params) {
		MapResponse response = new MapResponse();
		// TODO 逻辑删除
		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE);
		if (!response.success()) {
			return response;
		}
		if (orgParam != null){
			if (params == null || params.isEmpty()){
				response.setFail("传入数据为空！");
				return response;
			}
			if (params != null && !params.isEmpty()) {
				List<DeptClassify> deptClassifies = new ArrayList<>(params.size());
				for (DeptClassifyModel param : params) {
					if (param.getId() == null) {
						response.setFail("删除操作id不能为空！");
						throw new OrgCURDRunTimeException(response);
					}
					QDeptClassify qDeptClassify = QDeptClassify.deptClassify;
					Optional<DeptClassify> optional = deptClassifyRepository.findOne(qDeptClassify.id.eq(param.getId())
							.and(qDeptClassify.groupCode.eq(orgParam.groupCode)));
					if (!optional.isPresent()) {
						// TODO 数据不存在,直接报错返回
						response.setFail("数据不存在！");
						throw new OrgCURDRunTimeException(response);
					}
					DeptClassify entity = optional.get();
					entity.setStatus(0);
					entity.setUpdateTime(new Date());
					entity.setUpdateUser(orgParam.userId);

					deptClassifies.add(entity);
				}
				if (deptClassifies == null || deptClassifies.isEmpty()){
					response.setFail("无可删除的数据！");
					throw new OrgCURDRunTimeException(response);
				}
				deptClassifyRepository.saveAll(deptClassifies);
				List<String> returnValue = deptClassifies.stream().map(DeptClassify::getId).collect(Collectors.toList());
				response.add("ids", returnValue);
			}
		}
		return response;
	}
}

