package com.persagy.dmp.org.service;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.web.BaseResponse;
import com.persagy.common.web.MapResponse;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.org.common.OrgUtils;
import com.persagy.dmp.org.entity.Dept;
import com.persagy.dmp.org.entity.QDept;
import com.persagy.dmp.org.exception.OrgCURDRunTimeException;
import com.persagy.dmp.org.model.DeptClassifyModel;
import com.persagy.dmp.org.model.DeptModel;
import com.persagy.dmp.org.repository.DeptRepository;
import com.querydsl.core.types.dsl.BooleanExpression;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @author king
 */
@Service
public class DeptService extends OrgBaseService {

	@Autowired
	private DeptRepository deptRepository;

	@Autowired
	private CriteriaUtils criteriaUtils;

	private List<BooleanExpression> parse(ObjectNode criteria) {
		// TODO
		return null;
	}

	public PagedResponse<DeptModel> query(JacksonCriteria criteria) {
		MapResponse response = new MapResponse();
		PagedResponse<DeptModel> result = new PagedResponse<>();
		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE);
		if (!response.success()) {
			result.setFail(response.getMessage());
			return result;
		}
		criteria.add("groupCode", orgParam.groupCode);

		PagedResponse<Dept> resp = criteriaUtils.query(QDept.dept, this::parse, criteria);
		result.setCount(resp.getCount());
		List<Dept> dataList = resp.getData();
		if (dataList != null && dataList.size() > 0) {
			List<DeptModel> collect = dataList.stream().map(entity -> {
				DeptModel model = entity.toModel();
				return model;
			}).collect(Collectors.toList());
			result.setData(collect);
		}
		return result;
	}

	@Transactional
	public MapResponse create(List<DeptModel> params) {
		MapResponse response = new MapResponse();
		// TODO groupCode userId不可为空
		OrgParam orgParam = prepareParam(response);
		if (!response.success()) {
			return response;
		}
		if (orgParam != null) {
			if (params == null || params.isEmpty()){
				response.setFail("传入数据为空！");
				return response;
			}
			if (params != null && !params.isEmpty()) {
				List<Dept> depts = new ArrayList<>(params.size());
				for (DeptModel param : params) {
					Dept entity = Dept.fromModel(param);
					if (entity.getId() == null) {
						entity.setId(OrgUtils.getUUID());
					}
					// TODO companyId 不可为空
					if (entity.getCompanyId() == null){
						response.setFail("companyId 不可为空！");
						throw new OrgCURDRunTimeException(response);
					}
					// TODO deptClassifyCode 不可为空
					if ((entity.getDeptClassifyCode() == null)){
						response.setFail("deptClassifyCode 不可为空！");
						throw new OrgCURDRunTimeException(response);
					}
					entity.setStatus(1);
					entity.setGroupCode(orgParam.groupCode);
					entity.setCreateUser(orgParam.userId);
					Date date = new Date();
					entity.setCreateTime(date);
					entity.setUpdateTime(date);
					entity.setUpdateUser(orgParam.userId);
					depts.add(entity);
				}
				if (depts == null || depts.isEmpty()){
					response.setFail("无新建数据！");
					return response;
				}
				deptRepository.saveAll(depts);
				List<String> returnValue = depts.stream().map(Dept::getId).collect(Collectors.toList());
				response.add("ids", returnValue);
			}
		}
		return response;
	}

	@Transactional
	public MapResponse update(List<DeptModel> params) {
		MapResponse response = new MapResponse();
		// TODO
		OrgParam orgParam = prepareParam(response);
		if (orgParam != null) {
			if (params != null && !params.isEmpty()) {
				List<Dept> depts = new ArrayList<>(params.size());
				for (DeptModel param : params) {
					if (param.getId() == null) {
						response.setFail("更新操作id不能为空！");
						throw new OrgCURDRunTimeException(response);
					}
					QDept qDept = QDept.dept;
					Optional<Dept> optional = deptRepository.findOne(qDept.id.eq(param.getId())
							.and(qDept.groupCode.eq(orgParam.groupCode)));
					if (!optional.isPresent()){
						response.setFail("数据不存在！");
						throw new OrgCURDRunTimeException(response);
					}
					Dept one = optional.get();

					if (param.getDeptClassifyCode() != null) {
						one.setDeptClassifyCode(param.getDeptClassifyCode());
					}
					if (param.getCode() != null) {
						one.setCode(param.getCode());
					}
					if (param.getName() != null) {
						one.setName(param.getName());
					}
					if (param.getParentId() != null) {
						one.setParentId(param.getParentId());
					}
					if (param.getResourceFrom() != null) {
						one.setResourceFrom(param.getResourceFrom());
					}
					if (param.getManager() != null) {
						one.setManager(param.getManager());
					}
					if (param.getRemark() != null) {
						one.setRemark(param.getRemark());
					}
					one.setUpdateUser(orgParam.userId);
					one.setUpdateTime(new Date());

					depts.add(one);
				}
				if (depts == null || depts.isEmpty()){
					response.setFail("无更新数据！");
					throw new OrgCURDRunTimeException(response);
				}
				deptRepository.saveAll(depts);
				List<String> returnValue = depts.stream().map(Dept::getId).collect(Collectors.toList());
				response.add("ids", returnValue);
			}
		}
		return response;
	}

	/**
	 * TODO 确认部门删除后人员岗位如何处理
	 *
	 * @param params
	 * @return
	 */
	@Transactional
	public MapResponse delete(List<DeptModel> params) {
		MapResponse response = new MapResponse();
		// TODO 逻辑删除
		OrgParam orgParam = prepareParam(response);
		if (orgParam != null) {
			if (params != null && !params.isEmpty()) {
				List<Dept> depts = new ArrayList<>(params.size());
				for (DeptModel param : params) {
					if (param.getId() == null) {
						response.setFail("删除操作id不能为空！");
						throw new OrgCURDRunTimeException(response);
					}
					QDept qDept = QDept.dept;
					Optional<Dept> optional = deptRepository.findOne(qDept.id.eq(param.getId())
							.and(qDept.groupCode.eq(orgParam.groupCode)));

					if (!optional.isPresent()){
						response.setFail("数据不存在！");
						throw new OrgCURDRunTimeException(response);
					}
					Dept entity = optional.get();
					entity.setStatus(0);
					entity.setUpdateTime(new Date());
					entity.setUpdateUser(orgParam.userId);

					depts.add(entity);
				}
				if (depts == null || depts.isEmpty()){
					response.setFail("无可删除的数据！");
					throw new OrgCURDRunTimeException(response);
				}
				deptRepository.saveAll(depts);
				List<String> returnValue = depts.stream().map(Dept::getId).collect(Collectors.toList());
				response.add("ids", returnValue);
			}
		}
		return response;
	}
}

