package com.persagy.dmp.org.service;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.web.BaseResponse;
import com.persagy.common.web.MapResponse;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.org.common.OrgUtils;
import com.persagy.dmp.org.entity.DeptClassify;
import com.persagy.dmp.org.entity.Function;
import com.persagy.dmp.org.entity.QFunction;
import com.persagy.dmp.org.model.FunctionModel;
import com.persagy.dmp.org.repository.FunctionRepository;
import com.querydsl.core.types.dsl.BooleanExpression;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @author king
 */
@Service
public class FunctionService extends OrgBaseService {

	@Autowired
	private FunctionRepository functionRepository;

	@Autowired
	private CriteriaUtils criteriaUtils;

	private List<BooleanExpression> parse(ObjectNode criteria) {
		// TODO
		return null;
	}

	public PagedResponse<FunctionModel> query(JacksonCriteria criteria) {
		PagedResponse<Function> resp = criteriaUtils.query(QFunction.function, this::parse, criteria);
		PagedResponse<FunctionModel> result = new PagedResponse<>();
		result.setCount(resp.getCount());
		List<Function> dataList = resp.getData();
		if (dataList != null && dataList.size() > 0) {
			List<FunctionModel> collect = dataList.stream().map(entity -> {
				FunctionModel model = entity.toModel();
				return model;
			}).collect(Collectors.toList());
			result.setData(collect);
		}
		return result;
	}

	@Transactional
	public MapResponse create(List<FunctionModel> params) {
		MapResponse response = new MapResponse();
		// TODO
		if (params != null && !params.isEmpty()) {
			if (params == null || params.isEmpty()){
				response.setFail("传入数据为空！");
				return response;
			}
			List<Function> functions = new ArrayList<>(params.size());
			for (FunctionModel param : params) {
				Function entity = Function.fromModel(param);
				if (entity.getId() == null){
					entity.setId(OrgUtils.getUUID());
				}
				QFunction qFunction = QFunction.function;
				// TODO code 不可重复
				long codeFlag = functionRepository.count(qFunction.code.eq(entity.getCode()));
				if (codeFlag > 0) {
					response.setFail("部门分类 code 不能重复！");
					return response;
				}
				// TODO name 不可重复
				long nameFlag = functionRepository.count(qFunction.name.eq(entity.getName()));
				if (nameFlag > 0) {
					response.setFail("部门分类 name 不能重复！");
					return response;
				}
				entity.setStatus(1);
				// TODO userId 怎么传入?
//				entity.setCreateUser(DmpParameterStorage.getUserId());
				entity.setCreateTime(new Date());

				functions.add(entity);
			}
			if (functions == null || functions.isEmpty()){
				response.setFail("无新建数据！");
				return response;
			}
			functionRepository.saveAll(functions);
			List<String> returnValue = functions.stream().map(Function::getId).collect(Collectors.toList());
			response.add("ids", returnValue);
		}

		return response;
	}

	@Transactional
	public MapResponse update(List<FunctionModel> params) {
		MapResponse response = new MapResponse();
		// TODO
		if (params == null || params.isEmpty()){
			response.setFail("传入数据为空！");
			return response;
		}
		if (params != null && !params.isEmpty()) {
			List<Function> functions = new ArrayList<>(params.size());
			for (FunctionModel param : params) {
				if (param.getId() == null) {
					response.setFail("更新操作id不能为空！");
					return response;
				}
				Optional<Function> optional = functionRepository.findById(param.getId());
				if (!optional.isPresent()) {
					// TODO 数据不存在,直接报错返回
					response.setFail("数据不存在！");
					return response;
				}
				Function one = optional.get();

				QFunction qFunction = QFunction.function;
				if (param.getCode() != null){
					if (param.getCode() != one.getCode()){
						// TODO code 不可重复
						long codeFlag = functionRepository.count(qFunction.code.eq(param.getCode()));
						if (codeFlag > 1) {
							response.setFail("功能 code 不能重复！");
							return response;
						}
					}
					one.setCode(param.getCode());
				}
				if (param.getName() != null){
					if (param.getName() != one.getName()){
						// TODO name 不可重复
						long nameFlag = functionRepository.count(qFunction.name.eq(param.getName()));
						if (nameFlag > 1) {
							response.setFail("功能 name 不能重复！");
							return response;
						}
					}
					one.setName(param.getName());
				}
				if (param.getType() != null){
					one.setType(param.getType());
				}
				if (param.getParentId() != null){
					one.setParentId(param.getParentId());
				}
				if (param.getAppId() != null){
					one.setAppId(param.getAppId());
				}
				if (param.getFunctionUrl() != null){
					one.setFunctionUrl(param.getFunctionUrl());
				}
				if (param.getDescription() != null){
					one.setDescription(param.getDescription());
				}
				if (param.getIcon() != null){
					one.setIcon(param.getIcon());
				}
				if (param.getFunctionAlias() != null){
					one.setFunctionAlias(param.getFunctionAlias());
				}
				if (param.getFuncOrder() != null){
					one.setFuncOrder(param.getFuncOrder());
				}
				if (param.getObjType() != null){
					one.setObjType(param.getObjType());
				}
				if (param.getStatus() != null){
					one.setStatus(param.getStatus());
				}
				if (param.getRemark() != null){
					one.setRemark(param.getRemark());
				}

				// TODO userId 怎么传入?
//					one.setUpdateUser(orgParam.userId);
				one.setUpdateTime(new Date());

				functions.add(one);
			}
			if (functions == null || functions.isEmpty()){
				response.setFail("无更新数据！");
				return response;
			}
			functionRepository.saveAll(functions);
			List<String> returnValue = functions.stream().map(Function::getId).collect(Collectors.toList());
			response.add("ids", returnValue);
		}

		return response;
	}

	@Transactional
	public MapResponse delete(List<FunctionModel> params) {
		MapResponse response = new MapResponse();
		if (params == null || params.isEmpty()){
			response.setFail("传入数据为空！");
			return response;
		}
		// TODO 逻辑删除
		if (params != null && !params.isEmpty()) {
			List<Function> functions = new ArrayList<>(params.size());
			for (FunctionModel param : params) {
				if (param.getId() == null) {
					response.setResult(BaseResponse.FAIL);
					response.setMessage("删除操作id不能为空！");
					return response;
				}
				Optional<Function> optional = functionRepository.findById(param.getId());
				if (!optional.isPresent()) {
					// TODO 数据不存在,直接报错返回
					response.setResult(BaseResponse.FAIL);
					response.setMessage("数据不存在！");
					return response;
				}
				Function entity = optional.get();
				entity.setStatus(0);
				entity.setUpdateTime(new Date());
				// TODO userId 怎么传入?
//					entity.setUpdateUser(orgParam.userId);

				functions.add(entity);
			}
			if (functions == null || functions.isEmpty()){
				response.setFail("无删除数据！");
				return response;
			}
			functionRepository.saveAll(functions);
			List<String> returnValue = functions.stream().map(Function::getId).collect(Collectors.toList());
			response.add("ids", returnValue);
		}
		return response;
	}
}

