package com.persagy.dmp.org.service;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.dmp.org.common.OrgUtils;
import com.persagy.dmp.org.exception.OrgCURDRunTimeException;
import com.persagy.dmp.org.model.DeptModel;
import com.querydsl.core.types.dsl.BooleanExpression;
import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.web.PagedResponse;
import com.persagy.common.web.MapResponse;
import javax.transaction.Transactional;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;
import com.persagy.dmp.org.model.GroupAppRelModel;
import org.springframework.beans.factory.annotation.Autowired;
import com.persagy.dmp.org.entity.*;
import com.persagy.dmp.org.repository.GroupAppRelRepository;

@Service
public class GroupAppRelService extends OrgBaseService{

	@Autowired
	private GroupAppRelRepository groupAppRelRepository;

	@Autowired
	private CriteriaUtils criteriaUtils;

	private List<BooleanExpression> parse(ObjectNode criteria) {
		// TODO
		return null;
	}

	public PagedResponse<GroupAppRelModel> query(JacksonCriteria criteria) {
		MapResponse response = new MapResponse();
		PagedResponse<GroupAppRelModel> result = new PagedResponse<>();
		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE);
		if (!response.success()) {
			result.setFail(response.getMessage());
			return result;
		}
		criteria.add("groupCode", orgParam.groupCode);
		PagedResponse<GroupAppRel> resp = criteriaUtils.query(QGroupAppRel.groupAppRel, this::parse, criteria);
		result.setCount(resp.getCount());
		List<GroupAppRel> dataList = resp.getData();
		if (dataList != null && dataList.size() > 0) {
			List<GroupAppRelModel> collect = dataList.stream().map(entity -> {
				GroupAppRelModel model = entity.toModel();
				return model;
			}).collect(Collectors.toList());
			result.setData(collect);
		}
		return result;
	}

	@Transactional
	public MapResponse create(List<GroupAppRelModel> params) {
		MapResponse response = new MapResponse();
		// TODO
		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE);
		if (!response.success()) {
			return response;
		}
		if (orgParam != null) {
			if (params == null || params.isEmpty()){
				response.setFail("数据不存在！");
				return response;
			}
			if (params != null && !params.isEmpty()){
				List<GroupAppRel> groupAppRels = new ArrayList<>();
				for (GroupAppRelModel param : params) {
					GroupAppRel entity = GroupAppRel.fromModel(param);
					if (entity.getId() == null){
						entity.setId(OrgUtils.getUUID());
					}
					if (entity.getAppId() == null){
						response.setFail("appId不能为空！");
						throw new OrgCURDRunTimeException(response);
					}
					entity.setStatus(1);
					entity.setGroupCode(orgParam.groupCode);
					entity.setCreateUser(orgParam.userId);
					Date date = new Date();
					entity.setCreateTime(date);
					entity.setUpdateTime(date);
					entity.setUpdateUser(orgParam.userId);
					groupAppRels.add(entity);
				}
				if (groupAppRels == null || groupAppRels.isEmpty()){
					response.setFail("无新建数据！");
					throw new OrgCURDRunTimeException(response);
				}
				groupAppRelRepository.saveAll(groupAppRels);
				List<String> returnValue = groupAppRels.stream().map(GroupAppRel::getId).collect(Collectors.toList());
				response.add("ids", returnValue);
			}
		}
		return response;
	}

	@Transactional
	public MapResponse update(List<GroupAppRelModel> params) {
		MapResponse response = new MapResponse();
		// TODO
		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE);
		if (!response.success()) {
			return response;
		}
		if (orgParam != null) {
			if (params == null || params.isEmpty()) {
				response.setFail("数据不存在！");
				return response;
			}
			if (params != null && !params.isEmpty()) {
				List<GroupAppRel> groupAppRels = new ArrayList<>();
				for (GroupAppRelModel param : params) {
					if (param.getId() == null) {
						response.setFail("更新操作id不能为空！");
						throw new OrgCURDRunTimeException(response);
					}
					QGroupAppRel qGroupAppRel = QGroupAppRel.groupAppRel;
					Optional<GroupAppRel> optional = groupAppRelRepository.findOne(qGroupAppRel.id.eq(param.getId())
							.and(qGroupAppRel.groupCode.eq(orgParam.groupCode)));
					if (!optional.isPresent()) {
						// TODO 数据不存在,直接报错返回
						response.setFail("数据不存在！");
						throw new OrgCURDRunTimeException(response);
					}
					GroupAppRel one = optional.get();
					if (param.getAppId() != null){
						one.setAppId(param.getAppId());
					}
					if (param.getAuthorityType() != null){
						one.setAuthorityType(param.getAuthorityType());
					}
					if (param.getRemark() != null) {
						one.setRemark(param.getRemark());
					}
					if (param.getStatus() != null) {
						one.setStatus(one.getStatus());
					}
					one.setUpdateUser(orgParam.userId);
					one.setUpdateTime(new Date());

					groupAppRels.add(one);
				}
				if (groupAppRels == null || groupAppRels.isEmpty()){
					response.setFail("无新建数据！");
					throw new OrgCURDRunTimeException(response);
				}
				groupAppRelRepository.saveAll(groupAppRels);
				List<String> returnValue = groupAppRels.stream().map(GroupAppRel::getId).collect(Collectors.toList());
				response.add("ids", returnValue);
			}
		}
		return response;
	}

	@Transactional
	public MapResponse delete(List<GroupAppRelModel> params) {
		MapResponse response = new MapResponse();
		// TODO
		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE);
		if (!response.success()) {
			return response;
		}
		if (orgParam != null) {
			if (params == null || params.isEmpty()) {
				response.setFail("数据不存在！");
				return response;
			}
			if (params != null && !params.isEmpty()) {
				List<GroupAppRel> groupAppRels = new ArrayList<>();
				for (GroupAppRelModel param : params) {
					if (param.getId() == null) {
						response.setFail("删除操作id不能为空！");
						throw new OrgCURDRunTimeException(response);
					}
					QGroupAppRel qGroupAppRel = QGroupAppRel.groupAppRel;
					Optional<GroupAppRel> optional = groupAppRelRepository.findOne(qGroupAppRel.id.eq(param.getId())
							.and(qGroupAppRel.groupCode.eq(orgParam.groupCode)));
					if (!optional.isPresent()) {
						// TODO 数据不存在,直接报错返回
						response.setFail("数据不存在！");
						throw new OrgCURDRunTimeException(response);
					}
					GroupAppRel one = optional.get();

					one.setStatus(0);
					one.setUpdateUser(orgParam.userId);
					one.setUpdateTime(new Date());

					groupAppRels.add(one);
				}
				if (groupAppRels == null || groupAppRels.isEmpty()){
					response.setFail("无删除数据！");
					throw new OrgCURDRunTimeException(response);
				}
				groupAppRelRepository.saveAll(groupAppRels);
				List<String> returnValue = groupAppRels.stream().map(GroupAppRel::getId).collect(Collectors.toList());
				response.add("ids", returnValue);
			}
		}
		return response;
	}
}

