package com.persagy.dmp.org.service;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.dmp.org.common.OrgUtils;
import com.persagy.dmp.org.exception.OrgCURDRunTimeException;
import com.persagy.dmp.org.model.GroupAppRelModel;
import com.querydsl.core.types.dsl.BooleanExpression;
import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.web.PagedResponse;
import com.persagy.common.web.MapResponse;
import javax.transaction.Transactional;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;
import com.persagy.dmp.org.model.GroupFunctionRelModel;
import org.springframework.beans.factory.annotation.Autowired;
import com.persagy.dmp.org.entity.*;
import com.persagy.dmp.org.repository.GroupFunctionRelRepository;

@Service
public class GroupFunctionRelService  extends OrgBaseService{

	@Autowired
	private GroupFunctionRelRepository groupFunctionRelRepository;

	@Autowired
	private CriteriaUtils criteriaUtils;

	private List<BooleanExpression> parse(ObjectNode criteria) {
		// TODO
		return null;
	}

	public PagedResponse<GroupFunctionRelModel> query(JacksonCriteria criteria) {
		MapResponse response = new MapResponse();
		PagedResponse<GroupFunctionRelModel> result = new PagedResponse<>();
		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE);
		if (!response.success()) {
			result.setFail(response.getMessage());
			return result;
		}
		criteria.add("groupCode", orgParam.groupCode);

		PagedResponse<GroupFunctionRel> resp = criteriaUtils.query(QGroupFunctionRel.groupFunctionRel, this::parse, criteria);
		result.setCount(resp.getCount());
		List<GroupFunctionRel> dataList = resp.getData();
		if (dataList != null && dataList.size() > 0) {
			List<GroupFunctionRelModel> collect = dataList.stream().map(entity -> {
				GroupFunctionRelModel model = entity.toModel();
				return model;
			}).collect(Collectors.toList());
			result.setData(collect);
		}
		return result;
	}

	@Transactional
	public MapResponse create(List<GroupFunctionRelModel> params) {
		MapResponse response = new MapResponse();
		// TODO
		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE);
		if (!response.success()) {
			return response;
		}
		if (orgParam != null) {
			if (params == null || params.isEmpty()){
				response.setFail("数据不存在！");
				return response;
			}
			if (params != null && !params.isEmpty()){
				List<GroupFunctionRel> groupFunctionRels = new ArrayList<>();
				for (GroupFunctionRelModel param : params) {
					GroupFunctionRel entity = GroupFunctionRel.fromModel(param);
					if (entity.getId() == null){
						entity.setId(OrgUtils.getUUID());
					}
					if (entity.getFunctionId() == null){
						response.setFail("functionId 不能为空！");
						throw new OrgCURDRunTimeException(response);
					}
					entity.setStatus(1);
					entity.setGroupCode(orgParam.groupCode);
					entity.setCreateUser(orgParam.userId);
					Date date = new Date();
					entity.setCreateTime(date);
					entity.setUpdateTime(date);
					entity.setUpdateUser(orgParam.userId);

					groupFunctionRels.add(entity);
				}
				if (groupFunctionRels == null || groupFunctionRels.isEmpty()){
					response.setFail("无新建数据！");
					throw new OrgCURDRunTimeException(response);
				}
				groupFunctionRelRepository.saveAll(groupFunctionRels);
				List<String> returnValue = groupFunctionRels.stream().map(GroupFunctionRel::getId).collect(Collectors.toList());
				response.add("ids", returnValue);
			}
		}
		return response;
	}

	@Transactional
	public MapResponse update(List<GroupFunctionRelModel> params) {
		MapResponse response = new MapResponse();
		// TODO
		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE);
		if (!response.success()) {
			return response;
		}
		if (orgParam != null) {
			if (params == null || params.isEmpty()) {
				response.setFail("数据不存在！");
				return response;
			}
			if (params != null && !params.isEmpty()) {
				List<GroupFunctionRel> groupFunctionRels = new ArrayList<>();
				for (GroupFunctionRelModel param : params) {
					if (param.getId() == null) {
						response.setFail("更新操作id不能为空！");
						throw new OrgCURDRunTimeException(response);
					}
					QGroupFunctionRel qGroupFunctionRel = QGroupFunctionRel.groupFunctionRel;
					Optional<GroupFunctionRel> optional = groupFunctionRelRepository.findOne(qGroupFunctionRel.id.eq(param.getId())
							.and(qGroupFunctionRel.groupCode.eq(orgParam.groupCode)));
					if (!optional.isPresent()) {
						// TODO 数据不存在,直接报错返回
						response.setFail("数据不存在！");
						throw new OrgCURDRunTimeException(response);
					}
					GroupFunctionRel one = optional.get();
					if (param.getFunctionId() != null){
						one.setFunctionId(param.getFunctionId());
					}
					if (param.getAuthorityType() != null){
						one.setAuthorityType(param.getAuthorityType());
					}
					if (param.getRemark() != null) {
						one.setRemark(param.getRemark());
					}
					if (param.getStatus() != null) {
						one.setStatus(one.getStatus());
					}
					one.setUpdateUser(orgParam.userId);
					one.setUpdateTime(new Date());

					groupFunctionRels.add(one);
				}
				if (groupFunctionRels == null || groupFunctionRels.isEmpty()){
					response.setFail("无更新数据！");
					throw new OrgCURDRunTimeException(response);
				}
				groupFunctionRelRepository.saveAll(groupFunctionRels);
				List<String> returnValue = groupFunctionRels.stream().map(GroupFunctionRel::getId).collect(Collectors.toList());
				response.add("ids", returnValue);
			}
		}
		return response;
	}

	@Transactional
	public MapResponse delete(List<GroupFunctionRelModel> params) {
		MapResponse response = new MapResponse();
		// TODO
		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE);
		if (!response.success()) {
			return response;
		}
		if (orgParam != null) {
			if (params == null || params.isEmpty()) {
				response.setFail("数据不存在！");
				return response;
			}
			if (params != null && !params.isEmpty()) {
				List<GroupFunctionRel> groupFunctionRels = new ArrayList<>();
				for (GroupFunctionRelModel param : params) {
					if (param.getId() == null) {
						response.setFail("更新操作id不能为空！");
						return response;
					}
					QGroupFunctionRel qGroupFunctionRel = QGroupFunctionRel.groupFunctionRel;
					Optional<GroupFunctionRel> optional = groupFunctionRelRepository.findOne(qGroupFunctionRel.id.eq(param.getId())
							.and(qGroupFunctionRel.groupCode.eq(orgParam.groupCode)));
					if (!optional.isPresent()) {
						// TODO 数据不存在,直接报错返回
						response.setFail("数据不存在！");
						return response;
					}
					GroupFunctionRel one = optional.get();
					one.setStatus(0);
					one.setUpdateUser(orgParam.userId);
					one.setUpdateTime(new Date());

					groupFunctionRels.add(one);
				}
				if (groupFunctionRels == null || groupFunctionRels.isEmpty()){
					response.setFail("无更新数据！");
					return response;
				}
				groupFunctionRelRepository.saveAll(groupFunctionRels);
				List<String> returnValue = groupFunctionRels.stream().map(GroupFunctionRel::getId).collect(Collectors.toList());
				response.add("ids", returnValue);
			}
		}
		return response;
	}
}

