package com.persagy.dmp.org.service;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.dmp.config.DmpParameterStorage;
import com.persagy.dmp.org.common.OrgUtils;
import com.persagy.dmp.org.constant.OrgConstants;
import com.persagy.dmp.org.repository.CompanyRepository;
import com.persagy.dmp.rwd.model.DmpMessage;
import com.querydsl.core.types.dsl.BooleanExpression;
import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.web.PagedResponse;
import com.persagy.common.web.MapResponse;
import javax.transaction.Transactional;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;
import com.persagy.dmp.org.model.GroupModel;
import org.springframework.beans.factory.annotation.Autowired;
import com.persagy.dmp.org.entity.*;
import com.persagy.dmp.org.repository.GroupRepository;

/**
 * @author king
 */
@Service
public class GroupService extends OrgBaseService {

	@Autowired
	private GroupRepository groupRepository;

	@Autowired
	private CriteriaUtils criteriaUtils;


	@Autowired
	private InitService initService;

	@Autowired
	private CompanyRepository companyRepository;

	private List<BooleanExpression> parse(ObjectNode criteria) {
		// TODO
		return null;
	}

	public PagedResponse<GroupModel> query(JacksonCriteria criteria) {
		PagedResponse<Group> resp = criteriaUtils.query(QGroup.group, this::parse, criteria);
		PagedResponse<GroupModel> result = new PagedResponse<>();
		result.setCount(resp.getCount());
		List<Group> dataList = resp.getData();
		if (dataList != null && dataList.size() > 0) {
			List<GroupModel> collect = dataList.stream().map(entity -> {
				GroupModel model = entity.toModel();
				return model;
			}).collect(Collectors.toList());
			result.setData(collect);
		}
		return result;
	}

	@Transactional
	public MapResponse create(GroupModel param) {
		MapResponse response = new MapResponse();
		// TODO
		OrgParam orgParam = prepareParam(response, USER_ID);
		if (!response.success()) {
			return response;
		}
		Group entity = Group.fromModel(param);
		if (entity.getCode() == null){
			response.setFail("code 不能为空!");
			return response;
		}
		entity.setId(getUUID());
		entity.setStatus(1);
		entity.setCreateUser(orgParam.userId);
		entity.setCreateTime(new Date());
		groupRepository.save(entity);

		initService.initGroup();

		// TODO 创建集团时自动创建顶级公司
		Company company = new Company();

		company.setId(OrgUtils.getUUID());
		company.setGroupCode(entity.getCode());
		company.setCode(entity.getCode());
		company.setName(entity.getName());
		company.setParentId("0");
		company.setStatus(1);
		company.setCreateUser(orgParam.userId);
		company.setCreateTime(new Date());

		companyRepository.save(company);

		response.add("id", entity.getId());

		DmpMessage msg = new DmpMessage(OrgConstants.MESSAGE_GROUP_CREATE, entity.getId());
		msg.setGroupCode(entity.getCode());
		msg.setStr1(entity.getName());
		response.add(msg);

		return response;
	}

	@Transactional
	public MapResponse update(GroupModel param) {
		MapResponse response = new MapResponse();
		// TODO
		if (DmpParameterStorage.getUserId() == null){
			response.add("error", "userId不能为空！");
			return response;
		}
		String userId = DmpParameterStorage.getUserId();
		if (userId != null){
			Group one = groupRepository.getOne(param.getId());
			Group entity = Group.fromModel(param);
			entity.setUpdateUser(userId);
			entity.setUpdateTime(new Date());
			entity.setStatus(one.getStatus());
			entity.setCreateUser(one.getCreateUser());
			entity.setCreateTime(one.getCreateTime());
			groupRepository.save(entity);
			response.add("id", entity.getId());
		}
		return response;
	}

	@Transactional
	public MapResponse delete(GroupModel param) {
		MapResponse response = new MapResponse();
		// TODO 逻辑删除
		if (DmpParameterStorage.getUserId() == null){
			response.add("error", "userId不能为空！");
			return response;
		}
		String userId = DmpParameterStorage.getUserId();
		if (userId != null){
			Group entity = groupRepository.getOne(param.getId());
			entity.setStatus(0);
			entity.setUpdateTime(new Date());
			entity.setUpdateUser(userId);
			groupRepository.save(entity);
			response.add("id", entity.getId());
		}
		return response;
	}
}

