package com.persagy.dmp.org.service;

import com.persagy.common.web.MapResponse;
import com.persagy.dmp.config.DmpParameterStorage;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.sql.DataSource;
import javax.transaction.Transactional;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;

@Slf4j
@Service
public class InitService {

    @Autowired
    private DataSource dataSource;

    @Transactional
    public MapResponse initGroup() {
        MapResponse resp = new MapResponse();
        String groupCode = DmpParameterStorage.getGroupCode();
        String sql = null;
        Connection connection = null;
        Statement statement = null;
        try {
            connection = dataSource.getConnection();
            statement = connection.createStatement();

            /** 人员权限 */
            /** 公司 */
            sql = getCreateOrgCompanySql(groupCode);
            statement.executeUpdate(sql);

//            /** 部门 */
//            sql = getCreateOrgDeptSql(groupCode);
//            statement.executeUpdate(sql);
//
//            /** 部门分类 */
//            sql = getCreateOrgDeptClassifySql(groupCode);
//            statement.executeUpdate(sql);
//
//            /** 岗位 */
//            sql = getCreateOrgJobSql(groupCode);
//            statement.executeUpdate(sql);
//
//            /** 岗位分类 */
//            sql = getCreateOrgJobClassifySql(groupCode);
//            statement.executeUpdate(sql);
//
//            /** 岗位分类关系 */
//            sql = getCreateOrgJobClassifyRelSql(groupCode);
//            statement.executeUpdate(sql);
//
//            /** 项目表 */
//            sql = getCreateOrgProjectSql(groupCode);
//            statement.executeUpdate(sql);
//
//            /** 公司与项目关系表 */
//            sql = getCreateOrgCompanyProjectRelSql(groupCode);
//            statement.executeUpdate(sql);
//
//            /** 人员表 */
//            sql = getCreateOrgUserSql(groupCode);
//            statement.executeUpdate(sql);
//
//            /** 集团与应用关系表 */
//            sql = getCreateOrgGroupAppRelSql(groupCode);
//            statement.executeUpdate(sql);
//
//            /** 集团与功能关系表 */
//            sql = getCreateOrgGroupFunctionRelSql(groupCode);
//            statement.executeUpdate(sql);

            /** 任务 */
            /** 任务配置表 */
//            sql = getCreateTaskCfgSql(groupCode);
//            statement.executeUpdate(sql);

            /** 任务记录表 */
//            sql = getCreateTaskRecordSql(groupCode);
//            statement.executeUpdate(sql);

        } catch (SQLException e) {
            e.printStackTrace();
            resp.setFail(e.getMessage());
        } finally {
            close(statement);
            close(connection);
        }
        return resp;
    }

    /**
     * 公司表
     * @param groupCode 集团编码
     * @return
     */
    private String getCreateOrgCompanySql(String groupCode) {

        String sql = "CREATE TABLE `org_company_" + groupCode.toLowerCase() + "` (\n" +
                "  `id` varchar(80) NOT NULL,\n" +
                "  `code` varchar(80) NOT NULL COMMENT '公司编码',\n" +
                "  `name` varchar(80) DEFAULT NULL COMMENT '公司名称',\n" +
                "  `group_code` varchar(80) NOT NULL COMMENT '集团编码',\n" +
                "  `parent_id` varchar(80) DEFAULT NULL COMMENT '上级公司ID',\n" +
                "  `status` int(11) NOT NULL COMMENT '状态',\n" +
                "  `create_user` varchar(80) DEFAULT NULL COMMENT '创建用户',\n" +
                "  `update_user` varchar(80) DEFAULT NULL COMMENT '更新用户',\n" +
                "  `create_time` timestamp default current_timestamp comment '创建时间', \n" +
                "  `update_time` timestamp default current_timestamp on update current_timestamp comment '更新时间', \n" +
                "  `remark` varchar(500) DEFAULT NULL COMMENT '备注',\n" +
                "  PRIMARY KEY (`id`) USING BTREE\n" +
                ") ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='公司';";
        return sql;
    }

    /**
     * 公司与项目关系表
     * @param groupCode 集团编码
     * @return
     */
    private String getCreateOrgCompanyProjectRelSql(String groupCode) {

        String sql = "CREATE TABLE `org_company_project_rel_" + groupCode.toLowerCase() + "` (\n" +
                "  `id` varchar(60) NOT NULL,\n" +
                "  `company_id` varchar(60) NOT NULL COMMENT '公司code',\n" +
                "  `project_id` varchar(60) NOT NULL COMMENT '项目ID',\n" +
                "  `group_code` varchar(80) DEFAULT NULL COMMENT '集团编码',\n" +
                "  PRIMARY KEY (`id`) USING BTREE\n" +
                ") ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='公司与项目关系';";
        return sql;
    }

    /**
     * 部门表
     * @param groupCode 集团编码
     * @return
     */
    private String getCreateOrgDeptSql(String groupCode) {
        String sql = "CREATE TABLE `org_dept_" + groupCode.toLowerCase() + "` (\n" +
                "  `id` varchar(80) NOT NULL,\n" +
                "  `dept_classify_code` varchar(80) NOT NULL COMMENT '部门分类编码',\n" +
                "  `code` varchar(80) NOT NULL COMMENT '部门编码',\n" +
                "  `name` varchar(80) NOT NULL COMMENT '部门名称',\n" +
                "  `company_id` varchar(80) NOT NULL COMMENT '所属公司ID',\n" +
                "  `parent_id` varchar(80) NOT NULL COMMENT '上级部门ID,0:顶级部门',\n" +
                "  `group_code` varchar(80) NOT NULL COMMENT '集团编码',\n" +
                "  `resource_from` varchar(80) NOT NULL COMMENT '部门来源',\n" +
                "  `manager` varchar(80) NOT NULL COMMENT '部门负责人',\n" +
                "  `status` int(11) NOT NULL COMMENT '状态',\n" +
                "  `create_user` varchar(80) DEFAULT NULL COMMENT '创建用户',\n" +
                "  `update_user` varchar(80) DEFAULT NULL COMMENT '更新用户',\n" +
                "  `create_time` timestamp default current_timestamp comment '创建时间', \n" +
                "  `update_time` timestamp default current_timestamp on update current_timestamp comment '更新时间', \n" +
                "  `remark` varchar(500) DEFAULT NULL,\n" +
                "  PRIMARY KEY (`id`) USING BTREE\n" +
                ") ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='部门';";
        return sql;
    }

    /**
     * 部门分类表
     * @param groupCode 集团编码
     * @return
     */
    private String getCreateOrgDeptClassifySql(String groupCode) {
        String sql = "CREATE TABLE `org_dept_classify_" + groupCode.toLowerCase() + "` (\n" +
                "  `id` varchar(80) NOT NULL,\n" +
                "  `code` varchar(200) NOT NULL COMMENT '部门分类编码',\n" +
                "  `name` varchar(200) NOT NULL COMMENT '名称',\n" +
                "  `parent_id` varchar(200) NOT NULL COMMENT '父级分类ID',\n" +
                "  `order_num` int(11) NOT NULL COMMENT '排序',\n" +
                "  `group_code` varchar(80) NOT NULL COMMENT '集团ID',\n" +
                "  `status` int(11) NOT NULL COMMENT '状态',\n" +
                "  `create_user` varchar(80) DEFAULT NULL COMMENT '创建用户',\n" +
                "  `update_user` varchar(80) DEFAULT NULL COMMENT '更新用户',\n" +
                "  `create_time` timestamp default current_timestamp comment '创建时间', \n" +
                "  `update_time` timestamp default current_timestamp on update current_timestamp comment '更新时间', \n" +
                "  `remark` varchar(500) DEFAULT NULL,\n" +
                "  PRIMARY KEY (`id`) USING BTREE\n" +
                ") ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='部门分类';";
        return sql;
    }

    /**
     * 岗位表
     * @param groupCode 集团编码
     * @return
     */
    private String getCreateOrgJobSql(String groupCode) {
        String sql = "CREATE TABLE `org_job_" + groupCode.toLowerCase() + "` (\n" +
                "  `id` varchar(80) NOT NULL,\n" +
                "  `code` varchar(80) NOT NULL COMMENT '岗位编码',\n" +
                "  `name` varchar(80) NOT NULL COMMENT '岗位名称',\n" +
                "  `group_code` varchar(80) NOT NULL COMMENT '集团编码',\n" +
                "  `user_id` varchar(80) NOT NULL COMMENT '账号ID',\n" +
                "  `dept_id` varchar(80) NOT NULL COMMENT '部门ID',\n" +
                "  `job_classify_code` varchar(80) NOT NULL COMMENT '岗位类型',\n" +
                "  `resource_from` varchar(80) DEFAULT NULL COMMENT '部门来源',\n" +
                "  `status` int(11) NOT NULL COMMENT '状态',\n" +
                "  `create_user` varchar(80) DEFAULT NULL COMMENT '创建用户',\n" +
                "  `update_user` varchar(80) DEFAULT NULL COMMENT '更新用户',\n" +
                "  `create_time` timestamp default current_timestamp comment '创建时间', \n" +
                "  `update_time` timestamp default current_timestamp on update current_timestamp comment '更新时间', \n" +
                "  `remark` varchar(500) DEFAULT NULL,\n" +
                "  PRIMARY KEY (`id`) USING BTREE\n" +
                ") ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='岗位';";
        return sql;
    }

    /**
     * 岗位分类表
     * @param groupCode 集团编码
     * @return
     */
    private String getCreateOrgJobClassifySql(String groupCode) {
        String sql = "CREATE TABLE `org_job_classify_" + groupCode.toLowerCase() + "` (\n" +
                "  `id` varchar(80) NOT NULL,\n" +
                "  `code` varchar(80) NOT NULL COMMENT '岗位分类编码',\n" +
                "  `name` varchar(80) NOT NULL COMMENT '名称',\n" +
                "  `group_code` varchar(80) NOT NULL COMMENT '集团编码',\n" +
                "  `order_num` int(11) NOT NULL COMMENT '排序',\n" +
                "  `status` int(11) NOT NULL COMMENT '状态',\n" +
                "  `create_user` varchar(80) DEFAULT NULL COMMENT '创建用户',\n" +
                "  `update_user` varchar(80) DEFAULT NULL COMMENT '更新用户',\n" +
                "  `create_time` timestamp default current_timestamp comment '创建时间', \n" +
                "  `update_time` timestamp default current_timestamp on update current_timestamp comment '更新时间', \n" +
                "  `remark` varchar(500) DEFAULT NULL,\n" +
                "  PRIMARY KEY (`id`) USING BTREE\n" +
                ") ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='岗位分类';";
        return sql;
    }

    /**
     * 岗位分类关系表
     * @param groupCode 集团编码
     * @return
     */
    private String getCreateOrgJobClassifyRelSql(String groupCode) {
        String sql = "CREATE TABLE `org_job_classify_rel_" + groupCode.toLowerCase() + "` (\n" +
                "  `id` varchar(80) NOT NULL,\n" +
                "  `job_classify_code` varchar(80) NOT NULL COMMENT '岗位分类ID',\n" +
                "  `app_id` varchar(80) NOT NULL COMMENT '应用ID',\n" +
                "  `function_id` varchar(80) NOT NULL COMMENT '功能ID',\n" +
                "  `group_code` varchar(80) NOT NULL COMMENT '集团编码',\n" +
                "  `status` int(11) NOT NULL COMMENT '状态',\n" +
                "  `create_user` varchar(80) DEFAULT NULL COMMENT '创建用户',\n" +
                "  `update_user` varchar(80) DEFAULT NULL COMMENT '更新用户',\n" +
                "  `create_time` timestamp default current_timestamp comment '创建时间', \n" +
                "  `update_time` timestamp default current_timestamp on update current_timestamp comment '更新时间', \n" +
                "  `remark` varchar(500) DEFAULT NULL,\n" +
                "  PRIMARY KEY (`id`) USING BTREE\n" +
                ") ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='岗位分类与应用功能关系';";
        return sql;
    }

    /**
     * 人员表
     * @param groupCode 集团编码
     * @return
     */
    private String getCreateOrgProjectSql(String groupCode) {
        String sql = "CREATE TABLE `org_project_" + groupCode.toLowerCase() + "` (\n" +
                "  `id` varchar(80) NOT NULL,\n" +
                "  `name` varchar(80) NOT NULL COMMENT '部门名称',\n" +
                "  `group_code` varchar(80) NOT NULL COMMENT '集团编码',\n" +
                "  `status` int(11) NOT NULL COMMENT '状态',\n" +
                "  `create_user` varchar(80) DEFAULT NULL COMMENT '创建用户',\n" +
                "  `update_user` varchar(80) DEFAULT NULL COMMENT '更新用户',\n" +
                "  `create_time` timestamp default current_timestamp comment '创建时间', \n" +
                "  `update_time` timestamp default current_timestamp on update current_timestamp comment '更新时间', \n" +
                "  `remark` varchar(500) DEFAULT NULL,\n" +
                "  PRIMARY KEY (`id`) USING BTREE\n" +
                ") ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='项目';";
        return sql;
    }

    /**
     * 创建人员表
     * @param groupCode 集团编码
     * @return
     */
    private String getCreateOrgUserSql(String groupCode) {
        String sql = "CREATE TABLE `org_user_" + groupCode.toLowerCase() + "` (\n" +
                "  `id` varchar(80) NOT NULL,\n" +
                "  `name` varchar(80) DEFAULT NULL COMMENT '姓名',\n" +
                "  `username` varchar(80) DEFAULT NULL COMMENT '账号',\n" +
                "  `password` varchar(80) DEFAULT NULL COMMENT '密码',\n" +
                "  `id_number` varchar(80) DEFAULT NULL COMMENT '员工编码',\n" +
                "  `gender` int(11) DEFAULT NULL COMMENT '性别',\n" +
                "  `phone` varchar(80) DEFAULT NULL COMMENT '手机号',\n" +
                "  `mail` varchar(80) DEFAULT NULL COMMENT '邮箱',\n" +
                "  `birthday` date NULL DEFAULT NULL COMMENT '生日',\n" +
                "  `head_portrait` varchar(80) DEFAULT NULL COMMENT '头像',\n" +
                "  `other_account` varchar(80) DEFAULT NULL COMMENT '第三方账号',\n" +
                "  `user_from` varchar(80) DEFAULT NULL COMMENT '账号来源',\n" +
                "  `group_code` varchar(80) NOT NULL,\n" +
                "  `is_admin` varchar(80) DEFAULT NULL COMMENT '是否为PERSAGYADMIN',\n" +
                "  `last_login_time` datetime NULL DEFAULT NULL COMMENT '最后登录时间',\n" +
                "  `last_ip` varchar(20) DEFAULT NULL COMMENT '最后登录ip地址',\n" +
                "  `status` int(11) NOT NULL COMMENT '状态',\n" +
                "  `create_user` varchar(80) DEFAULT NULL COMMENT '创建用户',\n" +
                "  `update_user` varchar(80) DEFAULT NULL COMMENT '更新用户',\n" +
                "  `create_time` timestamp default current_timestamp comment '创建时间', \n" +
                "  `update_time` timestamp default current_timestamp on update current_timestamp comment '更新时间', \n" +
                "  `remark` varchar(500) DEFAULT NULL,\n" +
                "  `person_id` varchar(80) DEFAULT NULL,\n" +
                "  PRIMARY KEY (`id`) USING BTREE\n" +
                ") ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='人员账号';";
        return sql;
    }

    /**
     * 集团与应用关系表
     * @param groupCode 集团编码
     * @return
     */
    private String getCreateOrgGroupAppRelSql(String groupCode) {
        String sql = "CREATE TABLE `org_group_app_rel_" + groupCode.toLowerCase() + "` (\n" +
                "  `id` varchar(80) NOT NULL,\n" +
                "  `group_code` varchar(80) DEFAULT NULL COMMENT '集团编码',\n" +
                "  `app_id` varchar(80) DEFAULT NULL COMMENT '应用id',\n" +
                "  `authority_type` int(11) DEFAULT NULL COMMENT '权限类型(0-权限集,1-权限包)',\n" +
                "  `status` int(11) NOT NULL COMMENT '状态',\n" +
                "  `create_user` varchar(80) DEFAULT NULL COMMENT '创建用户',\n" +
                "  `update_user` varchar(80) DEFAULT NULL COMMENT '更新用户',\n" +
                "  `create_time` timestamp default current_timestamp comment '创建时间', \n" +
                "  `update_time` timestamp default current_timestamp on update current_timestamp comment '更新时间', \n" +
                "  `remark` varchar(500) DEFAULT NULL COMMENT '备注',\n" +
                "  PRIMARY KEY (`id`) USING BTREE\n" +
                ") ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='集团与应用关系';";
        return sql;
    }

    /**
     * 集团与功能关系表
     * @param groupCode 集团编码
     * @return
     */
    private String getCreateOrgGroupFunctionRelSql(String groupCode) {
        String sql = "CREATE TABLE `org_group_function_rel_" + groupCode.toLowerCase() + "` (\n" +
                "  `id` varchar(80) NOT NULL,\n" +
                "  `group_code` varchar(80) DEFAULT NULL COMMENT '集团编码',\n" +
                "  `function_id` varchar(80) DEFAULT NULL COMMENT '功能id',\n" +
                "  `authority_type` varchar(255) DEFAULT NULL COMMENT '权限类型(0-权限集,1-权限包)',\n" +
                "  `status` int(11) NOT NULL COMMENT '状态',\n" +
                "  `create_user` varchar(80) DEFAULT NULL COMMENT '创建用户',\n" +
                "  `update_user` varchar(80) DEFAULT NULL COMMENT '更新用户',\n" +
                "  `create_time` timestamp default current_timestamp comment '创建时间', \n" +
                "  `update_time` timestamp default current_timestamp on update current_timestamp comment '更新时间', \n" +
                "  `remark` varchar(500) DEFAULT NULL COMMENT '备注',\n" +
                "  PRIMARY KEY (`id`) USING BTREE\n" +
                ") ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='集团与功能关系';";
        return sql;
    }


    /**
     * 任务记录表
     * @param groupCode
     * @return
     */
    private String getCreateTaskRecordSql(String groupCode) {
        String sql = "CREATE TABLE `dpt_task_record` (\n" +
                "  `id` varchar(40) NOT NULL,\n" +
                "  `task_id` varchar(40) NOT NULL COMMENT '任务配置ID',\n" +
                "  `start_time` datetime NULL DEFAULT NULL COMMENT '开始时间',\n" +
                "  `end_time` datetime NULL DEFAULT NULL COMMENT '结束时间',\n" +
                "  `success` int(11) NOT NULL COMMENT '1是，0否，否表示发生了异常',\n" +
                "  `error_info` json NOT NULL COMMENT '错误信息',\n" +
                "  `group_code` varchar(80) DEFAULT NULL COMMENT '集团编码',\n" +
                "  PRIMARY KEY (`id`) USING BTREE\n" +
                ") ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='任务执行记录';";
        return sql;
    }

    private void close(Statement statement) {
        if (statement != null) {
            try {
                statement.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }
    }

    private void close(Connection connection) {
        if (connection != null) {
            try {
                connection.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }
    }
}
