package com.persagy.dmp.org.service;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.web.BaseResponse;
import com.persagy.dmp.org.common.OrgUtils;
import com.persagy.dmp.org.exception.OrgCURDRunTimeException;
import com.persagy.dmp.org.model.ApplicationModel;
import com.querydsl.core.types.dsl.BooleanExpression;
import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.web.PagedResponse;
import com.persagy.common.web.MapResponse;
import javax.transaction.Transactional;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;
import com.persagy.dmp.org.model.JobClassifyRelModel;
import org.springframework.beans.factory.annotation.Autowired;
import com.persagy.dmp.org.entity.*;
import com.persagy.dmp.org.repository.JobClassifyRelRepository;

/**
 * @author king
 */
@Service
public class JobClassifyRelService extends OrgBaseService {

	@Autowired
	private JobClassifyRelRepository jobClassifyRelRepository;

	@Autowired
	private CriteriaUtils criteriaUtils;

	private List<BooleanExpression> parse(ObjectNode criteria) {
		// TODO
		return null;
	}

	public PagedResponse<JobClassifyRelModel> query(JacksonCriteria criteria) {
		MapResponse response = new MapResponse();
		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE);
		PagedResponse<JobClassifyRelModel> result = new PagedResponse<>();
		if (!response.success()) {
			result.setFail(response.getMessage());
			return result;
		}
		criteria.add("groupCode", orgParam.groupCode);

		PagedResponse<JobClassifyRel> resp = criteriaUtils.query(QJobClassifyRel.jobClassifyRel, this::parse, criteria);
		result.setCount(resp.getCount());
		List<JobClassifyRel> dataList = resp.getData();
		if (dataList != null && dataList.size() > 0) {
			List<JobClassifyRelModel> collect = dataList.stream().map(entity -> {
				JobClassifyRelModel model = entity.toModel();
				return model;
			}).collect(Collectors.toList());
			result.setData(collect);
		}
		return result;
	}

	@Transactional
	public MapResponse create(List<JobClassifyRelModel> params) {
		MapResponse response = new MapResponse();
		// TODO
		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE);
		if (!response.success()) {
			return response;
		}
		if (orgParam != null){
			if (params == null || params.isEmpty()){
				response.setFail("传入数据为空！");
				return response;
			}
			if (params != null && !params.isEmpty()) {
				List<JobClassifyRel> jobClassifyRels = new ArrayList<>(params.size());
				for (JobClassifyRelModel param : params) {
					JobClassifyRel entity = JobClassifyRel.fromModel(param);
					if (entity.getId() == null){
						entity.setId(OrgUtils.getUUID());
					}

					if (param.getJobClassifyCode() == null){
						response.setFail("岗位分类编码不能为空！");
						throw new OrgCURDRunTimeException(response);
					}
					// TODO 应用id、功能id 不能同时为空
					if (entity.getAppId() == null && entity.getFunctionId() == null){
						response.setFail("appId、functionId 不能同时为空！");
						throw new OrgCURDRunTimeException(response);
					}
					entity.setStatus(1);
					entity.setGroupCode(orgParam.groupCode);
					entity.setCreateUser(orgParam.userId);
					Date date = new Date();
					entity.setCreateTime(date);
					entity.setUpdateTime(date);
					entity.setUpdateUser(orgParam.userId);

					jobClassifyRels.add(entity);
				}
				if (jobClassifyRels == null || jobClassifyRels.isEmpty()){
					response.setFail("无新建数据！");
					throw new OrgCURDRunTimeException(response);
				}
				jobClassifyRelRepository.saveAll(jobClassifyRels);
				List<String> returnValue = jobClassifyRels.stream().map(JobClassifyRel::getId).collect(Collectors.toList());
				response.add("ids", returnValue);
			}
		}
		return response;
	}

	@Transactional
	public MapResponse update(List<JobClassifyRelModel> params) {
		MapResponse response = new MapResponse();
		// TODO
		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE);
		if (!response.success()) {
			return response;
		}
		if (orgParam != null){
			if (params == null || params.isEmpty()){
				response.setFail("传入数据为空！");
				return response;
			}
			if (params != null && !params.isEmpty()) {
				List<JobClassifyRel> jobClassifyRels = new ArrayList<>(params.size());
				for (JobClassifyRelModel param : params) {
					if (param.getId() == null) {
						response.setFail("更新操作id不能为空！");
						throw new OrgCURDRunTimeException(response);
					}
					QJobClassifyRel qJobClassifyRel = QJobClassifyRel.jobClassifyRel;
					Optional<JobClassifyRel> optional = jobClassifyRelRepository.findOne(qJobClassifyRel.id.eq(param.getId())
							.and(qJobClassifyRel.groupCode.eq(orgParam.groupCode)));
					if (!optional.isPresent()) {
						// TODO 数据不存在,直接报错返回
						response.setFail("数据不存在！");
						throw new OrgCURDRunTimeException(response);
					}
					JobClassifyRel one = optional.get();

					if (param.getJobClassifyCode() != null){
						one.setJobClassifyCode(param.getJobClassifyCode());
					}
					if (param.getAppId() != null){
						one.setAppId(param.getAppId());
					}
					if (param.getFunctionId() != null){
						one.setFunctionId(param.getFunctionId());
					}
					if (param.getRemark() != null){
						one.setRemark(param.getRemark());
					}
					if (param.getStatus() != null){
						one.setStatus(param.getStatus());
					}
					one.setUpdateUser(orgParam.userId);
					one.setUpdateTime(new Date());

					jobClassifyRels.add(one);
				}
				if (jobClassifyRels == null || jobClassifyRels.isEmpty()){
					response.setFail("无更新数据！");
					throw new OrgCURDRunTimeException(response);
				}
				jobClassifyRelRepository.saveAll(jobClassifyRels);
				List<String> returnValue = jobClassifyRels.stream().map(JobClassifyRel::getId).collect(Collectors.toList());
				response.add("ids", returnValue);
			}
		}
		return response;
	}

	@Transactional
	public MapResponse delete(List<JobClassifyRelModel> params) {
		MapResponse response = new MapResponse();
		// TODO 逻辑删除
		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE);
		if (!response.success()) {
			return response;
		}
		if (orgParam != null){
			if (params == null || params.isEmpty()){
				response.setFail("传入数据为空！");
				return response;
			}
			if (params != null && !params.isEmpty()) {
				List<JobClassifyRel> jobClassifyRels = new ArrayList<>(params.size());
				for (JobClassifyRelModel param : params) {
					if (param.getId() == null) {
						response.setMessage("删除操作id不能为空！");
						throw new OrgCURDRunTimeException(response);
					}
					QJobClassifyRel qJobClassifyRel = QJobClassifyRel.jobClassifyRel;
					Optional<JobClassifyRel> optional = jobClassifyRelRepository.findOne(qJobClassifyRel.id.eq(param.getId())
							.and(qJobClassifyRel.groupCode.eq(orgParam.groupCode)));
					if (!optional.isPresent()) {
						// TODO 数据不存在,直接报错返回
						response.setFail("数据不存在！");
						throw new OrgCURDRunTimeException(response);
					}
					JobClassifyRel entity = optional.get();
					entity.setStatus(0);
					entity.setUpdateTime(new Date());
					entity.setUpdateUser(orgParam.userId);

					jobClassifyRels.add(entity);
				}
				if (jobClassifyRels == null || jobClassifyRels.isEmpty()){
					response.setFail("无删除数据！");
					throw new OrgCURDRunTimeException(response);
				}
				jobClassifyRelRepository.saveAll(jobClassifyRels);
				List<String> returnValue = jobClassifyRels.stream().map(JobClassifyRel::getId).collect(Collectors.toList());
				response.add("ids", returnValue);
			}
		}
		return response;
	}
}

