package com.persagy.dmp.org.service;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.dmp.org.common.OrgUtils;
import com.persagy.dmp.org.exception.OrgCURDRunTimeException;
import com.persagy.dmp.org.model.CompanyModel;
import com.persagy.dmp.org.model.DeptModel;
import com.querydsl.core.types.dsl.BooleanExpression;
import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.web.PagedResponse;
import com.persagy.common.web.MapResponse;
import javax.transaction.Transactional;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;
import com.persagy.dmp.org.model.JobClassifyModel;
import org.springframework.beans.factory.annotation.Autowired;
import com.persagy.dmp.org.entity.*;
import com.persagy.dmp.org.repository.JobClassifyRepository;

/**
 * @author king
 */
@Service
public class JobClassifyService extends OrgBaseService {

	@Autowired
	private JobClassifyRepository jobClassifyRepository;

	@Autowired
	private CriteriaUtils criteriaUtils;

	private List<BooleanExpression> parse(ObjectNode criteria) {
		// TODO
		return null;
	}

	public PagedResponse<JobClassifyModel> query(JacksonCriteria criteria) {
		MapResponse response = new MapResponse();
		PagedResponse<JobClassifyModel> result = new PagedResponse<>();
		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE);
		if (!response.success()) {
			result.setFail(response.getMessage());
			return result;
		}
		criteria.add("groupCode", orgParam.groupCode);

		PagedResponse<JobClassify> resp = criteriaUtils.query(QJobClassify.jobClassify, this::parse, criteria);
		result.setCount(resp.getCount());
		List<JobClassify> dataList = resp.getData();
		if (dataList != null && dataList.size() > 0) {
			List<JobClassifyModel> collect = dataList.stream().map(entity -> {
				JobClassifyModel model = entity.toModel();
				return model;
			}).collect(Collectors.toList());
			result.setData(collect);
		}
		return result;
	}

	@Transactional
	public MapResponse create(List<JobClassifyModel> params) {
		MapResponse response = new MapResponse();
		// TODO
		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE);
		if (!response.success()) {
			return response;
		}
		if (orgParam != null) {
			if (params == null || params.isEmpty()){
				response.setFail("传入数据为空！");
				return response;
			}
			if (params != null && !params.isEmpty()) {
				List<JobClassify> jobClassifies = new ArrayList<>(params.size());
				for (JobClassifyModel param : params) {
					JobClassify jobClassify = JobClassify.fromModel(param);
					if (jobClassify.getId() == null){
						jobClassify.setId(OrgUtils.getUUID());
					}
					if (jobClassify.getCode() == null){
						response.setFail("岗位分类 code 不能为空！");
						throw new OrgCURDRunTimeException(response);
					}
					if (jobClassify.getName() == null){
						response.setFail("岗位分类 name 不能为空！");
						throw new OrgCURDRunTimeException(response);
					}
					QJobClassify qJobClassify = QJobClassify.jobClassify;
					// TODO code 不可重复
					long codeFlag = jobClassifyRepository.count(qJobClassify.code.eq(param.getCode())
							.and(qJobClassify.groupCode.eq(orgParam.groupCode)));
					if (codeFlag > 0) {
						response.setFail("岗位分类 code 不能重复！");
						throw new OrgCURDRunTimeException(response);
					}

					// TODO name 不可重复
					long nameFlag = jobClassifyRepository.count(qJobClassify.name.eq(param.getName())
							.and(qJobClassify.groupCode.eq(orgParam.groupCode)));
					if (nameFlag > 0) {
						response.setFail("岗位分类 name 不能重复！");
						throw new OrgCURDRunTimeException(response);
					}

					jobClassify.setStatus(1);
					jobClassify.setGroupCode(orgParam.groupCode);
					jobClassify.setCreateUser(orgParam.userId);
					jobClassify.setCreateTime(new Date());
					Date date = new Date();
					jobClassify.setCreateTime(date);
					jobClassify.setUpdateTime(date);
					jobClassify.setUpdateUser(orgParam.userId);
					jobClassifies.add(jobClassify);
				}
				if (jobClassifies == null || jobClassifies.isEmpty()){
					response.setFail("无新建数据！");
					throw new OrgCURDRunTimeException(response);
				}
				jobClassifyRepository.saveAll(jobClassifies);
				List<String> returnValue = jobClassifies.stream().map(JobClassify::getId).collect(Collectors.toList());
				response.add("ids", returnValue);
			}
		}
		return response;
	}

	@Transactional
	public MapResponse update(List<JobClassifyModel> params) {
		MapResponse response = new MapResponse();
		// TODO
		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE);
		if (!response.success()) {
			return response;
		}
		if (orgParam != null) {
			if (params == null || params.isEmpty()){
				response.setFail("传入数据为空！");
				return response;
			}
			if (params != null && !params.isEmpty()) {
				List<JobClassify> jobClassifies = new ArrayList<>(params.size());
				for (JobClassifyModel param : params) {
					if (param.getId() == null){
						response.setFail("更新操作id不能为空！");
						throw new OrgCURDRunTimeException(response);
					}
					QJobClassify qJobClassify = QJobClassify.jobClassify;
					Optional<JobClassify> optional = jobClassifyRepository.findOne(qJobClassify.id.eq(param.getId())
							.and(qJobClassify.groupCode.eq(orgParam.groupCode)));
					if (!optional.isPresent()) {
						// TODO 数据不存在,直接报错返回
						response.setFail("数据不存在！");
						throw new OrgCURDRunTimeException(response);
					}
					JobClassify one = optional.get();

					if (param.getCode() != null){
						if (param.getCode() != one.getCode()){
							// TODO code 不可重复
							long codeFlag = jobClassifyRepository.count(qJobClassify.code.eq(param.getCode())
									.and(qJobClassify.groupCode.eq(orgParam.groupCode)));
							if (codeFlag > 1) {
								response.setFail("岗位分类 code 不能重复！");
								throw new OrgCURDRunTimeException(response);
							}
						}
						one.setCode(param.getCode());
					}
					if (param.getName() != null){
						if (param.getName() != one.getName()){
							// TODO name 不可重复
							long nameFlag = jobClassifyRepository.count(qJobClassify.name.eq(param.getName())
									.and(qJobClassify.groupCode.eq(orgParam.groupCode)));
							if (nameFlag > 1) {
								response.setFail("岗位分类 name 不能重复！");
								throw new OrgCURDRunTimeException(response);
							}
						}
						one.setName(param.getName());
					}
					if (param.getOrderNum() != null){
						one.setOrderNum(param.getOrderNum());
					}
					if (param.getStatus() != null){
						one.setStatus(param.getStatus());
					}
					if (param.getRemark() != null){
						one.setRemark(param.getRemark());
					}

					one.setUpdateUser(orgParam.userId);
					one.setUpdateTime(new Date());

					jobClassifies.add(one);
				}
				if (jobClassifies == null || jobClassifies.isEmpty()){
					response.setFail("无更新数据！");
					throw new OrgCURDRunTimeException(response);
				}
				jobClassifyRepository.saveAll(jobClassifies);
				List<String> returnValue = jobClassifies.stream().map(JobClassify::getId).collect(Collectors.toList());
				response.add("ids", returnValue);
			}
		}
		return response;
	}

	@Transactional
	public MapResponse delete(List<JobClassifyModel> params) {
		MapResponse response = new MapResponse();
		// TODO 逻辑删除
		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE);
		if (!response.success()) {
			return response;
		}
		if (orgParam != null) {
			if (params == null || params.isEmpty()){
				response.setFail("传入数据为空！");
				return response;
			}
			if (params != null && !params.isEmpty()) {
				List<JobClassify> jobClassifies = new ArrayList<>(params.size());
				for (JobClassifyModel param : params) {
					if (param.getId() == null){
						response.setFail("删除操作id不能为空！");
						throw new OrgCURDRunTimeException(response);
					}
					QJobClassify qJobClassify = QJobClassify.jobClassify;
					Optional<JobClassify> optional = jobClassifyRepository.findOne(qJobClassify.id.eq(param.getId())
							.and(qJobClassify.groupCode.eq(orgParam.groupCode)));
					if (!optional.isPresent()) {
						// TODO 数据不存在,直接报错返回
						response.setFail("数据不存在！");
						throw new OrgCURDRunTimeException(response);
					}
					JobClassify one = optional.get();
					one.setStatus(0);
					one.setUpdateTime(new Date());
					one.setUpdateUser(orgParam.userId);

					jobClassifies.add(one);
				}
				if (jobClassifies == null || jobClassifies.isEmpty()){
					response.setFail("无删除数据！");
					throw new OrgCURDRunTimeException(response);
				}
				jobClassifyRepository.saveAll(jobClassifies);
				List<String> returnValue = jobClassifies.stream().map(JobClassify::getId).collect(Collectors.toList());
				response.add("ids", returnValue);
			}
		}
		return response;
	}
}

