package com.persagy.dmp.org.service;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.google.common.collect.Lists;
import com.persagy.dmp.org.common.OrgUtils;
import com.persagy.dmp.org.exception.OrgCURDRunTimeException;
import com.persagy.dmp.org.model.ApplicationModel;
import com.persagy.dmp.org.model.CompanyModel;
import com.persagy.dmp.org.model.UserModel;
import com.querydsl.core.types.dsl.BooleanExpression;
import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.web.PagedResponse;
import com.persagy.common.web.MapResponse;
import javax.transaction.Transactional;

import lombok.var;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import com.persagy.dmp.org.model.JobModel;
import org.springframework.beans.factory.annotation.Autowired;
import com.persagy.dmp.org.entity.*;
import com.persagy.dmp.org.repository.JobRepository;

/**
 * @author king
 */
@Service
public class JobService extends OrgBaseService {

	@Autowired
	private JobRepository jobRepository;

	@Autowired
	private CriteriaUtils criteriaUtils;

	private List<BooleanExpression> parse(ObjectNode criteria) {
		// TODO
		return null;
	}

	public PagedResponse<JobModel> query(JacksonCriteria criteria) {
		MapResponse response = new MapResponse();
		PagedResponse<JobModel> result = new PagedResponse<>();
		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE);
		if (!response.success()) {
			result.setFail(response.getMessage());
			return result;
		}
		criteria.add("groupCode", orgParam.groupCode);

		PagedResponse<Job> resp = criteriaUtils.query(QJob.job, this::parse, criteria);
		result.setCount(resp.getCount());
		List<Job> dataList = resp.getData();
		if (dataList != null && dataList.size() > 0) {
			List<JobModel> collect = dataList.stream().map(entity -> {
				JobModel model = entity.toModel();
				return model;
			}).collect(Collectors.toList());
			result.setData(collect);
		}
		return result;
	}

	@Transactional
	public MapResponse create(List<JobModel> params) {
		MapResponse response = new MapResponse();
		// TODO
		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE);
		if (!response.success()) {
			return response;
		}
		if (orgParam != null){
			if (params == null || params.isEmpty()){
				response.setFail("传入数据为空！");
				return response;
			}
			if (params != null && !params.isEmpty()) {
				List<Job> jobs = new ArrayList<>(params.size());
				for (JobModel param : params) {
					Job entity = Job.fromModel(param);
					if (entity.getId() == null){
						entity.setId(OrgUtils.getUUID());
					}
					if (entity.getUserId() == null){
						response.setFail("userId 不能为空！");
						throw new OrgCURDRunTimeException(response);
					}
					if (entity.getDeptId() == null){
						response.setFail("deptId 不能为空！");
						throw new OrgCURDRunTimeException(response);
					}
					if (entity.getJobClassifyCode() == null){
						response.setFail("JobClassifyCode 不能为空！");
						throw new OrgCURDRunTimeException(response);
					}
					entity.setStatus(1);
					entity.setGroupCode(orgParam.groupCode);
					entity.setCreateUser(orgParam.userId);
					Date date = new Date();
					entity.setCreateTime(date);
					entity.setUpdateTime(date);
					entity.setUpdateUser(orgParam.userId);
					jobs.add(entity);
				}
				if (jobs == null || jobs.isEmpty()){
					response.setFail("无新建数据！");
					throw new OrgCURDRunTimeException(response);
				}
				jobRepository.saveAll(jobs);
				List<String> returnValue = jobs.stream().map(Job::getId).collect(Collectors.toList());
				response.add("ids", returnValue);
			}
		}
		return response;
	}

	@Transactional
	public MapResponse update(List<JobModel> params) {
		MapResponse response = new MapResponse();
		// TODO
		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE);
		if (!response.success()) {
			return response;
		}
		if (orgParam != null){
			if (params == null || params.isEmpty()){
				response.setFail("传入数据为空！");
				return response;
			}
			if (params != null && !params.isEmpty()) {
				List<Job> jobs = new ArrayList<>(params.size());
				for (JobModel param : params) {
					if (param.getId() == null) {
						response.setFail("更新操作id不能为空！");
						throw new OrgCURDRunTimeException(response);
					}
					QJob qJob = QJob.job;
					Optional<Job> optional = jobRepository.findOne(qJob.id.eq(param.getId())
							.and(qJob.groupCode.eq(orgParam.groupCode)));
					if (!optional.isPresent()) {
						// TODO 数据不存在,直接报错返回
						response.setFail("数据不存在！");
						throw new OrgCURDRunTimeException(response);
					}
					Job one = optional.get();
					if (param.getCode() != null){
						one.setCode(param.getCode());
					}
					if(param.getName() != null){
						one.setName(param.getName());
					}
					// TODO 该 userId 必须从此传入
					if(param.getUserId() != null){
						one.setUserId(param.getUserId());
					}
					if(param.getDeptId() != null){
						one.setDeptId(param.getDeptId());
					}
					if(param.getJobClassifyCode() != null){
						one.setJobClassifyCode(param.getJobClassifyCode());
					}
					if(param.getResourceFrom() != null){
						one.setResourceFrom(param.getResourceFrom());
					}
					if (param.getStatus() != null){
						one.setStatus(param.getStatus());
					}
					if(param.getRemark() != null){
						one.setRemark(param.getRemark());
					}

					one.setUpdateUser(orgParam.userId);
					one.setUpdateTime(new Date());
					jobs.add(one);
				}
				if (jobs == null || jobs.isEmpty()){
					response.setFail("无更新数据！");
					throw new OrgCURDRunTimeException(response);
				}
				jobRepository.saveAll(jobs);
				List<String> returnValue = jobs.stream().map(Job::getId).collect(Collectors.toList());
				response.add("ids", returnValue);
			}
		}
		return response;
	}

	@Transactional
	public MapResponse delete(List<JobModel> params) {
		MapResponse response = new MapResponse();
		// TODO 逻辑删除
		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE);
		if (!response.success()) {
			return response;
		}
		if (orgParam != null){
			if (params == null || params.isEmpty()){
				response.setFail("传入数据为空！");
				return response;
			}
			if (params != null && !params.isEmpty()) {
				List<Job> jobs = new ArrayList<>(params.size());
				for (JobModel param : params) {
					if (param.getId() == null) {
						response.setFail("删除操作id不能为空！");
						throw new OrgCURDRunTimeException(response);
					}
					QJob qJob = QJob.job;
					Optional<Job> optional = jobRepository.findOne(qJob.id.eq(param.getId())
							.and(qJob.groupCode.eq(orgParam.groupCode)));
					if (!optional.isPresent()) {
						// TODO 数据不存在,直接报错返回
						response.setFail("数据不存在！");
						throw new OrgCURDRunTimeException(response);
					}
					Job entity = optional.get();
					entity.setStatus(0);
					entity.setUpdateTime(new Date());
					entity.setUpdateUser(orgParam.userId);

					jobs.add(entity);
				}
				if (jobs == null || jobs.isEmpty()){
					response.setFail("无删除数据！");
					throw new OrgCURDRunTimeException(response);
				}
				jobRepository.saveAll(jobs);
				List<String> returnValue = jobs.stream().map(Job::getId).collect(Collectors.toList());
				response.add("ids", returnValue);
			}
		}
		return response;
	}

	/**
	 * 人员 jobs 批量修改
	 * 需要完善
	 * @param params
	 * @return
	 */
	@Transactional
	public MapResponse updateJobsByUserId(List<UserModel> params) {
		MapResponse response = new MapResponse();

		// TODO 逻辑删除
		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE);
		if (!response.success()) {
			return response;
		}
		if (orgParam != null){
			if (params == null || params.isEmpty()){
				response.setFail("传入数据为空！");
				return response;
			}
			if (params != null && !params.isEmpty()){
				for (UserModel param : params) {

					List<Job> jobs = param.getJobs();

					if (jobs != null && !jobs.isEmpty()) {
						String userId = param.getId();

						List<Job> saveJobs = new ArrayList<>();

						// TODO 根据 userId 查询现有的岗位关系
						QJob job = QJob.job;
						Iterable<Job> all = jobRepository.findAll(job.userId.eq(userId).and(job.groupCode.eq(orgParam.groupCode)));
						List<String> jobIds = Lists.newArrayList(all).stream().map(Job::getId).collect(Collectors.toList());

						// TODO 遍历要更新的集合，筛选需要删除的岗位
						for (Job j : jobs) {
							String jId = j.getId();
							if (jobIds.contains(jId)){
								jobIds.remove(jId);
							} else {
								saveJobs.add(j);
							}
						}

						jobs.remove(saveJobs);
						// TODO 更新
						List<JobModel> updateJobModels = jobs.stream().map(job1 -> {
							JobModel jobModel = job1.toModel();
							return jobModel;
						}).collect(Collectors.toList());
						update(updateJobModels);

						// TODO 新建
						List<JobModel> saveJobModels = saveJobs.stream().map(job1 -> {
							JobModel jobModel = job1.toModel();
							return jobModel;
						}).collect(Collectors.toList());
						create(saveJobModels);

						// TODO 逻辑删除岗位关系
						List<JobModel> deleteJobs = new ArrayList<>();
						for (String jobId : jobIds) {
							JobModel var2 = new JobModel();
							var2.setId(jobId);
							deleteJobs.add(var2);
						}
						delete(deleteJobs);
					}
				}
			}
		}
		return response;
	}
}

