package com.persagy.dmp.org.service;

import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.web.BaseResponse;
import com.persagy.common.web.MapResponse;
import com.persagy.dmp.config.DmpParameterStorage;
import com.persagy.dmp.org.repository.*;
import com.querydsl.jpa.impl.JPAQueryFactory;
import lombok.Getter;
import lombok.Setter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.UUID;

/**
 * @author king
 */
public class OrgBaseService {

	@Autowired
	protected JPAQueryFactory jpaQueryFactory;

	@Autowired
	protected CriteriaUtils criteriaUtils;

	@Autowired
	protected ApplicationRepository applicationRepository;

	@Autowired
	protected CompanyRepository companyRepository;

	@Autowired
	protected DeptClassifyRepository deptClassifyRepository;

	@Autowired
	protected DeptRepository deptRepository;

	@Autowired
	protected FunctionRepository functionRepository;

	@Autowired
	protected GroupRepository groupRepository;

	@Autowired
	protected JobRepository jobRepository;

	@Autowired
	protected JobClassifyRepository jobClassifyRepository;

	@Autowired
	protected JobClassifyRelRepository jobClassifyRelRepository;

	@Autowired
	protected ProjectRepository projectRepository;

	@Autowired
	protected UserRepository userRepository;


	public static final String USER_ID = "userId";
	public static final String GROUP_CODE = "groupCode";
	public static final String PROJECT_ID = "projectId";

	@Getter
	@Setter
	protected static class OrgParam {
		String groupCode;
		String userId;
		String projectId;
	}

	protected <T> OrgParam prepareParam(BaseResponse<T> resp, String... requiredColumns) {
		OrgParam param = new OrgParam();
		param.groupCode = DmpParameterStorage.getGroupCode();
		param.userId = DmpParameterStorage.getUserId();
		param.projectId = DmpParameterStorage.getProjectId();

		if(requiredColumns != null){
			for (int i = 0; i < requiredColumns.length; i++) {
				String column = requiredColumns[i];
				if (USER_ID.equals(column) && (param.userId == null || param.userId.isEmpty())) {
					resp.setFail("请求地址中的 userId 必须有值");
					return null;
				}
				if (GROUP_CODE.equals(column) && (param.groupCode == null || param.groupCode.isEmpty())){
					resp.setFail("请求地址中的 groupCode 必须有值");
					return null;
				}
				if (PROJECT_ID.equals(column) && (param.projectId == null || param.projectId.isEmpty())){
					resp.setFail("请求地址中的 projectId 必须有值");
					return null;
				}
			}
		}
		return param;
	}

	/**
	 * 替换自动生成 uuid 中 '-'
	 * @return
	 */
	public static String getUUID(){
		return UUID.randomUUID().toString().replace("-", "");
	}

	/**
	 * 获取时间
	 * @return
	 */
	public static Date getNowDate(){
		return new Date();
	}

	/**
	 * 获取时间戳
	 * @return
	 */
	public static Long getTimeStamp(){
		return System.currentTimeMillis();
	}
}
