package com.persagy.dmp.org.service;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.google.common.collect.Lists;
import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.web.MapResponse;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.org.common.OrgUtils;
import com.persagy.dmp.org.constant.OrgConstants;
import com.persagy.dmp.org.constant.OrgWithColumnConstant;
import com.persagy.dmp.org.entity.*;
import com.persagy.dmp.org.exception.OrgCURDRunTimeException;
import com.persagy.dmp.org.model.DeptClassifyModel;
import com.persagy.dmp.org.model.ProjectModel;
import com.persagy.dmp.org.repository.CompanyProjectRelRepository;
import com.persagy.dmp.org.repository.CompanyRepository;
import com.persagy.dmp.org.repository.ProjectRepository;
import com.persagy.dmp.rwd.model.DmpMessage;
import com.querydsl.core.types.dsl.BooleanExpression;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author king
 */
@Service
public class ProjectService extends OrgBaseService {

	@Autowired
	private ProjectRepository projectRepository;

	@Autowired
	private CriteriaUtils criteriaUtils;

	@Autowired
	private CompanyProjectRelRepository companyProjectRelRepository;

	@Autowired
	private CompanyRepository companyRepository;

	private List<BooleanExpression> parse(ObjectNode criteria) {
		// TODO
//
//		List<BooleanExpression> exps = new LinkedList<>();
//		QCompany qCompany = QCompany.company;
//		exps.addAll(CriteriaUtils.parse(qCompany.code, criteria.get("code")));
//		exps.addAll(CriteriaUtils.parse(qCompany.groupCode, criteria.get("groupCode")));
		return null;
	}

	public PagedResponse<ProjectModel> query(JacksonCriteria criteria) {

		MapResponse response = new MapResponse();
		PagedResponse<ProjectModel> result = new PagedResponse<>();
		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE);
		if (!response.success()) {
			result.setFail(response.getMessage());
			return result;
		}
		criteria.add("groupCode", orgParam.groupCode);

		PagedResponse<Project> resp = criteriaUtils.query(QProject.project, this::parse, criteria);
		result.setCount(resp.getCount());
		List<Project> dataList = resp.getData();

		Set<String> withColumns = criteria.getWithColumns();
		Boolean flag = withColumns != null && withColumns.contains(OrgWithColumnConstant.PROJECT_COMPANIES);

		if (dataList != null && dataList.size() > 0) {
			List<ProjectModel> collect = dataList.stream().map(entity -> {
				ProjectModel model = entity.toModel();

				// TODO 查询包含项目的公司
				if (flag) {

					String projectId = model.getId();
					QCompanyProjectRel qt = QCompanyProjectRel.companyProjectRel;
					Iterable<CompanyProjectRel> all = companyProjectRelRepository.findAll(qt.projectId.eq(projectId)
							.and(qt.groupCode.eq(orgParam.groupCode)));
					ArrayList<CompanyProjectRel> companyProjectRels = Lists.newArrayList(all);

					List<String> companyIds = companyProjectRels.stream().map(
							CompanyProjectRel::getCompanyId
					).collect(Collectors.toList());
					QCompany qCompany = QCompany.company;
					List<Company> companies = Lists.newArrayList(companyRepository.findAll(qCompany.id.in(companyIds)
							.and(qCompany.groupCode.eq(orgParam.groupCode))));
					model.setCompanies(companies);
				}
				return model;
			}).collect(Collectors.toList());
			result.setData(collect);
		}
		return result;
	}

	@Transactional
	public MapResponse create(List<ProjectModel> params) {
		MapResponse response = new MapResponse();
		// TODO
		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE);
		if (!response.success()) {
			return response;
		}
		if (orgParam != null){
			if (params == null || params.isEmpty()){
				response.setFail("传入数据为空！");
				return response;
			}
			if (params != null && !params.isEmpty()) {
				List<Project> projects = new ArrayList<>(params.size());
				for (ProjectModel param : params) {
					Project entity = Project.fromModel(param);
					if (param.getId() == null){
						response.setFail("项目 id 不能为空！");
						throw new OrgCURDRunTimeException(response);
					}

					QProject qProject = QProject.project;
					long projectIdFlag = projectRepository.count(qProject.id.eq(param.getId())
							.and(qProject.groupCode.eq(orgParam.groupCode)));
					if (projectIdFlag > 0) {
						response.setFail("项目 id 不能重复！");
						throw new OrgCURDRunTimeException(response);
					}
					entity.setStatus(1);
					entity.setGroupCode(orgParam.groupCode);
					entity.setCreateUser(orgParam.userId);
					Date date = new Date();
					entity.setCreateTime(date);
					entity.setUpdateTime(date);
					entity.setUpdateUser(orgParam.userId);
					projects.add(entity);
				}

				List<CompanyProjectRel> companyProjectRels = new ArrayList<>(projects.size());
				for (ProjectModel param : params) {
					// TODO 新建项目默认关联顶级公司
					String projectId = param.getId();
					QCompany qCompany = QCompany.company;
					Iterator<Company> iterator = companyRepository.findAll(qCompany.groupCode.eq(orgParam.groupCode)
							.and(qCompany.parentId.eq("0"))).iterator();

					ArrayList<Company> companies = Lists.newArrayList(iterator);
					if (companies == null || companies.size() == 0){
						response.setFail("该集团下不存在顶级公司，请创建！");
						throw new OrgCURDRunTimeException(response);
					}
					Company topCompany = companies.get(0);
					String topCompanyId = topCompany.getId();

					CompanyProjectRel topCpr = new CompanyProjectRel();
					topCpr.setId(OrgUtils.getUUID());
					topCpr.setProjectId(projectId);
					topCpr.setCompanyId(topCompanyId);
					topCpr.setGroupCode(orgParam.groupCode);
					companyProjectRels.add(topCpr);
				}
				if (projects == null || projects.isEmpty()){
					response.setFail("无新建数据！");
					throw new OrgCURDRunTimeException(response);
				}
				projectRepository.saveAll(projects);
				List<String> returnValue = projects.stream().map(Project::getId).collect(Collectors.toList());
				response.add("ids", returnValue);
				companyProjectRelRepository.saveAll(companyProjectRels);

				for (Project project : projects) {
					DmpMessage msg = new DmpMessage(OrgConstants.MESSAGE_PROJECT_CREATE, project.getId());
					msg.setGroupCode(orgParam.getGroupCode());
					msg.setProjectId(project.getId());
					msg.setUserId(orgParam.userId);
					msg.setStr1(project.getName());
					response.add(msg);
				}
			}
		}
		return response;
	}

	@Transactional
	public MapResponse update(List<ProjectModel> params) {
		MapResponse response = new MapResponse();
		// TODO
		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE);
		if (!response.success()) {
			return response;
		}
		if (orgParam != null){
			if (params == null || params.isEmpty()){
				response.setFail("传入数据为空！");
				return response;
			}
			if (params != null && !params.isEmpty()) {
				List<Project> projects = new ArrayList<>(params.size());
				for (ProjectModel param : params) {
					if (param.getId() == null){
						response.setFail("更新操作id不能为空！");
						throw new OrgCURDRunTimeException(response);
					}
					QProject qProject = QProject.project;
					Optional<Project> optional = projectRepository.findOne(qProject.id.eq(param.getId())
							.and(qProject.groupCode.eq(orgParam.groupCode)));
					if (!optional.isPresent()) {
						// TODO 数据不存在,直接报错返回
						response.setFail("数据不存在！");
						throw new OrgCURDRunTimeException(response);
					}
					Project one = optional.get();

					if (param.getName() != null){
						one.setName(param.getName());
					}
					if (param.getRemark() != null){
						one.setRemark(param.getRemark());
					}
					if (param.getStatus() != null){
						one.setStatus(param.getStatus());
					}
					one.setUpdateUser(orgParam.userId);
					one.setUpdateTime(new Date());

					projects.add(one);
				}
				if (projects == null || projects.isEmpty()){
					response.setFail("无更新数据！");
					throw new OrgCURDRunTimeException(response);
				}
				projectRepository.saveAll(projects);
				List<String> returnValue = projects.stream().map(Project::getId).collect(Collectors.toList());
				response.add("ids", returnValue);
			}
		}
		return response;
	}

	@Transactional
	public MapResponse delete(List<ProjectModel> params) {
		MapResponse response = new MapResponse();
		// TODO 逻辑删除
		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE);
		if (!response.success()) {
			return response;
		}
		if (orgParam != null){
			if (params == null || params.isEmpty()){
				response.setFail("传入数据为空！");
				return response;
			}
			if (params != null && !params.isEmpty()) {
				List<Project> projects = new ArrayList<>(params.size());
				for (ProjectModel param : params) {
					if (param.getId() == null) {
						response.setFail("删除操作id不能为空！");
						throw new OrgCURDRunTimeException(response);
					}
					QProject qProject = QProject.project;
					Optional<Project> optional = projectRepository.findOne(qProject.id.eq(param.getId())
							.and(qProject.groupCode.eq(orgParam.groupCode)));
					if (!optional.isPresent()) {
						// TODO 数据不存在,直接报错返回
						response.setFail("数据不存在！");
						throw new OrgCURDRunTimeException(response);
					}
					Project entity = optional.get();
					entity.setStatus(0);
					entity.setUpdateTime(new Date());
					entity.setUpdateUser(orgParam.userId);

					projects.add(entity);
				}
				if (projects == null || projects.isEmpty()){
					response.setFail("无可删除的数据！");
					throw new OrgCURDRunTimeException(response);
				}
				projectRepository.saveAll(projects);
				List<String> returnValue = projects.stream().map(Project::getId).collect(Collectors.toList());
				response.add("ids", returnValue);
			}
		}
		return response;
	}
}

