package com.persagy.dmp.org.service;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.google.common.collect.Lists;
import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.web.MapResponse;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.org.common.OrgUtils;
import com.persagy.dmp.org.constant.OrgQueryConstant;
import com.persagy.dmp.org.constant.OrgWithColumnConstant;
import com.persagy.dmp.org.entity.*;
import com.persagy.dmp.org.exception.OrgCURDRunTimeException;
import com.persagy.dmp.org.model.UserModel;
import com.persagy.dmp.org.repository.JobRepository;
import com.persagy.dmp.org.repository.UserRepository;
import com.querydsl.core.types.dsl.BooleanExpression;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author king
 */
@Service
public class UserService extends OrgBaseService {

	@Autowired
	private UserRepository userRepository;

	@Autowired
	private JobRepository jobRepository;


	@Autowired
	private CriteriaUtils criteriaUtils;

	private List<BooleanExpression> parse(ObjectNode criteria) {
		// TODO
		List<BooleanExpression> expressions = new LinkedList<>();

		if (criteria.get(OrgQueryConstant.COMPANY_USER) != null){
			QDept qDept = QDept.dept;
			List<String> deptIds = jpaQueryFactory.select(qDept.id)
					.from(qDept)
					.where(CriteriaUtils.parse(qDept.companyId,
							criteria.get(OrgQueryConstant.COMPANY_USER)).toArray(new BooleanExpression[]{}))
					.where(qDept.status.eq(1))
					.where(qDept.groupCode.eq(criteria.get("groupCode").asText()))
					.fetch();
			if (deptIds != null || !deptIds.isEmpty()){

				// TODO 根据部门id查询岗位
				QJob qJob = QJob.job;
				Iterable<Job> all = jobRepository.findAll(qJob.deptId.in(deptIds)
						.and(qJob.status.eq(1))
						.and(qJob.groupCode.eq(criteria.get("groupCode").asText())));

				if (all != null){

					// TODO 根据岗位信息查询 userId 集合
					List<Job> jobs = Lists.newArrayList(all);
					List<String> userIds = jobs.stream().map(Job::getUserId).collect(Collectors.toList());

					if (userIds != null || !userIds.isEmpty()){
						expressions.add(QUser.user.id.in(userIds));
					}
				}
			}
		} else if (criteria.get(OrgQueryConstant.DEPT_USER) != null){

			// TODO 查询某个部门下的所有用户
			QJob qJob = QJob.job;
			Iterable<Job> all = jobRepository.findAll(qJob.deptId.eq(criteria.get(OrgQueryConstant.DEPT_USER).toString())
					.and(qJob.status.eq(1))
					.and(qJob.groupCode.eq(criteria.get("groupCode").asText())));

			if (all != null){

				// TODO 根据岗位信息查询 userId 集合
				List<Job> jobs = Lists.newArrayList(all);
				List<String> userIds = jobs.stream().map(Job::getUserId).collect(Collectors.toList());

				if (userIds != null || !userIds.isEmpty()){
					expressions.add(QUser.user.id.in(userIds));
				}
			}
		}
		return expressions;
	}

	public PagedResponse<UserModel> query(JacksonCriteria criteria) {
		MapResponse response = new MapResponse();
		PagedResponse<UserModel> result = new PagedResponse<>();
		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE);
		if (!response.success()) {
			result.setFail(response.getMessage());
			return result;
		}
		criteria.add("groupCode", orgParam.groupCode);

		PagedResponse<User> resp = criteriaUtils.query(QUser.user, this::parse, criteria);
		result.setCount(resp.getCount());
		List<User> dataList = resp.getData();

		Set<String> withColumns = criteria.getWithColumns();
		boolean jobFlag = withColumns != null && withColumns.contains(OrgWithColumnConstant.USER_JOBS);

		if (dataList != null && dataList.size() > 0) {
			List<UserModel> collect = dataList.stream().map(entity -> {
				UserModel model = entity.toModel();
				// TODO 根据 userId 查询岗位
				if (jobFlag) {
					String userId = entity.getId();
					QJob job = QJob.job;
					Iterable<Job> jobs = jobRepository.findAll(job.userId.eq(userId)
							.and(job.groupCode.eq(orgParam.groupCode)));
					if (jobs != null){
						List<Job> jobList = Lists.newArrayList(jobs);
						model.setJobs(jobList);
					}
				}
				return model;
			}).collect(Collectors.toList());
			result.setData(collect);
		}
		return result;
	}

	@Transactional
	public MapResponse create(List<UserModel> params) {
		MapResponse response = new MapResponse();
		// TODO
		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE);
		if (!response.success()) {
			return response;
		}
		if (orgParam != null) {
			if (params == null || params.isEmpty()){
				response.setFail("传入数据为空！");
				return response;
			}
			if (params != null && !params.isEmpty()) {
				List<User> users = new ArrayList<>(params.size());
				for (UserModel param : params) {
					User entity = User.fromModel(param);
					if (entity.getId() == null){
						entity.setId(OrgUtils.getUUID());
					}

					entity.setStatus(1);
					entity.setGroupCode(orgParam.groupCode);
					entity.setCreateUser(orgParam.userId);
					Date date = new Date();
					entity.setCreateTime(date);
					entity.setUpdateTime(date);;
					entity.setUpdateUser(orgParam.userId);
					users.add(entity);
				}
				if (users == null || users.isEmpty()){
					response.setFail("无新建数据！");
					throw new OrgCURDRunTimeException(response);
				}
				userRepository.saveAll(users);
				List<String> returnValue = users.stream().map(User::getId).collect(Collectors.toList());
				response.add("ids", returnValue);
			}
		}
		return response;
	}

	@Transactional
	public MapResponse update(List<UserModel> params) {
		MapResponse response = new MapResponse();
		// TODO
		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE);
		if (!response.success()) {
			return response;
		}
		if (orgParam != null){
			if (params == null || params.isEmpty()){
				response.setFail("传入数据为空！");
				return response;
			}
			if (params != null && !params.isEmpty()){
				List<User> users = new ArrayList<>(params.size());
				for (UserModel param : params) {
					if (param.getId() == null){
						response.setFail("更新操作Id不能为空！");
						throw new OrgCURDRunTimeException(response);
					}
					QUser qUser = QUser.user;
					Optional<User> optional = userRepository.findOne(qUser.id.in(param.getId())
							.and(qUser.groupCode.eq(orgParam.groupCode)));
					if (!optional.isPresent()) {
						// TODO 数据不存在,直接报错返回
						response.setFail("数据不存在！");
						throw new OrgCURDRunTimeException(response);
					}
					User one = optional.get();

					if(param.getName() != null){
						one.setName(param.getName());
					}
					if (param.getUsername() != null){
						one.setUsername(param.getUsername());
					}
					if (param.getPassword() != null){
						one.setPassword(param.getPassword());
					}
					if (param.getIdNumber() != null){
						one.setIdNumber(param.getIdNumber());
					}
					if (param.getGender() != null){
						one.setGender(param.getGender());
					}
					if (param.getPhone() != null){
						one.setPhone(param.getPhone());
					}
					if (param.getMail() != null){
						one.setMail(param.getMail());
					}
					if (param.getBirthday() != null){
						one.setBirthday(param.getBirthday());
					}
					if (param.getHeadPortrait() != null){
						one.setHeadPortrait(param.getHeadPortrait());
					}
					if (param.getOtherAccount() != null){
						one.setOtherAccount(param.getOtherAccount());
					}
					if (param.getUserFrom() != null){
						one.setUserFrom(param.getUserFrom());
					}
					if (param.getIsAdmin() != null){
						one.setIsAdmin(param.getIsAdmin());
					}
					if (param.getLastIp() != null){
						one.setLastIp(param.getLastIp());
					}
					if (param.getLastLoginTime() != null){
						one.setLastLoginTime(param.getLastLoginTime());
					}
					if (param.getRemark() != null){
						one.setRemark(param.getRemark());
					}
					one.setUpdateUser(orgParam.userId);
					one.setUpdateTime(new Date());

					users.add(one);
				}
				if (users == null || users.isEmpty()){
					response.setFail("无更新数据！");
					throw new OrgCURDRunTimeException(response);
				}
				userRepository.saveAll(users);
				List<String> returnValue = users.stream().map(User::getId).collect(Collectors.toList());
				response.add("ids", returnValue);
			}
		}
		return response;
	}

	@Transactional
	public MapResponse delete(List<UserModel> params) {
		MapResponse response = new MapResponse();
		// TODO 逻辑删除
		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE);
		if (!response.success()) {
			return response;
		}
		if (orgParam != null){
			if (params == null || params.isEmpty()){
				response.setFail("传入数据为空！");
				throw new OrgCURDRunTimeException(response);
			}
			if (params != null && !params.isEmpty()){
				List<User> users = new ArrayList<>(params.size());
				for (UserModel param : params) {
					if (param.getId() == null){
						response.setFail("删除操作id不能为空！");
						throw new OrgCURDRunTimeException(response);
					}
					QUser qUser = QUser.user;
					Optional<User> optional = userRepository.findOne(qUser.id.eq(param.getId())
							.and(qUser.groupCode.eq(orgParam.groupCode)));
					if (!optional.isPresent()) {
						// TODO 数据不存在,直接报错返回
						response.setFail("数据不存在！");
						throw new OrgCURDRunTimeException(response);
					}
					User one = optional.get();
					one.setStatus(0);
					one.setUpdateTime(new Date());
					one.setUpdateUser(orgParam.userId);

					users.add(one);
				}
				if (users == null || users.isEmpty()){
					response.setFail("无删除数据！");
					throw new OrgCURDRunTimeException(response);
				}
				userRepository.saveAll(users);
				List<String> returnValue = users.stream().map(User::getId).collect(Collectors.toList());
				response.add("ids", returnValue);
			}
		}
		return response;
	}


}

