package com.persagy.dmp.rwd.datautils.service;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.JsonNodeFactory;
import com.persagy.common.json.JacksonMapper;
import com.persagy.dmp.common.http.HttpUtils;
import com.persagy.dmp.rwd.datautils.model.TransContext;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.LinkedList;
import java.util.List;

/**
 * @author: yaoll
 * @date: 2020-10-14
 * @verison: 1.0
 */
@Slf4j
@Service
public class TransObjectValidService {

	@Value("${persagy.dmp.rwd.server}")
	private String rwdServer;

	@Autowired
	private HttpUtils httpUtils;

	@Data
	public static class Model {
		private Boolean valid;
		private String obj_id;
		private String project_id;
		private String subtype;
		private String dtime;
		private String ctime;
		private String obj_type;

		@JsonIgnore
		public String getId() {
			if ("Pj".equals(obj_type)) {
				return obj_type + project_id;
			}
			return obj_type + project_id + obj_id;
		}

		public String getClassCode(TransContext ctx) {
			if ("Eq".equals(this.obj_type) || "Sp".equals(this.obj_type)) {
				return ctx.getClassCodeMapping().get(this.subtype);
			} else if ("Bd".equals(this.obj_type) || "Fl".equals(this.obj_type)) {
				return ctx.getClassCodeMapping().get(this.obj_type);
			}
			return null;
		}
	}

	public void trans(TransContext ctx) throws IOException {

		String file = ctx.getBasePath() + "/" + ctx.getDataFile();
		BufferedReader reader = new BufferedReader(new InputStreamReader(new FileInputStream(file)));

		ArrayNode objectArray = JsonNodeFactory.instance.arrayNode();
		List<String> deleteList = new LinkedList<>();
		String temp;
		while (true) {
			temp = reader.readLine();
			if (temp == null) {
				break;
			}
			Model model = JacksonMapper.toObject(temp, Model.class);
			if (model.valid) {
				continue;
			}
			if ("Si".equalsIgnoreCase(model.obj_type)) {
				continue;
			}
			if ("Ec".equalsIgnoreCase(model.obj_type)) {
				continue;
			}
//			String classCode = model.getClassCode(ctx);
//			if (classCode == null) {
//				log.error("obj error: {}", JacksonMapper.toSimpleJson(model));
//				continue;
//			}
			if (!model.valid) {
				deleteList.add(model.getId());
			}
		}
		reader.close();

		if (deleteList.size() > 0) {
			String url = rwdServer + "/rwd/instance/object/delete?groupCode=" + ctx.getGroupCode() + "&projectId=" + ctx.getProjectId();
			HttpUtils.HttpResult response = httpUtils.post(url, JacksonMapper.toSimpleJson(deleteList), 3600000);
			System.out.println(response.getContent());
		}

	}

}
