package com.persagy.dmp.rwd.datautils.service;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.JsonNodeFactory;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.json.JacksonMapper;
import com.persagy.dmp.common.http.HttpUtils;
import com.persagy.dmp.rwd.datautils.config.GraphMapping;
import com.persagy.dmp.rwd.datautils.model.TransContext;
import com.persagy.dmp.rwd.model.RelationInstanceModel;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.*;
import java.util.LinkedList;
import java.util.List;
import java.util.UUID;

/**
 * @author: yaoll
 * @date: 2020-11-01
 * @verison: 1.0
 */
@Slf4j
@Service
public class TransRelationService {

	@Value("${persagy.dmp.rwd.server}")
	private String rwdServer;

	@Autowired
	private HttpUtils httpUtils;

	@Data
	public static class Model {
		private String to_obj_type;
		private String to_obj_id;
		private String project_id;
		private String rel_type;
		private String from_obj_id;
		private String from_obj_type;
		private String graph_id;

		public String getGraphCode(){
			return this.graph_id.substring(2, this.graph_id.length() - 3);
		}

		@JsonIgnore
		public String getFromId() {
			if ("Pj".equals(from_obj_type)) {
				return from_obj_type + project_id;
			}
			return from_obj_type + project_id + from_obj_id;
		}

		@JsonIgnore
		public String getToId() {
			if ("Pj".equals(to_obj_type)) {
				return to_obj_type + project_id;
			}
			return to_obj_type + project_id + to_obj_id;
		}
	}

	private RelationInstanceModel toRelation(Model model){
		GraphMapping.Item mapping = GraphMapping.getMapping(model.getGraphCode(), model.rel_type);
		if (mapping == null) {
			return null;
		}
		RelationInstanceModel relation = new RelationInstanceModel();
		relation.setGraphCode(mapping.getNewGraph());
		relation.setRelCode(mapping.getNewRelcode());
		if (mapping.getSame()) {
			relation.setObjFrom(model.getFromId());
			relation.setObjTo(model.getToId());
		} else {
			relation.setObjFrom(model.getToId());
			relation.setObjTo(model.getFromId());
		}
		return relation;
	}

	public void trans(TransContext ctx) throws IOException {
		String file = ctx.getBasePath() + "/" + ctx.getDataFile();
		BufferedReader reader = new BufferedReader(new InputStreamReader(new FileInputStream(file)));
		ArrayNode objectArray = JsonNodeFactory.instance.arrayNode();
		String temp;
		List<RelationInstanceModel> data = new LinkedList<>();
		while (true) {
			temp = reader.readLine();
			if (temp == null) {
				break;
			}
			Model model = JacksonMapper.toObject(temp, Model.class);
			if ("Si".equalsIgnoreCase(model.from_obj_type) || "Si".equals(model.to_obj_type)) {
				continue;
			}
			if ("Ec".equalsIgnoreCase(model.from_obj_type) || "Ec".equals(model.to_obj_type)) {
				continue;
			}
			RelationInstanceModel relation = toRelation(model);
			if (relation != null) {
				data.add(relation);
			} else {
				log.error("relaton error: {}", temp);
			}

			ObjectNode obj = objectArray.addObject();
			obj.put("graphId", model.graph_id);
			obj.put("relCode", model.rel_type);
			obj.put("objFrom", model.getFromId());
			obj.put("objTo", model.getToId());
		}
		reader.close();

		if (objectArray.size() > 0) {
			String url = rwdServer + "/rwd/instance/relation/create?projectId=" + ctx.getProjectId();
			HttpUtils.HttpResult response = httpUtils.post(url, JacksonMapper.toSimpleJson(data), 3600000);
			System.out.println(response.getContent());
		}
	}

	/**
	 * 拼装SQL
	 * @param ctx
	 * @throws IOException
	 */
	public void transToSql(TransContext ctx) throws IOException {
		String outFile = "/relation-out.sql";
		String file = ctx.getBasePath() + "/" + ctx.getDataFile();
		BufferedReader reader = new BufferedReader(new InputStreamReader(new FileInputStream(file)));
		ArrayNode objectArray = JsonNodeFactory.instance.arrayNode();
		String temp;
		PrintWriter writer = new PrintWriter(new OutputStreamWriter(new FileOutputStream(ctx.getBasePath() + outFile)));

		String projectId = ctx.getProjectId();
		String insert = "insert into rwd_relation_"+ctx.getGroupCode().toLowerCase() + "_" + projectId.toLowerCase() + "(id,group_code,project_id,graph_code,graph_id,rel_code,obj_from,obj_to,status,create_time)values";
		while (true) {
			temp = reader.readLine();
			if (temp == null) {
				break;
			}
			Model model = JacksonMapper.toObject(temp, Model.class);
			if ("Si".equalsIgnoreCase(model.from_obj_type) || "Si".equals(model.to_obj_type)) {
				continue;
			}

			StringBuilder sql = new StringBuilder(insert);
			sql.append("('").append(UUID.randomUUID().toString().replace("-", "")).append("',");
			sql.append("'").append(ctx.getGroupCode()).append("',");
			sql.append("'").append(projectId).append("',");
			sql.append("'").append(model.getGraphCode()).append("',");
			sql.append("'").append(model.graph_id).append("',");
			sql.append("'").append(model.rel_type).append("',");
			sql.append("'").append(model.getFromId()).append("',");
			sql.append("'").append(model.getToId()).append("',");
			sql.append("1,now());");
			writer.println(sql.toString());

		}
		reader.close();
		writer.close();
	}

}
