package com.persagy.dmp.rwd.dic;

import com.persagy.dmp.rwd.enums.FuncidCategory;
import com.persagy.dmp.rwd.enums.FuncidDataType;

import java.util.LinkedList;
import java.util.List;
import java.util.Optional;

/**
 * @author: yaoll
 * @date: 2020-11-30
 * @verison: 1.0
 */
public class DataTypeMapping {

	private final static List<FuncidDefModel> list = new LinkedList<>();

	public static FuncidDefModel getDataType(String origDataType) {
		if (origDataType == null) {
			return null;
		}
		Optional<FuncidDefModel> first = list.stream()
				.filter(item -> origDataType.equals(item.getOrigDataType()))
				.findFirst();
		return first.isPresent() ? first.get() : null;
	}

	static {
		list.add(init("Num", "DOUBLE", "STATIC", "N", "N"));
		list.add(init("Nums", "DOUBLE", "STATIC", "Y", "N"));
		list.add(init("Numr", "DOUBLE", "STATIC", "N", "Y"));
		list.add(init("Numrs", "DOUBLE", "STATIC", "Y", "Y"));
		list.add(init("Bool", "BOOLEAN", "STATIC", "N", "N"));
		list.add(init("Bools", "BOOLEAN", "STATIC", "Y", "N"));
		list.add(init("Enum", "ENUM", "STATIC", "N", "N"));
		list.add(init("Enums", "ENUM", "STATIC", "Y", "N"));
		list.add(init("Menum", "MENUM", "STATIC", "N", "N")); // error
		list.add(init("MEnum", "MENUM", "STATIC", "N", "N"));
		list.add(init("MEnums", "MENUM", "STATIC", "Y", "N"));
		list.add(init("Str", "STRING", "STATIC", "N", "N"));
		list.add(init("Strs", "STRING", "STATIC", "Y", "N"));
		list.add(init("Time", "DATETIME", "STATIC", "N", "N"));
		list.add(init("Times", "DATETIME", "STATIC", "Y", "N"));
		list.add(init("Timer", "DATETIME", "STATIC", "N", "Y"));
		list.add(init("Timers", "DATETIME", "STATIC", "Y", "Y"));
		list.add(init("Dtime", "DATETIME", "STATIC", "N", "N")); // error
		list.add(init("DTime", "DATETIME", "STATIC", "N", "N"));
		list.add(init("DTimes", "DATETIME", "STATIC", "Y", "N"));
		list.add(init("DTimer", "DATETIME", "STATIC", "N", "Y"));
		list.add(init("DTimers", "DATETIME", "STATIC", "Y", "Y"));
		list.add(init("Attachment", "ATTACHMENT", "STATIC", "N", "N"));
		list.add(init("Attachments", "ATTACHMENT", "STATIC", "Y", "N"));
		list.add(init("Inst", "DOUBLE", "SEQUENTIAL", "N", "N"));
		list.add(init("Insts", "DOUBLE", "SEQUENTIAL", "Y", "N"));
		list.add(init("Acc", "DOUBLE", "SEQUENTIAL", "N", "N"));
		list.add(init("Accs", "DOUBLE", "SEQUENTIAL", "Y", "N"));
		list.add(init("Boolv", "BOOLEAN", "SEQUENTIAL", "N", "N"));
		list.add(init("Boolvs", "BOOLEAN", "SEQUENTIAL", "Y", "N"));
		list.add(init("Enumv", "ENUM", "SEQUENTIAL", "N", "N"));
		list.add(init("Enumvs", "ENUM", "SEQUENTIAL", "Y", "N"));
		list.add(init("MEnumv", "MENUM", "SEQUENTIAL", "N", "N"));
		list.add(init("MEnumvs", "MENUM", "SEQUENTIAL", "Y", "N"));
		list.add(init("Strv", "STRING", "SEQUENTIAL", "N", "N"));
		list.add(init("Strvs", "STRING", "SEQUENTIAL", "Y", "N"));
		list.add(init("Timev", "DATETIME", "SEQUENTIAL", "N", "N"));
		list.add(init("Timevs", "DATETIME", "SEQUENTIAL", "Y", "N"));
		list.add(init("DTimev", "DATETIME", "SEQUENTIAL", "N", "N"));
		list.add(init("DTimevs", "DATETIME", "SEQUENTIAL", "Y", "N"));
		list.add(init("Nump", "DOUBLE", "GRADATION", "N", "N"));
		list.add(init("Numps", "DOUBLE", "GRADATION", "Y", "N"));
		list.add(init("Boolp", "BOOLEAN", "GRADATION", "N", "N"));
		list.add(init("Boolps", "BOOLEAN", "GRADATION", "Y", "N"));
		list.add(init("Enump", "ENUM", "GRADATION", "N", "N"));
		list.add(init("Enumps", "ENUM", "GRADATION", "Y", "N"));
		list.add(init("MEnump", "MENUM", "GRADATION", "N", "N"));
		list.add(init("MEnumps", "MENUM", "GRADATION", "Y", "N"));
		list.add(init("Strp", "STRING", "GRADATION", "N", "N"));
		list.add(init("Strps", "STRING", "GRADATION", "Y", "N"));
		list.add(init("Timep", "DATETIME", "GRADATION", "N", "N"));
		list.add(init("Timeps", "DATETIME", "GRADATION", "Y", "N"));
		list.add(init("DTimep", "DATETIME", "GRADATION", "N", "N"));
		list.add(init("DTimeps", "DATETIME", "GRADATION", "Y", "N"));
		list.add(init("Numi", "DOUBLE", "PULSE", "N", "N"));
		list.add(init("Numis", "DOUBLE", "PULSE", "Y", "N"));
		list.add(init("Booli", "BOOLEAN", "PULSE", "N", "N"));
		list.add(init("Boolis", "BOOLEAN", "PULSE", "Y", "N"));
		list.add(init("Enumi", "ENUM", "PULSE", "N", "N"));
		list.add(init("Enumis", "ENUM", "PULSE", "Y", "N"));
		list.add(init("MEnumi", "MENUM", "PULSE", "N", "N"));
		list.add(init("MEnumis", "MENUM", "PULSE", "Y", "N"));
		list.add(init("Stri", "STRING", "PULSE", "N", "N"));
		list.add(init("Stris", "STRING", "PULSE", "Y", "N"));
		list.add(init("Timei", "DATETIME", "PULSE", "N", "N"));
		list.add(init("Timeis", "DATETIME", "PULSE", "Y", "N"));
		list.add(init("DTimei", "DATETIME", "PULSE", "N", "N"));
		list.add(init("DTimeis", "DATETIME", "PULSE", "Y", "N"));
	}

	private static FuncidDefModel init(String origType, String dataType, String category, String isMulti, String isRegion) {
		FuncidDefModel model = new FuncidDefModel();
		model.setOrigDataType(origType);
		model.setDataType(FuncidDataType.valueOf(dataType));
		model.setCategory(FuncidCategory.valueOf(category));
		model.setIsMultiple("Y".equalsIgnoreCase(isMulti));
		model.setIsRegion("Y".equalsIgnoreCase(isRegion));
		return model;
	}
}
