package com.persagy.dmp.rwd.dic;

import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.junit.Test;

import java.io.File;
import java.io.FileInputStream;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author: yaoll
 * @date: 2020-11-30
 * @verison: 1.0
 */
public class ExcelUtils {

	public static Object parseCell(Cell cell, String type) {
		if (cell == null) {
			return null;
		}
		CellType cellType = cell.getCellTypeEnum();
		if ("string".equalsIgnoreCase(type)) {
			if (cellType == CellType.STRING) {
				String val = cell.getStringCellValue();
				return val == null ? null : val.trim();
			} else if (cellType == CellType.NUMERIC) {
				return cell.getNumericCellValue() + "";
			}
		} else if ("integer".equalsIgnoreCase(type)) {
			if (cellType == CellType.STRING) {
				String stringCellValue = cell.getStringCellValue();
				return stringCellValue == null ? null : Double.valueOf(stringCellValue).intValue();
			} else if (cellType == CellType.NUMERIC) {
				return Double.valueOf(cell.getNumericCellValue()).intValue();
			}
		} else if ("double".equalsIgnoreCase(type)) {
			if (cellType == CellType.STRING) {
				String stringCellValue = cell.getStringCellValue();
				return stringCellValue == null ? null : Double.valueOf(stringCellValue);
			} else if (cellType == CellType.NUMERIC) {
				return cell.getNumericCellValue();
			}
		} else if ("date".equalsIgnoreCase(type)) {
			if (cellType == CellType.NUMERIC) {
				Date dateCellValue = cell.getDateCellValue();
				return dateCellValue == null ? null : dateCellValue;
			} else if (cellType == CellType.STRING) {
				String val = cell.getStringCellValue();
				return val == null ? null : val.trim();
			}
		}
		return null;
	}


	private String basepath = "/Users/devsaga/persagy/数据字典20201118/Ch2-物理世界";

	@Test
	public void test1() throws Exception {
		String path = basepath + "/2.2-设备设施类对象/2.2.3-系统";
		showPath(path);
	}

	public void showPath(String path) throws Exception {
		List<String> names = listFile(path);
		if (names != null) {
			for (String name : names) {
//				System.out.println("----------------------");
				String fileName = path + "/" + name;
				FileInputStream inputStream = new FileInputStream(fileName);
//				System.out.println(fileName);
				Workbook workbook = new XSSFWorkbook(inputStream);
				int numberOfSheets = workbook.getNumberOfSheets();
				for (int i = 0; i < numberOfSheets; i++) {
					Sheet sheet = workbook.getSheetAt(i);
					String sheetName = sheet.getSheetName();
					// funcids.addAll(readSheet("FFFA", "/2.2-设备设施类对象/2.2.3-系统/50-消防专业系统.xlsx", "FA火灾报警系统"));
					System.out.println("funcids.addAll(readSheet(\"" + sheetName.substring(0, 4) + "\", \"" + fileName + "\", \"" + sheetName + "\"));");
				}
			}
		}
	}

	@Test
	public void test2() throws Exception {
		showPathForEquipment(basepath + "/2.2-设备设施类对象/2.2.4-设备设施/" + "1-强电专业");
		showPathForEquipment(basepath + "/2.2-设备设施类对象/2.2.4-设备设施/" + "2-弱电专业");
		showPathForEquipment(basepath + "/2.2-设备设施类对象/2.2.4-设备设施/" + "3-空调专业");
		showPathForEquipment(basepath + "/2.2-设备设施类对象/2.2.4-设备设施/" + "4-给排水专业");
		showPathForEquipment(basepath + "/2.2-设备设施类对象/2.2.4-设备设施/" + "5-消防专业");
		showPathForEquipment(basepath + "/2.2-设备设施类对象/2.2.4-设备设施/" + "6-安防专业");
		showPathForEquipment(basepath + "/2.2-设备设施类对象/2.2.4-设备设施/" + "7-土建精装专业");
		showPathForEquipment(basepath + "/2.2-设备设施类对象/2.2.4-设备设施/" + "8-其他");
	}

	public void showPathForEquipment(String path) throws Exception {
		List<String> names = listFile(path);
		if (names != null) {
			for (String name : names) {
//				System.out.println("----------------------");
				String fileName = path + "/" + name;
				FileInputStream inputStream = new FileInputStream(fileName);
//				System.out.println(fileName);
				Workbook workbook = new XSSFWorkbook(inputStream);
				int numberOfSheets = workbook.getNumberOfSheets();
				for (int i = 0; i < numberOfSheets; i++) {
					Sheet sheet = workbook.getSheetAt(i);
					String sheetName = sheet.getSheetName();
					// funcids.addAll(readSheet("FFFA", "/2.2-设备设施类对象/2.2.3-系统/50-消防专业系统.xlsx", "FA火灾报警系统"));
					System.out.println("funcids.addAll(readSheet(\"" + sheetName.substring(0, 4) + "\", \"" + fileName + "\", \"" + sheetName + "\"));");
				}
			}
		}
	}

	@Test
	public void test3() throws Exception {
		showPathForComponent(basepath + "/2.2-设备设施类对象/2.2.5-部件");
	}

	public void showPathForComponent(String path) throws Exception {
		List<String> names = listFile(path);
		if (names != null) {
			for (String name : names) {
//				System.out.println("----------------------");
				String fileName = path + "/" + name;
				FileInputStream inputStream = new FileInputStream(fileName);
//				System.out.println(fileName);
				Workbook workbook = new XSSFWorkbook(inputStream);
				int numberOfSheets = workbook.getNumberOfSheets();
				for (int i = 0; i < numberOfSheets; i++) {
					Sheet sheet = workbook.getSheetAt(i);
					String sheetName = sheet.getSheetName();
					// funcids.addAll(readSheet("FFFA", "/2.2-设备设施类对象/2.2.3-系统/50-消防专业系统.xlsx", "FA火灾报警系统"));
					System.out.println("funcids.addAll(readSheet(\"" + sheetName.substring(0, 6) + "\", \"" + fileName + "\", \"" + sheetName + "\"));");
				}
			}
		}
	}


	private List<String> listFile(String path) {
		File file = new File(path);
		File[] files = file.listFiles();
		return Arrays.stream(files).map(File::getName).filter(item -> !item.startsWith("~")).collect(Collectors.toList());
	}

}
