package com.persagy.dmp.rwd.dic;

import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.JsonNodeFactory;
import com.fasterxml.jackson.databind.node.ObjectNode;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * @author: yaoll
 * @date: 2020-11-30
 * @verison: 1.0
 */
public class FuncidUtils {

	private final static Set<String> baseFuncid = new HashSet<>();

	static {
		baseFuncid.add("ProjID");
		baseFuncid.add("ProjName");
		baseFuncid.add("ProjLocalID");
		baseFuncid.add("ProjLocalName");
		baseFuncid.add("BuildID");
		baseFuncid.add("BuildName");
		baseFuncid.add("BuildLocalID");
		baseFuncid.add("BuildLocalName");
		baseFuncid.add("FloorID");
		baseFuncid.add("FloorName");
		baseFuncid.add("FloorLocalID");
		baseFuncid.add("FloorLocalName");
		baseFuncid.add("RoomID");
		baseFuncid.add("RoomName");
		baseFuncid.add("RoomLocalID");
		baseFuncid.add("RoomLocalName");
		baseFuncid.add("ShaftID");
		baseFuncid.add("ShaftName");
		baseFuncid.add("ShaftLocalID");
		baseFuncid.add("ShaftLocalName");
		baseFuncid.add("SysID");
		baseFuncid.add("SysName");
		baseFuncid.add("SysLocalID");
		baseFuncid.add("SysLocalName");
		baseFuncid.add("EquipID");
		baseFuncid.add("EquipName");
		baseFuncid.add("EquipLocalID");
		baseFuncid.add("EquipLocalName");
		baseFuncid.add("ToolID");
		baseFuncid.add("ToolName");
		baseFuncid.add("ToolLocalID");
		baseFuncid.add("ToolLocalName");
		baseFuncid.add("ConsumID");
		baseFuncid.add("ConsumName");
		baseFuncid.add("ConsumLocalID");
		baseFuncid.add("ConsumLocalName");

		baseFuncid.add("ECModelID");
		baseFuncid.add("ECModelName");
		baseFuncid.add("ECModelLocalID");
		baseFuncid.add("ECModelLocalName");

		baseFuncid.add("SubentryID");
		baseFuncid.add("SubentryName");
		baseFuncid.add("SubentryLocalID");
		baseFuncid.add("SubentryLocalName");

		baseFuncid.add("BranchID");
		baseFuncid.add("BranchName");
		baseFuncid.add("BranchLocalID");
		baseFuncid.add("BranchLocalName");

		baseFuncid.add("BeaconID");
		baseFuncid.add("BeaconlName");
		baseFuncid.add("BeaconLocalID");
		baseFuncid.add("BeaconLocalName");

		baseFuncid.add("TenantID");
		baseFuncid.add("TenantName");
		baseFuncid.add("TenantLocalID");
		baseFuncid.add("TenantLocalName");
	}

	public static boolean isBase(String code) {
		return baseFuncid.contains(code);
	}

	public static ArrayNode transEnumDataSource(String dataSource) {
		ArrayNode array = JsonNodeFactory.instance.arrayNode();
		if (dataSource == null) {
			return null;
		}
		// 包含换行符 直接返回
		if (dataSource.indexOf("\n") > -1) {
			return null;
		}
		// 预处理 ". " -> "."
		while (dataSource.indexOf(". ") > -1) {
			dataSource = dataSource.replace(". ", ".");
		}
		if (dataSource.startsWith("见《")) {
			return null;
		} else if (dataSource.indexOf(".") > -1) {

			// 预处理
			String tmp = dataSource.replace("（多选）", "")
					.replace("（可多选）", "")
					.replace("（可以多选）", "");

			// 尝试解析
			// 1.铸铁 2.钢 3.铝合金 4.铜管 5.复合型 99.其他
			String[] split = tmp.split(" ");
			if (split.length == 0) {
				return null;
			}
			Map<String, String> data = new HashMap<>();
			for (int i = 0; i < split.length; i++) {
				String temp = split[i];
				int idx = temp.indexOf(".");
				if (idx == -1) {
					continue;
				}
				String key = temp.substring(0, idx);
				String value = temp.substring(idx + 1);
				data.put(key, value);
				ObjectNode item = array.addObject();
				item.put("code", key);
				item.put("name", value);
			}
		}
		return array;
	}
}
