package com.persagy.dmp.rwd.dic;

import com.persagy.common.date.DateUtils;
import com.persagy.common.json.JacksonMapper;
import com.persagy.dmp.rwd.enums.ObjType;
import com.persagy.dmp.rwd.model.ClassDefModel;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.junit.Test;

import java.io.*;
import java.util.*;

/**
 * 导入类型定义
 *
 * @author: yaoll
 * @date: 2020-11-30
 * @verison: 1.0
 */
@Slf4j
public class ImportClass {

	private String basepath = "/Users/devsaga/persagy/数据字典20201118/Ch2-物理世界";

	private String outPath = "/Users/devsaga/persagy/数据字典20201118/class-" + DateUtils.format(new Date()) + ".dml.sql";

	@Test
	public void importClassData() throws Exception {

		// 所有类型
		List<ClassDefModel> classes = new ArrayList<>();

		// 顶级类型
		initBaseClass(classes);

		// 空间分区
		initSpace(classes);

		// 专业\系统\设备\部件
		initEquipment(classes);

		// 虚拟对象
		initVirtual(classes);

		// 工具
		initTool(classes);

		// 耗材
		initMaterial(classes);

		FileOutputStream fos = new FileOutputStream(outPath);
		PrintWriter writer = new PrintWriter(new BufferedOutputStream(fos));
		writer.println("truncate table rwd_def_class;");

		// 基本类型
		for (ClassDefModel item : classes) {
			if (item.getParentCode() == null) {
				writer.println(toSql(item));
			}
		}

		//
		List<String> parentCodes = Arrays.asList("space", "system", "equipment", "component", "virtual", "tool", "material");
		for (String parentCode : parentCodes) {
			for (ClassDefModel item : classes) {
				if (parentCode.equals(item.getParentCode())) {
					writer.println(toSql(item));
				}
			}
			writer.flush();
		}

		// 基本信息点
		for (ClassDefModel item : classes) {
			if (item.getObjType() == ObjType.project && item.getParentCode() == null) {
				writer.println(toFuncidSql(item, "id", "ProjID", "编码"));
				writer.println(toFuncidSql(item, "name", "ProjName", "名称"));
				writer.println(toFuncidSql(item, "localId", "ProjLocalID", "本地编码"));
				writer.println(toFuncidSql(item, "localName", "ProjLocalName", "本地名称"));
			} else if (item.getObjType() == ObjType.building && item.getParentCode() == null) {
				writer.println(toFuncidSql(item, "id", "BuildID", "编码"));
				writer.println(toFuncidSql(item, "name", "BuildName", "名称"));
				writer.println(toFuncidSql(item, "localId", "BuildLocalID", "本地编码"));
				writer.println(toFuncidSql(item, "localName", "BuildLocalName", "本地名称"));
			} else if (item.getObjType() == ObjType.floor && item.getParentCode() == null) {
				writer.println(toFuncidSql(item, "id", "FloorID", "编码"));
				writer.println(toFuncidSql(item, "name", "FloorName", "名称"));
				writer.println(toFuncidSql(item, "localId", "FloorLocalID", "本地编码"));
				writer.println(toFuncidSql(item, "localName", "FloorLocalName", "本地名称"));
			} else if (item.getObjType() == ObjType.space && item.getParentCode() == null) {
				writer.println(toFuncidSql(item, "id", "RoomID", "编码"));
				writer.println(toFuncidSql(item, "name", "RoomName", "名称"));
				writer.println(toFuncidSql(item, "localId", "RoomLocalID", "本地编码"));
				writer.println(toFuncidSql(item, "localName", "RoomLocalName", "本地名称"));
			} else if (item.getObjType() == ObjType.system && item.getParentCode() == null) {
				writer.println(toFuncidSql(item, "id", "SysID", "编码"));
				writer.println(toFuncidSql(item, "name", "SysName", "名称"));
				writer.println(toFuncidSql(item, "localId", "SysLocalID", "本地编码"));
				writer.println(toFuncidSql(item, "localName", "SysLocalName", "本地名称"));
			} else if (item.getObjType() == ObjType.equipment && item.getParentCode() == null) {
				writer.println(toFuncidSql(item, "id", "EquipID", "编码"));
				writer.println(toFuncidSql(item, "name", "EquipName", "名称"));
				writer.println(toFuncidSql(item, "localId", "EquipLocalID", "本地编码"));
				writer.println(toFuncidSql(item, "localName", "EquipLocalName", "本地名称"));
			} else if (item.getObjType() == ObjType.component && item.getParentCode() == null) {
				writer.println(toFuncidSql(item, "id", "EquipID", "编码"));
				writer.println(toFuncidSql(item, "name", "EquipName", "名称"));
				writer.println(toFuncidSql(item, "localId", "EquipLocalID", "本地编码"));
				writer.println(toFuncidSql(item, "localName", "EquipLocalName", "本地名称"));
			} else if (item.getObjType() == ObjType.shaft && item.getParentCode() == null) {
				writer.println(toFuncidSql(item, "id", "ShaftID", "编码"));
				writer.println(toFuncidSql(item, "name", "ShaftName", "名称"));
				writer.println(toFuncidSql(item, "localId", "ShaftLocalID", "本地编码"));
				writer.println(toFuncidSql(item, "localName", "ShaftLocalName", "本地名称"));
			} else if ("VOEm".equals(item.getCode())) {
				writer.println(toFuncidSql(item, "id", "ECModelID", "编码"));
				writer.println(toFuncidSql(item, "name", "ECModelName", "名称"));
				writer.println(toFuncidSql(item, "localId", "ECModelLocalID", "本地编码"));
				writer.println(toFuncidSql(item, "localName", "ECModelLocalName", "本地名称"));
			} else if ("VOSe".equals(item.getCode())) {
				writer.println(toFuncidSql(item, "id", "SubentryID", "编码"));
				writer.println(toFuncidSql(item, "name", "SubentryName", "名称"));
				writer.println(toFuncidSql(item, "localId", "SubentryLocalID", "本地编码"));
				writer.println(toFuncidSql(item, "localName", "SubentryLocalName", "本地名称"));
			} else if ("VOBr".equals(item.getCode())) {
				writer.println(toFuncidSql(item, "id", "BranchID", "编码"));
				writer.println(toFuncidSql(item, "name", "BranchName", "名称"));
				writer.println(toFuncidSql(item, "localId", "BranchLocalID", "本地编码"));
				writer.println(toFuncidSql(item, "localName", "BranchLocalName", "本地名称"));
			} else if ("VOBc".equals(item.getCode())) {
				writer.println(toFuncidSql(item, "id", "BeaconID", "编码"));
				writer.println(toFuncidSql(item, "name", "BeaconlName", "名称"));
				writer.println(toFuncidSql(item, "localId", "BeaconLocalID", "本地编码"));
				writer.println(toFuncidSql(item, "localName", "BeaconLocalName", "本地名称"));
			} else if ("VOTn".equals(item.getCode())) {
				writer.println(toFuncidSql(item, "id", "TenantID", "编码"));
				writer.println(toFuncidSql(item, "name", "TenantName", "名称"));
				writer.println(toFuncidSql(item, "localId", "TenantLocalID", "本地编码"));
				writer.println(toFuncidSql(item, "localName", "TenantLocalName", "本地名称"));
			} else if (item.getObjType() == ObjType.tool && item.getParentCode() == null) {
				writer.println(toFuncidSql(item, "id", "ToolID", "编码"));
				writer.println(toFuncidSql(item, "name", "ToolName", "名称"));
				writer.println(toFuncidSql(item, "localId", "ToolLocalID", "本地编码"));
				writer.println(toFuncidSql(item, "localName", "ToolLocalName", "本地名称"));
			} else if (item.getObjType() == ObjType.material && item.getParentCode() == null) {
				writer.println(toFuncidSql(item, "id", "ConsumID", "编码"));
				writer.println(toFuncidSql(item, "name", "ConsumName", "名称"));
				writer.println(toFuncidSql(item, "localId", "ConsumLocalID", "本地编码"));
				writer.println(toFuncidSql(item, "localName", "ConsumLocalName", "本地名称"));
			}
		}

//		classes.stream().filter(item -> item.getObjType() == ObjType.project).forEach(item -> {
//			writer.println(toFuncidSql(item, "id", "ProjID", "编码"));
//			writer.println(toFuncidSql(item, "name", "ProjName", "名称"));
//			writer.println(toFuncidSql(item, "localId", "ProjLocalID", "本地编码"));
//			writer.println(toFuncidSql(item, "localName", "ProjLocalName", "本地名称"));
//		});
//		classes.stream().filter(item -> item.getObjType().name().equals("building")).forEach(item -> {
//			writer.println(toFuncidSql(item, "id", "BuildID", "编码"));
//			writer.println(toFuncidSql(item, "name", "BuildName", "名称"));
//			writer.println(toFuncidSql(item, "localId", "BuildLocalID", "本地编码"));
//			writer.println(toFuncidSql(item, "localName", "BuildLocalName", "本地名称"));
//		});

		writer.close();

		for (ClassDefModel type : classes) {
			System.out.println(JacksonMapper.toSimpleJson(type));
		}
	}

	private String toSql(ClassDefModel model) {
		StringBuilder sql = new StringBuilder("insert into rwd_def_class( id, code, obj_type, name, alias_code, alias_name, major_code, system_code, equipment_code, parent_code) values ");
		sql.append("(");
		sql.append("'").append(UUID.randomUUID().toString().replace("-", "")).append("', "); // id
		sql.append("'").append(model.getCode()).append("', "); // code
		sql.append("'").append(model.getObjType().name()).append("', "); // obj_type
		sql.append("'").append(model.getName()).append("', "); // name
		sql.append("'").append(model.getCode()).append("', "); // alias_code
		sql.append("'").append(model.getName()).append("', "); // alias_name

		if (model.getMajorCode() != null) { // major_code
			sql.append("'").append(model.getMajorCode()).append("', ");
		} else {
			sql.append("null, ");
		}

		if (model.getSystemCode() != null) { // system_code
			sql.append("'").append(model.getSystemCode()).append("', ");
		} else {
			sql.append("null, ");
		}

		if (model.getEquipmentCode() != null) { // equipment_code
			sql.append("'").append(model.getEquipmentCode()).append("', ");
		} else {
			sql.append("null, ");
		}

		if (model.getParentCode() != null) { // parent_code
			sql.append("'").append(model.getParentCode()).append("', ");
		} else {
			sql.append("null, ");
		}

		// 去掉最后的分隔符 ", "
		sql.deleteCharAt(sql.length() - 1);
		sql.deleteCharAt(sql.length() - 1);
		sql.append(");");
		return sql.toString();
	}

	private String toFuncidSql(ClassDefModel model, String code, String origCode, String name) {
		StringBuilder buff = new StringBuilder("");
		buff.append("insert into rwd_def_funcid ");
		buff.append("(id,code,orig_code,name,alias_code,alias_name,class_code,type,group_code,project_id,category,");
		buff.append("first_tag,second_tag,priority,input_mode,unit,data_type,orig_data_type,is_multiple,is_region,formater,data_source,orig_data_source,note,sub_flag,weak_point) \n");
		buff.append("values (");
		buff.append("'").append(UUID.randomUUID().toString().replace("-", "")).append("',"); // id
		buff.append("'").append(code).append("',"); //code
		buff.append("'").append(origCode).append("',"); //orig_code
		buff.append("'").append(name).append("',"); //name
		buff.append("'").append(origCode).append("',"); //alias_code
		buff.append("'").append(name).append("',"); //alias_name
		buff.append("'").append(model.getCode()).append("',"); // class_code
		buff.append("'common',");// type
		buff.append("'0',");//
		buff.append("'0',");//
		buff.append("'STATIC',");//
		buff.append("null,");// first_tag
		buff.append("null,");
		buff.append("null,");
		// input_mode
		if ("id".equals(code) || "name".equals(code)) {
			buff.append("'X',");
		} else if ("localId".equals(code) || "localName".equals(code)) {
			buff.append("'B1',");
		} else {
			buff.append("null,");
		}

		buff.append("null,");
		buff.append("'STRING',");
		buff.append("'Str',");
		buff.append("0,");
		buff.append("0,");
		buff.append("null,");// formater
		buff.append("null,");
		buff.append("null,");
		buff.append("null,");
		buff.append("0,");
		buff.append("0);");
		return buff.toString();
	}

	private void initBaseClass(List<ClassDefModel> classes) {
		classes.add(prepareModel("project", "project", "项目", null, null, null));
		classes.add(prepareModel("building", "building", "建筑", null, null, null));
		classes.add(prepareModel("floor", "floor", "楼层", null, null, null));
		classes.add(prepareModel("space", "space", "空间", null, null, null));
		classes.add(prepareModel("system", "system", "系统", null, null, null));
		classes.add(prepareModel("equipment", "equipment", "设备", null, null, null));
		classes.add(prepareModel("component", "component", "部件", null, null, null));
		classes.add(prepareModel("shaft", "shaft", "竖井", null, null, null));
		classes.add(prepareModel("virtual", "virtual", "虚拟对象", null, null, null));
		classes.add(prepareModel("tool", "tool", "工具", null, null, null));
		classes.add(prepareModel("material", "material", "耗材", null, null, null));
	}

	private void initSpace(List<ClassDefModel> classes) {
		classes.add(prepareModel("space", "GeneralZone", "物业分区", "space", null, null));
		classes.add(prepareModel("space", "PowerSupplyZone", "供电分区", "space", null, null));
		classes.add(prepareModel("space", "LightingZone", "照明分区", "space", null, null));
		classes.add(prepareModel("space", "NetworkZone", "网络分区", "space", null, null));
		classes.add(prepareModel("space", "AirConditioningZone", "空调分区", "space", null, null));
		classes.add(prepareModel("space", "HeatingZone", "采暖分区", "space", null, null));
		classes.add(prepareModel("space", "CleanZone", "洁净分区", "space", null, null));
		classes.add(prepareModel("space", "DomesticWaterSupplyZone", "生活给水分区", "space", null, null));
		classes.add(prepareModel("space", "FireZone", "防火分区", "space", null, null));
		classes.add(prepareModel("space", "SecurityZone", "安防分区", "space", null, null));
		classes.add(prepareModel("space", "TenantZone", "租户分区", "space", null, null));
		classes.add(prepareModel("space", "FunctionZone", "功能分区", "space", null, null));
		classes.add(prepareModel("space", "OtherZone", "其他分区", "space", null, null));
	}

	private void initEquipment(List<ClassDefModel> classes) throws IOException {
		String classFile = basepath + "/2.2-设备设施类对象/2.2.1-设备设施类对象分类体系.xlsx";
		String sheetName = "设备（部件）类";

		FileInputStream inputStream = new FileInputStream(classFile);
		Workbook workbook = new XSSFWorkbook(inputStream);
		Sheet sheet = workbook.getSheet(sheetName);
		if (sheet == null) {
			return;
		}

		// 行号从0开始, 从第2行开始读数据
		Set<String> systemCodes = new HashSet<>();
		Set<String> equipmentCodes = new HashSet<>();
		int startRowNum = 2;
		int lastRowNum = sheet.getLastRowNum();
		CacheModel cache = new CacheModel();
		for (int rowIndex = startRowNum; rowIndex <= lastRowNum; rowIndex++) {
			Row row = sheet.getRow(rowIndex);
			if (row == null) {
				log.info("row[{}] is null", rowIndex);
				continue;
			}

			// 0专业, 3系统编码, 5系统名称, 6设备编码, 8设备名称
			CacheModel target = new CacheModel();
			target.majorCode = (String) ExcelUtils.parseCell(row.getCell(0), "string");// 专业编码
			target.systemCode = (String) ExcelUtils.parseCell(row.getCell(3), "string");//系统类编码
			target.systemName = (String) ExcelUtils.parseCell(row.getCell(5), "string");//系统类名称
			target.equipCode = (String) ExcelUtils.parseCell(row.getCell(6), "string");//设备设施类编码
			target.equipName = (String) ExcelUtils.parseCell(row.getCell(8), "string");//设备设施类名称
			target.componentCode = (String) ExcelUtils.parseCell(row.getCell(9), "string");//部件类编码
			target.componentName = (String) ExcelUtils.parseCell(row.getCell(11), "string");//部件类名称
//			System.out.println(rowIndex + ": " + target.majorCode + "\t" + target.systemCode + "\t" + target.equipCode + "\t" + target.componentCode);

			// 专业不为空
			if (target.majorCode != null) {
				cache.majorCode = target.majorCode;
				cache.systemCode = null;
				cache.systemName = null;
				cache.equipCode = null;
				cache.equipName = null;
				cache.componentCode = null;
				cache.componentName = null;
			} else {
				target.majorCode = cache.majorCode;
			}
			// 系统不为空
			if (target.systemCode != null) {
				cache.systemCode = target.systemCode;
				cache.systemName = target.systemName;
				cache.equipCode = null;
				cache.equipName = null;
				cache.componentCode = null;
				cache.componentName = null;
			} else {
				target.systemCode = cache.systemCode;
				target.systemName = cache.systemName;
			}
			// 设备不为空
			if (target.equipCode != null) {
				cache.equipCode = target.equipCode;
				cache.equipName = target.equipName;
				cache.componentCode = null;
				cache.componentName = null;
			} else {
				target.equipCode = cache.equipCode;
				target.equipName = cache.equipName;
			}
			// 部件不为空
			if (target.componentCode != null) {
			}

			// 拼装实际的系统编码和设备编码
			CacheModel object = new CacheModel();
			object.majorCode = target.majorCode;
			object.systemCode = target.majorCode + target.systemCode;
			object.systemName = target.systemName;
			object.equipCode = target.majorCode + target.equipCode;
			object.equipName = target.equipName;
			object.componentCode = target.componentCode;
			object.componentName = target.componentName;

			// 判断系统是否存在
			if (!systemCodes.contains(object.systemCode)) {
				ClassDefModel system = new ClassDefModel();
				system.setMajorCode(object.majorCode);
				system.setCode(object.systemCode);
				system.setName(object.systemName);
				system.setParentCode("system");
				system.setObjType(ObjType.system);
				classes.add(system);
				systemCodes.add(object.systemCode);
			}
			// 判断设备是否存在
			if (!equipmentCodes.contains(object.equipCode)) {
				ClassDefModel equipment = new ClassDefModel();
				equipment.setMajorCode(object.majorCode);
				equipment.setSystemCode(object.systemCode);
				equipment.setCode(object.equipCode);
				equipment.setName(object.equipName);
				equipment.setParentCode("equipment");
				equipment.setObjType(ObjType.equipment);
				classes.add(equipment);
				equipmentCodes.add(object.equipCode);
			}
			// 部件不为空
			if (object.componentCode != null) {
				ClassDefModel component = new ClassDefModel();
				component.setMajorCode(object.majorCode);
				component.setSystemCode(object.systemCode);
				component.setEquipmentCode(object.equipCode);
				component.setCode(object.majorCode + object.componentCode);
				component.setName(object.componentName);
				component.setParentCode("component");
				component.setObjType(ObjType.component);
				classes.add(component);
			}
		}
	}

	private void initVirtual(List<ClassDefModel> classes) {
		classes.add(prepareModel("virtual", "VOEm", "能耗模型", "virtual", null, null));
		classes.add(prepareModel("virtual", "VOSe", "分项", "virtual", null, null));
		classes.add(prepareModel("virtual", "VOBr", "支路", "virtual", null, null));
		classes.add(prepareModel("virtual", "VOBc", "信标", "virtual", null, null));
		classes.add(prepareModel("virtual", "VOTn", "租户", "virtual", null, null));
		classes.add(prepareModel("virtual", "VOWs", "工作历", "virtual", null, null));
		classes.add(prepareModel("virtual", "VOZn", "空间组合", "virtual", null, null));
		classes.add(prepareModel("virtual", "VOGp", "设备设施组合", "virtual", null, null));
	}

	private void initTool(List<ClassDefModel> classes) throws IOException {
		String classFile = basepath + "/2.3-耗材工具类对象/2.3.1-耗材工具类对象分类体系.xlsx";
		String sheetName = "工具";

		FileInputStream inputStream = new FileInputStream(classFile);
		Workbook workbook = new XSSFWorkbook(inputStream);
		Sheet sheet = workbook.getSheet(sheetName);
		if (sheet == null) {
			return;
		}

		// 行号从0开始, 从第2行开始读数据
		// 一级分类使用systemCode, 二级分类使用equipCode
		int startRowNum = 1;
		int lastRowNum = sheet.getLastRowNum();
		CacheModel cache = new CacheModel();
		for (int rowIndex = startRowNum; rowIndex <= lastRowNum; rowIndex++) {
			Row row = sheet.getRow(rowIndex);
			if (row == null) {
				log.info("row[{}] is null", rowIndex);
				continue;
			}
			// 0专业, 3系统编码, 5系统名称, 6设备编码, 8设备名称
			CacheModel target = new CacheModel();
			target.systemCode = (String) ExcelUtils.parseCell(row.getCell(0), "string");// 一级分类编码
			target.equipCode = (String) ExcelUtils.parseCell(row.getCell(3), "string");//二级分类编码
			target.componentCode = (String) ExcelUtils.parseCell(row.getCell(6), "string");//工具类编码
			target.componentName = (String) ExcelUtils.parseCell(row.getCell(8), "string");//工具类名称
//			System.out.println(rowIndex + ": " + target.majorCode + "\t" + target.systemCode + "\t" + target.equipCode + "\t" + target.componentCode);

			// 一级分类编码
			if (target.systemCode != null) {
				cache.systemCode = target.systemCode;
				cache.equipCode = null;
				cache.componentCode = null;
				cache.componentName = null;
			} else {
				target.systemCode = cache.systemCode;
			}
			// 二级分类编码
			if (target.equipCode != null) {
				cache.equipCode = target.equipCode;
				cache.componentCode = null;
				cache.componentName = null;
			} else {
				target.equipCode = cache.equipCode;
			}

			// 拼装实际的系统编码和设备编码
			CacheModel object = new CacheModel();
			object.systemCode = target.systemCode;
			object.equipCode = target.systemCode + target.equipCode;
			object.componentCode = target.componentCode;
			object.componentName = target.componentName;

			// 工具类编码
			if (object.componentCode != null) {
				ClassDefModel component = new ClassDefModel();
				component.setSystemCode(object.systemCode);
				component.setEquipmentCode(object.equipCode);
				component.setCode(object.componentCode);
				component.setName(object.componentName);
				component.setParentCode("tool");
				component.setObjType(ObjType.tool);
				classes.add(component);
			}
		}
	}

	private void initMaterial(List<ClassDefModel> classes) throws IOException {
		String classFile = basepath + "/2.3-耗材工具类对象/2.3.1-耗材工具类对象分类体系.xlsx";
		String sheetName = "耗材 ";

		FileInputStream inputStream = new FileInputStream(classFile);
		Workbook workbook = new XSSFWorkbook(inputStream);
		Sheet sheet = workbook.getSheet(sheetName);
		if (sheet == null) {
			return;
		}

		// 行号从0开始, 从第2行开始读数据
		// 一级分类使用systemCode, 二级分类使用equipCode
		int startRowNum = 1;
		int lastRowNum = sheet.getLastRowNum();
		CacheModel cache = new CacheModel();
		for (int rowIndex = startRowNum; rowIndex <= lastRowNum; rowIndex++) {
			Row row = sheet.getRow(rowIndex);
			if (row == null) {
				log.info("row[{}] is null", rowIndex);
				continue;
			}
			// 0专业, 3系统编码, 5系统名称, 6设备编码, 8设备名称
			CacheModel target = new CacheModel();
			target.systemCode = (String) ExcelUtils.parseCell(row.getCell(0), "string");// 一级分类编码
			target.equipCode = (String) ExcelUtils.parseCell(row.getCell(3), "string");//二级分类编码
			target.componentCode = (String) ExcelUtils.parseCell(row.getCell(6), "string");//耗材类编码
			target.componentName = (String) ExcelUtils.parseCell(row.getCell(8), "string");//耗材类名称
//			System.out.println(rowIndex + ": " + target.majorCode + "\t" + target.systemCode + "\t" + target.equipCode + "\t" + target.componentCode);

			// 一级分类编码
			if (target.systemCode != null) {
				cache.systemCode = target.systemCode;
				cache.equipCode = null;
				cache.componentCode = null;
				cache.componentName = null;
			} else {
				target.systemCode = cache.systemCode;
			}
			// 二级分类编码
			if (target.equipCode != null) {
				cache.equipCode = target.equipCode;
				cache.componentCode = null;
				cache.componentName = null;
			} else {
				target.equipCode = cache.equipCode;
			}

			// 拼装实际的系统编码和设备编码
			CacheModel object = new CacheModel();
			object.systemCode = target.systemCode;
			object.equipCode = target.systemCode + target.equipCode;
			object.componentCode = target.componentCode;
			object.componentName = target.componentName;

			// 耗材类编码
			if (object.componentCode != null) {
				ClassDefModel component = new ClassDefModel();
				component.setSystemCode(object.systemCode);
				component.setEquipmentCode(object.equipCode);
				component.setCode(object.componentCode);
				component.setName(object.componentName);
				component.setParentCode("material");
				component.setObjType(ObjType.material);
				classes.add(component);
			}
		}
	}

	public static final class CacheModel {
		String majorCode;
		String systemCode;
		String systemName;
		String equipCode;
		String equipName;
		String componentCode;
		String componentName;
	}

	private ClassDefModel prepareModel(String objType, String code, String name, String parentCode, String majorCode, String systemCode) {
		ClassDefModel model = new ClassDefModel();
		model.setObjType(ObjType.valueOf(objType));
		model.setCode(code);
		model.setName(name);
		model.setAliasCode(code);
		model.setAliasName(name);
		model.setParentCode(parentCode);
		model.setMajorCode(majorCode);
		model.setSystemCode(systemCode);
		return model;
	}

}
