package com.persagy.dmp.rwd.dic;

import com.persagy.common.date.DateUtils;
import com.persagy.common.json.JacksonMapper;
import com.persagy.dmp.rwd.enums.FuncidDataType;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.junit.Test;

import java.io.*;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.UUID;

/**
 * @author: yaoll
 * @date: 2020-11-30
 * @verison: 1.0
 */
@Slf4j
public class ImportFuncid {

	private String basepath = "/Users/devsaga/persagy/数据字典20201118/Ch2-物理世界";

	private String outPath = "/Users/devsaga/persagy/数据字典20201118/funcid-" + DateUtils.format(new Date()) + ".dml.sql";

	@Test
	public void importFuncidDef() throws IOException {
		List<FuncidDefModel> funcids = new LinkedList<>();
		importSpace(funcids);
		importSystem(funcids);
		importEquipment(funcids);
		importComponent(funcids);
		importVirtual(funcids);
		importTool(funcids);
		importMaterial(funcids);

		FileOutputStream fos = new FileOutputStream(outPath);
		PrintWriter writer = new PrintWriter(new BufferedOutputStream(fos));
		writer.println("delete from rwd_def_funcid where code not in ('id', 'name', 'localId', 'localName');");
		writer.println("commit;");
		funcids.forEach(item -> {
//			System.out.println(JacksonMapper.toSimpleJson(item));
			writer.println(toSql(item));
		});
		writer.println("commit;");
		writer.flush();
		writer.close();
	}

	private String toSql(FuncidDefModel model) {
		StringBuilder buff = new StringBuilder("");
		buff.append("insert into rwd_def_funcid ");
		buff.append("(id,code,orig_code,name,alias_code,alias_name,class_code,type,group_code,project_id,category,");
		buff.append("first_tag,second_tag,priority,input_mode,unit,data_type,orig_data_type,is_multiple,is_region,formater,data_source,orig_data_source,note,sub_flag,weak_point) \n");
		buff.append("values (");
		buff.append("'").append(UUID.randomUUID().toString().replace("-", "")).append("',"); // id
		buff.append("'").append(model.getCode()).append("',"); // code
		buff.append("'").append(model.getOrigCode()).append("',"); // orig_code
		buff.append("'").append(model.getName()).append("',"); // name
		buff.append("'").append(model.getAliasCode()).append("',"); // alias_code
		buff.append("'").append(model.getAliasName()).append("',"); // alias_name
		buff.append("'").append(model.getClassCode()).append("',"); // class_code
		buff.append("'").append(model.getType()).append("',"); // type
		buff.append("'").append(model.getGroupCode()).append("',"); // group_code
		buff.append("'").append(model.getProjectId()).append("',"); // project_id
		buff.append("'").append(model.getCategory().name()).append("',"); // category
		if (model.getFirstTag() != null) {
			buff.append("'").append(model.getFirstTag()).append("',"); // first_tag
		} else {
			buff.append("null,");
		}
		if (model.getSecondTag() != null) {
			buff.append("'").append(model.getSecondTag()).append("',"); // second_tag
		} else {
			buff.append("null,");
		}
		if (model.getPriority() != null) {
			buff.append("'").append(model.getPriority()).append("',"); // priority
		} else {
			buff.append("null,");
		}
		if (model.getInputMode() != null) {
			buff.append("'").append(model.getInputMode()).append("',"); // input_mode
		} else {
			buff.append("null,");
		}
		if (model.getUnit() != null) {
			buff.append("'").append(model.getUnit()).append("',"); // unit
		} else {
			buff.append("null,");
		}

		buff.append("'").append(model.getDataType().name()).append("',"); // data_type
		buff.append("'").append(model.getOrigDataType()).append("',"); // orig_data_type
		buff.append(model.getIsMultiple() ? 1 : 0).append(","); // is_multiple
		buff.append(model.getIsRegion() ? 1 : 0).append(","); // is_region
		if (model.getFormater() != null) {
			buff.append("'").append(model.getFormater()).append("',"); // formater
		} else {
			buff.append("null,");
		}
		if (model.getDataSource() != null) {
			buff.append("'").append(model.getDataSource().toString()).append("',"); // data_source
		} else {
			buff.append("null,");
		}
		if (model.getOrigDataSource() != null) { // orig_data_source
			buff.append("'").append(model.getOrigDataSource()).append("',"); // data_source
		} else {
			buff.append("null,");
		}
		if (model.getNote() != null) {
			buff.append("'").append(model.getNote()).append("',"); // note}
		} else {
			buff.append("null,");
		}
		buff.append(model.getSubFlag() ? 1 : 0).append(","); // sub_flag
		buff.append(model.getWeakPoint() ? 1 : 0).append(","); // weak_point
		buff.deleteCharAt(buff.length() - 1);
		buff.append(");");
		return buff.toString();
	}

	private FuncidDefModel toFuncidDefModel(CacheModel model) {
		FuncidDefModel funcid = new FuncidDefModel();
		funcid.setGroupCode("0");
		funcid.setProjectId("0");
		funcid.setType("common");
		funcid.setClassCode(model.classCode);
		funcid.setFirstTag(model.firstTag != null ? model.firstTag.replace("\n", "") : null);
		funcid.setSecondTag(model.secondTag != null ? model.secondTag.replace("\n", "") : null);
		funcid.setCode(model.origCode.substring(0, 1).toLowerCase() + model.origCode.substring(1));
		funcid.setOrigCode(model.origCode);
		funcid.setAliasCode(model.origCode);
		if (model.name != null) {
			funcid.setName(model.name.replace("\n", ""));
			funcid.setAliasName(model.name.replace("\n", ""));
		} else {
			log.error("funcid name is null : {}", JacksonMapper.toSimpleJson(funcid));
		}
		funcid.setUnit(model.unit);
		funcid.setPriority(model.priority);
		funcid.setInputMode(model.inputMode);
		funcid.setOrigDataType(model.origDataType);
		funcid.setSubFlag(false);
		funcid.setWeakPoint(false);
		FuncidDefModel dataType = DataTypeMapping.getDataType(model.origDataType);
		if (dataType != null) {
			funcid.setCategory(dataType.getCategory());
			funcid.setDataType(dataType.getDataType());
			funcid.setIsMultiple(dataType.getIsMultiple());
			funcid.setIsRegion(dataType.getIsRegion());
		} else {
			return null;
		}
		funcid.setSubFlag(false);
		if (model.remark != null) {
			funcid.setNote(model.remark.replace("\n", ""));
		}

		if (model.origDataSource != null) {
			funcid.setOrigDataSource(model.origDataSource.replace("\n", "nn")); // 去掉换行符
		}
		if (funcid.getDataType() == FuncidDataType.ENUM || funcid.getDataType() == FuncidDataType.MENUM || funcid.getDataType() == FuncidDataType.BOOLEAN) {
			funcid.setDataSource(FuncidUtils.transEnumDataSource(model.origDataSource)); // 解析datasource
		}
		return funcid;
	}

	private void importSpace(List<FuncidDefModel> funcids) throws IOException {
		funcids.addAll(readSheet("project", "/2.1-建筑空间类对象/2.1.1-项目.xlsx", "项目"));
		funcids.addAll(readSheet("building", "/2.1-建筑空间类对象/2.1.2-建筑.xlsx", "建筑"));
		funcids.addAll(readSheet("floor", "/2.1-建筑空间类对象/2.1.3-楼层.xlsx", "楼层"));
		funcids.addAll(readSheet("shaft", "/2.1-建筑空间类对象/2.1.4-竖井.xlsx", "竖井"));

		funcids.addAll(readSheet("GeneralZone", "/2.1-建筑空间类对象/2.1.5-空间.xlsx", "CMZ物业分区"));
		funcids.addAll(readSheet("PowerSupplyZone", "/2.1-建筑空间类对象/2.1.5-空间.xlsx", "PSZ供电分区"));
		funcids.addAll(readSheet("LightingZone", "/2.1-建筑空间类对象/2.1.5-空间.xlsx", "LTZ照明分区"));
		funcids.addAll(readSheet("NetworkZone", "/2.1-建筑空间类对象/2.1.5-空间.xlsx", "NWZ网络分区"));
		funcids.addAll(readSheet("AirConditioningZone", "/2.1-建筑空间类对象/2.1.5-空间.xlsx", "ACZ空调分区"));
		funcids.addAll(readSheet("HeatingZone", "/2.1-建筑空间类对象/2.1.5-空间.xlsx", "HTZ采暖分区"));
		funcids.addAll(readSheet("CleanZone", "/2.1-建筑空间类对象/2.1.5-空间.xlsx", "CLZ洁净分区"));
		funcids.addAll(readSheet("DomesticWaterSupplyZone", "/2.1-建筑空间类对象/2.1.5-空间.xlsx", "DWZ生活给水分区"));
		funcids.addAll(readSheet("FireZone", "/2.1-建筑空间类对象/2.1.5-空间.xlsx", "FRZ防火分区"));
		funcids.addAll(readSheet("SecurityZone", "/2.1-建筑空间类对象/2.1.5-空间.xlsx", "SCZ安防分区"));
		funcids.addAll(readSheet("TenantZone", "/2.1-建筑空间类对象/2.1.5-空间.xlsx", "TNZ租户分区"));
		funcids.addAll(readSheet("FunctionZone", "/2.1-建筑空间类对象/2.1.5-空间.xlsx", "FCZ功能分区"));
		funcids.addAll(readSheet("OtherZone", "/2.1-建筑空间类对象/2.1.5-空间.xlsx", "OTZ其他分区"));
	}

	private void importSystem(List<FuncidDefModel> funcids) throws IOException {
		funcids.addAll(readSheet("system", "/2.2-设备设施类对象/2.2.2-设备设施类对象通用属性.xlsx", "系统"));

		funcids.addAll(readSheet("SETD", "/2.2-设备设施类对象/2.2.3-系统/10-强电专业系统.xlsx", "TD变配电系统 "));
		funcids.addAll(readSheet("SEDG", "/2.2-设备设施类对象/2.2.3-系统/10-强电专业系统.xlsx", "DG柴发机房系统"));
		funcids.addAll(readSheet("SEBP", "/2.2-设备设施类对象/2.2.3-系统/10-强电专业系统.xlsx", "BP备用电源系统"));
		funcids.addAll(readSheet("SEEP", "/2.2-设备设施类对象/2.2.3-系统/10-强电专业系统.xlsx", "EP机房动力系统"));
		funcids.addAll(readSheet("SEOP", "/2.2-设备设施类对象/2.2.3-系统/10-强电专业系统.xlsx", "OP室外动力系统"));
		funcids.addAll(readSheet("SEEL", "/2.2-设备设施类对象/2.2.3-系统/10-强电专业系统.xlsx", "EL电梯系统"));
		funcids.addAll(readSheet("SELT", "/2.2-设备设施类对象/2.2.3-系统/10-强电专业系统.xlsx", "LT照明系统"));
		funcids.addAll(readSheet("SETP", "/2.2-设备设施类对象/2.2.3-系统/10-强电专业系统.xlsx", "TP租户配电系统"));
		funcids.addAll(readSheet("SELP", "/2.2-设备设施类对象/2.2.3-系统/10-强电专业系统.xlsx", "LP雷电防护系统"));
		funcids.addAll(readSheet("SESD", "/2.2-设备设施类对象/2.2.3-系统/10-强电专业系统.xlsx", "SD强电人防系统"));

		funcids.addAll(readSheet("WEBA", "/2.2-设备设施类对象/2.2.3-系统/20-弱电专业系统.xlsx", "BA建筑设备控制系统"));
		funcids.addAll(readSheet("WETV", "/2.2-设备设施类对象/2.2.3-系统/20-弱电专业系统.xlsx", "TV电视信号系统"));
		funcids.addAll(readSheet("WEMI", "/2.2-设备设施类对象/2.2.3-系统/20-弱电专业系统.xlsx", "MI多媒体信息发布系统  "));
		funcids.addAll(readSheet("WEWT", "/2.2-设备设施类对象/2.2.3-系统/20-弱电专业系统.xlsx", "WT无线对讲系统   "));
		funcids.addAll(readSheet("WEGM", "/2.2-设备设施类对象/2.2.3-系统/20-弱电专业系统.xlsx", "GM车库管理系统"));
		funcids.addAll(readSheet("WELN", "/2.2-设备设施类对象/2.2.3-系统/20-弱电专业系统.xlsx", "LN本地网络系统"));
		funcids.addAll(readSheet("WEMA", "/2.2-设备设施类对象/2.2.3-系统/20-弱电专业系统.xlsx", "MA移动通信信号放大系统"));
		funcids.addAll(readSheet("WETM", "/2.2-设备设施类对象/2.2.3-系统/20-弱电专业系统.xlsx", "TM租户管理系统"));
		funcids.addAll(readSheet("WEPM", "/2.2-设备设施类对象/2.2.3-系统/20-弱电专业系统.xlsx", "PM客流管理系统"));
		funcids.addAll(readSheet("WEEM", "/2.2-设备设施类对象/2.2.3-系统/20-弱电专业系统.xlsx", "EM能源管理系统"));
		funcids.addAll(readSheet("WEMM", "/2.2-设备设施类对象/2.2.3-系统/20-弱电专业系统.xlsx", "MM气象监测系统"));
		funcids.addAll(readSheet("WECM", "/2.2-设备设施类对象/2.2.3-系统/20-弱电专业系统.xlsx", "CM收银管理系统"));
		funcids.addAll(readSheet("WEOF", "/2.2-设备设施类对象/2.2.3-系统/20-弱电专业系统.xlsx", "OF办公系统"));

		funcids.addAll(readSheet("ACCC", "/2.2-设备设施类对象/2.2.3-系统/30-空调专业系统.xlsx", "CC中央供冷系统 "));
		funcids.addAll(readSheet("ACCH", "/2.2-设备设施类对象/2.2.3-系统/30-空调专业系统.xlsx", "CH中央供热系统"));
		funcids.addAll(readSheet("ACAT", "/2.2-设备设施类对象/2.2.3-系统/30-空调专业系统.xlsx", "AT空调末端系统"));
		funcids.addAll(readSheet("ACVT", "/2.2-设备设施类对象/2.2.3-系统/30-空调专业系统.xlsx", "VT通风系统"));
		funcids.addAll(readSheet("ACKL", "/2.2-设备设施类对象/2.2.3-系统/30-空调专业系统.xlsx", "KL厨房排油烟系统"));
		funcids.addAll(readSheet("ACAD", "/2.2-设备设施类对象/2.2.3-系统/30-空调专业系统.xlsx", "AD空调人防系统"));

		funcids.addAll(readSheet("WSDW", "/2.2-设备设施类对象/2.2.3-系统/40-给排水专业系统.xlsx", "DW生活给水系统"));
		funcids.addAll(readSheet("WSDH", "/2.2-设备设施类对象/2.2.3-系统/40-给排水专业系统.xlsx", "DH生活热水系统"));
		funcids.addAll(readSheet("WSRW", "/2.2-设备设施类对象/2.2.3-系统/40-给排水专业系统.xlsx", "RW中水系统"));
		funcids.addAll(readSheet("WSDD", "/2.2-设备设施类对象/2.2.3-系统/40-给排水专业系统.xlsx", "DD直饮水系统"));
		funcids.addAll(readSheet("WSST", "/2.2-设备设施类对象/2.2.3-系统/40-给排水专业系统.xlsx", "ST污水处理系统"));
		funcids.addAll(readSheet("WSSI", "/2.2-设备设施类对象/2.2.3-系统/40-给排水专业系统.xlsx", "SI喷灌系统"));
		funcids.addAll(readSheet("WSWS", "/2.2-设备设施类对象/2.2.3-系统/40-给排水专业系统.xlsx", "WS水景系统"));
		funcids.addAll(readSheet("WSWP", "/2.2-设备设施类对象/2.2.3-系统/40-给排水专业系统.xlsx", "WP泳池系统"));
		funcids.addAll(readSheet("WSFG", "/2.2-设备设施类对象/2.2.3-系统/40-给排水专业系统.xlsx", "FG燃气系统"));
		funcids.addAll(readSheet("WSWH", "/2.2-设备设施类对象/2.2.3-系统/40-给排水专业系统.xlsx", "WH给排水电伴热系统"));
		funcids.addAll(readSheet("WSWD", "/2.2-设备设施类对象/2.2.3-系统/40-给排水专业系统.xlsx", "WD给排水人防系统"));

		funcids.addAll(readSheet("FFFA", "/2.2-设备设施类对象/2.2.3-系统/50-消防专业系统.xlsx", "FA火灾报警系统"));
		funcids.addAll(readSheet("FFGA", "/2.2-设备设施类对象/2.2.3-系统/50-消防专业系统.xlsx", "GA燃气报警系统"));
		funcids.addAll(readSheet("FFEA", "/2.2-设备设施类对象/2.2.3-系统/50-消防专业系统.xlsx", "EA漏电火灾报警系统"));
		funcids.addAll(readSheet("FFBM", "/2.2-设备设施类对象/2.2.3-系统/50-消防专业系统.xlsx", "BM紧急广播及背景音乐系统"));
		funcids.addAll(readSheet("FFFS", "/2.2-设备设施类对象/2.2.3-系统/50-消防专业系统.xlsx", "FS消防给水系统"));
		funcids.addAll(readSheet("FFGE", "/2.2-设备设施类对象/2.2.3-系统/50-消防专业系统.xlsx", "GE气体灭火系统"));
		funcids.addAll(readSheet("FFFE", "/2.2-设备设施类对象/2.2.3-系统/50-消防专业系统.xlsx", "FE灭火器系统"));
		funcids.addAll(readSheet("FFSC", "/2.2-设备设施类对象/2.2.3-系统/50-消防专业系统.xlsx", "SC防排烟系统"));
		funcids.addAll(readSheet("FFFR", "/2.2-设备设施类对象/2.2.3-系统/50-消防专业系统.xlsx", "FR防火卷帘门系统"));
		funcids.addAll(readSheet("FFEE", "/2.2-设备设施类对象/2.2.3-系统/50-消防专业系统.xlsx", "EE应急照明及疏散指示系统"));
		funcids.addAll(readSheet("FFKE", "/2.2-设备设施类对象/2.2.3-系统/50-消防专业系统.xlsx", "KE厨房自动灭火系统"));

		funcids.addAll(readSheet("SPVS", "/2.2-设备设施类对象/2.2.3-系统/60-安防专业系统.xlsx", "VS视频监控系统"));
		funcids.addAll(readSheet("SPNP", "/2.2-设备设施类对象/2.2.3-系统/60-安防专业系统.xlsx", "NP巡更系统"));
		funcids.addAll(readSheet("SPEG", "/2.2-设备设施类对象/2.2.3-系统/60-安防专业系统.xlsx", "EG门禁系统"));
		funcids.addAll(readSheet("SPIA", "/2.2-设备设施类对象/2.2.3-系统/60-安防专业系统.xlsx", "IA入侵报警系统"));
	}

	private void importEquipment(List<FuncidDefModel> funcids) throws IOException {
		funcids.addAll(readSheet("equipment", "/2.2-设备设施类对象/2.2.2-设备设施类对象通用属性.xlsx", "设备设施"));
		funcids.addAll(readSheet("SEBPBP", "/2.2-设备设施类对象/2.2.4-设备设施/1-强电专业/13-备用电源系统设备.xlsx", "BPBP备用电源"));
		funcids.addAll(readSheet("SEBPSP", "/2.2-设备设施类对象/2.2.4-设备设施/1-强电专业/13-备用电源系统设备.xlsx", "BPSP太阳能电池板"));
		funcids.addAll(readSheet("SEBPWG", "/2.2-设备设施类对象/2.2.4-设备设施/1-强电专业/13-备用电源系统设备.xlsx", "BPWG风力发电装置"));
		funcids.addAll(readSheet("SEBPHT", "/2.2-设备设施类对象/2.2.4-设备设施/1-强电专业/13-备用电源系统设备.xlsx", "BPHT备用电源主机"));
		funcids.addAll(readSheet("SEBPCU", "/2.2-设备设施类对象/2.2.4-设备设施/1-强电专业/13-备用电源系统设备.xlsx", "BPCU备用电源控制箱"));
		funcids.addAll(readSheet("SELPAT", "/2.2-设备设施类对象/2.2.4-设备设施/1-强电专业/19-雷电防护系统设备.xlsx", "LPAT接闪器"));
		funcids.addAll(readSheet("SELPDL", "/2.2-设备设施类对象/2.2.4-设备设施/1-强电专业/19-雷电防护系统设备.xlsx", "LPDL引下线"));
		funcids.addAll(readSheet("SELPGD", "/2.2-设备设施类对象/2.2.4-设备设施/1-强电专业/19-雷电防护系统设备.xlsx", "LPGD接地装置"));
		funcids.addAll(readSheet("SEOPCU", "/2.2-设备设施类对象/2.2.4-设备设施/1-强电专业/15-室外动力系统设备.xlsx", "OPCU室外动力控制箱"));
		funcids.addAll(readSheet("SETPCU", "/2.2-设备设施类对象/2.2.4-设备设施/1-强电专业/18-租户配电系统设备.xlsx", "TPCU租户配电控制箱"));
		funcids.addAll(readSheet("SETPBP", "/2.2-设备设施类对象/2.2.4-设备设施/1-强电专业/18-租户配电系统设备.xlsx", "TPBP租户母线插接箱"));
		funcids.addAll(readSheet("SETDTF", "/2.2-设备设施类对象/2.2.4-设备设施/1-强电专业/11-变配电系统设备.xlsx", "TDTF变压器"));
		funcids.addAll(readSheet("SETDHS", "/2.2-设备设施类对象/2.2.4-设备设施/1-强电专业/11-变配电系统设备.xlsx", "TDHS高压开关柜"));
		funcids.addAll(readSheet("SETDLS", "/2.2-设备设施类对象/2.2.4-设备设施/1-强电专业/11-变配电系统设备.xlsx", "TDLS低压开关柜"));
		funcids.addAll(readSheet("SETDDS", "/2.2-设备设施类对象/2.2.4-设备设施/1-强电专业/11-变配电系统设备.xlsx", "TDDS直流屏"));
		funcids.addAll(readSheet("SETDIV", "/2.2-设备设施类对象/2.2.4-设备设施/1-强电专业/11-变配电系统设备.xlsx", "TDIV逆变器"));
		funcids.addAll(readSheet("SETDBP", "/2.2-设备设施类对象/2.2.4-设备设施/1-强电专业/11-变配电系统设备.xlsx", "TDBP变配电母线插接箱"));
		funcids.addAll(readSheet("SETDCU", "/2.2-设备设施类对象/2.2.4-设备设施/1-强电专业/11-变配电系统设备.xlsx", "TDCU变配电控制箱"));
		funcids.addAll(readSheet("SEEPCU", "/2.2-设备设施类对象/2.2.4-设备设施/1-强电专业/14-机房动力系统设备.xlsx", "EPCU机房动力控制箱"));
		funcids.addAll(readSheet("SEDGDG", "/2.2-设备设施类对象/2.2.4-设备设施/1-强电专业/12-柴发机房系统设备.xlsx", "DGDG柴油发电机"));
		funcids.addAll(readSheet("SEDGDT", "/2.2-设备设施类对象/2.2.4-设备设施/1-强电专业/12-柴发机房系统设备.xlsx", "DGDT柴发储油箱"));
		funcids.addAll(readSheet("SEDGPP", "/2.2-设备设施类对象/2.2.4-设备设施/1-强电专业/12-柴发机房系统设备.xlsx", "DGPP柴发输油泵"));
		funcids.addAll(readSheet("SEDGJH", "/2.2-设备设施类对象/2.2.4-设备设施/1-强电专业/12-柴发机房系统设备.xlsx", "DGJH柴发水套加热器"));
		funcids.addAll(readSheet("SEDGEP", "/2.2-设备设施类对象/2.2.4-设备设施/1-强电专业/12-柴发机房系统设备.xlsx", "DGEP柴发尾气净化装置"));
		funcids.addAll(readSheet("SEDGCU", "/2.2-设备设施类对象/2.2.4-设备设施/1-强电专业/12-柴发机房系统设备.xlsx", "DGCU柴发控制箱"));
		funcids.addAll(readSheet("SEELEL", "/2.2-设备设施类对象/2.2.4-设备设施/1-强电专业/16-电梯系统设备.xlsx", "ELEL直梯"));
		funcids.addAll(readSheet("SEELES", "/2.2-设备设施类对象/2.2.4-设备设施/1-强电专业/16-电梯系统设备.xlsx", "ELES扶梯"));
		funcids.addAll(readSheet("SEELCU", "/2.2-设备设施类对象/2.2.4-设备设施/1-强电专业/16-电梯系统设备.xlsx", "ELCU电梯控制箱"));
		funcids.addAll(readSheet("SEELET", "/2.2-设备设施类对象/2.2.4-设备设施/1-强电专业/16-电梯系统设备.xlsx", "ELET电梯曳引机"));
		funcids.addAll(readSheet("SELTLT", "/2.2-设备设施类对象/2.2.4-设备设施/1-强电专业/17-照明系统设备.xlsx", "LTLT照明灯具"));
		funcids.addAll(readSheet("SELTCU", "/2.2-设备设施类对象/2.2.4-设备设施/1-强电专业/17-照明系统设备.xlsx", "LTCU照明控制箱"));
		funcids.addAll(readSheet("SELTCP", "/2.2-设备设施类对象/2.2.4-设备设施/1-强电专业/17-照明系统设备.xlsx", "LTCP照明控制面板"));
		funcids.addAll(readSheet("SESDCU", "/2.2-设备设施类对象/2.2.4-设备设施/1-强电专业/1A-强电人防系统设备.xlsx", "SDCU强电人防控制箱"));
		funcids.addAll(readSheet("WEBAHT", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/21-建筑设备控制系统设备.xlsx", "BAHT楼控主机"));
		funcids.addAll(readSheet("WEBACU", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/21-建筑设备控制系统设备.xlsx", "BACU楼控控制箱"));
		funcids.addAll(readSheet("WEBADC", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/21-建筑设备控制系统设备.xlsx", "BADC楼控数据采集器"));
		funcids.addAll(readSheet("WEBAPS", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/21-建筑设备控制系统设备.xlsx", "BAPS楼控电源供应器"));
		funcids.addAll(readSheet("WEBAEC", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/21-建筑设备控制系统设备.xlsx", "BAEC楼控机柜"));
		funcids.addAll(readSheet("WEBADS", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/21-建筑设备控制系统设备.xlsx", "BADS楼控显示屏"));
		funcids.addAll(readSheet("WEBACP", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/21-建筑设备控制系统设备.xlsx", "BACP楼控控制面板"));
		funcids.addAll(readSheet("WETVFE", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/22-电视信号系统设备.xlsx", "TVFE电视前端设备"));
		funcids.addAll(readSheet("WETVTV", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/22-电视信号系统设备.xlsx", "TVTV电视机"));
		funcids.addAll(readSheet("WETVCU", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/22-电视信号系统设备.xlsx", "TVCU电视信号控制箱"));
		funcids.addAll(readSheet("WELNHT", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/26-本地网络系统设备.xlsx", "LNHT服务器主机"));
		funcids.addAll(readSheet("WELNRT", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/26-本地网络系统设备.xlsx", "LNRT路由器"));
		funcids.addAll(readSheet("WELNEX", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/26-本地网络系统设备.xlsx", "LNEX交换机"));
		funcids.addAll(readSheet("WELNHF", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/26-本地网络系统设备.xlsx", "LNHF硬件防火墙"));
		funcids.addAll(readSheet("WELNAC", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/26-本地网络系统设备.xlsx", "LNAC无线控制器"));
		funcids.addAll(readSheet("WELNAP", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/26-本地网络系统设备.xlsx", "LNAP无线AP点位"));
		funcids.addAll(readSheet("WELNCU", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/26-本地网络系统设备.xlsx", "LNCU网络控制箱"));
		funcids.addAll(readSheet("WEMMHT", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/2B-气象监测系统.xlsx", "MMHT气象监测主机"));
		funcids.addAll(readSheet("WEMMCU", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/2B-气象监测系统.xlsx", "MMCU气象监测控制箱"));
		funcids.addAll(readSheet("WEMMMS", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/2B-气象监测系统.xlsx", "MMMS气象监测站"));
		funcids.addAll(readSheet("WEMAHT", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/27-移动通信信号放大系统设备.xlsx", "MAHT移动通信信号放大主机"));
		funcids.addAll(readSheet("WEMAAT", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/27-移动通信信号放大系统设备.xlsx", "MAAT移动通信信号放大天线"));
		funcids.addAll(readSheet("WEMACU", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/27-移动通信信号放大系统设备.xlsx", "MACU移动通信信号放大控制箱"));
		funcids.addAll(readSheet("WEMIHT", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/23-多媒体信息发布系统设备.xlsx", "MIHT多媒体主机"));
		funcids.addAll(readSheet("WEMIPL", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/23-多媒体信息发布系统设备.xlsx", "MIPL播放器"));
		funcids.addAll(readSheet("WEMIDS", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/23-多媒体信息发布系统设备.xlsx", "MIDS多媒体显示屏"));
		funcids.addAll(readSheet("WEMIST", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/23-多媒体信息发布系统设备.xlsx", "MIST自助终端机"));
		funcids.addAll(readSheet("WEMIFE", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/23-多媒体信息发布系统设备.xlsx", "MIFE多媒体前端设备"));
		funcids.addAll(readSheet("WEMICU", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/23-多媒体信息发布系统设备.xlsx", "MICU多媒体控制箱"));
		funcids.addAll(readSheet("WEOFTL", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/2D-办公系统设备.xlsx", "OFTL办公电话"));
		funcids.addAll(readSheet("WEOFCM", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/2D-办公系统设备.xlsx", "OFCM办公摄像头"));
		funcids.addAll(readSheet("WEOFCP", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/2D-办公系统设备.xlsx", "OFCP办公控制面板"));
		funcids.addAll(readSheet("WEOFCU", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/2D-办公系统设备.xlsx", "OFCU办公控制箱"));
		funcids.addAll(readSheet("WEOFDS", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/2D-办公系统设备.xlsx", "OFDS办公显示屏"));
		funcids.addAll(readSheet("WEOFFE", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/2D-办公系统设备.xlsx", "OFFE办公前端设备"));
		funcids.addAll(readSheet("WEOFHT", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/2D-办公系统设备.xlsx", "OFHT办公主机"));
		funcids.addAll(readSheet("WEOFSP", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/2D-办公系统设备.xlsx", "OFSP办公扬声器"));
		funcids.addAll(readSheet("WEOFCK", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/2D-办公系统设备.xlsx", "OFCK办公时钟"));
		funcids.addAll(readSheet("WEOFPT", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/2D-办公系统设备.xlsx", "OFPT办公打印机"));
		funcids.addAll(readSheet("WEOFMP", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/2D-办公系统设备.xlsx", "OFMP办公话筒"));
		funcids.addAll(readSheet("WEOFPJ", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/2D-办公系统设备.xlsx", "OFPJ办公投影仪"));
		funcids.addAll(readSheet("WEOFPS", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/2D-办公系统设备.xlsx", "OFPS办公投影幕"));
		funcids.addAll(readSheet("WEOFTV", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/2D-办公系统设备.xlsx", "OFTV办公电视机"));
		funcids.addAll(readSheet("WEGMHT", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/25-车库管理系统设备.xlsx", "GMHT车库主机"));
		funcids.addAll(readSheet("WEGMCU", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/25-车库管理系统设备.xlsx", "GMCU车库控制箱"));
		funcids.addAll(readSheet("WEGMCM", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/25-车库管理系统设备.xlsx", "GMCM车库摄像头"));
		funcids.addAll(readSheet("WEGMDS", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/25-车库管理系统设备.xlsx", "GMDS车库显示屏"));
		funcids.addAll(readSheet("WEGMGT", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/25-车库管理系统设备.xlsx", "GMGT进出闸机"));
		funcids.addAll(readSheet("WEGMMP", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/25-车库管理系统设备.xlsx", "GMMP机械车位"));
		funcids.addAll(readSheet("WEGMCP", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/25-车库管理系统设备.xlsx", "GMCP充电桩"));
		funcids.addAll(readSheet("WEGMPD", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/25-车库管理系统设备.xlsx", "GMPD车位检测器"));
		funcids.addAll(readSheet("WEGMGL", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/25-车库管理系统设备.xlsx", "GMGL车位引导灯"));
		funcids.addAll(readSheet("WEGMLP", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/25-车库管理系统设备.xlsx", "GMLP升降防撞柱"));
		funcids.addAll(readSheet("WEGMPL", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/25-车库管理系统设备.xlsx", "GMPL车库地锁"));
		funcids.addAll(readSheet("WEWTWT", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/24-无线对讲系统设备.xlsx", "WTWT对讲机"));
		funcids.addAll(readSheet("WEWTRS", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/24-无线对讲系统设备.xlsx", "WTRS中继台"));
		funcids.addAll(readSheet("WEWTCU", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/24-无线对讲系统设备.xlsx", "WTCU无线对讲控制箱"));
		funcids.addAll(readSheet("WEPMHT", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/29-客流管理系统设备.xlsx", "PMHT客流管理主机"));
		funcids.addAll(readSheet("WEPMCU", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/29-客流管理系统设备.xlsx", "PMCU客流管理控制箱"));
		funcids.addAll(readSheet("WECMHT", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/2C-收银管理系统设备.xlsx", "CMHT收银管理主机"));
		funcids.addAll(readSheet("WECMCU", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/2C-收银管理系统设备.xlsx", "CMCU收银管理控制箱"));
		funcids.addAll(readSheet("WEEMHT", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/2A-能源管理系统设备.xlsx", "EMHT能源管理主机"));
		funcids.addAll(readSheet("WEEMCU", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/2A-能源管理系统设备.xlsx", "EMCU能源管理控制箱"));
		funcids.addAll(readSheet("WETMHT", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/28-租户管理系统设备.xlsx", "TMHT租户管理主机"));
		funcids.addAll(readSheet("WETMCU", "/2.2-设备设施类对象/2.2.4-设备设施/2-弱电专业/28-租户管理系统设备.xlsx", "TMCU租户管理控制箱"));
		funcids.addAll(readSheet("ACCCCC", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/31-中央供冷系统设备.xlsx", "CCCC离心式冷水机组"));
		funcids.addAll(readSheet("ACCCSC", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/31-中央供冷系统设备.xlsx", "CCSC螺杆式冷水机组"));
		funcids.addAll(readSheet("ACCCAC", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/31-中央供冷系统设备.xlsx", "CCAC吸收式冷水机组"));
		funcids.addAll(readSheet("ACCCAH", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/31-中央供冷系统设备.xlsx", "CCAH空调空气源热泵"));
		funcids.addAll(readSheet("ACCCWH", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/31-中央供冷系统设备.xlsx", "CCWH空调水源热泵"));
		funcids.addAll(readSheet("ACCCGH", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/31-中央供冷系统设备.xlsx", "CCGH空调地源热泵"));
		funcids.addAll(readSheet("ACCCOT", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/31-中央供冷系统设备.xlsx", "CCOT冷却塔"));
		funcids.addAll(readSheet("ACCCCP", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/31-中央供冷系统设备.xlsx", "CCCP供冷冷冻水泵"));
		funcids.addAll(readSheet("ACCCOP", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/31-中央供冷系统设备.xlsx", "CCOP供冷冷却水泵"));
		funcids.addAll(readSheet("ACCCGP", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/31-中央供冷系统设备.xlsx", "CCGP供冷乙二醇泵"));
		funcids.addAll(readSheet("ACCCFP", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/31-中央供冷系统设备.xlsx", "CCFP供冷补水泵"));
		funcids.addAll(readSheet("ACCCCF", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/31-中央供冷系统设备.xlsx", "CCCF供冷定压补水装置"));
		funcids.addAll(readSheet("ACCCPT", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/31-中央供冷系统设备.xlsx", "CCPT供冷压力罐"));
		funcids.addAll(readSheet("ACCCHE", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/31-中央供冷系统设备.xlsx", "CCHE供冷换热器"));
		funcids.addAll(readSheet("ACCCVD", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/31-中央供冷系统设备.xlsx", "CCVD供冷真空脱气机"));
		funcids.addAll(readSheet("ACCCCD", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/31-中央供冷系统设备.xlsx", "CCCD供冷水加药装置"));
		funcids.addAll(readSheet("ACCCSD", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/31-中央供冷系统设备.xlsx", "CCSD供冷软化水装置"));
		funcids.addAll(readSheet("ACCCTU", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/31-中央供冷系统设备.xlsx", "CCTU供冷全程水处理仪"));
		funcids.addAll(readSheet("ACCCST", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/31-中央供冷系统设备.xlsx", "CCST供冷蓄冷热槽"));
		funcids.addAll(readSheet("ACCCDB", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/31-中央供冷系统设备.xlsx", "CCDB供冷分水器"));
		funcids.addAll(readSheet("ACCCCL", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/31-中央供冷系统设备.xlsx", "CCCL供冷集水器"));
		funcids.addAll(readSheet("ACCCWT", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/31-中央供冷系统设备.xlsx", "CCWT供冷水箱"));
		funcids.addAll(readSheet("ACCCOC", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/31-中央供冷系统设备.xlsx", "CCOC供冷在线清洗装置"));
		funcids.addAll(readSheet("ACCCTC", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/31-中央供冷系统设备.xlsx", "CCTC供冷伴热电缆"));
		funcids.addAll(readSheet("ACCCCU", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/31-中央供冷系统设备.xlsx", "CCCU供冷控制箱"));
		funcids.addAll(readSheet("ACVTSF", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/34-通风系统设备.xlsx", "VTSF通风风机"));
		funcids.addAll(readSheet("ACVTAC", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/34-通风系统设备.xlsx", "VTAC热风幕"));
		funcids.addAll(readSheet("ACVTAP", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/34-通风系统设备.xlsx", "VTAP空气净化器"));
		funcids.addAll(readSheet("ACVTDO", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/34-通风系统设备.xlsx", "VTDO通风除味装置"));
		funcids.addAll(readSheet("ACVTIO", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/34-通风系统设备.xlsx", "VTIO通风风口"));
		funcids.addAll(readSheet("ACVTMF", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/34-通风系统设备.xlsx", "VTMF通风消声器"));
		funcids.addAll(readSheet("ACVTPC", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/34-通风系统设备.xlsx", "VTPC通风静压箱"));
		funcids.addAll(readSheet("ACVTCU", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/34-通风系统设备.xlsx", "VTCU通风控制箱"));
		funcids.addAll(readSheet("ACVTCP", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/34-通风系统设备.xlsx", "VTCP通风控制面板"));
		funcids.addAll(readSheet("ACATAH", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/33-空调末端系统设备.xlsx", "ATAH空调机组"));
		funcids.addAll(readSheet("ACATFU", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/33-空调末端系统设备.xlsx", "ATFU新风机组"));
		funcids.addAll(readSheet("ACATFC", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/33-空调末端系统设备.xlsx", "ATFC风机盘管"));
		funcids.addAll(readSheet("ACATSA", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/33-空调末端系统设备.xlsx", "ATSA分体空调"));
		funcids.addAll(readSheet("ACATSI", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/33-空调末端系统设备.xlsx", "ATSI分体空调室内机"));
		funcids.addAll(readSheet("ACATSO", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/33-空调末端系统设备.xlsx", "ATSO分体空调室外机"));
		funcids.addAll(readSheet("ACATVR", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/33-空调末端系统设备.xlsx", "ATVR变频多联机"));
		funcids.addAll(readSheet("ACATVI", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/33-空调末端系统设备.xlsx", "ATVI变频多联机室内机"));
		funcids.addAll(readSheet("ACATVO", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/33-空调末端系统设备.xlsx", "ATVO变频多联机室外机"));
		funcids.addAll(readSheet("ACATVA", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/33-空调末端系统设备.xlsx", "ATVA变风量箱"));
		funcids.addAll(readSheet("ACATFA", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/33-空调末端系统设备.xlsx", "ATFA地板空调器"));
		funcids.addAll(readSheet("ACATIO", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/33-空调末端系统设备.xlsx", "ATIO空调风口"));
		funcids.addAll(readSheet("ACATRD", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/33-空调末端系统设备.xlsx", "ATRD散热器"));
		funcids.addAll(readSheet("ACATIR", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/33-空调末端系统设备.xlsx", "ATIR红外辐射单元"));
		funcids.addAll(readSheet("ACATFH", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/33-空调末端系统设备.xlsx", "ATFH地板采暖设备"));
		funcids.addAll(readSheet("ACATHD", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/33-空调末端系统设备.xlsx", "ATHD空调加湿器"));
		funcids.addAll(readSheet("ACATMF", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/33-空调末端系统设备.xlsx", "ATMF空调消声器"));
		funcids.addAll(readSheet("ACATPC", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/33-空调末端系统设备.xlsx", "ATPC空调静压箱"));
		funcids.addAll(readSheet("ACATCA", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/33-空调末端系统设备.xlsx", "ATCA定风量箱"));
		funcids.addAll(readSheet("ACATCU", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/33-空调末端系统设备.xlsx", "ATCU空调末端控制箱"));
		funcids.addAll(readSheet("ACKLSF", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/35-厨房排油烟系统设备.xlsx", "KLSF厨房排油烟风机"));
		funcids.addAll(readSheet("ACKLAP", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/35-厨房排油烟系统设备.xlsx", "KLAP厨房油烟净化器"));
		funcids.addAll(readSheet("ACKLCU", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/35-厨房排油烟系统设备.xlsx", "KLCU厨房排油烟控制箱"));
		funcids.addAll(readSheet("ACKLIO", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/35-厨房排油烟系统设备.xlsx", "KLIO厨房排油烟风口"));
		funcids.addAll(readSheet("ACCHCB", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/32-中央供热系统设备.xlsx", "CHCB供热燃煤锅炉"));
		funcids.addAll(readSheet("ACCHFB", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/32-中央供热系统设备.xlsx", "CHFB供热燃油锅炉"));
		funcids.addAll(readSheet("ACCHGB", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/32-中央供热系统设备.xlsx", "CHGB供热燃气锅炉"));
		funcids.addAll(readSheet("ACCHEB", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/32-中央供热系统设备.xlsx", "CHEB供热电锅炉"));
		funcids.addAll(readSheet("ACCHHP", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/32-中央供热系统设备.xlsx", "CHHP供热水泵"));
		funcids.addAll(readSheet("ACCHFP", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/32-中央供热系统设备.xlsx", "CHFP供热补水泵"));
		funcids.addAll(readSheet("ACCHCF", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/32-中央供热系统设备.xlsx", "CHCF供热定压补水装置"));
		funcids.addAll(readSheet("ACCHPT", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/32-中央供热系统设备.xlsx", "CHPT供热压力罐"));
		funcids.addAll(readSheet("ACCHHE", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/32-中央供热系统设备.xlsx", "CHHE供热换热器"));
		funcids.addAll(readSheet("ACCHVD", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/32-中央供热系统设备.xlsx", "CHVD供热真空脱气机"));
		funcids.addAll(readSheet("ACCHCD", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/32-中央供热系统设备.xlsx", "CHCD供热水加药装置"));
		funcids.addAll(readSheet("ACCHSD", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/32-中央供热系统设备.xlsx", "CHSD供热软化水装置"));
		funcids.addAll(readSheet("ACCHTU", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/32-中央供热系统设备.xlsx", "CHTU供热全程水处理仪"));
		funcids.addAll(readSheet("ACCHST", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/32-中央供热系统设备.xlsx", "CHST供热蓄冷热槽"));
		funcids.addAll(readSheet("ACCHDB", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/32-中央供热系统设备.xlsx", "CHDB供热分水器"));
		funcids.addAll(readSheet("ACCHCL", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/32-中央供热系统设备.xlsx", "CHCL供热集水器"));
		funcids.addAll(readSheet("ACCHWT", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/32-中央供热系统设备.xlsx", "CHWT供热水箱"));
		funcids.addAll(readSheet("ACCHOC", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/32-中央供热系统设备.xlsx", "CHOC供热在线清洗装置"));
		funcids.addAll(readSheet("ACCHBF", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/32-中央供热系统设备.xlsx", "CHBF生物质炊暖炉"));
		funcids.addAll(readSheet("ACCHCU", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/32-中央供热系统设备.xlsx", "CHCU供热控制箱"));
		funcids.addAll(readSheet("ACADSF", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/36-空调人防系统设备.xlsx", "ADSF空调人防风机"));
		funcids.addAll(readSheet("ACADFD", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/36-空调人防系统设备.xlsx", "ADFD滤毒除湿机"));
		funcids.addAll(readSheet("ACADCU", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/36-空调人防系统设备.xlsx", "ADCU空调人防控制箱"));
		funcids.addAll(readSheet("ACADIO", "/2.2-设备设施类对象/2.2.4-设备设施/3-空调专业/36-空调人防系统设备.xlsx", "ADIO空调人防风口"));
		funcids.addAll(readSheet("WSWSCF", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/47-水景系统设备.xlsx", "WSCF水景循环过滤机组"));
		funcids.addAll(readSheet("WSWSSN", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/47-水景系统设备.xlsx", "WSSN水景喷头"));
		funcids.addAll(readSheet("WSWSCU", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/47-水景系统设备.xlsx", "WSCU水景控制箱"));
		funcids.addAll(readSheet("WSWSLT", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/47-水景系统设备.xlsx", "WSLT水景灯具"));
		funcids.addAll(readSheet("WSWSPP", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/47-水景系统设备.xlsx", "WSPP水景水泵"));
		funcids.addAll(readSheet("WSSTLU", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/45-污水处理系统设备.xlsx", "STLU污废水一体式提升机组"));
		funcids.addAll(readSheet("WSSTSP", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/45-污水处理系统设备.xlsx", "STSP潜污泵"));
		funcids.addAll(readSheet("WSSTOR", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/45-污水处理系统设备.xlsx", "STOR餐饮除油成套设备"));
		funcids.addAll(readSheet("WSSTST", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/45-污水处理系统设备.xlsx", "STST化粪池"));
		funcids.addAll(readSheet("WSSTCP", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/45-污水处理系统设备.xlsx", "STCP集水坑"));
		funcids.addAll(readSheet("WSSTOS", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/45-污水处理系统设备.xlsx", "STOS隔油池"));
		funcids.addAll(readSheet("WSSTCU", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/45-污水处理系统设备.xlsx", "STCU污水控制箱"));
		funcids.addAll(readSheet("WSRWSU", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/43-中水系统设备.xlsx", "RWSU中水供水机组"));
		funcids.addAll(readSheet("WSRWTU", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/43-中水系统设备.xlsx", "RWTU中水处理机组"));
		funcids.addAll(readSheet("WSRWWT", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/43-中水系统设备.xlsx", "RWWT中水储水箱"));
		funcids.addAll(readSheet("WSRWRC", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/43-中水系统设备.xlsx", "RWRC雨水收集设备"));
		funcids.addAll(readSheet("WSRWPT", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/43-中水系统设备.xlsx", "RWPT中水供水压力罐"));
		funcids.addAll(readSheet("WSRWPP", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/43-中水系统设备.xlsx", "RWPP中水供水水泵"));
		funcids.addAll(readSheet("WSRWCU", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/43-中水系统设备.xlsx", "RWCU中水控制箱"));
		funcids.addAll(readSheet("WSWDWT", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/4B-给排水人防系统设备.xlsx", "WDWT给排水人防储水箱"));
		funcids.addAll(readSheet("WSWDCU", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/4B-给排水人防系统设备.xlsx", "WDCU给排水人防控制箱"));
		funcids.addAll(readSheet("WSDHCB", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/42-生活热水系统设备.xlsx", "DHCB生活热水燃煤锅炉"));
		funcids.addAll(readSheet("WSDHFB", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/42-生活热水系统设备.xlsx", "DHFB生活热水燃油锅炉"));
		funcids.addAll(readSheet("WSDHGB", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/42-生活热水系统设备.xlsx", "DHGB生活热水燃气锅炉"));
		funcids.addAll(readSheet("WSDHEB", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/42-生活热水系统设备.xlsx", "DHEB生活热水电锅炉"));
		funcids.addAll(readSheet("WSDHAH", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/42-生活热水系统设备.xlsx", "DHAH生活热水空气源热泵"));
		funcids.addAll(readSheet("WSDHGH", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/42-生活热水系统设备.xlsx", "DHGH生活热水燃气热水器"));
		funcids.addAll(readSheet("WSDHEH", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/42-生活热水系统设备.xlsx", "DHEH生活热水电热水器"));
		funcids.addAll(readSheet("WSDHSC", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/42-生活热水系统设备.xlsx", "DHSC生活热水太阳能集热器"));
		funcids.addAll(readSheet("WSDHHA", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/42-生活热水系统设备.xlsx", "DHHA生活热水辅热设备"));
		funcids.addAll(readSheet("WSDHHE", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/42-生活热水系统设备.xlsx", "DHHE生活热水换热器"));
		funcids.addAll(readSheet("WSDHSU", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/42-生活热水系统设备.xlsx", " DHSU生活热水供水机组"));
		funcids.addAll(readSheet("WSDHHP", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/42-生活热水系统设备.xlsx", "DHHP生活热水水泵"));
		funcids.addAll(readSheet("WSDHSD", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/42-生活热水系统设备.xlsx", "DHSD生活热水软化水装置"));
		funcids.addAll(readSheet("WSDHCF", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/42-生活热水系统设备.xlsx", "DHCF生活热水定压补水装置"));
		funcids.addAll(readSheet("WSDHPT", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/42-生活热水系统设备.xlsx", "DHPT生活热水压力罐"));
		funcids.addAll(readSheet("WSDHST", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/42-生活热水系统设备.xlsx", "DHST生活热水蓄冷热槽"));
		funcids.addAll(readSheet("WSDHWT", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/42-生活热水系统设备.xlsx", "DHWT生活热水储水箱"));
		funcids.addAll(readSheet("WSDHCU", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/42-生活热水系统设备.xlsx", "DHCU生活热水控制箱"));
		funcids.addAll(readSheet("WSWPCF", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/48-泳池系统设备.xlsx", "WPCF泳池循环过滤机组"));
		funcids.addAll(readSheet("WSWPPP", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/48-泳池系统设备.xlsx", "WPPP泳池循环水泵"));
		funcids.addAll(readSheet("WSWPHE", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/48-泳池系统设备.xlsx", "WPHE泳池换热器"));
		funcids.addAll(readSheet("WSWPOS", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/48-泳池系统设备.xlsx", "WPOS泳池臭氧消毒器"));
		funcids.addAll(readSheet("WSWPQM", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/48-泳池系统设备.xlsx", "WPQM泳池水质监测仪"));
		funcids.addAll(readSheet("WSWPPB", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/48-泳池系统设备.xlsx", "WPPB泳池PH值平衡投药泵"));
		funcids.addAll(readSheet("WSWPOP", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/48-泳池系统设备.xlsx", "WPOP泳池氧消毒剂投药泵"));
		funcids.addAll(readSheet("WSWPWT", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/48-泳池系统设备.xlsx", "WPWT泳池水箱"));
		funcids.addAll(readSheet("WSWPCU", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/48-泳池系统设备.xlsx", "WPCU泳池控制箱"));
		funcids.addAll(readSheet("WSFGPA", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/49-燃气系统设备.xlsx", "FGPA燃气调压箱"));
		funcids.addAll(readSheet("WSFGLP", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/49-燃气系统设备.xlsx", "FGLP液化石油气瓶"));
		funcids.addAll(readSheet("WSDDTU", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/44-直饮水系统设备.xlsx", "DDTU直饮水处理机组"));
		funcids.addAll(readSheet("WSDDIH", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/44-直饮水系统设备.xlsx", "DDIH直饮水即时加热器"));
		funcids.addAll(readSheet("WSDDCU", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/44-直饮水系统设备.xlsx", "DDCU直饮水控制箱"));
		funcids.addAll(readSheet("WSSISU", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/46-喷灌系统设备.xlsx", "SISU喷灌机组"));
		funcids.addAll(readSheet("WSSISN", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/46-喷灌系统设备.xlsx", "SISN喷灌喷头"));
		funcids.addAll(readSheet("WSSICU", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/46-喷灌系统设备.xlsx", "SICU喷灌控制箱"));
		funcids.addAll(readSheet("WSDWSU", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/41-生活给水系统设备.xlsx", "DWSU生活给水供水机组"));
		funcids.addAll(readSheet("WSDWTS", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/41-生活给水系统设备.xlsx", "DWTS生活给水水箱消毒器"));
		funcids.addAll(readSheet("WSDWUS", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/41-生活给水系统设备.xlsx", "DWUS生活给水紫外线消毒器"));
		funcids.addAll(readSheet("WSDWWT", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/41-生活给水系统设备.xlsx", "DWWT生活给水储水箱"));
		funcids.addAll(readSheet("WSDWPT", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/41-生活给水系统设备.xlsx", "DWPT生活给水压力罐"));
		funcids.addAll(readSheet("WSDWPP", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/41-生活给水系统设备.xlsx", "DWPP生活给水水泵"));
		funcids.addAll(readSheet("WSDWCU", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/41-生活给水系统设备.xlsx", "DWCU生活给水控制箱"));
		funcids.addAll(readSheet("WSWHTC", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/4A-给排水电伴热系统设备.xlsx", "WHTC给排水伴热电缆"));
		funcids.addAll(readSheet("WSWHIB", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/4A-给排水电伴热系统设备.xlsx", "WHIB给排水伴热保温器"));
		funcids.addAll(readSheet("WSWHCU", "/2.2-设备设施类对象/2.2.4-设备设施/4-给排水专业/4A-给排水电伴热系统设备.xlsx", "WHCU给排水伴热控制箱"));
		funcids.addAll(readSheet("FFGAHT", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/52-燃气报警系统设备.xlsx", "GAHT燃气报警主机"));
		funcids.addAll(readSheet("FFGASE", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/52-燃气报警系统设备.xlsx", "GASE燃气探测器"));
		funcids.addAll(readSheet("FFGACV", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/52-燃气报警系统设备.xlsx", "GACV燃气切断阀"));
		funcids.addAll(readSheet("FFGACU", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/52-燃气报警系统设备.xlsx", "GACU燃气报警控制箱"));
		funcids.addAll(readSheet("FFFEFE", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/57-灭火器系统设备.xlsx", "FEFE灭火器"));
		funcids.addAll(readSheet("FFGEBG", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/56-气体灭火系统设备.xlsx", "GEBG气体灭火瓶组"));
		funcids.addAll(readSheet("FFGESB", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/56-气体灭火系统设备.xlsx", "GESB气体灭火启动瓶"));
		funcids.addAll(readSheet("FFGEHT", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/56-气体灭火系统设备.xlsx", "GEHT气体灭火主机"));
		funcids.addAll(readSheet("FFGECU", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/56-气体灭火系统设备.xlsx", "GECU气体灭火控制箱"));
		funcids.addAll(readSheet("FFGESN", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/56-气体灭火系统设备.xlsx", "GESN气体灭火喷头"));
		funcids.addAll(readSheet("FFFAHT", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/51-火灾报警系统设备.xlsx", "FAHT火灾报警主机"));
		funcids.addAll(readSheet("FFFADS", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/51-火灾报警系统设备.xlsx", "FADS火灾报警显示屏"));
		funcids.addAll(readSheet("FFFALP", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/51-火灾报警系统设备.xlsx", "FALP联动琴台"));
		funcids.addAll(readSheet("FFFASE", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/51-火灾报警系统设备.xlsx", "FASE火灾探测器"));
		funcids.addAll(readSheet("FFFAAL", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/51-火灾报警系统设备.xlsx", "FAAL火灾报警声光报警器"));
		funcids.addAll(readSheet("FFFAMA", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/51-火灾报警系统设备.xlsx", "FAMA消防手动报警按钮"));
		funcids.addAll(readSheet("FFFATL", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/51-火灾报警系统设备.xlsx", "FATL消防电话"));
		funcids.addAll(readSheet("FFFACU", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/51-火灾报警系统设备.xlsx", "FACU火灾报警控制箱"));
		funcids.addAll(readSheet("FFKECU", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/5B-厨房自动灭火系统设备.xlsx", "KECU厨房自动灭火控制箱"));
		funcids.addAll(readSheet("FFKEEC", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/5B-厨房自动灭火系统设备.xlsx", "KEEC厨房自动灭火灭火剂容器"));
		funcids.addAll(readSheet("FFKESN", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/5B-厨房自动灭火系统设备.xlsx", "KESN厨房自动灭火喷头"));
		funcids.addAll(readSheet("FFEEHT", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/5A-应急照明及疏散指示系统设备.xlsx", "EEHT应急照明及疏散指示主机"));
		funcids.addAll(readSheet("FFEECU", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/5A-应急照明及疏散指示系统设备.xlsx", "EECU应急照明及疏散指示控制箱"));
		funcids.addAll(readSheet("FFEEEL", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/5A-应急照明及疏散指示系统设备.xlsx", "EEEL应急照明灯具"));
		funcids.addAll(readSheet("FFEEEI", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/5A-应急照明及疏散指示系统设备.xlsx", "EEEI疏散指示灯牌"));
		funcids.addAll(readSheet("FFFRFR", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/59-防火卷帘门系统设备.xlsx", "FRFR防火卷帘门"));
		funcids.addAll(readSheet("FFFRFD", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/59-防火卷帘门系统设备.xlsx", "FRFD防火门"));
		funcids.addAll(readSheet("FFFRHT", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/59-防火卷帘门系统设备.xlsx", "FRHT防火卷帘门主机"));
		funcids.addAll(readSheet("FFFRCP", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/59-防火卷帘门系统设备.xlsx", "FRCP防火卷帘门控制面板"));
		funcids.addAll(readSheet("FFFRCU", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/59-防火卷帘门系统设备.xlsx", "FRCU防火卷帘门控制箱"));
		funcids.addAll(readSheet("FFEAHT", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/53-漏电火灾报警系统设备.xlsx", "EAHT漏电火灾报警主机"));
		funcids.addAll(readSheet("FFEALS", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/53-漏电火灾报警系统设备.xlsx", "EALS漏电探测器"));
		funcids.addAll(readSheet("FFEACU", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/53-漏电火灾报警系统设备.xlsx", "EACU漏电火灾报警控制箱"));
		funcids.addAll(readSheet("FFBMFE", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/54-紧急广播及背景音乐系统设备.xlsx", "BMFE广播前端设备"));
		funcids.addAll(readSheet("FFBMHT", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/54-紧急广播及背景音乐系统设备.xlsx", "BMHT广播主机"));
		funcids.addAll(readSheet("FFBMCU", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/54-紧急广播及背景音乐系统设备.xlsx", "BMCU广播分区控制箱"));
		funcids.addAll(readSheet("FFBMDS", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/54-紧急广播及背景音乐系统设备.xlsx", "BMDS广播显示屏"));
		funcids.addAll(readSheet("FFBMMP", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/54-紧急广播及背景音乐系统设备.xlsx", "BMMP广播话筒"));
		funcids.addAll(readSheet("FFBMSP", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/54-紧急广播及背景音乐系统设备.xlsx", "BMSP广播扬声器"));
		funcids.addAll(readSheet("FFFSHP", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/55-消防给水系统设备.xlsx", "FSHP消火栓供水加压泵"));
		funcids.addAll(readSheet("FFFSSP", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/55-消防给水系统设备.xlsx", "FSSP喷淋供水加压泵"));
		funcids.addAll(readSheet("FFFSMP", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/55-消防给水系统设备.xlsx", "FSMP水喷雾供水加压泵"));
		funcids.addAll(readSheet("FFFSAP", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/55-消防给水系统设备.xlsx", "FSAP停机坪消防供水加压泵"));
		funcids.addAll(readSheet("FFFSWM", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/55-消防给水系统设备.xlsx", "FSWM消防水炮设备"));
		funcids.addAll(readSheet("FFFSCF", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/55-消防给水系统设备.xlsx", "FSCF消防给水定压补水装置"));
		funcids.addAll(readSheet("FFFSPT", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/55-消防给水系统设备.xlsx", "FSPT消防给水压力罐"));
		funcids.addAll(readSheet("FFFSFT", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/55-消防给水系统设备.xlsx", "FSFT泡沫储罐组"));
		funcids.addAll(readSheet("FFFSAC", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/55-消防给水系统设备.xlsx", "FSAC消防空压机"));
		funcids.addAll(readSheet("FFFSWV", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/55-消防给水系统设备.xlsx", "FSWV湿式报警阀组"));
		funcids.addAll(readSheet("FFFSDV", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/55-消防给水系统设备.xlsx", "FSDV干式报警阀组"));
		funcids.addAll(readSheet("FFFSPV", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/55-消防给水系统设备.xlsx", "FSPV预作用报警阀组"));
		funcids.addAll(readSheet("FFFSHB", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/55-消防给水系统设备.xlsx", "FSHB消火栓箱"));
		funcids.addAll(readSheet("FFFSOH", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/55-消防给水系统设备.xlsx", "FSOH室外消火栓"));
		funcids.addAll(readSheet("FFFSPC", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/55-消防给水系统设备.xlsx", "FSPC消防水泵接合器"));
		funcids.addAll(readSheet("FFFSTC", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/55-消防给水系统设备.xlsx", "FSTC消防伴热电缆"));
		funcids.addAll(readSheet("FFFSET", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/55-消防给水系统设备.xlsx", "FSET消防末端试水装置"));
		funcids.addAll(readSheet("FFFSWT", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/55-消防给水系统设备.xlsx", "FSWT消防水箱"));
		funcids.addAll(readSheet("FFFSTS", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/55-消防给水系统设备.xlsx", "FSTS消防水箱消毒器"));
		funcids.addAll(readSheet("FFFSFP", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/55-消防给水系统设备.xlsx", "FSFP消防补水泵"));
		funcids.addAll(readSheet("FFFSSN", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/55-消防给水系统设备.xlsx", "FSSN消防喷头"));
		funcids.addAll(readSheet("FFFSCP", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/55-消防给水系统设备.xlsx", "FSCP消火栓控制面板"));
		funcids.addAll(readSheet("FFFSHT", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/55-消防给水系统设备.xlsx", "FSHT消防给水主机"));
		funcids.addAll(readSheet("FFFSCU", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/55-消防给水系统设备.xlsx", "FSCU消防给水控制箱"));
		funcids.addAll(readSheet("FFSCEF", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/58-防排烟系统设备.xlsx", "SCEF防排烟风机"));
		funcids.addAll(readSheet("FFSCIO", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/58-防排烟系统设备.xlsx", "SCIO排烟风口"));
		funcids.addAll(readSheet("FFSCHW", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/58-防排烟系统设备.xlsx", "SCHW电动挡烟垂壁"));
		funcids.addAll(readSheet("FFSCSD", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/58-防排烟系统设备.xlsx", "SCSD防火排烟阀"));
		funcids.addAll(readSheet("FFSCCU", "/2.2-设备设施类对象/2.2.4-设备设施/5-消防专业/58-防排烟系统设备.xlsx", "SCCU防排烟控制箱"));
		funcids.addAll(readSheet("SPNPHT", "/2.2-设备设施类对象/2.2.4-设备设施/6-安防专业/62-巡更系统设备.xlsx", "NPHT巡更主机"));
		funcids.addAll(readSheet("SPNPCU", "/2.2-设备设施类对象/2.2.4-设备设施/6-安防专业/62-巡更系统设备.xlsx", "NPCU巡更控制箱"));
		funcids.addAll(readSheet("SPNPCP", "/2.2-设备设施类对象/2.2.4-设备设施/6-安防专业/62-巡更系统设备.xlsx", "NPCP巡更通讯座"));
		funcids.addAll(readSheet("SPNPPS", "/2.2-设备设施类对象/2.2.4-设备设施/6-安防专业/62-巡更系统设备.xlsx", "NPPS巡更棒"));
		funcids.addAll(readSheet("SPIAHT", "/2.2-设备设施类对象/2.2.4-设备设施/6-安防专业/64-入侵报警系统设备.xlsx", "IAHT入侵报警主机"));
		funcids.addAll(readSheet("SPIACU", "/2.2-设备设施类对象/2.2.4-设备设施/6-安防专业/64-入侵报警系统设备.xlsx", "IACU入侵报警控制箱"));
		funcids.addAll(readSheet("SPIAMA", "/2.2-设备设施类对象/2.2.4-设备设施/6-安防专业/64-入侵报警系统设备.xlsx", "IAMA入侵报警手动报警按钮"));
		funcids.addAll(readSheet("SPIAAL", "/2.2-设备设施类对象/2.2.4-设备设施/6-安防专业/64-入侵报警系统设备.xlsx", "IAAL入侵报警声光报警器"));
		funcids.addAll(readSheet("SPIACP", "/2.2-设备设施类对象/2.2.4-设备设施/6-安防专业/64-入侵报警系统设备.xlsx", "IACP入侵报警控制面板"));
		funcids.addAll(readSheet("SPIASE", "/2.2-设备设施类对象/2.2.4-设备设施/6-安防专业/64-入侵报警系统设备.xlsx", "IASE入侵报警探测器"));
		funcids.addAll(readSheet("SPEGHT", "/2.2-设备设施类对象/2.2.4-设备设施/6-安防专业/63-门禁系统设备.xlsx", "EGHT门禁主机"));
		funcids.addAll(readSheet("SPEGCU", "/2.2-设备设施类对象/2.2.4-设备设施/6-安防专业/63-门禁系统设备.xlsx", "EGCU门禁控制箱"));
		funcids.addAll(readSheet("SPEGCR", "/2.2-设备设施类对象/2.2.4-设备设施/6-安防专业/63-门禁系统设备.xlsx", "EGCR门禁读卡器"));
		funcids.addAll(readSheet("SPEGWT", "/2.2-设备设施类对象/2.2.4-设备设施/6-安防专业/63-门禁系统设备.xlsx", "EGWT门禁对讲机"));
		funcids.addAll(readSheet("SPEGTK", "/2.2-设备设施类对象/2.2.4-设备设施/6-安防专业/63-门禁系统设备.xlsx", "EGTK破胎器"));
		funcids.addAll(readSheet("SPEGDL", "/2.2-设备设施类对象/2.2.4-设备设施/6-安防专业/63-门禁系统设备.xlsx", "EGDL门禁门锁"));
		funcids.addAll(readSheet("SPEGBG", "/2.2-设备设施类对象/2.2.4-设备设施/6-安防专业/63-门禁系统设备.xlsx", "EGBG破玻璃按钮"));
		funcids.addAll(readSheet("SPEGCP", "/2.2-设备设施类对象/2.2.4-设备设施/6-安防专业/63-门禁系统设备.xlsx", "EGCP门禁控制面板"));
		funcids.addAll(readSheet("SPEGFG", "/2.2-设备设施类对象/2.2.4-设备设施/6-安防专业/63-门禁系统设备.xlsx", "EGFG速通门"));
		funcids.addAll(readSheet("SPVSHT", "/2.2-设备设施类对象/2.2.4-设备设施/6-安防专业/61-视频监控系统设备.xlsx", "VSHT监控主机"));
		funcids.addAll(readSheet("SPVSCM", "/2.2-设备设施类对象/2.2.4-设备设施/6-安防专业/61-视频监控系统设备.xlsx", "VSCM监控摄像头"));
		funcids.addAll(readSheet("SPVSFE", "/2.2-设备设施类对象/2.2.4-设备设施/6-安防专业/61-视频监控系统设备.xlsx", "VSFE监控前端设备"));
		funcids.addAll(readSheet("SPVSVM", "/2.2-设备设施类对象/2.2.4-设备设施/6-安防专业/61-视频监控系统设备.xlsx", "VSVM监视器"));
		funcids.addAll(readSheet("SPVSDR", "/2.2-设备设施类对象/2.2.4-设备设施/6-安防专业/61-视频监控系统设备.xlsx", "VSDR硬盘录像机"));
		funcids.addAll(readSheet("SPVSCP", "/2.2-设备设施类对象/2.2.4-设备设施/6-安防专业/61-视频监控系统设备.xlsx", "VSCP监控控制面板"));
		funcids.addAll(readSheet("SPVSCU", "/2.2-设备设施类对象/2.2.4-设备设施/6-安防专业/61-视频监控系统设备.xlsx", "VSCU监控控制箱"));
		funcids.addAll(readSheet("CFIDWS", "/2.2-设备设施类对象/2.2.4-设备设施/7-土建精装专业/73-室内装饰.xlsx", "IDWS盥洗盆"));
		funcids.addAll(readSheet("CFIDTL", "/2.2-设备设施类对象/2.2.4-设备设施/7-土建精装专业/73-室内装饰.xlsx", "IDTL坐便器"));
		funcids.addAll(readSheet("CFIDSP", "/2.2-设备设施类对象/2.2.4-设备设施/7-土建精装专业/73-室内装饰.xlsx", "IDSP蹲便器"));
		funcids.addAll(readSheet("CFIDUR", "/2.2-设备设施类对象/2.2.4-设备设施/7-土建精装专业/73-室内装饰.xlsx", "IDUR小便器"));
		funcids.addAll(readSheet("CFIDHD", "/2.2-设备设施类对象/2.2.4-设备设施/7-土建精装专业/73-室内装饰.xlsx", "IDHD干手机"));
		funcids.addAll(readSheet("CFIDBT", "/2.2-设备设施类对象/2.2.4-设备设施/7-土建精装专业/73-室内装饰.xlsx", "IDBT浴缸"));
		funcids.addAll(readSheet("CFIDSK", "/2.2-设备设施类对象/2.2.4-设备设施/7-土建精装专业/73-室内装饰.xlsx", "IDSK洗碗槽"));
		funcids.addAll(readSheet("CFIDSC", "/2.2-设备设施类对象/2.2.4-设备设施/7-土建精装专业/73-室内装饰.xlsx", "IDSC插座"));
		funcids.addAll(readSheet("CFIDFN", "/2.2-设备设施类对象/2.2.4-设备设施/7-土建精装专业/73-室内装饰.xlsx", "IDFN家具"));
		funcids.addAll(readSheet("CFIDCL", "/2.2-设备设施类对象/2.2.4-设备设施/7-土建精装专业/73-室内装饰.xlsx", "IDCL吊顶"));
		funcids.addAll(readSheet("CFBECW", "/2.2-设备设施类对象/2.2.4-设备设施/7-土建精装专业/72-围护结构.xlsx", "BECW幕墙"));
		funcids.addAll(readSheet("CFBEBC", "/2.2-设备设施类对象/2.2.4-设备设施/7-土建精装专业/72-围护结构.xlsx", "BEBC呼吸式幕墙"));
		funcids.addAll(readSheet("CFBELR", "/2.2-设备设施类对象/2.2.4-设备设施/7-土建精装专业/72-围护结构.xlsx", "BELR采光顶"));
		funcids.addAll(readSheet("CFBEDR", "/2.2-设备设施类对象/2.2.4-设备设施/7-土建精装专业/72-围护结构.xlsx", "BEDR门"));
		funcids.addAll(readSheet("CFBEWN", "/2.2-设备设施类对象/2.2.4-设备设施/7-土建精装专业/72-围护结构.xlsx", "BEWN窗"));
		funcids.addAll(readSheet("CFBEAW", "/2.2-设备设施类对象/2.2.4-设备设施/7-土建精装专业/72-围护结构.xlsx", "BEAW雨篷"));
		funcids.addAll(readSheet("CFBEES", "/2.2-设备设施类对象/2.2.4-设备设施/7-土建精装专业/72-围护结构.xlsx", "BEES电动遮阳帘"));
		funcids.addAll(readSheet("CFBEMT", "/2.2-设备设施类对象/2.2.4-设备设施/7-土建精装专业/72-围护结构.xlsx", "BEMT遮阳电机"));
		funcids.addAll(readSheet("CFBECT", "/2.2-设备设施类对象/2.2.4-设备设施/7-土建精装专业/72-围护结构.xlsx", "BECT普通遮阳帘"));
		funcids.addAll(readSheet("CFCSFD", "/2.2-设备设施类对象/2.2.4-设备设施/7-土建精装专业/71-土建结构.xlsx", "CSFD基础"));
		funcids.addAll(readSheet("CFCSCL", "/2.2-设备设施类对象/2.2.4-设备设施/7-土建精装专业/71-土建结构.xlsx", "CSCL柱"));
		funcids.addAll(readSheet("CFCSGD", "/2.2-设备设施类对象/2.2.4-设备设施/7-土建精装专业/71-土建结构.xlsx", "CSGD梁"));
		funcids.addAll(readSheet("CFCSTR", "/2.2-设备设施类对象/2.2.4-设备设施/7-土建精装专业/71-土建结构.xlsx", "CSTR桁架"));
		funcids.addAll(readSheet("CFCSWL", "/2.2-设备设施类对象/2.2.4-设备设施/7-土建精装专业/71-土建结构.xlsx", "CSWL墙"));
		funcids.addAll(readSheet("CFCSFL", "/2.2-设备设施类对象/2.2.4-设备设施/7-土建精装专业/71-土建结构.xlsx", "CSFL楼板"));
		funcids.addAll(readSheet("CFCSRF", "/2.2-设备设施类对象/2.2.4-设备设施/7-土建精装专业/71-土建结构.xlsx", "CSRF屋顶"));
		funcids.addAll(readSheet("CFCSSW", "/2.2-设备设施类对象/2.2.4-设备设施/7-土建精装专业/71-土建结构.xlsx", "CSSW楼梯"));
		funcids.addAll(readSheet("CFPKPS", "/2.2-设备设施类对象/2.2.4-设备设施/7-土建精装专业/75-停车场.xlsx", "PKPS停车位"));
		funcids.addAll(readSheet("CFCADM", "/2.2-设备设施类对象/2.2.4-设备设施/7-土建精装专业/74-建筑附件.xlsx", "CADM装饰材料"));
		funcids.addAll(readSheet("CFCAHD", "/2.2-设备设施类对象/2.2.4-设备设施/7-土建精装专业/74-建筑附件.xlsx", "CAHD栏杆"));
		funcids.addAll(readSheet("CFCASB", "/2.2-设备设施类对象/2.2.4-设备设施/7-土建精装专业/74-建筑附件.xlsx", "CASB减速带"));
		funcids.addAll(readSheet("CFCAST", "/2.2-设备设施类对象/2.2.4-设备设施/7-土建精装专业/74-建筑附件.xlsx", "CAST地桩"));
		funcids.addAll(readSheet("CFCADB", "/2.2-设备设施类对象/2.2.4-设备设施/7-土建精装专业/74-建筑附件.xlsx", "CADB展示板"));
		funcids.addAll(readSheet("CFCAID", "/2.2-设备设施类对象/2.2.4-设备设施/7-土建精装专业/74-建筑附件.xlsx", "CAID标识"));
		funcids.addAll(readSheet("CFCAWC", "/2.2-设备设施类对象/2.2.4-设备设施/7-土建精装专业/74-建筑附件.xlsx", "CAWC擦窗机"));
		funcids.addAll(readSheet("CFCADD", "/2.2-设备设施类对象/2.2.4-设备设施/7-土建精装专业/74-建筑附件.xlsx", "CADD排水沟"));
		funcids.addAll(readSheet("CFCAWB", "/2.2-设备设施类对象/2.2.4-设备设施/7-土建精装专业/74-建筑附件.xlsx", "CAWB垃圾桶"));
		funcids.addAll(readSheet("CFCACR", "/2.2-设备设施类对象/2.2.4-设备设施/7-土建精装专业/74-建筑附件.xlsx", "CACR扫地机器人"));
		funcids.addAll(readSheet("CFCAMC", "/2.2-设备设施类对象/2.2.4-设备设施/7-土建精装专业/74-建筑附件.xlsx", "CAMC井盖"));
		funcids.addAll(readSheet("OTXXFZ", "/2.2-设备设施类对象/2.2.4-设备设施/8-其他/87-其他系统设备.xlsx", "XXFZ冷柜（恒温柜）"));
		funcids.addAll(readSheet("OTXXDC", "/2.2-设备设施类对象/2.2.4-设备设施/8-其他/87-其他系统设备.xlsx", "XXDC风干柜"));
		funcids.addAll(readSheet("OTXXAC", "/2.2-设备设施类对象/2.2.4-设备设施/8-其他/87-其他系统设备.xlsx", "XXAC冷风机"));
		funcids.addAll(readSheet("OTXXSC", "/2.2-设备设施类对象/2.2.4-设备设施/8-其他/87-其他系统设备.xlsx", "XXSC消毒柜"));
		funcids.addAll(readSheet("OTXXUA", "/2.2-设备设施类对象/2.2.4-设备设施/8-其他/87-其他系统设备.xlsx", "XXUA无人机"));
		funcids.addAll(readSheet("OTXXSV", "/2.2-设备设施类对象/2.2.4-设备设施/8-其他/87-其他系统设备.xlsx", "XXSV炉具"));
		funcids.addAll(readSheet("OTLAPL", "/2.2-设备设施类对象/2.2.4-设备设施/8-其他/83-液体管道及附件.xlsx", "LAPL液体管道"));
		funcids.addAll(readSheet("OTLAFI", "/2.2-设备设施类对象/2.2.4-设备设施/8-其他/83-液体管道及附件.xlsx", "LAFI水流指示器"));
		funcids.addAll(readSheet("OTLAFT", "/2.2-设备设施类对象/2.2.4-设备设施/8-其他/83-液体管道及附件.xlsx", "LAFT水管过滤器"));
		funcids.addAll(readSheet("OTSECB", "/2.2-设备设施类对象/2.2.4-设备设施/8-其他/85-传感器.xlsx", "SECB组合式传感器"));
		funcids.addAll(readSheet("OTSETP", "/2.2-设备设施类对象/2.2.4-设备设施/8-其他/85-传感器.xlsx", "SETP温度传感器"));
		funcids.addAll(readSheet("OTSERH", "/2.2-设备设施类对象/2.2.4-设备设施/8-其他/85-传感器.xlsx", "SERH湿度传感器"));
		funcids.addAll(readSheet("OTSEPS", "/2.2-设备设施类对象/2.2.4-设备设施/8-其他/85-传感器.xlsx", "SEPS压力传感器"));
		funcids.addAll(readSheet("OTSECT", "/2.2-设备设施类对象/2.2.4-设备设施/8-其他/85-传感器.xlsx", "SECT浓度传感器"));
		funcids.addAll(readSheet("OTSESP", "/2.2-设备设施类对象/2.2.4-设备设施/8-其他/85-传感器.xlsx", "SESP速度传感器"));
		funcids.addAll(readSheet("OTSEDP", "/2.2-设备设施类对象/2.2.4-设备设施/8-其他/85-传感器.xlsx", "SEDP位移传感器"));
		funcids.addAll(readSheet("OTSEFL", "/2.2-设备设施类对象/2.2.4-设备设施/8-其他/85-传感器.xlsx", "SEFL流量传感器"));
		funcids.addAll(readSheet("OTSEHT", "/2.2-设备设施类对象/2.2.4-设备设施/8-其他/85-传感器.xlsx", "SEHT热量传感器"));
		funcids.addAll(readSheet("OTSEEE", "/2.2-设备设施类对象/2.2.4-设备设施/8-其他/85-传感器.xlsx", "SEEE电度传感器"));
		funcids.addAll(readSheet("OTSESD", "/2.2-设备设施类对象/2.2.4-设备设施/8-其他/85-传感器.xlsx", "SESD声音传感器"));
		funcids.addAll(readSheet("OTSEVD", "/2.2-设备设施类对象/2.2.4-设备设施/8-其他/85-传感器.xlsx", "SEVD图像传感器"));
		funcids.addAll(readSheet("OTSEPF", "/2.2-设备设施类对象/2.2.4-设备设施/8-其他/85-传感器.xlsx", "SEPF人流量传感器"));
		funcids.addAll(readSheet("OTSEIL", "/2.2-设备设施类对象/2.2.4-设备设施/8-其他/85-传感器.xlsx", "SEIL照度传感器"));
		funcids.addAll(readSheet("OTAEOV", "/2.2-设备设施类对象/2.2.4-设备设施/8-其他/86-执行器.xlsx", "AEOV通断阀"));
		funcids.addAll(readSheet("OTAEAV", "/2.2-设备设施类对象/2.2.4-设备设施/8-其他/86-执行器.xlsx", "AEAV调节阀"));
		funcids.addAll(readSheet("OTAECV", "/2.2-设备设施类对象/2.2.4-设备设施/8-其他/86-执行器.xlsx", "AECV止回阀"));
		funcids.addAll(readSheet("OTAERV", "/2.2-设备设施类对象/2.2.4-设备设施/8-其他/86-执行器.xlsx", "AERV减压阀"));
		funcids.addAll(readSheet("OTAESV", "/2.2-设备设施类对象/2.2.4-设备设施/8-其他/86-执行器.xlsx", "AESV安全阀"));
		funcids.addAll(readSheet("OTAERL", "/2.2-设备设施类对象/2.2.4-设备设施/8-其他/86-执行器.xlsx", "AERL继电器"));
		funcids.addAll(readSheet("OTAEFC", "/2.2-设备设施类对象/2.2.4-设备设施/8-其他/86-执行器.xlsx", "AEFC变频器"));
		funcids.addAll(readSheet("OTAEIL", "/2.2-设备设施类对象/2.2.4-设备设施/8-其他/86-执行器.xlsx", "AEIL提示灯"));
		funcids.addAll(readSheet("OTAEAL", "/2.2-设备设施类对象/2.2.4-设备设施/8-其他/86-执行器.xlsx", "AEAL声光报警器"));
		funcids.addAll(readSheet("OTWAEW", "/2.2-设备设施类对象/2.2.4-设备设施/8-其他/82-弱电线路及附件.xlsx", "WAEW弱电线路"));
		funcids.addAll(readSheet("OTSAEW", "/2.2-设备设施类对象/2.2.4-设备设施/8-其他/81-强电线路及附件.xlsx", "SAEW强电线路"));
		funcids.addAll(readSheet("OTAAPL", "/2.2-设备设施类对象/2.2.4-设备设施/8-其他/84-气体管道及附件.xlsx", "AAPL气体管道"));
		funcids.addAll(readSheet("OTAAFT", "/2.2-设备设施类对象/2.2.4-设备设施/8-其他/84-气体管道及附件.xlsx", "AAFT风道过滤器"));
	}

	private void importComponent(List<FuncidDefModel> funcids) throws IOException {
		funcids.addAll(readSheet("component", "/2.2-设备设施类对象/2.2.2-设备设施类对象通用属性.xlsx", "部件"));

		funcids.addAll(readSheet("SETDHSIP", "/2.2-设备设施类对象/2.2.5-部件/1-强电专业部件.xlsx", "TDHSIP综合保护装置"));
		funcids.addAll(readSheet("SETDLSDW", "/2.2-设备设施类对象/2.2.5-部件/1-强电专业部件.xlsx", "TDLSDW低压配电抽屉"));
		funcids.addAll(readSheet("WEBADCBP", "/2.2-设备设施类对象/2.2.5-部件/2-弱电专业部件.xlsx", "BADCBP总线端口"));
		funcids.addAll(readSheet("ACCCGHBP", "/2.2-设备设施类对象/2.2.5-部件/3-空调专业部件.xlsx", "CCGHBP地埋管"));
		funcids.addAll(readSheet("ACCCOTFN", "/2.2-设备设施类对象/2.2.5-部件/3-空调专业部件.xlsx", "CCOTFN冷却塔风机"));
		//funcids.addAll(readSheet("Sheet1", "/2.2-设备设施类对象/2.2.5-部件/4-给排水专业部件.xlsx", "Sheet1"));
		//funcids.addAll(readSheet("Sheet1", "/2.2-设备设施类对象/2.2.5-部件/5-消防专业部件.xlsx", "Sheet1"));
		//funcids.addAll(readSheet("Sheet1", "/2.2-设备设施类对象/2.2.5-部件/6-安防专业部件.xlsx", "Sheet1"));
		funcids.addAll(readSheet("CFCSFDCS", "/2.2-设备设施类对象/2.2.5-部件/7-土建精装专业部件.xlsx", "CSFDCS基础垫层"));
		funcids.addAll(readSheet("CFCSFDWB", "/2.2-设备设施类对象/2.2.5-部件/7-土建精装专业部件.xlsx", "CSFDWB基础防水板"));
		funcids.addAll(readSheet("CFCSFDGD", "/2.2-设备设施类对象/2.2.5-部件/7-土建精装专业部件.xlsx", "CSFDGD基础梁"));
		funcids.addAll(readSheet("CFCSFDPN", "/2.2-设备设施类对象/2.2.5-部件/7-土建精装专业部件.xlsx", "CSFDPN基础嵌板"));
		funcids.addAll(readSheet("CFCSFDAA", "/2.2-设备设施类对象/2.2.5-部件/7-土建精装专业部件.xlsx", "CSFDAA基础抗浮锚杆"));
		funcids.addAll(readSheet("CFCSFDPL", "/2.2-设备设施类对象/2.2.5-部件/7-土建精装专业部件.xlsx", "CSFDPL基础桩"));
		funcids.addAll(readSheet("CFCSFDED", "/2.2-设备设施类对象/2.2.5-部件/7-土建精装专业部件.xlsx", "CSFDED基础电梯基坑"));
		funcids.addAll(readSheet("CFCSFDCP", "/2.2-设备设施类对象/2.2.5-部件/7-土建精装专业部件.xlsx", "CSFDCP基础集水坑"));
		funcids.addAll(readSheet("CFCSWLBD", "/2.2-设备设施类对象/2.2.5-部件/7-土建精装专业部件.xlsx", "CSWLBD墙体"));
		funcids.addAll(readSheet("CFCSWLIL", "/2.2-设备设施类对象/2.2.5-部件/7-土建精装专业部件.xlsx", "CSWLIL墙保温层"));
		funcids.addAll(readSheet("CFCSWLEP", "/2.2-设备设施类对象/2.2.5-部件/7-土建精装专业部件.xlsx", "CSWLEP墙预埋件"));
		funcids.addAll(readSheet("CFCSWLCT", "/2.2-设备设施类对象/2.2.5-部件/7-土建精装专业部件.xlsx", "CSWLCT墙涂层"));
		funcids.addAll(readSheet("CFCSSWBD", "/2.2-设备设施类对象/2.2.5-部件/7-土建精装专业部件.xlsx", "CSSWBD楼梯主体"));
		funcids.addAll(readSheet("CFCSSWRW", "/2.2-设备设施类对象/2.2.5-部件/7-土建精装专业部件.xlsx", "CSSWRW楼梯坡道"));
		funcids.addAll(readSheet("CFCSSWAR", "/2.2-设备设施类对象/2.2.5-部件/7-土建精装专业部件.xlsx", "CSSWAR楼梯扶手"));
		funcids.addAll(readSheet("CFBECWKL", "/2.2-设备设施类对象/2.2.5-部件/7-土建精装专业部件.xlsx", "BECWKL幕墙龙骨"));
		funcids.addAll(readSheet("CFBECWPN", "/2.2-设备设施类对象/2.2.5-部件/7-土建精装专业部件.xlsx", "BECWPN幕墙嵌板"));
		funcids.addAll(readSheet("CFBECWCP", "/2.2-设备设施类对象/2.2.5-部件/7-土建精装专业部件.xlsx", "BECWCP幕墙组件"));
		funcids.addAll(readSheet("CFBELRKL", "/2.2-设备设施类对象/2.2.5-部件/7-土建精装专业部件.xlsx", "BELRKL采光顶龙骨"));
		funcids.addAll(readSheet("CFBELRPN", "/2.2-设备设施类对象/2.2.5-部件/7-土建精装专业部件.xlsx", "BELRPN采光顶嵌板"));
		funcids.addAll(readSheet("CFBELRCP", "/2.2-设备设施类对象/2.2.5-部件/7-土建精装专业部件.xlsx", "BELRCP采光顶组件"));
		//funcids.addAll(readSheet("Sheet1", "/2.2-设备设施类对象/2.2.5-部件/8-其他专业部件.xlsx", "Sheet1"));

	}

	private void importVirtual(List<FuncidDefModel> funcids) throws IOException {
		funcids.addAll(readSheet("VOEm", "/2.5-虚拟对象.xlsx", "1能耗模型"));
		funcids.addAll(readSheet("VOSe", "/2.5-虚拟对象.xlsx", "2分项"));
		funcids.addAll(readSheet("VOBr", "/2.5-虚拟对象.xlsx", "3支路"));
		funcids.addAll(readSheet("VOBc", "/2.5-虚拟对象.xlsx", "4信标"));
	}

	private void importTool(List<FuncidDefModel> funcids) throws IOException {
		funcids.addAll(readSheet("tool", "/2.3-耗材工具类对象/2.3.2-耗材工具类对象通用属性.xlsx", "工具"));
	}

	private void importMaterial(List<FuncidDefModel> funcids) throws IOException {
		funcids.addAll(readSheet("material", "/2.3-耗材工具类对象/2.3.2-耗材工具类对象通用属性.xlsx", "耗材"));
	}

	@Data
	private static class CacheModel {
		private String classCode;
		private String firstTag;
		private String secondTag;
		private String origCode;
		private String name;
		private String unit;
		private String origDataType;
		private String priority; // 优先级
		private String inputMode;
		private String origDataSource; // 字典选择
		private String remark; // 备注
	}

	private List<FuncidDefModel> readSheet(String classCode, String fileName, String sheetName) throws IOException {
		List<FuncidDefModel> data = new LinkedList<>();
		String classFile = basepath + fileName;
		FileInputStream inputStream = new FileInputStream(classFile);
		Workbook workbook = new XSSFWorkbook(inputStream);
		Sheet sheet = workbook.getSheet(sheetName);
		if (sheet == null) {
			return data;
		}

		// 行号从0开始, 从第2行开始读数据
		int startRowNum = 2;
		int lastRowNum = sheet.getLastRowNum();
		CacheModel cache = new CacheModel();
		for (int rowIndex = startRowNum; rowIndex <= lastRowNum; rowIndex++) {
			Row row = sheet.getRow(rowIndex);
			if (row == null) {
				log.info("file: {}, sheet: {}, row[{}] is null", fileName, sheetName, rowIndex);
				continue;
			}
			// 0:firstTag, 1:secondTag, 2:name, 3:origCode, 4:unit, 5:origDataType, 6:priorite, 7:imputMode, 8:origDataSource, 9:remark
			CacheModel target = new CacheModel();
			target.classCode = classCode;
			target.firstTag = (String) ExcelUtils.parseCell(row.getCell(0), "string");
			target.secondTag = (String) ExcelUtils.parseCell(row.getCell(1), "string");
			target.name = (String) ExcelUtils.parseCell(row.getCell(2), "string");
			target.origCode = (String) ExcelUtils.parseCell(row.getCell(3), "string");
			target.unit = (String) ExcelUtils.parseCell(row.getCell(4), "string");
			target.origDataType = (String) ExcelUtils.parseCell(row.getCell(5), "string");
			target.priority = (String) ExcelUtils.parseCell(row.getCell(6), "string");
			target.inputMode = (String) ExcelUtils.parseCell(row.getCell(7), "string");
			target.origDataSource = (String) ExcelUtils.parseCell(row.getCell(8), "string");
			target.remark = (String) ExcelUtils.parseCell(row.getCell(9), "string");

			if (target.origCode == null || "".equals(target.origCode) || "`".equals(target.origCode)) {
				continue;
			}
			if (FuncidUtils.isBase(target.origCode)) {
				continue;
			}

			// firstTag不为空
			if (target.firstTag != null) {
				cache.firstTag = target.firstTag;
				cache.secondTag = null;
			} else {
				target.firstTag = cache.firstTag;
			}

			// secondTag
			if (target.secondTag != null) {
				cache.secondTag = target.secondTag;
			} else {
				target.secondTag = cache.secondTag;
			}
			FuncidDefModel funcidDefModel = toFuncidDefModel(target);
			if (funcidDefModel == null) {
				log.info("file: {}, sheet: {}, row: {} data type [{}] error", fileName, sheetName, rowIndex, target.origDataType);
			} else {
				data.add(funcidDefModel);
			}
		}
		return data;
	}

}

