package com.persagy.dmp.rwd.funcid.parse.parser;

import com.fasterxml.jackson.databind.JsonNode;
import com.persagy.dmp.rwd.funcid.parse.model.RegionValue;
import com.persagy.dmp.rwd.model.FuncidDefModel;
import lombok.Getter;
import lombok.Setter;

import java.util.List;

@Getter
@Setter
public class ParserContext<T> {


	private T value;
	private List<T> arrayValue;
	private RegionValue<T> regionValue;
	private List<RegionValue<T>> regionArray;

	private FuncidDefModel def;

	private JsonNode source;
	private boolean isNull;
	private boolean hasError;
	private StringBuilder error;

	public ParserContext(FuncidDefModel def) {
		this(def, null);
	}

	public ParserContext(FuncidDefModel def, JsonNode source) {
		this.def = def;
		this.source = source;
		this.isNull = false;
		this.hasError = false;
		this.error = new StringBuilder();
	}

	public void addError(String error) {
		if (!this.hasError) {
			this.hasError = true;
		}
		this.error.append(error);
	}

	public Object getSourceValue() {
		Boolean isMultiple = def.getIsMultiple();
		Boolean isRegion = def.getIsRegion();
		if (isMultiple && isRegion) {
			return regionArray;
		}
		if (isMultiple && !isRegion) {
			return arrayValue;
		}
		if (!isMultiple && isRegion) {
			return regionValue;
		}
		if (!isMultiple && !isRegion) {
			return value;
		}
		return null;
	}
}
