package com.persagy.dmp.rwd.funcid.parse.parser.impl;

import com.fasterxml.jackson.databind.JsonNode;
import com.persagy.dmp.rwd.enums.FuncidDataType;
import com.persagy.dmp.rwd.funcid.parse.parser.ParserContext;
import com.persagy.dmp.rwd.funcid.parse.parser.ParserUtils;
import com.persagy.dmp.rwd.model.FuncidDefModel;

import java.util.List;

/**
 * @author: yaoll
 * @date: 2020-09-10
 * @verison: 1.0
 */
public class AttachmentParser extends AbstractParser<String> {

	@Override
	public void checkDef(ParserContext context) {
		super.checkDef(context);
	}

	@Override
	public void checkValue(ParserContext<String> context) {
		super.checkValue(context);
		if (context.isHasError()) {
			return;
		}

		FuncidDefModel def = context.getDef();
		JsonNode source = context.getSource();

		FuncidDataType dataType = def.getDataType();
		String sourceStr = source.toString();

		Boolean isMultiple = def.getIsMultiple();
		Boolean isRegion = def.getIsRegion();

		if (isMultiple && isRegion) {
			// 复数, 区间
			context.addError("不支持[Attachment 复数 区间]");
			return;
		}

		if (isMultiple && !isRegion) {
			// 复数 非区间
			if (!ParserUtils.isArray(source, dataType)) {
				context.addError("值错误[" + sourceStr + "]");
				return;
			}
			List<String> value = ParserUtils.parseToList(sourceStr, String.class);
			if (value == null || value.size() == 0) {
				context.addError("值错误[" + sourceStr + "]");
				return;
			}
			context.setArrayValue(value);
		}

		if (!isMultiple && isRegion) {
			// 单数 区间
			context.addError("不支持[Attachment 单数 区间]");
			return;
		}

		if (!isMultiple && !isRegion) {
			// 单数 非区间
			if (!ParserUtils.is(source, dataType)) {
				context.addError("值错误[" + sourceStr + "]");
				return;
			}
			String value = ParserUtils.parse(sourceStr, String.class);
			context.setValue(value);
		}
	}


}
