package com.persagy.dmp.rwd.funcid.parse.parser.impl;

import com.fasterxml.jackson.databind.JsonNode;
import com.persagy.dmp.rwd.enums.FuncidDataType;
import com.persagy.dmp.rwd.funcid.parse.parser.ParserContext;
import com.persagy.dmp.rwd.funcid.parse.parser.ParserUtils;
import com.persagy.dmp.rwd.model.FuncidDefModel;

import java.util.List;


/**
 * 布尔型解析器
 * <p>
 * 不支持区间
 * <p>
 * 假设信息点编码的值为Info01, 入参格式:<br/>
 * 单数 非区间: { "Info01": true }<br/>
 * 复数 非区间: { "Info01": [ true, false, false] }<br/>
 */
public class BooleanParser extends AbstractParser<Boolean> {

	@Override
	public void checkDef(ParserContext context) {
		if (context.getDef().getIsRegion()) {
			context.addError("Boolean类型不支持区间");
		}
	}

	@Override
	public void checkValue(ParserContext<Boolean> context) {
		super.checkValue(context);
		if (context.isHasError()) {
			return;
		}

		FuncidDefModel def = context.getDef();
		JsonNode source = context.getSource();
		Boolean isMultiple = def.getIsMultiple();
		FuncidDataType dataType = def.getDataType();
		String sourceStr = source.toString();
		String error = "值错误, 需要[ true or false], 传入[" + sourceStr + "]";
		if (isMultiple) {
			// 复数
			if (!ParserUtils.isArray(source, dataType)) {
				context.addError(error);
				return;
			}
			List<Boolean> value = ParserUtils.parseToList(sourceStr, Boolean.class);
			if (value == null || value.size() == 0) {
				context.addError(error);
				return;
			}
			context.setArrayValue(value);
		}
		if (!isMultiple) {
			// 单数 非区间
			if (!ParserUtils.is(source, dataType)) {
				context.addError(error);
				return;
			}
			Boolean value = ParserUtils.parse(sourceStr, Boolean.class);
			if (value == null) {
				context.addError(error);
				return;
			}
			context.setValue(value);
		}
	}
}
