package com.persagy.dmp.rwd.funcid.parse.parser.impl;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.JsonNodeType;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.dmp.rwd.enums.FuncidDataType;
import com.persagy.dmp.rwd.funcid.parse.parser.ParserContext;
import com.persagy.dmp.rwd.funcid.parse.parser.ParserUtils;
import com.persagy.dmp.rwd.model.FuncidDefModel;
import lombok.Getter;
import lombok.Setter;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author: yaoll
 * @date: 2020-09-11
 * @verison: 1.0
 */
public class EnumParser extends AbstractParser<String> {


	@Override
	public void checkDef(ParserContext context) {
		super.checkDef(context);
	}

	@Override
	public void checkValue(ParserContext<String> context) {
		super.checkValue(context);
		if (context.isHasError()) {
			return;
		}

		FuncidDefModel def = context.getDef();
		JsonNode source = context.getSource();
		FuncidDataType dataType = def.getDataType();
		String sourceStr = source.toString();
		JsonNode dataSource = def.getDataSource();
		if (dataSource == null) {
			context.addError("值定义错误:dataSource为空");
			return;
		}
		if (dataSource.getNodeType() != JsonNodeType.ARRAY) {
			context.addError("值定义错误:dataSource不是数组");
			return;
		}

		String dataSourceStr = dataSource.toString();
		Map<String, EnumNode> enumValues = getEnumValues(dataSource);
		if (enumValues == null || enumValues.size() == 0) {
			context.addError("dataSource[" + dataSourceStr + "]错误");
			return;
		}

		String error = "值错误, 需要[" + dataSourceStr + "], 传入[" + sourceStr + "]";

		Boolean isMultiple = def.getIsMultiple();
		Boolean isRegion = def.getIsRegion();
		if (isMultiple && isRegion) {
			// 复数, 区间
			context.addError("不支持[Enum 复数 区间]");
			return;
		}

		if (isMultiple && !isRegion) {
			// 复数 非区间
			if (!ParserUtils.isArray(source, dataType)) {
				context.addError(error);
				return;
			}

			List<String> value = ParserUtils.parseToList(sourceStr, String.class);
			if (value == null || value.size() == 0) {
				context.addError(error);
				return;
			}
			for (String code : value) {
				if (!enumValues.containsKey(code)) {
					context.addError(error);
					return;
				}
			}
			context.setArrayValue(value);
		}

		if (!isMultiple && isRegion) {
			// 单数, 区间
			context.addError("不支持[Enum 单数 区间]");
			return;
		}

		if (!isMultiple && !isRegion) {
			// 单数 非区间
			if (!ParserUtils.is(source, dataType)) {
				context.addError(error);
				return;
			}
			String value = ParserUtils.parse(sourceStr, String.class);
			if (value == null) {
				context.addError(error);
				return;
			}
			// 区间匹配
			if (!enumValues.containsKey(value)) {
				context.addError(error);
				return;
			}
			context.setValue(value);
		}
	}

	@Getter
	@Setter
	public class EnumNode {
		private String code;
		private String parentCode;

		public EnumNode(ObjectNode param) {
			this.code = param.get("code").asText();
			JsonNode parentCode = param.get("parentCode");
			if (parentCode != null) {
				this.parentCode = parentCode.asText();
			}
		}

	}

	private Map<String, EnumNode> getEnumValues(JsonNode dataSource) {
		Map<String, EnumNode> result = new HashMap<>();
		ArrayNode array = (ArrayNode) dataSource;
		for (JsonNode jsonNode : array) {
			ObjectNode node = (ObjectNode) jsonNode;
			if (node.has("code")) {
				result.put(node.get("code").asText(), new EnumNode(node));
			}
		}
		return result;
	}

}
