package com.persagy.dmp.rwd.funcid.parse.utils;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.JsonNodeType;
import com.fasterxml.jackson.databind.node.ObjectNode;

public class ObjectNodeUtils {

	private static final String PATTERN_LONG = "-?[0-9]+?";
	private static final String PATTERN_DOUBLE = "-?[0-9]+(\\.[0-9]+)?";

	public static boolean isString(JsonNode node) {
		if (node == null) {
			return false;
		}
		JsonNodeType nodeType = node.getNodeType();
		if (nodeType == JsonNodeType.ARRAY) {
			return false;
		} else if(nodeType == JsonNodeType.OBJECT){
			return false;
		} else if(nodeType == JsonNodeType.POJO){
			return false;
		}
		return true;
	}

	public static String toString(JsonNode node) {
		if (node == null) {
			return null;
		}
		return node.asText();
	}

	public static String getString(ObjectNode obj, String key) {
		if (obj == null) {
			return null;
		}
		if (!obj.has(key)) {
			return null;
		}
		JsonNode node = obj.get(key);
		return node.asText();
	}

	public static boolean isLong(JsonNode node) {
		if (node == null) {
			return false;
		}
		JsonNodeType nodeType = node.getNodeType();
		if (nodeType == JsonNodeType.NUMBER) {
			return true;
		}
		if (nodeType == JsonNodeType.STRING) {
			String str = node.asText();
			return str.matches(PATTERN_LONG);
		}
		return false;
	}

	public static Long toLong(JsonNode node) {
		if (node == null) {
			return null;
		}
		JsonNodeType nodeType = node.getNodeType();
		if (nodeType == JsonNodeType.NUMBER) {
			return node.asLong();
		}
		if (nodeType == JsonNodeType.STRING) {
			String str = node.asText();
			if (str.matches(PATTERN_LONG)) {
				return Long.parseLong(node.asText());
			}
		}
		return null;
	}

	public static Long getLong(ObjectNode obj, String key) {
		if (obj == null) {
			return null;
		}
		if (!obj.has(key)) {
			return null;
		}
		JsonNode node = obj.get(key);
		return toLong(node);
	}

	public static boolean isDouble(JsonNode node) {
		if (node == null) {
			return false;
		}
		JsonNodeType nodeType = node.getNodeType();
		if (nodeType == JsonNodeType.NUMBER) {
			return true;
		}
		if (nodeType == JsonNodeType.STRING) {
			String str = node.asText();
			return str.matches(PATTERN_DOUBLE);
		}
		return false;
	}

	public static Double toDouble(JsonNode node) {
		if (node == null) {
			return null;
		}
		JsonNodeType nodeType = node.getNodeType();
		if (nodeType == JsonNodeType.NUMBER) {
			return node.asDouble();
		}
		if (nodeType == JsonNodeType.STRING) {
			String str = node.asText();
			if (str.matches(PATTERN_DOUBLE)) {
				return Double.parseDouble(str);
			}
		}
		return null;
	}

	public static Double getDouble(ObjectNode obj, String key) {
		if (obj == null) {
			return null;
		}
		if (!obj.has(key)) {
			return null;
		}
		JsonNode node = obj.get(key);
		return toDouble(node);
	}

	public static boolean isBoolean(JsonNode node) {
		if (node == null) {
			return false;
		}
		JsonNodeType nodeType = node.getNodeType();
		if (nodeType == JsonNodeType.BOOLEAN) {
			return true;
		}
		if (nodeType == JsonNodeType.STRING) {
			String value = node.asText().toLowerCase();
			return "true".equals(value) || "false".equals(value);
		}
		return false;
	}

	public static Boolean toBoolean(JsonNode node) {
		if (node == null) {
			return null;
		}
		JsonNodeType nodeType = node.getNodeType();
		if (nodeType == JsonNodeType.BOOLEAN) {
			return node.asBoolean();
		}
		if (nodeType == JsonNodeType.STRING) {
			String str = node.asText().toLowerCase();
			return "true".equals(str) ? true : "false".equals(str) ? false : null;
		}
		return null;
	}

	public static Boolean getBoolean(ObjectNode obj, String key) {
		if (obj == null) {
			return null;
		}
		if (!obj.has(key)) {
			return null;
		}
		JsonNode node = obj.get(key);
		JsonNodeType nodeType = node.getNodeType();
		if (nodeType == JsonNodeType.BOOLEAN) {
			return node.asBoolean();
		}
		if (nodeType == JsonNodeType.STRING) {
			return Boolean.parseBoolean(node.asText());
		}
		return null;
	}

}
