package com.persagy.dmp.rwd.funcid.parse.parser.impl;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.json.JacksonMapper;
import com.persagy.dmp.rwd.enums.FuncidCategory;
import com.persagy.dmp.rwd.enums.FuncidDataType;
import com.persagy.dmp.rwd.funcid.parse.parser.InfoCodeParser;
import com.persagy.dmp.rwd.funcid.parse.parser.ParserContext;
import com.persagy.dmp.rwd.model.FuncidDefModel;
import org.junit.Assert;

import java.util.HashMap;
import java.util.Map;

/**
 * @author: yaoll
 * @date: 2020-09-22
 * @verison: 1.0
 */
public class TestBaseParser {
	protected String code = "test_code";

	protected InfoCodeParser parser;

	protected <T extends Comparable> Object doAssert(FuncidDefModel def, Boolean isMultiple, Boolean isRegion, ArrayNode source, Object param,
													 boolean success, Object target) {
		def.setIsMultiple(isMultiple);
		def.setIsRegion(isRegion);
		def.setDataSource(source);

		StringBuilder buff = new StringBuilder("----------------- ");
		buff.append(def.getIsMultiple() ? "复数" : "单数").append(" ");
		buff.append(def.getIsRegion() ? "区间" : "非区间").append(" ");
		buff.append(source == null ? "无限制" : ("限制" + source.toString())).append(" ");
		buff.append("  参数: ").append(JacksonMapper.toSimpleJson(param));
		System.out.println(buff.toString());

		ParserContext<T> context = new ParserContext<T>(def, JacksonMapper.toObject(JacksonMapper.toSimpleJson(param), JsonNode.class));
		parser.checkValue(context);


		ObjectNode object = prepareObject(param);
		System.out.println(JacksonMapper.toSimpleJson(context));
		System.out.println(JacksonMapper.toSimpleJson(object));

		Assert.assertEquals(success, !context.isHasError());

		if (!context.isHasError()) {

			Object value = context.getSourceValue();

			System.out.println("value = " + JacksonMapper.toSimpleJson(value));
			if (target != null) {
				if (target instanceof Number) {
					Assert.assertEquals(true, ((Number) target).doubleValue() == ((Number) value).doubleValue());
				} else {
					Assert.assertEquals(target, value);
				}
			}
			System.out.println("\n");
		}
		return context.getValue();
	}


	protected ObjectNode prepareObject(Object value) {
		Map<String, Object> map = new HashMap<>();
		map.put(code, value);
		return JacksonMapper.toObject(JacksonMapper.toSimpleJson(map), ObjectNode.class);
//		JacksonCriteria object = JsonNodeFactory.instance.objectNode();
//		String format = MessageFormat.format("'{'\"{0}\": {1}'}'", code, value);
//		return JsonMapper.fromJsonSimple(format, JacksonCriteria.class);
	}

	protected FuncidDefModel prepareModel(FuncidCategory category, FuncidDataType type, Boolean isMultiple, Boolean isRegion, String source, String format) {
		FuncidDefModel def = new FuncidDefModel();
		def.setCode(code);
		def.setCategory(category);
		def.setDataType(type);
		def.setIsMultiple(isMultiple);
		def.setIsRegion(isRegion);
		def.setDataSource(JacksonMapper.toObject(source, ArrayNode.class));
		def.setFormater(format);
		return def;
	}

	// < <= => >
	protected Map<String, Object> paramRegion(Object $gt, Object $gte, Object $lte, Object $lt) {
		Map<String, Object> region = new HashMap<>();
		if ($lt != null) {
			region.put("$lt", $lt);
		}
		if ($lte != null) {
			region.put("$lte", $lte);
		}
		if ($gte != null) {
			region.put("$gte", $gte);
		}
		if ($gt != null) {
			region.put("$gt", $gt);
		}
		return region;
	}

	protected ObjectNode prepareRegion(Object $gt, Object $gte, Object $lte, Object $lt) {
		return JacksonMapper.toObject(JacksonMapper.toSimpleJson(paramRegion($gt, $gte, $lte, $lt)), ObjectNode.class);
	}
}
