package com.persagy.dmp.rwd.funcid.parse.parser.impl;

import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.JsonNodeFactory;
import com.persagy.dmp.rwd.enums.FuncidCategory;
import com.persagy.dmp.rwd.enums.FuncidDataType;
import com.persagy.dmp.rwd.model.FuncidDefModel;
import org.junit.Before;
import org.junit.Test;

import java.util.Arrays;

/**
 * @author: yaoll
 * @date: 2020-09-22
 * @verison: 1.0
 */
public class TestBooleanParser extends TestBaseParser{

	FuncidDefModel def;
	ArrayNode region;
	Object param;

	@Before
	public void before() {
		parser = new BooleanParser();

		def = prepareModel(FuncidCategory.STATIC, FuncidDataType.BOOLEAN, false, false, null, null);
		region = JsonNodeFactory.instance.arrayNode();
		region.add(prepareRegion(10L, null, 50L, null));
		region.add(prepareRegion(200L, null, 250L, null));
	}

	@Test
	public void testParser() {
		param = true;
		doAssert(def, false, false, null, param, true, true); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
//		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
//		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, true, true); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
//		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
//		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = Arrays.asList(true, false, true);
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, true, null); // 复数 非区间 无限制
//		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
//		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, true, null);// 复数 非区间 有限制
//		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
//		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = "true";
		doAssert(def, false, false, null, param, true, true); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
//		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
//		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, true, true); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
//		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
//		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = "false";
		doAssert(def, false, false, null, param, true, false); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
//		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
//		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, true, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
//		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
//		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = "abc";
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
//		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
//		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
//		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
//		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = "[\"abc\"]";
		param = Arrays.asList("abc");
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
//		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
//		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
//		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
//		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = "{\"$gt\": 12, \"$lt\": 44}";
		param = prepareRegion(12L, null, null, 44L);
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
//		doAssert(def, false, true, null, param, true, null); // 单数 区间 无限制
//		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
//		doAssert(def, false, true, region, param, true, null);// 单数 区间 有限制
//		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = "{\"$gt\": 112, \"$lt\": 144}";
		param = prepareRegion(112L, null, null, 144L);
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
//		doAssert(def, false, true, null, param, true, null); // 单数 区间 无限制
//		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
//		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
//		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = "[{\"$gt\": 33, \"$lt\": 44}]";
		param = Arrays.asList(prepareRegion(33L, null, null, 44L));
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
//		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
//		doAssert(def, true, true, null, param, true, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
//		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
//		doAssert(def, true, true, region, param, true, null);// 复数 区间 有限制

		param = "[{\"$gt\": 33, \"$lt\": 144}]";
		param = Arrays.asList(prepareRegion(33L, null, null, 144L));
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
//		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
//		doAssert(def, true, true, null, param, true, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
//		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
//		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = "[{\"$gt\": 133, \"$lt\": 144}]";
		param = Arrays.asList(prepareRegion(133L, null, null, 144L));
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
//		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
//		doAssert(def, true, true, null, param, true, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
//		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
//		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

	}
}
