package com.persagy.dmp.rwd.funcid.parse.parser.impl;

import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.JsonNodeFactory;
import com.persagy.dmp.rwd.enums.FuncidCategory;
import com.persagy.dmp.rwd.enums.FuncidDataType;
import com.persagy.dmp.rwd.model.FuncidDefModel;
import org.junit.Before;
import org.junit.Test;

import java.util.Arrays;

/**
 * @author: yaoll
 * @date: 2020-09-22
 * @verison: 1.0
 */
public class TestDateTimeParser  extends TestBaseParser {

	FuncidDefModel def;
	ArrayNode region;
	Object param;

	@Before
	public void before() {
		parser = new DateTimeParser();

		def = prepareModel(FuncidCategory.STATIC, FuncidDataType.DATETIME, false, false, null, "MMdd");
		region = JsonNodeFactory.instance.arrayNode();
		region.add(prepareRegion("0301", null, "0401", null));
		region.add(prepareRegion("0901", null, "1201", null));
	}


	@Test
	public void test1() {
		param = "0150";
		doAssert(def, false, false, null, param, true, "0150"); // 单数 非区间 无限制
	}

	@Test
	public void testParser() {
		param = "0150";
		doAssert(def, false, false, null, param, true, "0150"); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, "0150"); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = "0305";
		doAssert(def, false, false, null, param, true, "0305"); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, true, "0305"); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制


		param = "0335";
		doAssert(def, false, false, null, param, true, "0335"); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, true, "0335"); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制


		param = "20200101";
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = Arrays.asList(1, 2, 3);
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = Arrays.asList("0102", "0103", "0104");
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, true, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = Arrays.asList("0302", "0303", "0304");
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, true, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, true, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = "abcd";
		doAssert(def, false, false, null, param, true, null); // 单数 非区间 无限制 // TODO 目前只校验入参字符串长度
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = Arrays.asList("abcd");
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, true, null); // 复数 非区间 无限制 // TODO 目前只校验入参字符串长度
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = paramRegion("0105", null, null, "0110");
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, true, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = paramRegion("0301", null, null, "0315");
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, true, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, true, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = Arrays.asList(paramRegion("0101", null, null, "0131"));
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, true, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = Arrays.asList(paramRegion("0305", null, null, "0310"));
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, true, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, true, null);// 复数 区间 有限制

		param = Arrays.asList(paramRegion("20200101", null, null, "20200901"));
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制


		def = prepareModel(FuncidCategory.STATIC, FuncidDataType.STRING, false, false, null, "HHmmss");
		region = JsonNodeFactory.instance.arrayNode();
		region.add(prepareRegion("010000", null, "020000", null));
		param = "2900";
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = "290000";
		doAssert(def, false, false, null, param, true, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制
	}


}
