package com.persagy.dmp.rwd.funcid.parse.parser.impl;

import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.JsonNodeFactory;
import com.persagy.dmp.rwd.enums.FuncidCategory;
import com.persagy.dmp.rwd.enums.FuncidDataType;
import com.persagy.dmp.rwd.model.FuncidDefModel;
import org.junit.Before;
import org.junit.Test;

import java.util.Arrays;

/**
 * @author: yaoll
 * @date: 2020-09-22
 * @verison: 1.0
 */
public class TestDoubleParser extends TestBaseParser {

	FuncidDefModel def;
	ArrayNode region;
	Object param;

	@Before
	public void before() {
		parser = new DoubleParser();

		def = prepareModel(FuncidCategory.STATIC, FuncidDataType.DOUBLE, false, false, null, null);
		region = JsonNodeFactory.instance.arrayNode();
		region.add(prepareRegion(10L, null, 50L, null));
		region.add(prepareRegion(200L, null, 250L, null));
	}

	@Test
	public void test1() {
		param = Arrays.asList(31, 32, 35);
		doAssert(def, true, false, region, param, true, null);// 复数 非区间 有限制
	}

	@Test
	public void testParser() {

		Object param;

		param = 50.1;
		doAssert(def, false, false, null, param, true, 50.1); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, 50.1); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = 123;
		doAssert(def, false, false, null, param, true, 123.0); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = 23.01; //
		doAssert(def, false, false, null, param, true, 23.01); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, true, 23.01); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = 123.85;
		doAssert(def, false, false, null, param, true, 123.85); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = Arrays.asList(1, 2, 3);
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, true, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = Arrays.asList(31, 32, 35);
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, true, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, true, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = "true";
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = "0815";
		doAssert(def, false, false, null, param, true, 815.0); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = "abc";
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = Arrays.asList("abc");
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = paramRegion(12, null, null, 44);
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, true, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, true, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = paramRegion(112, null, null, 144);
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, true, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = Arrays.asList(paramRegion(33.0, null, null, 44.2));
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, true, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, true, null);// 复数 区间 有限制

		param = Arrays.asList(paramRegion(33, null, null, 144));
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, true, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = Arrays.asList(paramRegion(133, null, null, 144));
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, true, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制


		param = Arrays.asList(paramRegion(233, null, null, 244));
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, true, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, true, null);// 复数 区间 有限制
	}
}
