package com.persagy.dmp.rwd.funcid.parse.parser.impl;

import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.JsonNodeFactory;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.dmp.rwd.enums.FuncidCategory;
import com.persagy.dmp.rwd.enums.FuncidDataType;
import com.persagy.dmp.rwd.model.FuncidDefModel;
import org.junit.Before;
import org.junit.Test;

import java.util.Arrays;

/**
 * @author: yaoll
 * @date: 2020-09-22
 * @verison: 1.0
 */
public class TestEnumParser extends TestBaseParser {

	FuncidDefModel def;
	ArrayNode region;
	Object param;

	@Before
	public void before() {
		parser = new EnumParser();
		def = prepareModel(FuncidCategory.STATIC, FuncidDataType.ENUM, false, false, null, null);

		ArrayNode array = JsonNodeFactory.instance.arrayNode();
		ObjectNode obj = array.addObject();
		obj.put("code", "GC");
		obj.put("name", "国产");

		obj = array.addObject();
		obj.put("code", "HZ");
		obj.put("name", "合资");

		obj = array.addObject();
		obj.put("code", "JK");
		obj.put("name", "进口");
		def.setDataSource(array);

		region = JsonNodeFactory.instance.arrayNode();
		region.add(prepareRegion(10L, null, 50L, null));
	}

	@Test
	public void testParser() {
		ArrayNode array = JsonNodeFactory.instance.arrayNode();
		ObjectNode obj = array.addObject();
		obj.put("code", "GC");
		obj.put("name", "国产");

		obj = array.addObject();
		obj.put("code", "HZ");
		obj.put("name", "合资");

		obj = array.addObject();
		obj.put("code", "JK");
		obj.put("name", "进口");
		region = array;

		Object param;

		param = "GC";
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, true, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = "ABC";
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制


		param = "[\"GC\",\"JK\"]";
		param = Arrays.asList("GC", "JK");
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, true, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = "[\"GC\",\"JK\",\"AA\"]";
		param = Arrays.asList("GC", "JK", "AA");
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = "[[\"GC\"],[\"JK\",\"HZ\"]]";
		param = Arrays.asList(Arrays.asList("GC"), Arrays.asList("JK", "HZ"));
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = "{\"$gt\": 112, \"$lt\": 144}";
		param = paramRegion(112, null, null, 144);
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = "[{\"$gt\": 33, \"$lt\": 44}]";
		param = Arrays.asList(paramRegion(33, null, null, 44));
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = "[{\"$gt\": 33, \"$lt\": 144}]";
		param = Arrays.asList(paramRegion(33, null, null, 44));
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = "[{\"$gt\": 133, \"$lt\": 144}]";
		param = Arrays.asList(paramRegion(133, null, null, 144));
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

	}
}
