package com.persagy.dmp.rwd.funcid.parse.parser.impl;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.JsonNodeFactory;
import com.persagy.common.json.JacksonMapper;
import com.persagy.dmp.rwd.enums.FuncidCategory;
import com.persagy.dmp.rwd.enums.FuncidDataType;
import com.persagy.dmp.rwd.funcid.parse.model.RegionValue;
import com.persagy.dmp.rwd.funcid.parse.parser.ParserUtils;
import com.persagy.dmp.rwd.model.FuncidDefModel;
import org.junit.Before;
import org.junit.Test;

import java.util.Arrays;
import java.util.List;

/**
 * @author: yaoll
 * @date: 2020-09-22
 * @verison: 1.0
 */
public class TestIntegerParser extends TestBaseParser {

	@Before
	public void before() {
		super.parser = new IntegerParser();
	}

	@Test
	public void test1() {

		FuncidDefModel def = prepareModel(FuncidCategory.STATIC, FuncidDataType.INTEGER, false, false, null, null);
		ArrayNode region = JsonNodeFactory.instance.arrayNode();
		region.add(prepareRegion(10L, null, 50L, null));
		Object param;

		param = paramRegion(10, null, 50, null);// "{\"$gt\": 10, \"$lte\": 50}";
		doAssert(def, false, true, region, param, true, null);// 单数 区间 有限制
	}

	@Test
	public void testParser() {

		FuncidDefModel def = prepareModel(FuncidCategory.STATIC, FuncidDataType.INTEGER, false, false, null, null);
		ArrayNode region = JsonNodeFactory.instance.arrayNode();
		region.add(prepareRegion(10L, null, 50L, null));
		Object param;

		param = 10;
		doAssert(def, false, false, null, param, true, 10L); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, 10L); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = 23L;
		doAssert(def, false, false, null, param, true, 23L); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, true, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = 123;
		doAssert(def, false, false, null, param, true, 123L); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = 23.01; // 截取整数部分
		doAssert(def, false, false, null, param, true, 23L); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, true, 23L); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = 123.85;
		doAssert(def, false, false, null, param, true, 123L); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = Arrays.asList(1, 2, 3);
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, true, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = Arrays.asList(31, 32, 33);
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, true, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, true, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = true;
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = "0815";
		doAssert(def, false, false, null, param, true, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = "abc";
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = Arrays.asList("abc");
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = paramRegion(10, null, 50, null);// "{\"$gt\": 10, \"$lte\": 50}";
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, true, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, true, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = paramRegion(112, null, null, 144);// "{\"$gt\": 10, \"$lte\": 50}";
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, true, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = Arrays.asList(paramRegion(112, null, null, 144));
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, true, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = Arrays.asList(paramRegion(33, null, null, 144));
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, true, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = Arrays.asList(paramRegion(133, null, null, 144));
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, true, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

	}


	@Test
	public void test2() {
		String dataSource = "[{\"$gt\":10,\"$lte\":50}]";
		ArrayNode o = JacksonMapper.toObject(dataSource, ArrayNode.class);

		List<RegionValue<Long>> regionValues1 = ParserUtils.parseDataSource(o, Long.class);
		System.out.println(regionValues1);
		System.out.println(JacksonMapper.toSimpleJson(regionValues1));
		for (RegionValue<Long> regionValue : regionValues1) {
			System.out.println("region: " + JacksonMapper.toSimpleJson(regionValue));
			System.out.println(regionValue.$gt.getClass());
		}

		List<RegionValue<Long>> regionValues = JacksonMapper.toObject(dataSource, new TypeReference<List<RegionValue<Long>>>() {
		});

		for (RegionValue<Long> regionValue : regionValues) {
			System.out.println("region: " + JacksonMapper.toSimpleJson(regionValue));
			System.out.println(regionValue.$gt.getClass());
		}


	}
}