package com.persagy.dmp.rwd.funcid.parse.parser.impl;

import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.JsonNodeFactory;
import com.persagy.dmp.rwd.enums.FuncidCategory;
import com.persagy.dmp.rwd.enums.FuncidDataType;
import com.persagy.dmp.rwd.model.FuncidDefModel;
import org.junit.Before;
import org.junit.Test;

import java.util.Arrays;

/**
 * @author: yaoll
 * @date: 2020-09-22
 * @verison: 1.0
 */
public class TestStringParser extends TestBaseParser {

	FuncidDefModel def;
	ArrayNode region;
	Object param;

	@Before
	public void before() {
		super.parser = new StringParser();
		def = prepareModel(FuncidCategory.STATIC, FuncidDataType.STRING, false, false, null, null);
		region = JsonNodeFactory.instance.arrayNode();
		region.add(prepareRegion(10L, null, 50L, null));
	}

	@Test
	public void testParser() {
		Object param;

		param = 23;
		doAssert(def, false, false, null, param, true, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = "12345";
		doAssert(def, false, false, null, param, true, "12345"); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = "123456789012345";
		doAssert(def, false, false, null, param, true, "123456789012345"); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, true, "123456789012345"); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = "123456789";
		doAssert(def, false, false, null, param, true, "123456789"); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, "123456789"); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = "[1,2,3]";
		doAssert(def, false, false, null, param, true, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = "\"[1,2,3]\"";
		doAssert(def, false, false, null, param, true, "\"[1,2,3]\""); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, "\"[1,2,3]\""); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = "true";
		doAssert(def, false, false, null, param, true, "true"); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = "\"0123\"";
		doAssert(def, false, false, null, param, true, "\"0123\""); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = "\"abc\"";
		doAssert(def, false, false, null, param, true, "\"abc\""); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = "[\"abc\"]";
		doAssert(def, false, false, null, param, true, "[\"abc\"]"); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = Arrays.asList("abcdefg","123456");
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, true, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = Arrays.asList("abcdefg12345678","123456abcdefghijkl");
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, true, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, true, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = paramRegion(12, null, null, 44);
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = paramRegion(112, null, null, 144);
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = Arrays.asList(paramRegion(33.0, null, null, 44.2));
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制

		param = Arrays.asList(paramRegion(112, null, null, 144));
		doAssert(def, false, false, null, param, false, null); // 单数 非区间 无限制
		doAssert(def, true, false, null, param, false, null); // 复数 非区间 无限制
		doAssert(def, false, true, null, param, false, null); // 单数 区间 无限制
		doAssert(def, true, true, null, param, false, null);  // 复数 区间 无限制
		doAssert(def, false, false, region, param, false, null); // 单数 非区间 有限制
		doAssert(def, true, false, region, param, false, null);// 复数 非区间 有限制
		doAssert(def, false, true, region, param, false, null);// 单数 区间 有限制
		doAssert(def, true, true, region, param, false, null);// 复数 区间 有限制


	}
}