package com.persagy.dmp.rwd;

import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JpaUtils;
import com.persagy.dmp.common.SpringUtils;
import com.persagy.dmp.common.http.HttpUtils;
import com.persagy.dmp.rwd.funcid.parse.parser.InfoCodeParserFactory;
import com.querydsl.jpa.impl.JPAQueryFactory;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.annotation.EnableRabbit;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.cloud.client.discovery.EnableDiscoveryClient;
import org.springframework.cloud.openfeign.EnableFeignClients;
import org.springframework.context.ConfigurableApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.core.env.ConfigurableEnvironment;

import javax.persistence.EntityManager;

/**
 * @author: yaoll
 * @date: 2020-10-22
 * @verison: 1.0
 */
@Slf4j
@EnableRabbit
@EnableDiscoveryClient
@EnableFeignClients
@SpringBootApplication
public class RwdApp {

	public static void main(String[] args) {
		ConfigurableApplicationContext ctx = SpringApplication.run(RwdApp.class, args);
		ConfigurableEnvironment env = ctx.getEnvironment();
		log.info("profile                  = {}", env.getProperty("spring.profiles.active"));
	}

	@Bean
	public InfoCodeParserFactory infoCodeParserFactory() {
		return new InfoCodeParserFactory();
	}

	@Bean
	public JPAQueryFactory jpaQueryFactory(EntityManager entityManager) {
		return new JPAQueryFactory(entityManager);
	}

	@Bean
	public CriteriaUtils criteriaUtils(@Autowired JPAQueryFactory jpaQueryFactory) {
		return new CriteriaUtils(jpaQueryFactory);
	}

	@Bean
	public JpaUtils jpaUtils(@Autowired EntityManager entityManager) {
		return new JpaUtils(entityManager);
	}

	@Bean
	public HttpUtils httpUtils() {
		return new HttpUtils();
	}

	@Bean
	public SpringUtils springUtils() {
		return new SpringUtils();
	}
}
