package com.persagy.dmp.rwd.entity;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.dmp.rwd.model.ObjectInstanceModel;
import com.vladmihalcea.hibernate.type.json.JsonStringType;
import lombok.Getter;
import lombok.Setter;
import org.hibernate.annotations.*;

import javax.persistence.*;
import javax.persistence.Entity;
import javax.persistence.Table;
import java.util.Date;
import java.util.List;

@Setter
@Getter
@Entity
@Table(name = "rwd_object")
@TypeDef(name = "json", typeClass = JsonStringType.class)
@IdClass(GroupProjectPK.class)
public class ObjectInstance {

	@Id
	private String id;
	@Id
	private String groupCode;
	@Id
	private String projectId;
	private String name;
	private String localId;
	private String localName;
	@Column(updatable = false)
	private String objType;
	@Column(updatable = false)
	private String classCode;

	@Column(updatable = false)
	private Integer grouping;

	@Type(type = "json")
	private ObjectNode infos;

	@Type(type = "json")
	private List<String> virtualInfoCodes;

	private Integer valid;
	@Column(updatable = false)
	@CreationTimestamp
	private Date createTime;
	@UpdateTimestamp
	private Date updateTime;
	@Column(updatable = false)
	private String createUser;
	private String updateUser;
	@Column(updatable = false)
	private String createApp;
	private String updateApp;

	public static ObjectInstance fromModel(ObjectInstanceModel model) {
		ObjectInstance entity = new ObjectInstance();
		entity.setId(model.getId());
		entity.setName(model.getName());
		entity.setLocalId(model.getLocalId());
		entity.setLocalName(model.getLocalName());
		entity.setProjectId(model.getProjectId());
		entity.setObjType(model.getObjType());
		entity.setClassCode(model.getClassCode());
		entity.setGrouping(model.getGrouping());
		entity.setValid(model.getValid());
		entity.setCreateTime(model.getCreateTime());
		entity.setUpdateTime(model.getUpdateTime());
		entity.setCreateUser(model.getCreateUser());
		entity.setUpdateUser(model.getUpdateUser());
		entity.setCreateApp(model.getCreateApp());
		entity.setUpdateApp(model.getUpdateApp());
		entity.setInfos(model.getInfos());
		entity.setVirtualInfoCodes(model.getVirtualInfoCodes());
		return entity;
	}

	public ObjectInstanceModel toModel() {
		ObjectInstanceModel model = new ObjectInstanceModel();
		model.setId(this.id);
		model.setName(this.getName());
		model.setLocalId(this.getLocalId());
		model.setLocalName(this.getLocalName());
		model.setProjectId(this.getProjectId());
		model.setObjType(this.getObjType());
		model.setClassCode(this.getClassCode());
		model.setGrouping(this.getGrouping());
		model.setValid(this.getValid());
		model.setCreateTime(this.getCreateTime());
		model.setUpdateTime(this.getUpdateTime());
		model.setCreateUser(this.getCreateUser());
		model.setUpdateUser(this.getUpdateUser());
		model.setCreateApp(this.getCreateApp());
		model.setUpdateApp(this.getUpdateApp());
		model.setInfos(this.getInfos());
		model.setVirtualInfoCodes(this.getVirtualInfoCodes());
		return model;
	}

}
