package com.persagy.dmp.rwd.service;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.web.MapResponse;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.config.DmpParameterStorage;
import com.persagy.dmp.rwd.config.RwdConstants;
import com.persagy.dmp.rwd.entity.AppClass;
import com.persagy.dmp.rwd.entity.QAppClass;
import com.persagy.dmp.rwd.model.AppClassModel;
import com.querydsl.core.types.dsl.BooleanExpression;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class AppClassService extends BaseService{

	private List<BooleanExpression> parse(ObjectNode criteria) {
		// TODO
		return null;
	}

	public PagedResponse<AppClassModel> query(JacksonCriteria criteria) {
		PagedResponse<AppClassModel> result = new PagedResponse<>();
		super.checkParam(result, RwdConstants.QUERY_GROUPCODE);
		if (!result.success()) {
			return result;
		}
		if (!criteria.getCriteria().has("groupCode")) {
			criteria.add("groupCode", DmpParameterStorage.getGroupCode());
		}
		PagedResponse<AppClass> resp = criteriaUtils.query(QAppClass.appClass, this::parse, criteria);
		result.setCount(resp.getCount());
		List<AppClass> dataList = resp.getData();
		if (dataList != null && dataList.size() > 0) {
			List<AppClassModel> collect = dataList.stream().map(entity -> {
				AppClassModel model = entity.toModel();
				return model;
			}).collect(Collectors.toList());
			result.setData(collect);
		}
		return result;
	}

	@Transactional
	public MapResponse create(AppClassModel param) {
		MapResponse response = new MapResponse();
		super.checkParam(response, RwdConstants.QUERY_GROUPCODE);
		if (!response.success()) {
			return response;
		}
		// TODO
		if (param.getProjectId() == null) {
			response.setFail("[projectId] is required");
			return response;
		}
		AppClass entity = AppClass.fromModel(param);
		entity.setGroupCode(DmpParameterStorage.getGroupCode());
		appClassRepository.save(entity);
		response.add("id", entity.getId());
		return response;
	}

	@Transactional
	public MapResponse update(AppClassModel param) {
		MapResponse response = new MapResponse();
		super.checkParam(response, RwdConstants.QUERY_GROUPCODE);
		if (!response.success()) {
			return response;
		}
		// TODO
		AppClass entity = AppClass.fromModel(param);
		appClassRepository.save(entity);
		response.add("id", entity.getId());
		return response;
	}

	@Transactional
	public MapResponse delete(AppClassModel param) {
		MapResponse response = new MapResponse();
		super.checkParam(response, RwdConstants.QUERY_GROUPCODE);
		if (!response.success()) {
			return response;
		}
		// TODO
		appClassRepository.deleteById(param.getId());
		return response;
	}
}

