package com.persagy.dmp.rwd.service;

import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JpaUtils;
import com.persagy.common.web.BaseResponse;
import com.persagy.dmp.config.DmpParameterStorage;
import com.persagy.dmp.rwd.config.RwdConstants;
import com.persagy.dmp.rwd.repository.*;
import com.querydsl.jpa.impl.JPAQueryFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.persistence.EntityManager;
import java.util.Map;

/**
 * @author: yaoll
 * @date: 2020-09-16
 * @verison: 1.0
 */
@Service
public class BaseService {

    @Autowired
    protected JpaUtils jpaUtils;

    @Autowired
    protected EntityManager entityManager;

    @Autowired
    protected JPAQueryFactory jpaQueryFactory;

    @Autowired
    protected CriteriaUtils criteriaUtils;

    @Autowired
    protected AppClassRepository appClassRepository;

    @Autowired
    protected AppFuncidRepository appFuncidRepository;

    @Autowired
    protected ClassDefRepository classDefRepository;

    @Autowired
    protected FuncidDefRepository funcidDefRepository;

    @Autowired
    protected GraphDefRepository graphDefRepository;

    @Autowired
    protected GraphInstanceRepository graphInstanceRepository;

    @Autowired
    protected ObjectInstanceRepository objectInstanceRepository;

    @Autowired
    protected RelationInstanceRepository relationInstanceRepository;

    @Autowired
    protected SchemaRepository schemaRepository;

    @Autowired
    protected SchemaClassRepository schemaClassRepository;

    @Autowired
    protected SchemaFuncidRepository schemaFuncidRepository;

    @Autowired
    protected SchemaProjectRepository schemaProjectRepository;

    protected static class OrgParam {
        String groupCode;
        String schemaId;
        String projectId;
        String appId;
        String userId;
    }

    protected <T> OrgParam prepareParam(BaseResponse<T> resp, String... requestParams) {
        OrgParam param = new OrgParam();
        param.groupCode = DmpParameterStorage.getGroupCode();
        param.schemaId = DmpParameterStorage.getSchemaId();
        param.projectId = DmpParameterStorage.getProjectId();
        param.appId = DmpParameterStorage.getAppId();
        param.userId = DmpParameterStorage.getUserId();

        if (requestParams == null) {
            return param;
        }
        checkParam(resp, requestParams);
        return param;
    }

    protected void checkParam(BaseResponse resp, String... requestParams) {
        if (requestParams == null) {
            return;
        }
        for (String paramName : requestParams) {
            if (RwdConstants.QUERY_GROUPCODE.equals(paramName) && DmpParameterStorage.getGroupCode() == null) {
                resp.setFail("集团编码不可为空");
                return;
            }

            if (RwdConstants.QUERY_PROJECTID.equals(paramName) && DmpParameterStorage.getProjectId() == null) {
                resp.setFail("项目ID不可为空");
                return;
            }

            if (RwdConstants.QUERY_USERID.equals(paramName) && DmpParameterStorage.getUserId() == null) {
                resp.setFail("用户ID不可为空");
                return;
            }

            if (RwdConstants.QUERY_APPID.equals(paramName) && DmpParameterStorage.getAppId() == null) {
                resp.setFail("应用ID不可为空");
                return;
            }
        }
    }

    protected String getInfoCodeValue(String groupCode, String projectId, String objectId, String infoCode) {
        if ("classCode".equals(infoCode)) {
            String sql = "select class_code from rwd_object where group_code = ? and project_id = ? and id = ?";
            Map<String, Object> result = jpaUtils.queryOne(sql, groupCode, projectId, objectId);
            if (result != null && result.size() > 0) {
                return (String) result.get("class_code");
            } else {
                return null;
            }
        } else {
            String sql = "select class_code, json_unquote(json_extract(infos, '$." + infoCode + "')) meter from rwd_object where group_code = ? and project_id = ? and id = ?";
            Map<String, Object> result = jpaUtils.queryOne(sql, groupCode, projectId, objectId);
            if (result != null && result.size() > 0) {
                return (String) result.get("meter");
            } else {
                return null;
            }
        }
    }


}
