package com.persagy.dmp.rwd.service;

import com.fasterxml.jackson.databind.node.JsonNodeFactory;
import com.persagy.common.web.MapResponse;
import com.persagy.dmp.config.DmpParameterStorage;
import com.persagy.dmp.rwd.entity.GraphDef;
import com.persagy.dmp.rwd.entity.GraphInstance;
import com.persagy.dmp.rwd.entity.ObjectInstance;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.sql.DataSource;
import javax.transaction.Transactional;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Collections;
import java.util.Date;
import java.util.List;

/**
 * @author: yaoll
 * @date: 2020-09-29
 * @verison: 1.0
 */
@Slf4j
@Service
public class InitService extends BaseService {

	@Autowired
	private DataSource dataSource;


	@Transactional
	public MapResponse initGroup() {
		MapResponse resp = new MapResponse();
		String groupCode = DmpParameterStorage.getGroupCode();
		String sql = null;
		Connection connection = null;
		Statement statement = null;
		try {
			connection = dataSource.getConnection();
			statement = connection.createStatement();
			// 创建类型表
			sql = getCreateGroupClassSql(groupCode);
			statement.executeUpdate(sql);

			// 创建信息点表
			sql = getCreateGroupFuncidSql(groupCode);
			statement.executeUpdate(sql);

			sql = getCreateSchemaSql(groupCode);
			statement.executeUpdate(sql);

			sql = getCreateSchemaClassSql(groupCode);
			statement.executeUpdate(sql);

			sql = getCreateSchemaFuncidSql(groupCode);
			statement.executeUpdate(sql);

			sql = getCreateSchemaProjectSql(groupCode);
			statement.executeUpdate(sql);

			sql = getCreateAppClassSql(groupCode);
			statement.executeUpdate(sql);

			sql = getCreateAppFuncidSql(groupCode);
			statement.executeUpdate(sql);
		} catch (SQLException e) {
			e.printStackTrace();
			resp.setFail(e.getMessage());
		} finally {
			close(statement);
			close(connection);
		}
		return resp;
	}

	@Transactional
	public MapResponse initProject(String projectName) {
		MapResponse resp = new MapResponse();
		String groupCode = DmpParameterStorage.getGroupCode();
		String projectId = DmpParameterStorage.getProjectId();
		String sql = null;
		Connection connection = null;
		Statement statement = null;
		try {
			connection = dataSource.getConnection();
			statement = connection.createStatement();
			// 创建对象表
			sql = getCreateObjectSql(groupCode, projectId);
			log.info("create rwd_object");
			System.out.println(sql);
			statement.executeUpdate(sql);

			// 创建图实例表
			sql = getCreateGraphSql(groupCode, projectId);
			log.info("create rwd_graph");
			System.out.println(sql);
			statement.executeUpdate(sql);

			// 创建关系表
			sql = getCreateRelationSql(groupCode, projectId);
			log.info("create rwd_relation");
			System.out.println(sql);
			statement.executeUpdate(sql);

			// 插入项目对象
			ObjectInstance pro = new ObjectInstance();
			pro.setId(projectId);
			pro.setGroupCode(groupCode);
			pro.setProjectId(projectId);
			pro.setName(projectName);
			pro.setObjType("project");
			pro.setClassCode("project");
			pro.setInfos(JsonNodeFactory.instance.objectNode());
			pro.setGrouping(1);
			pro.setVirtualInfoCodes(Collections.emptyList());
			pro.setValid(1);
			pro.setCreateTime(new Date());
			pro.setCreateUser(DmpParameterStorage.getUserId());
			pro.setCreateApp(DmpParameterStorage.getAppId() == null ? "dmp" : DmpParameterStorage.getAppId());
			objectInstanceRepository.save(pro);

			// 插入图实例数据
			List<GraphDef> all = graphDefRepository.findAll();
			for (GraphDef def : all) {
				GraphInstance graph = new GraphInstance();
				graph.setId("Gt" + def.getCode() + "001");
				graph.setGroupCode(groupCode);
				graph.setProjectId(projectId);
				graph.setGraphCode(def.getCode());
				graphInstanceRepository.save(graph);
			}

		} catch (SQLException e) {
			e.printStackTrace();
			resp.setFail(e.getMessage());
		} finally {
			close(statement);
			close(connection);
		}
		return resp;
	}

	private String getCreateGroupClassSql(String groupCode) {
		StringBuilder sql = new StringBuilder();
		sql.append(" CREATE TABLE `rwd_def_class_" + groupCode.toLowerCase() + "` (       \n");
		sql.append("   `id` varchar(40) NOT NULL,                                         \n");
		sql.append("   `code` varchar(40) NOT NULL ,                                      \n");
		sql.append("   `obj_type` varchar(40) DEFAULT NULL ,                              \n");
		sql.append("   `name` varchar(40) NOT NULL ,                                      \n");
		sql.append("   `type` varchar(40) NOT NULL DEFAULT 'common',                      \n");
		sql.append("   `group_code` varchar(40) NOT NULL DEFAULT '0',                     \n");
		sql.append("   `project_id` varchar(40) NOT NULL DEFAULT '0',                     \n");
		sql.append("   `alias_code` varchar(40) NOT NULL ,                                \n");
		sql.append("   `alias_name` varchar(40) NOT NULL ,                                \n");
		sql.append("   `major_code` varchar(40) DEFAULT NULL ,                            \n");
		sql.append("   `system_code` varchar(40) DEFAULT NULL ,                           \n");
		sql.append("   `equipment_code` varchar(40) DEFAULT NULL ,                        \n");
		sql.append("   `parent_code` varchar(40) DEFAULT NULL ,                           \n");
		sql.append("   PRIMARY KEY (`id`),                                                \n");
		sql.append("   KEY `code` (`code`),                                               \n");
		sql.append("   KEY `alias_code` (`alias_code`),                                   \n");
		sql.append("   KEY `parent_code` (`parent_code`),                                 \n");
		sql.append("   KEY `major_code` (`major_code`),                                   \n");
		sql.append("   KEY `type` (`type`,`group_code`,`project_id`,`code`)               \n");
		sql.append(" ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4                              \n");
		return sql.toString();
	}

	private String getCreateGroupFuncidSql(String groupCode) {
		StringBuilder sql = new StringBuilder();
		sql.append(" CREATE TABLE `rwd_def_funcid_" + groupCode.toLowerCase() + "` (              \n");
		sql.append("   `id` varchar(40) NOT NULL,                                                 \n");
		sql.append("   `code` varchar(100) NOT NULL,                                              \n");
		sql.append("   `orig_code` varchar(100) DEFAULT NULL,                                     \n");
		sql.append("   `name` varchar(40) DEFAULT NULL,                                           \n");
		sql.append("   `alias_code` varchar(100) DEFAULT NULL,                                    \n");
		sql.append("   `alias_name` varchar(100) DEFAULT NULL,                                    \n");
		sql.append("   `class_code` varchar(40) DEFAULT NULL,                                     \n");
		sql.append("   `type` varchar(40) NOT NULL DEFAULT 'common',                              \n");
		sql.append("   `group_code` varchar(40) NOT NULL DEFAULT '0',                             \n");
		sql.append("   `project_id` varchar(40) NOT NULL DEFAULT '0',                             \n");
		sql.append("   `category` varchar(40) DEFAULT NULL,                                       \n");
		sql.append("   `first_tag` varchar(40) DEFAULT NULL,                                      \n");
		sql.append("   `second_tag` varchar(40) DEFAULT NULL,                                     \n");
		sql.append("   `priority` varchar(10) DEFAULT NULL,                                       \n");
		sql.append("   `input_mode` varchar(40) DEFAULT NULL,                                     \n");
		sql.append("   `unit` varchar(40) DEFAULT NULL,                                           \n");
		sql.append("   `data_type` varchar(40) DEFAULT NULL,                                      \n");
		sql.append("   `orig_data_type` varchar(40) DEFAULT NULL,                                 \n");
		sql.append("   `is_multiple` int(11) DEFAULT NULL,                                        \n");
		sql.append("   `is_region` int(11) DEFAULT NULL,                                          \n");
		sql.append("   `formater` varchar(40) DEFAULT NULL,                                       \n");
		sql.append("   `data_source` json DEFAULT NULL,                                           \n");
		sql.append("   `orig_data_source` text,                                                   \n");
		sql.append("   `note` text,                                                               \n");
		sql.append("   `sub_flag` int(11) DEFAULT NULL,                                           \n");
		sql.append("   `weak_point` int(11) DEFAULT NULL,                                         \n");
		sql.append("   PRIMARY KEY (`id`),                                                        \n");
		sql.append("   KEY `class_code` (`class_code`,`code`),                                    \n");
		sql.append("   KEY `class_code_2` (`class_code`,`orig_code`)                              \n");
		sql.append(" ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4                                      \n");
		return sql.toString();
	}

	private String getCreateSchemaSql(String groupCode) {
		StringBuilder sql = new StringBuilder();
		sql.append(" CREATE TABLE `rwd_schema_" + groupCode.toLowerCase() + "` (                                  \n");
		sql.append("   `id` varchar(40) NOT NULL,                                                   \n");
		sql.append("   `name` varchar(40) DEFAULT NULL,                                             \n");
		sql.append("   `group_code` varchar(40) NOT NULL,                                           \n");
		sql.append("   `remark` varchar(255) DEFAULT NULL,                                          \n");
		sql.append("   `status` int(11) DEFAULT NULL,                                               \n");
		sql.append("   PRIMARY KEY (`id`),                                                          \n");
		sql.append("   KEY `group_code` (`group_code`)                                              \n");
		sql.append(" ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4                                        \n");
		return sql.toString();
	}
	private String getCreateSchemaClassSql(String groupCode) {
		StringBuilder sql = new StringBuilder();
		sql.append(" CREATE TABLE `rwd_schema_class_" + groupCode.toLowerCase() + "` (                            \n");
		sql.append("   `id` varchar(40) NOT NULL,                                                   \n");
		sql.append("   `schema_id` varchar(40) DEFAULT NULL,                                        \n");
		sql.append("   `group_code` varchar(40) NOT NULL,                                           \n");
		sql.append("   `class_code` varchar(40) DEFAULT NULL,                                       \n");
		sql.append("   PRIMARY KEY (`id`),                                                          \n");
		sql.append("   KEY `schema_id` (`schema_id`)                                                \n");
		sql.append(" ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4                                        \n");
		return sql.toString();
	}
	private String getCreateSchemaFuncidSql(String groupCode) {
		StringBuilder sql = new StringBuilder();
		sql.append(" CREATE TABLE `rwd_schema_funcid_" + groupCode.toLowerCase() + "` (                           \n");
		sql.append("   `id` varchar(40) NOT NULL,                                                   \n");
		sql.append("   `schema_id` varchar(40) DEFAULT NULL,                                        \n");
		sql.append("   `group_code` varchar(40) NOT NULL,                                           \n");
		sql.append("   `class_code` varchar(40) DEFAULT NULL,                                       \n");
		sql.append("   `info_code` varchar(100) DEFAULT NULL,                                       \n");
		sql.append("   PRIMARY KEY (`id`),                                                          \n");
		sql.append("   KEY `schema_id` (`schema_id`,`class_code`,`info_code`)                       \n");
		sql.append(" ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4                                        \n");
		return sql.toString();
	}
	private String getCreateSchemaProjectSql(String groupCode) {
		StringBuilder sql = new StringBuilder();
		sql.append(" CREATE TABLE `rwd_schema_project_" + groupCode.toLowerCase() + "` (                          \n");
		sql.append("   `id` varchar(40) NOT NULL,                                                   \n");
		sql.append("   `schema_id` varchar(40) DEFAULT NULL,                                        \n");
		sql.append("   `group_code` varchar(40) NOT NULL,                                           \n");
		sql.append("   `project_id` varchar(40) DEFAULT NULL,                                       \n");
		sql.append("   PRIMARY KEY (`id`),                                                          \n");
		sql.append("   KEY `schema_id` (`schema_id`),                                               \n");
		sql.append("   KEY `project_id` (`project_id`)                                              \n");
		sql.append(" ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4                                        \n");
		return sql.toString();
	}
	private String getCreateAppClassSql(String groupCode) {
		StringBuilder sql = new StringBuilder();
		sql.append(" CREATE TABLE `rwd_app_class_" + groupCode.toLowerCase() + "` (                              \n");
		sql.append("   `id` varchar(40) NOT NULL,                                                   \n");
		sql.append("   `app_id` varchar(40) DEFAULT NULL,                                           \n");
		sql.append("   `class_code` varchar(40) DEFAULT NULL,                                       \n");
		sql.append("   `group_code` varchar(40) NOT NULL,                                           \n");
		sql.append("   `project_id` varchar(40) DEFAULT NULL,                                       \n");
		sql.append("   PRIMARY KEY (`id`),                                                          \n");
		sql.append("   KEY `app_id` (`app_id`,`class_code`)                                         \n");
		sql.append(" ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4                                        \n");
		return sql.toString();
	}
	private String getCreateAppFuncidSql(String groupCode) {
		StringBuilder sql = new StringBuilder();
		sql.append(" CREATE TABLE `rwd_app_funcid_" + groupCode.toLowerCase() + "` (                              \n");
		sql.append("   `id` varchar(40) NOT NULL,                                                   \n");
		sql.append("   `app_id` varchar(40) DEFAULT NULL,                                           \n");
		sql.append("   `group_code` varchar(40) NOT NULL,                                           \n");
		sql.append("   `project_id` varchar(40) DEFAULT NULL,                                       \n");
		sql.append("   `class_code` varchar(40) DEFAULT NULL,                                       \n");
		sql.append("   `info_code` varchar(100) DEFAULT NULL,                                       \n");
		sql.append("   PRIMARY KEY (`id`),                                                          \n");
		sql.append("   KEY `app_id` (`app_id`,`project_id`,`class_code`,`info_code`)                \n");
		sql.append(" ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4                                        \n");
		return sql.toString();
	}

	private String getCreateObjectSql(String groupCode, String projectId) {
		StringBuilder sql = new StringBuilder();
		sql.append(" CREATE TABLE `rwd_object_" + groupCode.toLowerCase() + "_" + projectId.toLowerCase() + "` (  \n");
		sql.append("   `id` varchar(100) NOT NULL,                                                    \n");
		sql.append("   `name` varchar(100) DEFAULT NULL COMMENT '对象名称',                            \n");
		sql.append("   `local_id` varchar(40) DEFAULT NULL COMMENT '对象本地编码',                     \n");
		sql.append("   `local_name` varchar(100) DEFAULT NULL COMMENT '对象本地名称',                  \n");
		sql.append("   `group_code` varchar(40) NOT NULL,                                         \n");
		sql.append("   `project_id` varchar(40) NOT NULL COMMENT '项目id',                            \n");
		sql.append("   `obj_type` varchar(40) NOT NULL COMMENT '对象分类',                             \n");
		sql.append("   `class_code` varchar(40) NOT NULL COMMENT '对象类型编码',                       \n");
		sql.append("   `grouping` int(11) NOT NULL DEFAULT '1' COMMENT '1单个对象,2对象组',            \n");
		sql.append("   `valid` int(11) NOT NULL DEFAULT '1' COMMENT '对象状态:1有效,0无效',            \n");
		sql.append("   `infos` json DEFAULT NULL COMMENT '边类型编码',                                \n");
		sql.append("   `virtual_info_codes` json DEFAULT NULL COMMENT '虚点清单',                     \n");
		sql.append("   `create_time` timestamp NOT NULL default current_timestamp COMMENT '创建时间',                           \n");
		sql.append("   `update_time` timestamp NOT NULL default current_timestamp on update current_timestamp COMMENT '最后更新时间',               \n");
		sql.append("   `create_user` varchar(40) DEFAULT NULL COMMENT '创建用户',                     \n");
		sql.append("   `update_user` varchar(40) DEFAULT NULL COMMENT '最后更新用户',                  \n");
		sql.append("   `create_app` varchar(40) DEFAULT NULL COMMENT '创建应用',                      \n");
		sql.append("   `update_app` varchar(40) DEFAULT NULL COMMENT '最后更新应用',                   \n");
		sql.append("   PRIMARY KEY (`id`) USING BTREE,                                               \n");
		sql.append("   KEY `project_id_class_code` (`class_code`,`create_time`),        \n");
		sql.append("   KEY `project_id_create_time` (`create_time`),                    \n");
		sql.append("   KEY `project_id_update_time` (`update_time`)                     \n");
		sql.append(" ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='对象实例表'                     \n");
		return sql.toString();
	}

	private String getCreateGraphSql(String groupCode, String projectId) {
		StringBuilder sql = new StringBuilder();
		sql.append(" CREATE TABLE `rwd_graph_" + groupCode.toLowerCase() + "_" + projectId.toLowerCase() + "` (                        \n");
		sql.append("   `id` varchar(40) NOT NULL,                                        \n");
		sql.append("   `group_code` varchar(40) NOT NULL,                            \n");
		sql.append("   `project_id` varchar(40) NOT NULL,                                \n");
		sql.append("   `graph_code` varchar(40) NOT NULL,                                \n");
		sql.append("   `infos` json DEFAULT NULL,                                        \n");
		sql.append("   PRIMARY KEY (`id`),                                               \n");
		sql.append("   KEY `graph_code` (`graph_code`)                                   \n");
		sql.append(" ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='图实例表'           \n");
		return sql.toString();
	}

	private String getCreateRelationSql(String groupCode, String projectId) {
		StringBuilder sql = new StringBuilder();
		sql.append(" CREATE TABLE `rwd_relation_" + groupCode.toLowerCase() + "_" + projectId.toLowerCase() + "` (                        \n");
		sql.append("   `id` varchar(40) NOT NULL,                                           \n");
		sql.append("   `group_code` varchar(40) NOT NULL,                               \n");
		sql.append("   `project_id` varchar(40) NOT NULL,                                   \n");
		sql.append("   `graph_id` varchar(40) DEFAULT NULL,                                 \n");
		sql.append("   `graph_code` varchar(40) DEFAULT NULL,                               \n");
		sql.append("   `rel_code` varchar(40) DEFAULT NULL,                                 \n");
		sql.append("   `obj_from` varchar(100) DEFAULT NULL,                                \n");
		sql.append("   `obj_to` varchar(100) DEFAULT NULL,                                  \n");
		sql.append("   `status` int(11) NOT NULL DEFAULT '1',                               \n");
		sql.append("   `create_time` timestamp NOT NULL default current_timestamp COMMENT '创建时间',                           \n");
		sql.append("   `update_time` timestamp NOT NULL default current_timestamp on update current_timestamp COMMENT '最后更新时间',               \n");
		sql.append("   `create_user` varchar(40) DEFAULT NULL,                              \n");
		sql.append("   `update_user` varchar(40) DEFAULT NULL,                              \n");
		sql.append("   `create_app` varchar(40) DEFAULT NULL,                               \n");
		sql.append("   `update_app` varchar(40) DEFAULT NULL,                               \n");
		sql.append("   PRIMARY KEY (`id`) USING BTREE,                                      \n");
		sql.append("   KEY `gc_r_f_t` (`graph_code`,`rel_code`,`obj_from`,`obj_to`),        \n");
		sql.append("   KEY `gi_r_f_t` (`graph_id`,`rel_code`,`obj_from`,`obj_to`),          \n");
		sql.append("   KEY `gc_r_t` (`graph_code`,`rel_code`,`obj_to`),                     \n");
		sql.append("   KEY `gi_r_t` (`graph_id`,`rel_code`,`obj_to`)                        \n");
		sql.append(" ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4                                \n");
		return sql.toString();
	}

	private void close(Statement statement) {
		if (statement != null) {
			try {
				statement.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
	}

	private void close(Connection connection) {
		if (connection != null) {

			try {
				connection.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
	}

}
