package com.persagy.dmp.rwd.service;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.JsonNodeFactory;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.date.DateUtils;
import com.persagy.common.json.JacksonMapper;
import com.persagy.common.web.MapResponse;
import com.persagy.dmp.config.DmpParameterStorage;
import com.persagy.dmp.rwd.config.RwdConstants;
import com.persagy.dmp.rwd.entity.ObjectInstance;
import com.persagy.dmp.rwd.entity.QObjectInstance;
import com.persagy.dmp.rwd.entity.QSchemaProject;
import com.persagy.dmp.rwd.entity.SchemaProject;
import com.persagy.dmp.rwd.enums.FuncidCategory;
import com.persagy.dmp.rwd.enums.ObjType;
import com.persagy.dmp.rwd.funcid.parse.parser.InfoCodeParser;
import com.persagy.dmp.rwd.funcid.parse.parser.InfoCodeParserFactory;
import com.persagy.dmp.rwd.funcid.parse.parser.ParserContext;
import com.persagy.dmp.rwd.model.ClassDefModel;
import com.persagy.dmp.rwd.model.DmpMessage;
import com.persagy.dmp.rwd.model.FuncidDefModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.*;

/**
 * @author: yaoll
 * @date: 2020-09-22
 * @verison: 1.0
 */
@Service
public class ObjectInstanceCreateService extends BaseService {

	@Autowired
	private FuncidDefService funcidDefService;

	@Autowired
	private ClassDefService classDefService;

	@Autowired
	private InfoCodeParserFactory infoCodeParserFactory;

	@Transactional
	public MapResponse create(ArrayNode paramList) {

		MapResponse response = new MapResponse();
		if (paramList == null || paramList.size() == 0) {
			return response;
		}

		OrgParam orgParam = super.prepareParam(response, RwdConstants.QUERY_GROUPCODE, RwdConstants.QUERY_PROJECTID);
		if (!response.success()) {
			return response;
		}
		QSchemaProject pqt = QSchemaProject.schemaProject;
		Optional<SchemaProject> one = schemaProjectRepository.findOne(pqt.projectId.eq(orgParam.projectId).and(pqt.groupCode.eq(orgParam.groupCode)));
		if (one.isPresent()) {
			orgParam.schemaId = one.get().getSchemaId();
		} else {
			response.setFail("当前项目未配置集团方案");
			return response;
		}

		// 类型缓存, TODO 判断应用是否订阅了该类型
		Map<String, ClassDefModel> classCache = new HashMap<>();
		// 信息点缓存
		Map<String, List<FuncidDefModel>> funcidCache = new HashMap<>();

		// 校验
		for (JsonNode jsonNode : paramList) {
			ObjectNode param = (ObjectNode) jsonNode;
			check(response, orgParam, classCache, funcidCache, param);
			if (!response.success()) {
				return response;
			}
		}
		// 赋值
		String now = DateUtils.format(new Date());
		for (JsonNode jsonNode : paramList) {
			ObjectNode param = (ObjectNode) jsonNode;
			String classCode = JacksonMapper.getString(param, RwdConstants.OBJECT_CLASSCODE);
			ClassDefModel classDefModel = classCache.get(classCode);
			ObjectInstance entity;
			String id = JacksonMapper.getString(param, RwdConstants.OBJECT_ID);
			boolean isNew = true;
			if (id == null) {
				// 数据不存在
				entity = new ObjectInstance();
				entity.setId(prepareObjectId(DmpParameterStorage.getProjectId(), classDefModel));
				entity.setClassCode(classCode);
				entity.setGroupCode(orgParam.groupCode);
				entity.setProjectId(orgParam.projectId);
				entity.setValid(RwdConstants.OBJECT_VALID_VALID);
				entity.setName(JacksonMapper.getString(param, RwdConstants.OBJECT_NAME));
				entity.setLocalId(JacksonMapper.getString(param, RwdConstants.OBJECT_LOCALID));
				entity.setLocalName(JacksonMapper.getString(param, RwdConstants.OBJECT_LOCALNAME));
				entity.setObjType(classDefModel.getObjType().name());
				entity.setGrouping(JacksonMapper.getInteger(param, RwdConstants.OBJECT_GROUPING, 1));
				entity.setVirtualInfoCodes(getStringList(param, RwdConstants.OBJECT_VIRTUAL_INFO_CODES));
				entity.setInfos(JsonNodeFactory.instance.objectNode());
				entity.setCreateTime(new Date());
				entity.setCreateUser(JacksonMapper.getString(param, RwdConstants.OBJECT_CREATE_USER));
				entity.setCreateApp(JacksonMapper.getString(param, RwdConstants.OBJECT_CREATE_APP, orgParam.appId));
			} else {
				QObjectInstance qt = QObjectInstance.objectInstance;
				Optional<ObjectInstance> opt = objectInstanceRepository.findOne(qt.groupCode.eq(orgParam.groupCode).and(qt.projectId.eq(orgParam.projectId).and(qt.id.eq(id))));
				if (!opt.isPresent()) {
					// 数据不存在
					entity = new ObjectInstance();
					entity.setId(id);
					entity.setClassCode(classCode);
					entity.setGroupCode(orgParam.groupCode);
					entity.setProjectId(orgParam.projectId);
					entity.setValid(RwdConstants.OBJECT_VALID_VALID);
					entity.setName(JacksonMapper.getString(param, RwdConstants.OBJECT_NAME));
					entity.setLocalId(JacksonMapper.getString(param, RwdConstants.OBJECT_LOCALID));
					entity.setLocalName(JacksonMapper.getString(param, RwdConstants.OBJECT_LOCALNAME));
					entity.setObjType(classDefModel.getObjType().name());
					entity.setGrouping(JacksonMapper.getInteger(param, RwdConstants.OBJECT_GROUPING, 1));
					entity.setVirtualInfoCodes(getStringList(param, RwdConstants.OBJECT_VIRTUAL_INFO_CODES));
					entity.setInfos(JsonNodeFactory.instance.objectNode());
					entity.setCreateTime(new Date());
					entity.setCreateUser(JacksonMapper.getString(param, RwdConstants.OBJECT_CREATE_USER));
					entity.setCreateApp(JacksonMapper.getString(param, RwdConstants.OBJECT_CREATE_APP, orgParam.appId));
				} else {
					// 数据已存在
					isNew = false;
					entity = opt.get();
					String name = JacksonMapper.getString(param, RwdConstants.OBJECT_NAME);
					if (name != null) {
						entity.setName(name);
					}
					String localId = JacksonMapper.getString(param, RwdConstants.OBJECT_LOCALID);
					if (localId != null) {
						entity.setLocalId(localId);
					}
					String localName = JacksonMapper.getString(param, RwdConstants.OBJECT_LOCALNAME);
					if (localName != null) {
						entity.setLocalName(localName);
					}
					if (param.has(RwdConstants.OBJECT_VIRTUAL_INFO_CODES)) {
						entity.setVirtualInfoCodes(getStringList(param, RwdConstants.OBJECT_VIRTUAL_INFO_CODES));
					}
					entity.setUpdateTime(new Date());
					entity.setUpdateUser(JacksonMapper.getString(param, RwdConstants.OBJECT_CREATE_USER));
					entity.setUpdateApp(JacksonMapper.getString(param, RwdConstants.OBJECT_CREATE_APP, orgParam.appId));
				}
			}

			// 已订阅的信息点
			List<FuncidDefModel> funcidList = funcidCache.get(classCode);
			ObjectNode values = entity.getInfos();
			for (FuncidDefModel funcidDefModel : funcidList) {
				JsonNode source = null;
				String code = funcidDefModel.getCode();
				if (RwdConstants.isBasicColumn(code)) {
					continue;
				}
				if (param.has(code)) {
					source = param.get(code);
				} else if (param.has(funcidDefModel.getOrigCode())) {
					source = param.get(funcidDefModel.getOrigCode());
				}
				if (source != null) {
					if (funcidDefModel.getCategory() == FuncidCategory.STATIC) {
						InfoCodeParser parser = infoCodeParserFactory.getParser(funcidDefModel.getDataType());
						ParserContext parserContext = new ParserContext(funcidDefModel, source);
						parser.checkValue(parserContext);
						Object value = parserContext.getValue();
						values.putPOJO(code, value);
					} else {
						// 动态值不需校验
						values.set(code, source);
					}
				}
			}

			objectInstanceRepository.save(entity);
			entityManager.flush();
			entityManager.clear();

			String msgType = isNew ? RwdConstants.MESSAGE_INSTANCE_OBJECT_CREATE : RwdConstants.MESSAGE_INSTANCE_OBJECT_UPDATE;
			DmpMessage message = new DmpMessage(msgType, entity.getId());
			message.setGroupCode(orgParam.groupCode);
			message.setProjectId(orgParam.projectId);
			message.setStr1(entity.getObjType());
			message.setStr2(entity.getClassCode());
			message.setAppId(orgParam.appId);
			message.setUserId(DmpParameterStorage.getUserId());
			response.add(message);

		}
		return response;
	}

	private MapResponse check(MapResponse response, OrgParam orgParam, Map<String, ClassDefModel> classCache, Map<String, List<FuncidDefModel>> funcidCache, ObjectNode param) {
		String classCode = JacksonMapper.getString(param, RwdConstants.OBJECT_CLASSCODE);
		if (classCode == null) {
			response.setFail("[classCode] is required");
			return response;
		}

		if (!classCache.containsKey(classCode)) {
			JacksonCriteria criteria = JacksonCriteria.newInstance();
			criteria.add("code", classCode);
			List<ClassDefModel> data = classDefService.queryClass(orgParam.groupCode, orgParam.schemaId, orgParam.projectId, orgParam.appId, false, criteria).getData();
			if (data.size() > 0) {
				classCache.put(classCode, data.get(0));
			}
		}
		if (!classCache.containsKey(classCode)) {
			response.setFail("未订阅类型[" + classCode + "]或类型错误");
			return response;
		}
		if (!funcidCache.containsKey(classCode)) {
			JacksonCriteria criteria = JacksonCriteria.newInstance();
			criteria.add("classCode", classCode);
			List<FuncidDefModel> data = funcidDefService.queryFuncid(orgParam.groupCode, orgParam.schemaId, orgParam.projectId, orgParam.appId, criteria).getData();
			funcidCache.put(classCode, data);
		}
		// 已订阅的信息点
		List<FuncidDefModel> funcidList = funcidCache.get(classCode);
		for (FuncidDefModel funcidDefModel : funcidList) {
			JsonNode source = null;
			String code = funcidDefModel.getCode();
			if (RwdConstants.isBasicColumn(code)) {
				continue;
			}
			if (param.has(code)) {
				source = param.get(code);
			} else if (param.has(funcidDefModel.getOrigCode())) {
				source = param.get(funcidDefModel.getOrigCode());
			}
			if (source != null) {
				if (funcidDefModel.getCategory() == FuncidCategory.STATIC) {
					InfoCodeParser parser = infoCodeParserFactory.getParser(funcidDefModel.getDataType());
					ParserContext parserContext = new ParserContext(funcidDefModel, source);
					parser.checkValue(parserContext);
					if (parserContext.isHasError()) {
						response.setFail(parserContext.getError().toString());
						return response;
					}
				}
			}
		}
		return response;
	}


	private String prepareObjectId(String projectId, ClassDefModel model) {
		ObjType objType = model.getObjType();
		if (objType == ObjType.project) {
			return projectId;
		} else if (objType == ObjType.building) {
			return "Bd" + projectId.substring(2) + UUID.randomUUID().toString().replace("-", "");
		} else if (objType == ObjType.floor) {
			return "Fl" + projectId.substring(2) + UUID.randomUUID().toString().replace("-", "");
		} else if (objType == ObjType.space) {
			return "Sp" + projectId.substring(2) + UUID.randomUUID().toString().replace("-", "");
		} else if (objType == ObjType.system) {
			return "Sy" + projectId.substring(2) + UUID.randomUUID().toString().replace("-", "");
		} else if (objType == ObjType.equipment) {
			return "Eq" + projectId.substring(2) + UUID.randomUUID().toString().replace("-", "");
		} else if (objType == ObjType.shaft) {
			return "Sh" + projectId.substring(2) + UUID.randomUUID().toString().replace("-", "");
		} else if (objType == ObjType.component) {
			return "Ec" + projectId.substring(2) + UUID.randomUUID().toString().replace("-", "");
		} else if (objType == ObjType.virtual) {
			return "Vo" + projectId.substring(2) + UUID.randomUUID().toString().replace("-", "");
		}
		return UUID.randomUUID().toString().replace("-", "");
	}

	public static final List<String> getStringList(ObjectNode obj, String key) {
		ArrayNode array = JacksonMapper.getArray(obj, key, JsonNodeFactory.instance.arrayNode());
		List<String> list = new ArrayList<>(array.size());
		for (JsonNode jsonNode : array) {
			list.add(jsonNode.asText());
		}
		return list;
	}

}
