package com.persagy.dmp.rwd.service;

import com.persagy.common.web.MapResponse;
import com.persagy.dmp.config.DmpParameterStorage;
import com.persagy.dmp.rwd.config.RwdConstants;
import com.persagy.dmp.rwd.entity.ObjectInstance;
import com.persagy.dmp.rwd.entity.QObjectInstance;
import com.persagy.dmp.rwd.model.DmpMessage;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.List;
import java.util.Optional;

/**
 * @author: yaoll
 * @date: 2020-09-22
 * @verison: 1.0
 */
@Service
public class ObjectInstanceDeleteService extends BaseService {

	@Transactional
	public MapResponse delete(List<String> idList) {

		MapResponse response = new MapResponse();
		super.checkParam(response, RwdConstants.QUERY_GROUPCODE, RwdConstants.QUERY_PROJECTID);
		if (!response.success()) {
			return response;
		}
		if (idList == null || idList.size() == 0) {
			return response;
		}

		QObjectInstance qt = QObjectInstance.objectInstance;
		for (String id : idList) {
			Optional<ObjectInstance> one = objectInstanceRepository.findOne(qt.id.eq(id)
					.and(qt.groupCode.eq(DmpParameterStorage.getGroupCode()))
					.and(qt.projectId.eq(DmpParameterStorage.getProjectId()))
					.and(qt.valid.eq(1)));
			if (one.isPresent()) {
				ObjectInstance entity = one.get();
				entity.setValid(RwdConstants.OBJECT_VALID_INAVALID);
				entity.setUpdateApp(DmpParameterStorage.getAppId());
				entity.setUpdateUser(DmpParameterStorage.getUserId());

				objectInstanceRepository.save(entity);
				entityManager.flush();
				entityManager.clear();

				// TODO 关系处理

				// 消息通知
				DmpMessage message = new DmpMessage(RwdConstants.MESSAGE_INSTANCE_OBJECT_DELETE, id);
				message.setGroupCode(DmpParameterStorage.getGroupCode());
				message.setProjectId(DmpParameterStorage.getProjectId());
				message.setStr1(entity.getObjType());
				message.setStr2(entity.getClassCode());
				message.setAppId(DmpParameterStorage.getAppId());
				message.setUserId(DmpParameterStorage.getUserId());
				response.add(message);
			}
		}
		return response;
	}

}
