package com.persagy.dmp.rwd.service;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.JsonNodeFactory;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.date.DateUtils;
import com.persagy.common.json.JacksonMapper;
import com.persagy.common.web.MapResponse;
import com.persagy.dmp.config.DmpParameterStorage;
import com.persagy.dmp.rwd.config.RwdConstants;
import com.persagy.dmp.rwd.entity.ObjectInstance;
import com.persagy.dmp.rwd.entity.QObjectInstance;
import com.persagy.dmp.rwd.entity.QSchemaProject;
import com.persagy.dmp.rwd.entity.SchemaProject;
import com.persagy.dmp.rwd.enums.FuncidCategory;
import com.persagy.dmp.rwd.funcid.parse.parser.InfoCodeParser;
import com.persagy.dmp.rwd.funcid.parse.parser.InfoCodeParserFactory;
import com.persagy.dmp.rwd.funcid.parse.parser.ParserContext;
import com.persagy.dmp.rwd.model.ClassDefModel;
import com.persagy.dmp.rwd.model.DmpMessage;
import com.persagy.dmp.rwd.model.FuncidDefModel;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author: yaoll
 * @date: 2020-09-23
 * @verison: 1.0
 */
@Slf4j
@Service
public class ObjectInstanceUpdateService extends BaseService {

	@Autowired
	private FuncidDefService funcidDefService;

	@Autowired
	private ClassDefService classDefService;

	@Autowired
	private InfoCodeParserFactory infoCodeParserFactory;

	@Transactional(propagation = Propagation.REQUIRED)
	public MapResponse update(ArrayNode paramList) {

		MapResponse response = new MapResponse();
		if (paramList == null || paramList.size() == 0) {
			return response;
		}

		OrgParam orgParam = super.prepareParam(response, RwdConstants.QUERY_GROUPCODE, RwdConstants.QUERY_PROJECTID);
		if (!response.success()) {
			return response;
		}

		QSchemaProject pqt = QSchemaProject.schemaProject;
		Optional<SchemaProject> one = schemaProjectRepository.findOne(pqt.projectId.eq(orgParam.projectId).and(pqt.groupCode.eq(orgParam.groupCode)));
		if (one.isPresent()) {
			orgParam.schemaId = one.get().getSchemaId();
		} else {
			response.setFail("当前项目未配置集团方案");
			return response;
		}

		// 类型缓存, TODO 判断应用是否订阅了该类型
		Map<String, ClassDefModel> classCache = new HashMap<>();
		// 信息点缓存, key: classCode
		Map<String, List<FuncidDefModel>> funcidCache = new HashMap<>();
		Map<String, Set<String>> infoCodeCache = new HashMap<>();

		// 校验
		QObjectInstance qt = QObjectInstance.objectInstance;
		for (JsonNode jsonNode : paramList) {
			ObjectNode param = (ObjectNode) jsonNode;
			String id = JacksonMapper.getString(param, RwdConstants.OBJECT_ID);
			if (id == null) {
				response.setFail("id is required");
				return response;
			}
			if ("Eq44030500190026e03ce3554a66ad66269a85106482".equals(id)) {
				log.info("id = {}", id);
			}
			String classCode = super.getInfoCodeValue(orgParam.groupCode, orgParam.projectId, id, RwdConstants.OBJECT_CLASSCODE);
			if (classCode == null) {
				response.setFail("object[" + id + "] not exists");
				return response;
			}
			if (!classCache.containsKey(classCode)) {
				JacksonCriteria criteria = JacksonCriteria.newInstance();
				criteria.add("classCode", classCode);
				List<ClassDefModel> data = classDefService.queryClass(orgParam.groupCode, orgParam.schemaId, orgParam.projectId, orgParam.appId, false, criteria).getData();
				if (data.size() > 0) {
					classCache.put(classCode, data.get(0));
				}
			}
			if (!classCache.containsKey(classCode)) {
				response.setFail("未订阅类型[" + classCode + "]");
				return response;
			}

			if (!funcidCache.containsKey(classCode)) {
				JacksonCriteria criteria = JacksonCriteria.newInstance();
				criteria.add("classCode", classCode);
				List<FuncidDefModel> data = funcidDefService.queryFuncid(orgParam.groupCode, orgParam.schemaId, orgParam.projectId, orgParam.appId, criteria).getData();
				funcidCache.put(classCode, data);
				infoCodeCache.put(classCode, data.stream().map(FuncidDefModel::getCode).collect(Collectors.toSet()));
			}
			// 已订阅的信息点
			List<FuncidDefModel> funcidList = funcidCache.get(classCode);
			for (FuncidDefModel funcidDefModel : funcidList) {
				JsonNode source = null;
				String code = funcidDefModel.getCode();
				if (RwdConstants.isBasicColumn(code)) {
					continue;
				}
				if (param.has(code)) {
					source = param.get(code);
				} else if (param.has(funcidDefModel.getOrigCode())) {
					source = param.get(funcidDefModel.getOrigCode());
				}
				if (source != null) {
					if (funcidDefModel.getCategory() == FuncidCategory.STATIC) {
						// 静态点
						InfoCodeParser parser = infoCodeParserFactory.getParser(funcidDefModel.getDataType());
						ParserContext parserContext = new ParserContext(funcidDefModel, source);
						parser.checkValue(parserContext);
						if (parserContext.isHasError()) {
							response.setFail(parserContext.getError().toString());
							return response;
						}
					}
				}
			}
		}
		// 赋值
		for (JsonNode jsonNode : paramList) {
			ObjectNode param = (ObjectNode) jsonNode;
			save(response, funcidCache, infoCodeCache, orgParam, param);
		}

		return response;
	}

	public void save(MapResponse response, Map<String, List<FuncidDefModel>> funcidCache, Map<String, Set<String>> infoCodeCache, OrgParam orgParam, ObjectNode param) {
		QObjectInstance qt = QObjectInstance.objectInstance;
		String id = JacksonMapper.getString(param, RwdConstants.OBJECT_ID);
		Optional<ObjectInstance> opt = objectInstanceRepository.findOne(qt.id.eq(id).and(qt.groupCode.eq(orgParam.groupCode)).and(qt.projectId.eq(orgParam.projectId)));
		ObjectInstance entity = opt.get();
		String name = JacksonMapper.getString(param, RwdConstants.OBJECT_NAME);
		if (name != null) {
			entity.setName(name);
		}
		String localId = JacksonMapper.getString(param, RwdConstants.OBJECT_LOCALID);
		if (localId != null) {
			entity.setLocalId(localId);
		}
		String localName = JacksonMapper.getString(param, RwdConstants.OBJECT_LOCALNAME);
		if (localName != null) {
			entity.setLocalName(localName);
		}
		String updateUser = JacksonMapper.getString(param, RwdConstants.OBJECT_UPDATE_USER);
		if (updateUser != null) {
			entity.setUpdateUser(updateUser);
		}
		String updateApp = JacksonMapper.getString(param, RwdConstants.OBJECT_UPDATE_APP, orgParam.appId);
		if (updateApp != null) {
			entity.setUpdateApp(updateApp);
		}
		if (param.has(RwdConstants.OBJECT_VIRTUAL_INFO_CODES)) {
			entity.setVirtualInfoCodes(getStringList(param, RwdConstants.OBJECT_VIRTUAL_INFO_CODES));
		}
//			entityList.add(entity);
		String classCode = entity.getClassCode();
		// 已订阅的信息点
		List<FuncidDefModel> funcidList = funcidCache.get(classCode);
		for (FuncidDefModel funcidDefModel : funcidList) {
			JsonNode source = null;
			String code = funcidDefModel.getCode();
			if (RwdConstants.isBasicColumn(code)) {
				continue;
			}
			if (param.has(code)) {
				source = param.get(code);
			} else if (param.has(funcidDefModel.getOrigCode())) {
				source = param.get(funcidDefModel.getOrigCode());
			}
			if (source != null) {
				if (funcidDefModel.getCategory() == FuncidCategory.STATIC) {
					// 静态点
					InfoCodeParser parser = infoCodeParserFactory.getParser(funcidDefModel.getDataType());
					ParserContext parserContext = new ParserContext(funcidDefModel, source);
					parser.checkValue(parserContext);
					Object value = parserContext.getValue();
					if (value != null) {
						if (entity.getInfos() == null) {
							entity.setInfos(JsonNodeFactory.instance.objectNode());
						}
						entity.getInfos().putPOJO(code, value);
					}
				} else {
					// 动态点
					if (entity.getInfos() == null) {
						entity.setInfos(JsonNodeFactory.instance.objectNode());
					}
					entity.getInfos().set(code, source);
				}
			}
		}

		// 处理要删除的信息点
		Set<String> infoCodes = infoCodeCache.get(classCode);
		if (param.has(RwdConstants.OBJECT_UPDATE_REMOVE)) {
			ArrayNode array = JacksonMapper.getArray(param, RwdConstants.OBJECT_UPDATE_REMOVE, JsonNodeFactory.instance.arrayNode());
			for (JsonNode node : array) {
				String infoCode = node.asText();
				if (RwdConstants.isBasicColumn(infoCode)) {
					if (RwdConstants.OBJECT_NAME.equals(infoCode)) {
						entity.setName(null);
					} else if (RwdConstants.OBJECT_LOCALID.equals(infoCode)) {
						entity.setLocalId(null);
					} else if (RwdConstants.OBJECT_LOCALNAME.equals(infoCode)) {
						entity.setLocalName(null);
					} else if (RwdConstants.OBJECT_VIRTUAL_INFO_CODES.equals(infoCode)) {
						entity.setVirtualInfoCodes(Collections.emptyList());
					}
				} else if (infoCodes.contains(infoCode)) {
					entity.getInfos().remove(infoCode);
				}
			}
		}

		objectInstanceRepository.save(entity);
		entityManager.flush();
		entityManager.clear();
		String now = DateUtils.format(new Date());
		DmpMessage message = new DmpMessage(RwdConstants.MESSAGE_INSTANCE_OBJECT_UPDATE, entity.getId());
		message.setGroupCode(orgParam.groupCode);
		message.setProjectId(orgParam.projectId);
		message.setStr1(entity.getObjType());
		message.setStr2(entity.getClassCode());
		message.setAppId(orgParam.appId);
		message.setUserId(DmpParameterStorage.getUserId());
		message.setSendTime(now);
		response.add(message);
	}

	public static final List<String> getStringList(ObjectNode obj, String key) {
		ArrayNode array = JacksonMapper.getArray(obj, key, JsonNodeFactory.instance.arrayNode());
		List<String> list = new ArrayList<>(array.size());
		for (JsonNode jsonNode : array) {
			list.add(jsonNode.asText());
		}
		return list;
	}
}
