package com.persagy.dmp.rwd.service;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.web.MapResponse;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.config.DmpParameterStorage;
import com.persagy.dmp.rwd.config.RwdConstants;
import com.persagy.dmp.rwd.entity.QSchemaFuncid;
import com.persagy.dmp.rwd.entity.SchemaFuncid;
import com.persagy.dmp.rwd.model.SchemaFuncidModel;
import com.persagy.dmp.rwd.repository.SchemaFuncidRepository;
import com.querydsl.core.types.dsl.BooleanExpression;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class SchemaFuncidService extends BaseService {

	@Autowired
	private SchemaFuncidRepository schemaFuncidRepository;

	@Autowired
	private CriteriaUtils criteriaUtils;

	private List<BooleanExpression> parse(ObjectNode criteria) {
		// TODO
		return null;
	}

	public PagedResponse<SchemaFuncidModel> query(JacksonCriteria criteria) {
		PagedResponse<SchemaFuncidModel> result = new PagedResponse<>();
		super.checkParam(result, RwdConstants.QUERY_GROUPCODE);
		if (!result.success()) {
			return result;
		}
		criteria.add("groupCode", DmpParameterStorage.getGroupCode());
		PagedResponse<SchemaFuncid> resp = criteriaUtils.query(QSchemaFuncid.schemaFuncid, this::parse, criteria);
		result.setCount(resp.getCount());
		List<SchemaFuncid> dataList = resp.getData();
		if (dataList != null && dataList.size() > 0) {
			List<SchemaFuncidModel> collect = dataList.stream().map(entity -> {
				SchemaFuncidModel model = entity.toModel();
				return model;
			}).collect(Collectors.toList());
			result.setData(collect);
		}
		return result;
	}

	@Transactional
	public MapResponse create(SchemaFuncidModel param) {
		MapResponse response = new MapResponse();
		super.checkParam(response, RwdConstants.QUERY_GROUPCODE);
		if (!response.success()) {
			return response;
		}
		// TODO
		SchemaFuncid entity = SchemaFuncid.fromModel(param);
		entity.setGroupCode(DmpParameterStorage.getGroupCode());
		schemaFuncidRepository.save(entity);
		response.add("id", entity.getId());
		return response;
	}

	@Transactional
	public MapResponse update(SchemaFuncidModel param) {
		MapResponse response = new MapResponse();
		super.checkParam(response, RwdConstants.QUERY_GROUPCODE);
		if (!response.success()) {
			return response;
		}
		// TODO
		SchemaFuncid entity = SchemaFuncid.fromModel(param);
		schemaFuncidRepository.save(entity);
		response.add("id", entity.getId());
		return response;
	}

	@Transactional
	public MapResponse delete(SchemaFuncidModel param) {
		MapResponse response = new MapResponse();
		super.checkParam(response, RwdConstants.QUERY_GROUPCODE);
		if (!response.success()) {
			return response;
		}
		// TODO
		schemaFuncidRepository.deleteById(param.getId());
		return response;
	}
}

