package com.persagy.dmp.rwd.service;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.web.MapResponse;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.config.DmpParameterStorage;
import com.persagy.dmp.rwd.config.RwdConstants;
import com.persagy.dmp.rwd.entity.QSchemaProject;
import com.persagy.dmp.rwd.entity.SchemaProject;
import com.persagy.dmp.rwd.model.SchemaProjectModel;
import com.querydsl.core.types.dsl.BooleanExpression;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

@Service
public class SchemaProjectService extends BaseService {

	private List<BooleanExpression> parse(ObjectNode criteria) {
		// TODO
		return null;
	}

	public PagedResponse<SchemaProjectModel> query(JacksonCriteria criteria) {
		PagedResponse<SchemaProjectModel> result = new PagedResponse<>();
		super.checkParam(result, RwdConstants.QUERY_GROUPCODE);
		if (!result.success()) {
			return result;
		}
		criteria.add("groupCode", DmpParameterStorage.getGroupCode());
		PagedResponse<SchemaProject> resp = criteriaUtils.query(QSchemaProject.schemaProject, this::parse, criteria);
		result.setCount(resp.getCount());
		List<SchemaProject> dataList = resp.getData();
		if (dataList != null && dataList.size() > 0) {
			List<SchemaProjectModel> collect = dataList.stream().map(entity -> {
				SchemaProjectModel model = entity.toModel();
				return model;
			}).collect(Collectors.toList());
			result.setData(collect);
		}
		return result;
	}

	@Transactional
	public MapResponse create(SchemaProjectModel param) {
		MapResponse response = new MapResponse();
		super.checkParam(response, RwdConstants.QUERY_GROUPCODE);
		if (!response.success()) {
			return response;
		}
		// TODO
		SchemaProject entity = SchemaProject.fromModel(param);
		if (entity.getId() == null) {
			entity.setId(UUID.randomUUID().toString().replace("-", ""));
		}
		entity.setGroupCode(DmpParameterStorage.getGroupCode());
		schemaProjectRepository.save(entity);
		response.add("id", entity.getId());
		return response;
	}

	@Transactional
	public MapResponse update(SchemaProjectModel param) {
		MapResponse response = new MapResponse();
		super.checkParam(response, RwdConstants.QUERY_GROUPCODE);
		if (!response.success()) {
			return response;
		}
		// TODO
		SchemaProject entity = SchemaProject.fromModel(param);
		schemaProjectRepository.save(entity);
		response.add("id", entity.getId());
		return response;
	}

	@Transactional
	public MapResponse delete(SchemaProjectModel param) {
		MapResponse response = new MapResponse();
		super.checkParam(response, RwdConstants.QUERY_GROUPCODE);
		if (!response.success()) {
			return response;
		}
		QSchemaProject qt = QSchemaProject.schemaProject;
		BooleanExpression exp = qt.groupCode.eq(DmpParameterStorage.getGroupCode()).and(qt.id.eq(param.getId()));
		boolean flag = false;
		if (param.getId() != null) {
			exp = exp.and(qt.id.eq(param.getId()));
			flag = true;
		}
		if (param.getProjectId() != null) {
			exp = exp.and(qt.projectId.eq(param.getProjectId()));
			flag = true;
		}
		if (flag) {
			Iterable<SchemaProject> all = schemaProjectRepository.findAll(exp);
			if (all != null) {
				for (SchemaProject entity : all) {
					schemaProjectRepository.delete(entity);
				}
			}
		}
		return response;
	}
}

