package com.persagy.dmp.rwd.service;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.web.MapResponse;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.config.DmpParameterStorage;
import com.persagy.dmp.rwd.config.RwdConstants;
import com.persagy.dmp.rwd.entity.QSchema;
import com.persagy.dmp.rwd.entity.Schema;
import com.persagy.dmp.rwd.model.SchemaModel;
import com.querydsl.core.types.dsl.BooleanExpression;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.List;
import java.util.Optional;
import java.util.UUID;
import java.util.stream.Collectors;

@Service
public class SchemaService extends BaseService {

	@Autowired
	private CriteriaUtils criteriaUtils;

	private List<BooleanExpression> parse(ObjectNode criteria) {
		// TODO
		return null;
	}

	public PagedResponse<SchemaModel> query(JacksonCriteria criteria) {
		PagedResponse<SchemaModel> result = new PagedResponse<>();
		super.checkParam(result, RwdConstants.QUERY_GROUPCODE);
		if (!result.success()) {
			return result;
		}
		criteria.add(RwdConstants.OBJECT_GROUPCODE, DmpParameterStorage.getGroupCode());
		PagedResponse<Schema> resp = criteriaUtils.query(QSchema.schema, this::parse, criteria);
		result.setCount(resp.getCount());
		List<Schema> dataList = resp.getData();
		if (dataList != null && dataList.size() > 0) {
			List<SchemaModel> collect = dataList.stream().map(entity -> {
				SchemaModel model = entity.toModel();
				return model;
			}).collect(Collectors.toList());
			result.setData(collect);
		}
		return result;
	}

	@Transactional
	public MapResponse create(SchemaModel param) {
		MapResponse response = new MapResponse();
		super.checkParam(response, RwdConstants.QUERY_GROUPCODE);
		if (!response.success()) {
			return response;
		}
		if (DmpParameterStorage.getGroupCode() == null) {
			response.setFail("集团编码不可为空");
			return response;
		}
		// TODO
		Schema entity = Schema.fromModel(param);
		if (entity.getId() == null) {
			entity.setId(UUID.randomUUID().toString().replace("-", ""));
		}
		entity.setGroupCode(DmpParameterStorage.getGroupCode());
		entity.setStatus(1);
		schemaRepository.save(entity);
		response.add("id", entity.getId());
		return response;
	}

	@Transactional
	public MapResponse update(SchemaModel param) {
		MapResponse response = new MapResponse();
		super.checkParam(response, RwdConstants.QUERY_GROUPCODE);
		if (!response.success()) {
			return response;
		}
		if (DmpParameterStorage.getGroupCode() == null) {
			response.setFail("集团编码不可为空");
			return response;
		}
		// TODO
		QSchema qt = QSchema.schema;
		String schemaId = param.getId();
		Optional<Schema> opt = schemaRepository.findOne(qt.id.eq(schemaId).and(qt.groupCode.eq(DmpParameterStorage.getGroupCode())));
		if (!opt.isPresent()) {
			response.setFail("数据不存在");
		}

		Schema entity = opt.get();
		if (param.getName() != null) {
			entity.setName(param.getName());
		}
		if (param.getRemark() != null) {
			entity.setRemark(param.getRemark());
		}
		schemaRepository.save(entity);
		response.add("id", entity.getId());
		return response;
	}

	@Transactional
	public MapResponse delete(SchemaModel param) {
		MapResponse response = new MapResponse();
		super.checkParam(response, RwdConstants.QUERY_GROUPCODE);
		if (!response.success()) {
			return response;
		}
		// TODO
		schemaRepository.deleteById(param.getId());
		return response;
	}
}

