package com.persagy.dmp.rwd.service.iot;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.JsonNodeFactory;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.json.JacksonMapper;
import com.persagy.common.web.ListResponse;
import com.persagy.dmp.common.http.HttpUtils;
import com.persagy.dmp.config.DmpParameterStorage;
import com.persagy.dmp.rwd.config.RwdConstants;
import com.persagy.dmp.rwd.model.iot.CurrentDataModel;
import lombok.Getter;
import lombok.Setter;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class CurrentIotService extends BaseIotService {

	/**
	 * 查询实时数据
	 *
	 * @param param
	 * @return
	 */
	public ListResponse<CurrentDataModel> getCurrentData(List<CurrentDataModel> param) {
		ListResponse<CurrentDataModel> resp = new ListResponse<>();
		super.checkParam(resp, RwdConstants.QUERY_GROUPCODE, RwdConstants.QUERY_PROJECTID);
		if (!resp.success()) {
			return resp;
		}
		if (param == null || param.size() == 0) {
			return resp;
		}
		String groupCode = DmpParameterStorage.getGroupCode();
		String projectId = DmpParameterStorage.getProjectId();
		String appId = DmpParameterStorage.getAppId();
		Map<String, Set<String>> infoCodeCache = new HashMap<>(); // 设备类型对应的信息点
		String projectLocalId = projectId.substring(2); // TODO
		resp.setData(param);
		List<PointParam> parameters = new LinkedList<>();
		for (CurrentDataModel model : param) {
			if (model.getObjectId() == null) {
				model.setError("对象id不可为空");
				continue;
			}
			String infoCode = model.getInfoCode();
			if (infoCode == null) {
				model.setError("信息点编码不可为空");
				continue;
			}
//			String classCode = getInfoCodeValue(projectId, model.getObjectId(), "classCode");
//			if (classCode == null) {
//				model.setError("对象不存在或已失效");
//				continue;
//			}
//			TODO 暂不校验订阅
//			if (!infoCodeCache.containsKey(classCode)) {
//				infoCodeCache.put(classCode, getInfoCode(projectId, appId, classCode));
//			}
//			Set<String> strings = infoCodeCache.get(classCode);
//			if (!strings.contains(infoCode)) {
//				model.setError("infoCode is not subcribed or is static");
//				continue;
//			}

			String infoCodeValue = getInfoCodeValue(groupCode, projectId, model.getObjectId(), infoCode);
			if (infoCodeValue == null) {
				model.setError("value of infoCode[" + model.getObjectId() + ":" + infoCode + "] is null");
				continue;
			}

			model.setInfoValue(infoCodeValue);
			parameters.add(new PointParam(model.getMeter(), model.getFunction()));
		}

		if (parameters.size() == 0) {
			return resp;
		}

		String url = iotDataServer + "/presentreportbatch_post";
		Map<String, Object> requestBody = new HashMap<>();
		requestBody.put("building", projectLocalId);
		requestBody.put("points", parameters);
		HttpUtils.HttpResult httpResult = httpUtils.post(url, JacksonMapper.toSimpleJson(requestBody), 30000);
		String content = httpResult.getContent();
		if (!httpResult.success()) {
			resp.setFail(content);
			return resp;
		}
		ObjectNode response = JacksonMapper.toObject(content, ObjectNode.class);
		ArrayNode list = JacksonMapper.getArray(response, "points", JsonNodeFactory.instance.arrayNode());
		for (JsonNode jsonNode : list) {
			ObjectNode item = (ObjectNode) jsonNode;
			String meter = JacksonMapper.getString(item, "meter");
			String function = JacksonMapper.getString(item, "funcid");
			for (CurrentDataModel model : param) {
				if (meter.equals(model.getMeter()) && function.equals(model.getFunction())) {
					model.setData(item.get("data"));
					model.setTime(JacksonMapper.getString(item, "time"));
				}
			}
		}
		return resp;
	}

	@Getter
	@Setter
	public static class PointParam {
		String meter;
		String funcid;

		public PointParam(String meter, String function) {
			this.meter = meter;
			this.funcid = function;
		}
	}
}
