package com.persagy.dmp.rwd.service.iot;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.JsonNodeFactory;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.json.JacksonMapper;
import com.persagy.common.web.ListResponse;
import com.persagy.dmp.config.DmpParameterStorage;
import com.persagy.dmp.common.http.HttpUtils;
import com.persagy.dmp.rwd.config.RwdConstants;
import com.persagy.dmp.rwd.model.iot.history.InfoCodeModel;
import com.persagy.dmp.rwd.model.iot.history.RequestModel;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class HistoryIotService extends BaseIotService {

	private void checkRequest(RequestModel request, ListResponse<InfoCodeModel> resp) {
		String startTime = request.getStartTime();
		if (startTime == null) {
			resp.setFail("startTime is required");
			return;
		}
		String endTime = request.getEndTime();
		if (endTime == null) {
			resp.setFail("endTime is required");
			return;
		}
		String period = request.getPeriod();
		if (period == null) {
			resp.setFail("period is required");
			return;
		}
		List<InfoCodeModel> params = request.getParams();
		if (params == null || params.size() == 0) {
			resp.setFail("params is required");
			return;
		}
	}

	private void checkParam(String groupCode, String projectId, String appId, InfoCodeModel param, ArrayNode pointList, Map<String, Set<String>> infoCodeCache) {

		String objectId = param.getObjectId();
		if (objectId == null) {
			param.setStatus("error");
			param.setError("objectId is not required");
			return;
		}
		String infoCode = param.getInfoCode();
		if (infoCode == null) {
			param.setStatus("error");
			param.setError("infoCode is not required");
			return;
		}

		String classCode = getInfoCodeValue(groupCode, projectId, objectId, "classCode");
		if (classCode == null) {
			param.setStatus("error");
			param.setError("对象不存在或已失效");
			return;
		}
//		TODO 暂时不控制订阅
//		if (!infoCodeCache.containsKey(classCode)) {
//			infoCodeCache.put(classCode, getInfoCode(projectId, appId, classCode));
//		}
//		Set<String> strings = infoCodeCache.get(classCode);
//		if (strings.contains(infoCodeCache)) {
//			param.setStatus("error");
//			param.setError("infoCode is not subcribed or is static");
//			return;
//		}

		String infoCodeValue = getInfoCodeValue(groupCode, projectId, objectId, infoCode);
		if (infoCodeValue == null) {
			param.setStatus("error");
			param.setError("value of infoCode[" + objectId + ":" + infoCode + "] is null");
			return;
		}

		int idx = infoCodeValue.lastIndexOf("-");
		if (idx == -1) {

		} else {
			String meter = infoCodeValue.substring(0, idx);
			String function = infoCodeValue.substring(idx + 1);
			param.setMeter(meter);
			param.setFunction(function);
			ObjectNode point = pointList.addObject();
			point.put("meter", meter);
			point.put("funcid", Long.parseLong(function));
		}
	}

	public ListResponse<InfoCodeModel> getHistroyData(RequestModel request) {
		ListResponse<InfoCodeModel> resp = new ListResponse<>();
		super.checkParam(resp, RwdConstants.QUERY_GROUPCODE, RwdConstants.QUERY_PROJECTID);
		if (!resp.success()) {
			return resp;
		}
		String groupCode = DmpParameterStorage.getGroupCode();
		String projectId = DmpParameterStorage.getProjectId();
		String appId = DmpParameterStorage.getAppId();
		checkRequest(request, resp);
		if (!resp.success()) {
			return resp;
		}

		List<InfoCodeModel> params = request.getParams();
		resp.setData(params);

		Map<String, Set<String>> infoCodeCache = new HashMap<>();
		ArrayNode pointList = JsonNodeFactory.instance.arrayNode();
		for (InfoCodeModel param : params) {
			param.setStatus("success");
			checkParam(groupCode, projectId, appId, param, pointList, infoCodeCache);
		}
		if (pointList.size() == 0) {
			return resp;
		}
		ObjectNode requestNode = JsonNodeFactory.instance.objectNode();
		requestNode.put("building", projectId.substring(2)); // TODO

		for (JsonNode jsonNode : pointList) {
			ObjectNode item = (ObjectNode) jsonNode;
			item.put("time_period", request.getPeriod());
			item.put("time_from", request.getStartTime());
			item.put("time_to", request.getEndTime());
		}
		requestNode.set("points", pointList);

		HttpUtils.HttpResult httpResult = httpUtils.post(iotDataServer + "/fjd_query_batch_post", requestNode.toString(), 30000);
		String content = httpResult.getContent();
		if (!httpResult.success()) {
			resp.setFail(content);
			return resp;
		}

		ObjectNode responseNode = JacksonMapper.toObject(content, ObjectNode.class);
		ArrayNode points = JacksonMapper.getArray(responseNode, "points");

		for (JsonNode point : points) {
			ObjectNode meterItem = (ObjectNode) point;
			String meter = JacksonMapper.getString(meterItem, "meter");
			String function = JacksonMapper.getInteger(meterItem, "funcid") + "";
			for (InfoCodeModel param : params) {
				if (!(meter + "-" + function).equals(param.getMeter() + "-" + param.getFunction())) {
					continue;
				}
				ArrayNode data = JsonNodeFactory.instance.arrayNode();
				ArrayNode datas = JacksonMapper.getArray(meterItem, "datas");
				if (datas != null && datas.size() > 0) {
					for (JsonNode jsonNode : datas) {
						ObjectNode dataItem = (ObjectNode) jsonNode;
						ObjectNode item = data.addObject();
						item.put("time", JacksonMapper.getString(dataItem, "data_time"));
						item.put("data", JacksonMapper.getDouble(dataItem, "data_value"));
					}
				}
				param.setData(data);
			}
		}

		return resp;
	}

}
