package com.persagy.dmp.rwd.service.iot;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.JsonNodeFactory;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.json.JacksonMapper;
import com.persagy.common.web.ListResponse;
import com.persagy.dmp.config.DmpParameterStorage;
import com.persagy.dmp.common.http.HttpUtils;
import com.persagy.dmp.rwd.config.RwdConstants;
import com.persagy.dmp.rwd.model.iot.SettingDataModel;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class SettingIotService extends BaseIotService {

	public ListResponse<SettingDataModel> setting(List<SettingDataModel> param) {
		ListResponse<SettingDataModel> resp = new ListResponse<>();
		super.checkParam(resp, RwdConstants.QUERY_GROUPCODE, RwdConstants.QUERY_PROJECTID);
		if (!resp.success()) {
			return resp;
		}
		String groupCode = DmpParameterStorage.getGroupCode();
		String projectId = DmpParameterStorage.getProjectId();
		resp.setData(param);
		// 设定参数
		ArrayNode pointValueList = JsonNodeFactory.instance.arrayNode();
		Map<String, Set<String>> virtualPointCache = new HashMap<>();
		for (SettingDataModel model : param) {
			model.setStatus("success");

			String objectId = model.getObjectId();
			String infoCode = model.getInfoCode();
			String infoCodeValue = getInfoCodeValue(groupCode, projectId, objectId, infoCode);

			if (infoCodeValue == null) {
				model.setStatus("error");
				model.setError("value of infoCode[" + objectId + ":" + infoCode + "] is null");
				continue;
			}

			int idx = infoCodeValue.lastIndexOf("-");
			if (idx == -1) {
				continue;
			}
			String meter = infoCodeValue.substring(0, idx);
			String function = infoCodeValue.substring(idx + 1);
			model.setMeter(meter);
			model.setFunction(function);
			ObjectNode point = pointValueList.addObject();
			point.put("meter", meter);
			point.put("funcid", Long.parseLong(function));
			point.put("data", model.getValue()); // TODO 要细分类型

			if (!virtualPointCache.containsKey(objectId)) {
				String sql = "select virtual_info_codes from rwd_object where group_code = ? and project_id = ? and id = ?";
				Map<String, Object> result = jpaUtils.queryOne(sql, groupCode, projectId, objectId);
				if (result != null && result.size() > 0) {
					String codes = (String) result.get("virtual_info_codes");
					if (codes == null) {
						codes = "[]";
					}
					Set<String> set = JacksonMapper.toObject(codes, HashSet.class, String.class);
					virtualPointCache.put(objectId, set);
				} else {
					virtualPointCache.put(objectId, Collections.emptySet());
				}
			}
			Set<String> virtualPointList = virtualPointCache.get(objectId);
			point.put("virtual", virtualPointList.contains(infoCode));
		}

		if (pointValueList.size() == 0) {
			return resp;
		}
		String projectLocalId = projectId.substring(2); // TODO
		ObjectNode requestNode = JsonNodeFactory.instance.objectNode();
		requestNode.put("building", projectLocalId);
		requestNode.set("points", pointValueList);

		HttpUtils.HttpResult httpResult = httpUtils.post(iotSettingServer + "/pointsetbatch_post", requestNode.toString(), 30000);
		String content = httpResult.getContent();
		if (!httpResult.success()) {
			resp.setFail(content);
			return resp;
		}

		ObjectNode responseNode = JacksonMapper.toObject(content, ObjectNode.class);
		ArrayNode list = JacksonMapper.getArray(responseNode, "points", JsonNodeFactory.instance.arrayNode());
		for (JsonNode jsonNode : list) {
			ObjectNode item = (ObjectNode) jsonNode;
			String meter1 = JacksonMapper.getString(item, "meter");
			String function1 = JacksonMapper.getInteger(item, "funcid") + "";
			for (SettingDataModel model : param) {
				if (model.getMeter().equals(meter1) && model.getFunction().equals(function1)) {
					// 楼号-表号-功能号-时间作为唯一标识
					String exeCode = projectLocalId + ":" + meter1 + ":" + function1 + ":" + JacksonMapper.getString(item, "receivetime");
					model.setExeCode(exeCode);
				}
			}
		}
		return resp;
	}

	public ListResponse<SettingDataModel> settingQuery(List<String> param) {
		ListResponse<SettingDataModel> resp = new ListResponse<>();
		if (param == null || param.size() == 0) {
			return resp;
		}

		String projectId = DmpParameterStorage.getProjectId();
		String projectLocalId = projectId.substring(2);

		ObjectNode requestNode = JsonNodeFactory.instance.objectNode();
		requestNode.put("building", projectLocalId);
		ArrayNode points = requestNode.putArray("points");
		for (String str : param) {
			ObjectNode point = points.addObject();
			String[] split = str.split(":");
			point.put("meter", split[1]);
			point.put("funcid", Integer.parseInt(split[2]));
			point.put("receivetime", split[3]);
		}

		HttpUtils.HttpResult httpResult = httpUtils.post(iotSettingServer + "/pointsetresultbatch_post", requestNode.toString(), 30000);
		String content = httpResult.getContent();
		if (!httpResult.success()) {
			resp.setFail(content);
			return resp;
		}

		ObjectNode responseNode = JacksonMapper.toObject(content, ObjectNode.class);
		ArrayNode responseData = JacksonMapper.getArray(responseNode, "points", JsonNodeFactory.instance.arrayNode());
		List<SettingDataModel> list = new LinkedList<>();
		for (JsonNode datum : responseData) {
			ObjectNode item = (ObjectNode) datum;
			String meter1 = JacksonMapper.getString(item, "meter");
			String function1 = JacksonMapper.getInteger(item, "funcid") + "";
			String receivetime1 = JacksonMapper.getString(item, "receivetime");
			String exeCode = projectLocalId + ":" + meter1 + ":" + function1 + ":" + receivetime1;
			SettingDataModel model = new SettingDataModel();
			model.setExeCode(exeCode);
			model.setExeResult(JacksonMapper.getString(item, "status"));
			list.add(model);
		}
		resp.setData(list);
		return resp;
	}

}
