package com.persagy.dmp.rwd.sharding;

import org.apache.shardingsphere.api.sharding.complex.ComplexKeysShardingAlgorithm;
import org.apache.shardingsphere.api.sharding.complex.ComplexKeysShardingValue;

import java.util.Collection;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * @author: yaoll
 * @date: 2020-10-30
 * @verison: 1.0
 */
public class GroupProjectShardingAlgorithm implements ComplexKeysShardingAlgorithm<String> {

	@Override
	public Collection<String> doSharding(Collection<String> availableTargetNames, ComplexKeysShardingValue<String> shardingValue) {
		String logicTableName = shardingValue.getLogicTableName();
		Map<String, Collection<String>> columnNameAndShardingValuesMap = shardingValue.getColumnNameAndShardingValuesMap();
		Collection<String> groupCodes = columnNameAndShardingValuesMap.get("group_code");
		Collection<String> projectIds = columnNameAndShardingValuesMap.get("project_id");
		Set<String> result = new HashSet<>();
		if (groupCodes == null || groupCodes.size() == 0 || projectIds == null || projectIds.size() == 0) {
			result.add(logicTableName);
		} else {
			for (String groupCode : groupCodes) {
				for (String projectId : projectIds) {
					if ("0".equals(groupCode)) {
						result.add(logicTableName);
					} else {
						result.add(logicTableName + "_" + groupCode.toLowerCase() + "_" + projectId.toLowerCase());
					}
				}
			}
		}
		return result;
	}

}
