package com.persagy.dmp.dpt.controller;

import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.web.MapResponse;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.dpt.constant.TaskConstant;
import com.persagy.dmp.dpt.entity.TaskCfg;
import com.persagy.dmp.dpt.model.TaskCfgModel;
import com.persagy.dmp.dpt.model.TaskRecordModel;
import com.persagy.dmp.dpt.service.TaskCfgService;
import com.persagy.dmp.dpt.service.TaskRecordService;
import com.persagy.dmp.rwd.model.DmpMessage;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.*;

@Slf4j
@RestController
@RequestMapping("/task/cfg")
public class TaskCfgController {

	@Autowired
	private TaskCfgService service;

	@Autowired
	private TaskRecordService taskRecordService;

	@Autowired
	private RabbitTemplate rabbitTemplate;

	@PostMapping("/query")
	public PagedResponse<TaskCfgModel> query(@RequestBody JacksonCriteria criteria) {
		return service.query(criteria);
	}
	
	@PostMapping("/create")
	public MapResponse create(@RequestBody TaskCfgModel param){
		MapResponse mapResponse = service.create(param);
		// TODO 新增任务广播 MQ
		String taskId = param.getId();

		return mapResponse;
	}
	
	@PostMapping("/update")
	public MapResponse update(@RequestBody TaskCfgModel param){
		MapResponse mapResponse = service.update(param);
		// TODO  更新任务广播 MQ
		String taskId = param.getId();
		Map<String,Object> taskMQ = new HashMap<>();
		taskMQ.put("taskId",taskId);
		taskMQ.put("type","update");
		taskMQ.put("groupCode", param.getGroupCode());
		taskMQ.put("appId", param.getAppId());
		taskMQ.put("projectId", param.getProjectId());
		rabbitTemplate.convertAndSend("TaskExchanges", "", taskMQ);
		return mapResponse;
	}
	
	@PostMapping("/delete")
	public MapResponse delete(@RequestBody TaskCfgModel param){
		MapResponse mapResponse = service.delete(param);
		// TODO  删除任务广播 MQ
		String taskId = param.getId();
		Map<String,Object> taskMQ = new HashMap<>();
		taskMQ.put("taskId",taskId);
		taskMQ.put("type","delete");
		taskMQ.put("groupCode", param.getGroupCode());
		taskMQ.put("appId", param.getAppId());
		taskMQ.put("projectId", param.getProjectId());
		rabbitTemplate.convertAndSend("TaskExchanges", "", taskMQ);
		return mapResponse;
	}

}
