package com.persagy.dmp.dpt.entity;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.vladmihalcea.hibernate.type.json.JsonStringType;
import lombok.Getter;
import lombok.Setter;
import com.persagy.dmp.dpt.model.TaskCfgModel;
import org.hibernate.annotations.GenericGenerator;
import org.hibernate.annotations.Type;
import org.hibernate.annotations.TypeDef;

import javax.persistence.GeneratedValue;
import javax.persistence.*;
import java.util.Date;

@Setter
@Getter
@Entity
@TypeDef(name = "json", typeClass = JsonStringType.class)
@Table(name = "dpt_task_cfg")
public class TaskCfg {

	@Id
	@GeneratedValue(generator = "assignedId")
	@GenericGenerator(name = "assignedId", strategy = "assigned")
	private String id;
	private String groupCode;
	private String appId;
	private String projectId;
	private String code;
	private String name;
	private String type;

	@Type(type = "json")
	private ObjectNode config;
	@Type(type = "json")
	private ObjectNode params;
	@Type(type = "json")
	private ObjectNode timeRule;
	private Date startTime;
	private Date endTime;
	private Integer responseTime;
	private String level;
	private Integer status;
	private String remark;

	public static TaskCfg fromModel(TaskCfgModel model) {
		TaskCfg entity = new TaskCfg();
		entity.setId(model.getId());
		entity.setGroupCode(model.getGroupCode());
		entity.setAppId(model.getAppId());
		entity.setProjectId(model.getProjectId());
		entity.setCode(model.getCode());
		entity.setName(model.getName());
		entity.setType(model.getType());
		entity.setConfig(model.getConfig());
		entity.setParams(model.getParams());
		entity.setTimeRule(model.getTimeRule());
		entity.setStartTime(model.getStartTime());
		entity.setEndTime(model.getEndTime());
		entity.setResponseTime(model.getResponseTime());
		entity.setLevel(model.getLevel());
		entity.setStatus(model.getStatus());
		entity.setRemark(model.getRemark());
		return entity;
	}

	public TaskCfgModel toModel() {
		TaskCfgModel model = new TaskCfgModel();
		model.setId(this.id);
		model.setGroupCode(this.getGroupCode());
		model.setAppId(this.getAppId());
		model.setProjectId(this.getProjectId());
		model.setCode(this.getCode());
		model.setName(this.getName());
		model.setType(this.getType());
		model.setConfig(this.getConfig());
		model.setParams(this.getParams());
		model.setTimeRule(this.getTimeRule());
		model.setStartTime(this.getStartTime());
		model.setEndTime(this.getEndTime());
		model.setResponseTime(this.getResponseTime());
		model.setLevel(this.getLevel());
		model.setStatus(this.getStatus());
		model.setRemark(this.getRemark());
		return model;
	}

}
