package com.persagy.dmp.dpt.jms;

import com.persagy.common.json.JacksonMapper;
import com.persagy.dmp.rwd.model.DmpMessage;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.*;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * @author: yaoll
 * @date: 2020-10-22
 * @verison: 1.0
 */
@Slf4j
@Configuration
public class JmsConfig {

	@Value("${persagy.dmp.exchange}")
	private String exchange;

	@Value("${persagy.dmp.task.routingKey}")
	private String taskRoutingKey;

	@Value("${persagy.dmp.task.queue}")
	private String taskQueue;

	@Autowired
	private MessageProcesser messageProcesser;

	@Bean
	public TopicExchange exchange() {
		return new TopicExchange(exchange);
	}

	@Bean
	public Queue queue() {
		return new Queue(taskQueue, true);
	}

	@Bean
	public Binding taskBinding() {
		return BindingBuilder.bind(queue()).to(exchange()).with(taskRoutingKey);
	}

	// @RabbitListener(queues = "${persagy.dmp.task.queue}")    //监听器监听指定的Queue
	public void processC(DmpMessage message) {
		log.info("============================== Receive {}: {}", taskQueue, JacksonMapper.toSimpleJson(message));
		messageProcesser.listen(message);
	}

}
