package com.persagy.dmp.dpt.service;

import com.persagy.common.web.BaseResponse;
import com.persagy.dmp.config.DmpParameterStorage;
import com.persagy.dmp.dpt.repository.TaskCfgRepository;
import com.persagy.dmp.dpt.repository.TaskRecordRepository;
import lombok.Getter;
import lombok.Setter;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.UUID;

public class TaskBaseService {

    @Autowired
    protected TaskCfgRepository taskCfgRepository;

    @Autowired
    protected TaskRecordRepository taskRecordRepository;

    @Autowired
    protected RabbitTemplate rabbitTemplate;

    public static final String USER_ID = "userId";
    public static final String APP_ID = "appId";
    public static final String GROUP_CODE = "groupCode";
    public static final String PROJECT_ID = "projectId";

    @Getter
    @Setter
    protected static class OrgParam {
        String groupCode;
        String userId;
        String appId;
        String projectId;
    }

    protected <T> OrgParam prepareParam(BaseResponse<T> resp, String... requiredColumns) {
        OrgParam param = new OrgParam();
        param.groupCode = DmpParameterStorage.getGroupCode();
        param.userId = DmpParameterStorage.getUserId();
        param.projectId = DmpParameterStorage.getProjectId();
        param.appId = DmpParameterStorage.getAppId();
        if(requiredColumns != null){
            for (int i = 0; i < requiredColumns.length; i++) {
                String column = requiredColumns[i];
                if (USER_ID.equals(column) && (param.userId == null || param.userId.isEmpty())) {
                    resp.setFail("请求地址中的 userId 必须有值");
                    return null;
                }
                if (GROUP_CODE.equals(column) && (param.groupCode == null || param.groupCode.isEmpty())){
                    resp.setFail("请求地址中的 groupCode 必须有值");
                    return null;
                }
                if (PROJECT_ID.equals(column) && (param.projectId == null || param.projectId.isEmpty())){
                    resp.setFail("请求地址中的 projectId 必须有值");
                    return null;
                }
                if (APP_ID.equals(column) && (param.appId == null || param.appId.isEmpty())){
                    resp.setFail("请求地址中的 appId 必须有值");
                    return null;
                }
            }
        }
        return param;
    }

    public static String getUUID(){
        return UUID.randomUUID().toString().replace("-", "");
    }

}
