package com.persagy.dmp.dpt.service;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.dmp.dpt.constant.TaskConstant;
import com.persagy.dmp.dpt.model.TaskRecordModel;
import com.persagy.dmp.rwd.model.DmpMessage;
import com.querydsl.core.types.dsl.BooleanExpression;
import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.web.PagedResponse;
import com.persagy.common.web.MapResponse;
import javax.transaction.Transactional;

import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;
import com.persagy.dmp.dpt.model.TaskCfgModel;
import org.springframework.beans.factory.annotation.Autowired;
import com.persagy.dmp.dpt.entity.*;
import com.persagy.dmp.dpt.repository.TaskCfgRepository;

import java.util.stream.Stream;

@Service
public class TaskCfgService extends TaskBaseService{

	@Autowired
	private TaskCfgRepository taskCfgRepository;

	@Autowired
	private CriteriaUtils criteriaUtils;

	private List<BooleanExpression> parse(ObjectNode criteria) {
		// TODO
		return null;
	}

	public PagedResponse<TaskCfgModel> query(JacksonCriteria criteria) {
		PagedResponse<TaskCfg> resp = criteriaUtils.query(QTaskCfg.taskCfg, this::parse, criteria);
		PagedResponse<TaskCfgModel> result = new PagedResponse<>();
		result.setCount(resp.getCount());
		List<TaskCfg> dataList = resp.getData();
		if (dataList != null && dataList.size() > 0) {
			List<TaskCfgModel> collect = dataList.stream().map(entity -> {
				TaskCfgModel model = entity.toModel();
				return model;
			}).collect(Collectors.toList());
			result.setData(collect);
		}
		return result;
	}

	@Transactional
	public MapResponse create(TaskCfgModel param) {
		MapResponse response = new MapResponse();
		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE, PROJECT_ID, APP_ID);
		if (!response.success()) {
			response.setFail(response.getMessage());
			return response;
		}
		TaskCfg entity = TaskCfg.fromModel(param);

		entity.setId(getUUID());
		entity.setAppId(orgParam.appId);
		entity.setGroupCode(orgParam.groupCode);
		entity.setProjectId(orgParam.projectId);

		ObjectNode config = entity.getConfig();
		System.out.println(config.get(TaskConstant.CONFIG_EXCHANGE).asText());
		if (config.get(TaskConstant.CONFIG_EXCHANGE).asText().isEmpty()){
			response.setFail("exchange 不能为空！");
			return response;
		}
		if (config.get(TaskConstant.CONFIG_ROUTING).asText().isEmpty()){
			response.setFail("routing 不能为空！");
			return response;
		}
		entity.setStatus(1);
		taskCfgRepository.save(entity);
		response.add("id", entity.getId());

		DmpMessage msg = new DmpMessage(TaskConstant.TASK_CFG_CREATE, entity.getId());
		msg.setGroupCode(orgParam.groupCode);
		msg.setProjectId(orgParam.projectId);
		msg.setUserId(orgParam.userId);
		msg.setAppId(orgParam.appId);
		response.add(msg);

		return response;
	}

	@Transactional
	public MapResponse update(TaskCfgModel param) {
		MapResponse response = new MapResponse();
		// TODO
		prepareParam(response, USER_ID, GROUP_CODE, PROJECT_ID, APP_ID);
		if (!response.success()) {
			response.setFail(response.getMessage());
			return response;
		}
		TaskCfg taskCfg = TaskCfg.fromModel(param);

		if (taskCfg.getId() == null){
			response.setFail("更新操作id不能为空！");
			return response;
		}
		Optional<TaskCfg> optional = taskCfgRepository.findById(taskCfg.getId());
		if (!optional.isPresent()){
			response.setFail("数据不存在！");
			return response;
		}
		TaskCfg one = optional.get();

		if (taskCfg.getConfig() != null){
			ObjectNode config = taskCfg.getConfig();
			if (config.get(TaskConstant.CONFIG_EXCHANGE).asText().isEmpty()){
				response.setFail("exchange 不能为空！");
				return response;
			}
			if (config.get(TaskConstant.CONFIG_ROUTING).asText().isEmpty()){
				response.setFail("routing 不能为空！");
				return response;
			}
		}

		/** 集团编码不支持修改 */
		taskCfg.setGroupCode(one.getGroupCode());

		/** 项目 Id 不支持修改 */
		taskCfg.setProjectId(one.getProjectId());

		taskCfgRepository.save(one);
		response.add("id", one.getId());
		return response;
	}

	@Transactional
	public MapResponse delete(TaskCfgModel param) {
		MapResponse response = new MapResponse();
		/** 查询要删除的文件是否存在 */
		Optional<TaskCfg> optional = taskCfgRepository.findById(param.getId());
		if (!optional.isPresent()){
			response.setFail("数据不存在！");
			return response;
		}
		TaskCfg one = optional.get();

		/** 逻辑删除 */
		one.setStatus(0);

		taskCfgRepository.save(one);
		response.add("id", one.getId());
		return response;
	}
}

