package com.persagy.dmp.dpt.service;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.web.MapResponse;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.dpt.entity.QTaskRecord;
import com.persagy.dmp.dpt.entity.TaskCfg;
import com.persagy.dmp.dpt.entity.TaskRecord;
import com.persagy.dmp.dpt.model.TaskRecordModel;
import com.persagy.dmp.dpt.repository.TaskRecordRepository;
import com.querydsl.core.types.dsl.BooleanExpression;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class TaskRecordService extends TaskBaseService{

	@Autowired
	private TaskRecordRepository taskRecordRepository;

	@Autowired
	private CriteriaUtils criteriaUtils;

	private List<BooleanExpression> parse(ObjectNode criteria) {
		// TODO
		return null;
	}

	public PagedResponse<TaskRecordModel> query(JacksonCriteria criteria) {
		PagedResponse<TaskRecord> resp = criteriaUtils.query(QTaskRecord.taskRecord, this::parse, criteria);
		PagedResponse<TaskRecordModel> result = new PagedResponse<>();
		result.setCount(resp.getCount());
		List<TaskRecord> dataList = resp.getData();
		if (dataList != null && dataList.size() > 0) {
			List<TaskRecordModel> collect = dataList.stream().map(entity -> {
				TaskRecordModel model = entity.toModel();
				return model;
			}).collect(Collectors.toList());
			result.setData(collect);
		}
		return result;
	}

	@Transactional
	public MapResponse create(TaskRecordModel param) {
		MapResponse response = new MapResponse();
		// TODO
		TaskRecord entity = TaskRecord.fromModel(param);
		taskRecordRepository.save(entity);
		response.add("id", entity.getId());
		return response;
	}

	@Transactional
	public MapResponse update(TaskRecordModel param) {
		MapResponse response = new MapResponse();
		// TODO
		TaskRecord entity = TaskRecord.fromModel(param);
		taskRecordRepository.save(entity);
		response.add("id", entity.getId());
		return response;
	}

	@Transactional
	public MapResponse delete(TaskRecordModel param) {
		MapResponse response = new MapResponse();
		// TODO
		taskRecordRepository.deleteById(param.getId());
		return response;
	}

	/**
	 * 回调接口保存任务记录
	 * @param taskRecord
	 */
	@Transactional
	public void save(TaskRecord taskRecord){
		taskRecordRepository.save(taskRecord);
	}

	public MapResponse saveAndSendMq(List<String> taskIds) {
		MapResponse response = new MapResponse();

		OrgParam orgParam = prepareParam(response, USER_ID, GROUP_CODE, PROJECT_ID, APP_ID);
		if (!response.success()) {
			response.setFail(response.getMessage());
			return response;
		}
		if (taskIds == null || taskIds.isEmpty()) {
			response.setFail("无数据！");
			return response;
		}
		List<TaskCfg> taskCfgs = taskCfgRepository.findAllById(taskIds);
		if (taskCfgs == null || taskCfgs.isEmpty()){
			response.setFail("没有要发送的数据！");
			return response;
		}

		/** 按级别排序 */
		taskCfgs.sort((t1, t2) -> t2.getLevel().compareTo(t1.getLevel()));

		for (TaskCfg taskCfg : taskCfgs) {
			TaskRecord taskRecord = new TaskRecord();
			String taskId = taskCfg.getId();
			String taskRecordId = getUUID();
			Date startDate = new Date();
			/** 新增 taskRecord */
			taskRecord.setId(taskRecordId);
			taskRecord.setTaskId(taskId);
			taskRecord.setStartTime(startDate);
			taskRecord.setErrorInfo(taskCfg.getConfig());
			taskRecord.setSuccess(0);

			/** 保存任务记录 */
			save(taskRecord);

			/** 获取发送的路由与路由键 */
			String exchange = taskCfg.getConfig().get("exchange").textValue();
			String routing = taskCfg.getConfig().get("routing").textValue();

			/** 封装MQ消息 */
			Map<String, Object> taskMQ = new HashMap<>();
			taskMQ.put("params", taskCfg.getParams());
			taskMQ.put("taskId", taskId);
			taskMQ.put("taskRecordId",taskRecordId);
			taskMQ.put(GROUP_CODE, orgParam.groupCode);
			taskMQ.put(USER_ID, orgParam.userId);
			taskMQ.put(APP_ID, orgParam.appId);
			taskMQ.put(PROJECT_ID, orgParam.projectId);

			/** 发送MQ */
			rabbitTemplate.convertAndSend(exchange, routing, taskMQ);
		}
		return response;
	}

}

